@@ -23,6 +23,7 @@
  * the AssociationList elements; IT NEEDS IT'S OWN CODE! */
 
 define([
+    'dojo/_base/lang',
     'dojo/Deferred',
     './metadata',
     './ipa',
@@ -30,16 +31,26 @@ define([
     './navigation',
     './phases',
     './reg',
+    './rpc',
     './spec_util',
     './text',
     './facet',
     './search',
     './dialog'],
-        function(Deferred, metadata_provider, IPA, $, navigation,
-                 phases, reg, su, text) {
+        function(lang, Deferred, metadata_provider, IPA, $, navigation,
+                 phases, reg, rpc, su, text) {
 
+/**
+ * Association module
+ * @class association
+ * @singleton
+ */
 var exp = {};
 
+/**
+ * Associator base class
+ * @class
+ */
 IPA.associator = function (spec) {
 
     spec = spec || {};
@@ -65,7 +76,10 @@ IPA.associator = function (spec) {
 
 
 /**
+ * Serial associator
  * This associator is built for the case where each association requires a separate rpc
+ * @class
+ * @extends IPA.associator
  */
 IPA.serial_associator = function(spec) {
 
@@ -80,7 +94,7 @@ IPA.serial_associator = function(spec) {
             return;
         }
 
-        var batch = IPA.batch_command({
+        var batch = rpc.batch_command({
             on_success: that.on_success,
             on_error: that.on_error
         });
@@ -92,7 +106,7 @@ IPA.serial_associator = function(spec) {
             options = {};
             options[that.entity.name] = that.pkey;
 
-            command = IPA.command({
+            command = rpc.command({
                 entity: that.other_entity.name,
                 method: that.method,
                 args: args,
@@ -112,6 +126,7 @@ IPA.serial_associator = function(spec) {
 /**
  * This associator is for the common case where all the asociations can be sent
  * in a single rpc
+ * @class
  */
 IPA.bulk_associator = function(spec) {
 
@@ -126,7 +141,7 @@ IPA.bulk_associator = function(spec) {
             return;
         }
 
-        var command = IPA.command({
+        var command = rpc.command({
             entity: that.entity.name,
             method: that.method,
             args: [that.pkey],
@@ -148,6 +163,8 @@ IPA.bulk_associator = function(spec) {
 /**
  * This dialog is for adding value of multivalued attribute which behaves like
  * association attribute.
+ * @class
+ * @extends IPA.entity_adder_dialog
  */
 IPA.attribute_adder_dialog = function(spec) {
 
@@ -206,6 +223,8 @@ IPA.attribute_adder_dialog = function(spec) {
 
 /**
  * This dialog is used for adding associations between two entities.
+ * @class
+ * @extends IPA.adder_dialog
  */
 IPA.association_adder_dialog = function(spec) {
 
@@ -216,6 +235,12 @@ IPA.association_adder_dialog = function(spec) {
     that.entity = IPA.get_entity(spec.entity);
     that.pkey = spec.pkey;
 
+    /**
+     * Map of options for search method
+     * @property {Object}
+     */
+    that.search_options = spec.search_options;
+
     that.other_entity = IPA.get_entity(spec.other_entity);
     that.attribute_member = spec.attribute_member;
 
@@ -260,6 +285,10 @@ IPA.association_adder_dialog = function(spec) {
         }
 
         var options = { all: true };
+        if (that.search_options) {
+            lang.mixin(options, that.search_options);
+        }
+
         var relationships = that.other_entity.metadata.relationships;
 
         /* TODO: better generic handling of different relationships! */
@@ -285,7 +314,7 @@ IPA.association_adder_dialog = function(spec) {
             }
         }
 
-        IPA.command({
+        rpc.command({
             entity: that.other_entity.name,
             method: 'find',
             args: [that.get_filter()],
@@ -310,6 +339,8 @@ IPA.association_adder_dialog = function(spec) {
 
 /**
  * This dialog is used for removing associations between two entities.
+ * @class
+ * @extends IPA.deleter_dialog
  */
 IPA.association_deleter_dialog = function (spec) {
 
@@ -347,7 +378,10 @@ IPA.association_deleter_dialog = function (spec) {
     return that;
 };
 
-
+/**
+ * Association config
+ * @class
+ */
 IPA.association_config = function (spec) {
 
     spec = spec || {};
@@ -362,6 +396,11 @@ IPA.association_config = function (spec) {
     return that;
 };
 
+/**
+ * Association table widget
+ * @class
+ * @extends IPA.table_widget
+ */
 IPA.association_table_widget = function (spec) {
 
     spec = spec || {};
@@ -454,30 +493,24 @@ IPA.association_table_widget = function (spec) {
 
         that.table_create(container);
 
-        that.remove_button = IPA.action_button({
+        that.remove_button = IPA.button_widget({
             name: 'remove',
             label: '@i18n:buttons.remove',
-            icon: 'remove-icon',
-            'class': 'action-button-disabled',
-            click: function() {
-                if (!that.remove_button.hasClass('action-button-disabled')) {
-                    that.remove_handler();
-                }
-                return false;
-            }
-        }).appendTo(that.buttons);
+            icon: 'fa-trash-o',
+            enabled: false,
+            button_class: 'btn btn-link',
+            click: that.remove_handler
+        });
+        that.remove_button.create(that.buttons);
 
-        that.add_button = IPA.action_button({
+        that.add_button = IPA.button_widget({
             name: 'add',
             label: '@i18n:buttons.add',
-            icon: 'add-icon',
-            click: function() {
-                if (!that.add_button.hasClass('action-button-disabled')) {
-                    that.add_handler();
-                }
-                return false;
-            }
-        }).appendTo(that.buttons);
+            icon: 'fa-plus',
+            button_class: 'btn btn-link',
+            click: that.add_handler
+        });
+        that.add_button.create(that.buttons);
     };
 
     that.add_handler = function() {
@@ -493,7 +526,7 @@ IPA.association_table_widget = function (spec) {
                 that.show_add_dialog();
             };
 
-            dialog.open(that.container);
+            dialog.open();
 
         } else {
             that.show_add_dialog();
@@ -513,7 +546,7 @@ IPA.association_table_widget = function (spec) {
                 that.show_remove_dialog();
             };
 
-            dialog.open(that.container);
+            dialog.open();
 
         } else {
             that.show_remove_dialog();
@@ -522,14 +555,13 @@ IPA.association_table_widget = function (spec) {
 
     that.set_enabled = function(enabled) {
         that.table_set_enabled(enabled);
-        if (enabled) {
-            if(that.add_button) {
-                that.add_button.removeClass('action-button-disabled');
-            }
-        } else {
-            $('.action-button', that.table).addClass('action-button-disabled');
+        if (!enabled) {
             that.unselect_all();
         }
+        if (that.add_button) {
+            that.add_button.set_enabled(enabled);
+            that.remove_button.set_enabled(false);
+        }
     };
 
     that.select_changed = function() {
@@ -537,11 +569,7 @@ IPA.association_table_widget = function (spec) {
         var values = that.get_selected_values();
 
         if (that.remove_button) {
-            if (values.length === 0) {
-                that.remove_button.addClass('action-button-disabled');
-            } else {
-                that.remove_button.removeClass('action-button-disabled');
-            }
+            that.remove_button.set_enabled(values.length > 0);
         }
     };
 
@@ -571,7 +599,6 @@ IPA.association_table_widget = function (spec) {
                 that.add_record(that.values[i]);
             }
         }
-        that.updated.notify([], that);
     };
 
     that.create_add_dialog = function() {
@@ -623,14 +650,14 @@ IPA.association_table_widget = function (spec) {
             );
         };
 
-        dialog.open(that.container);
+        dialog.open();
     };
 
     that.add = function(values, on_success, on_error) {
 
         var pkey = that.facet.get_pkey();
 
-        var command = IPA.command({
+        var command = rpc.command({
             entity: that.entity.name,
             method: that.add_method,
             args: [pkey],
@@ -687,14 +714,14 @@ IPA.association_table_widget = function (spec) {
         };
 
 
-        dialog.open(that.container);
+        dialog.open();
     };
 
     that.remove = function(values, on_success, on_error) {
 
         var pkey = that.facet.get_pkey();
 
-        var command = IPA.command({
+        var command = rpc.command({
             entity: that.entity.name,
             method: that.remove_method,
             args: [pkey],
@@ -735,6 +762,11 @@ IPA.association_table_widget = function (spec) {
     return that;
 };
 
+/**
+ * Association table field
+ * @class
+ * @extends IPA.field
+ */
 IPA.association_table_field = function (spec) {
 
     spec = spec || {};
@@ -752,7 +784,7 @@ IPA.association_table_field = function (spec) {
         }
 
         var pkey = that.facet.get_pkey();
-        IPA.command({
+        rpc.command({
             entity: that.entity.name,
             method: 'show',
             args: [pkey],
@@ -771,6 +803,10 @@ IPA.association_table_field = function (spec) {
     return that;
 };
 
+/**
+ * Association facet pre-op
+ * @member association
+ */
 exp.association_facet_pre_op = function(spec, context) {
 
     var has_indirect_attribute_member = function(spec) {
@@ -851,17 +887,17 @@ exp.association_facet_pre_op = function(spec, context) {
         {
             name: 'refresh',
             label: '@i18n:buttons.refresh',
-            icon: 'reset-icon'
+            icon: 'fa-refresh'
         },
         {
             name: 'remove',
             label: '@i18n:buttons.remove',
-            icon: 'remove-icon'
+            icon: 'fa-trash-o'
         },
         {
             name: 'add',
             label: '@i18n:buttons.add',
-            icon: 'add-icon'
+            icon: 'fa-plus'
         });
 
     spec.state = spec.state || {};
@@ -880,6 +916,12 @@ exp.association_facet_pre_op = function(spec, context) {
     return spec;
 };
 
+/**
+ * Association facet
+ * @class association.association_facet
+ * @alternateClassName IPA.association_facet
+ * @extends facet.table_facet
+ */
 exp.association_facet = IPA.association_facet = function (spec, no_init) {
 
     spec = spec || {};
@@ -905,6 +947,12 @@ exp.association_facet = IPA.association_facet = function (spec, no_init) {
 
     that.adder_columns = $.ordered_map();
 
+    /**
+     * Map of search options for adder dialog
+     * @property {Object}
+     */
+    that.search_options = spec.search_options;
+
     that.get_adder_column = function(name) {
         return that.adder_columns.get(name);
     };
@@ -982,57 +1030,58 @@ exp.association_facet = IPA.association_facet = function (spec, no_init) {
         that.facet_create_header(container);
 
         if (that.indirect_attribute_member) {
-
-            var div = $('<div/>', {
-                'class': 'right-aligned-facet-controls'
-            }).appendTo(that.controls);
-
-            div.append(text.get('@i18n:association.show_results'));
-            div.append(' ');
-
-            var name = that.entity.name+'-'+that.attribute_member+'-'+that.other_entity.name+'-type-radio';
-            var direct_id = name + '-direct';
-
-            that.direct_radio = $('<input/>', {
-                id: direct_id,
-                type: 'radio',
-                name: name,
-                value: 'direct',
-                click: function() {
-                    that.association_type = $(this).val();
-                    that.refresh();
-                    return true;
-                }
-            }).appendTo(div);
-
-            $('<label/>', {
-                text: text.get('@i18n:association.direct_membership'),
-                'for': direct_id
-            }).appendTo(div);
-
-            div.append(' ');
-
-            var indirect_id = name + '-indirect';
-
-            that.indirect_radio = $('<input/>', {
-                id: indirect_id,
-                type: 'radio',
-                name: name,
-                value: 'indirect',
-                click: function() {
-                    that.association_type = $(this).val();
-                    that.refresh();
-                    return true;
-                }
-            }).appendTo(div);
-
-            $('<label/>', {
-                text: text.get('@i18n:association.indirect_membership'),
-                'for': indirect_id
-            }).appendTo(div);
+            that.create_membership_radios(that.controls_right);
         }
 
-        that.create_control_buttons(that.controls);
+        that.create_control_buttons(that.controls_left);
+    };
+
+    that.create_membership_radios = function(container) {
+
+        var div = $('<div/>', { 'class': 'association-direction'}).appendTo(container);
+        div.append(text.get('@i18n:association.show_results'));
+        div.append(' ');
+
+        var name = that.entity.name+'-'+that.attribute_member+'-'+that.other_entity.name+'-type-radio';
+        var direct_id = name + '-direct';
+
+        that.direct_radio = $('<input/>', {
+            id: direct_id,
+            type: 'radio',
+            name: name,
+            value: 'direct',
+            click: function() {
+                that.association_type = $(this).val();
+                that.refresh();
+                return true;
+            }
+        }).appendTo(div);
+
+        $('<label/>', {
+            text: text.get('@i18n:association.direct_membership'),
+            'for': direct_id
+        }).appendTo(div);
+
+        div.append(' ');
+
+        var indirect_id = name + '-indirect';
+
+        that.indirect_radio = $('<input/>', {
+            id: indirect_id,
+            type: 'radio',
+            name: name,
+            value: 'indirect',
+            click: function() {
+                that.association_type = $(this).val();
+                that.refresh();
+                return true;
+            }
+        }).appendTo(div);
+
+        $('<label/>', {
+            text: text.get('@i18n:association.indirect_membership'),
+            'for': indirect_id
+        }).appendTo(div);
     };
 
     that.get_attribute_name = function() {
@@ -1068,7 +1117,8 @@ exp.association_facet = IPA.association_facet = function (spec, no_init) {
             pkey: pkey,
             other_entity: that.other_entity,
             attribute_member: that.attribute_member,
-            exclude: pkeys
+            exclude: pkeys,
+            search_options: that.search_options
         });
 
         var adder_columns = that.adder_columns.values;
@@ -1103,7 +1153,7 @@ exp.association_facet = IPA.association_facet = function (spec, no_init) {
             associator.execute();
         };
 
-        dialog.open(that.container);
+        dialog.open();
     };
 
     that.show_remove_dialog = function() {
@@ -1143,6 +1193,7 @@ exp.association_facet = IPA.association_facet = function (spec, no_init) {
                 method: that.remove_method,
                 on_success: function(data) {
                     that.refresh();
+                    that.table.unselect_all();
 
                     var succeeded = IPA.get_succeeded(data);
                     var msg = text.get('@i18n:association.removed').replace('${count}', succeeded);
@@ -1156,7 +1207,7 @@ exp.association_facet = IPA.association_facet = function (spec, no_init) {
             associator.execute();
         };
 
-        dialog.open(that.container);
+        dialog.open();
     };
 
     that.get_records_map = function(data) {
@@ -1186,7 +1237,7 @@ exp.association_facet = IPA.association_facet = function (spec, no_init) {
 
         var pkeys = that.get_pkeys();
 
-        var command = IPA.command({
+        var command = rpc.command({
             entity: that.entity.name,
             method: 'show',
             args: pkeys
@@ -1223,6 +1274,10 @@ exp.association_facet = IPA.association_facet = function (spec, no_init) {
     return that;
 };
 
+/**
+ * Attribute facet pre-op
+ * @member association
+ */
 exp.attribute_facet_pre_op = function(spec, context) {
 
     var entity = context.entity;
@@ -1266,17 +1321,17 @@ exp.attribute_facet_pre_op = function(spec, context) {
         {
             name: 'refresh',
             label: '@i18n:buttons.refresh',
-            icon: 'reset-icon'
+            icon: 'fa-refresh'
         },
         {
             name: 'remove',
             label: '@i18n:buttons.remove',
-            icon: 'remove-icon'
+            icon: 'fa-trash-o'
         },
         {
             name: 'add',
             label: '@i18n:buttons.add',
-            icon: 'add-icon'
+            icon: 'fa-plus'
         });
 
     spec.state = spec.state || {};
@@ -1295,6 +1350,12 @@ exp.attribute_facet_pre_op = function(spec, context) {
     return spec;
 };
 
+/**
+ * Association facet
+ * @class association.attribute_facet
+ * @alternateClassName IPA.attribute_facet
+ * @extends facet.table_facet
+ */
 exp.attribute_facet = IPA.attribute_facet = function(spec, no_init) {
 
     spec = spec || {};
@@ -1337,7 +1398,7 @@ exp.attribute_facet = IPA.attribute_facet = function(spec, no_init) {
 
         var pkey = that.get_pkeys();
 
-        var command = IPA.command({
+        var command = rpc.command({
             entity: that.entity.name,
             method: 'show',
             args: pkey
@@ -1379,7 +1440,7 @@ exp.attribute_facet = IPA.attribute_facet = function(spec, no_init) {
         dialog.added.attach(function() {
             that.refresh();
         });
-        dialog.open(that.container);
+        dialog.open();
     };
 
     that.show_remove_dialog = function() {
@@ -1403,6 +1464,7 @@ exp.attribute_facet = IPA.attribute_facet = function(spec, no_init) {
                 function(data) {
                     that.load(data);
                     that.show_content();
+                    that.table.unselect_all();
 
                     var succeeded = IPA.get_succeeded(data);
                     var msg = text.get('@i18n:association.removed').replace('${count}', succeeded);
@@ -1415,14 +1477,14 @@ exp.attribute_facet = IPA.attribute_facet = function(spec, no_init) {
         };
 
 
-        dialog.open(that.container);
+        dialog.open();
     };
 
     that.remove = function(values, on_success, on_error) {
 
         var pkey = that.get_pkeys();
 
-        var command = IPA.command({
+        var command = rpc.command({
             entity: that.entity.name,
             method: that.remove_method,
             args: pkey,
@@ -1455,6 +1517,11 @@ exp.attribute_facet = IPA.attribute_facet = function(spec, no_init) {
     return that;
 };
 
+/**
+ * Attriute read-only evaluator
+ * @class IPA.attr_read_only_evaluator
+ * @extends IPA.state_evaluator
+ */
 IPA.attr_read_only_evaluator = function(spec) {
 
     spec.name = spec.name || 'attr_read_only_evaluator';
