@@ -19,18 +19,22 @@
  */
 
 define([
+    'dojo/_base/declare',
+    './field',
     './metadata',
     './ipa',
     './jquery',
     './navigation',
     './phases',
     './reg',
+    './rpc',
     './text',
     './details',
     './search',
     './association',
     './entity'],
-    function(metadata_provider, IPA, $, navigation, phases, reg, text) {
+    function(declare, field_mod, metadata_provider, IPA, $, navigation,
+        phases, reg, rpc, text) {
 
 var exp = IPA.automember = {};
 
@@ -208,7 +212,7 @@ IPA.automember.rule_search_facet = function(spec) {
         }
         dialog.pkey_prefix = that.managed_entity_pkey_prefix();
         dialog.group_type = that.group_type;
-        dialog.open(that.container);
+        dialog.open();
     };
 
     that.create_remove_dialog = function() {
@@ -250,7 +254,7 @@ IPA.automember.rule_details_facet = function(spec) {
 
     spec.widgets = [
             {
-            $type: 'details_table_section',
+            $type: 'details_section',
             name: 'general',
             label: '@i18n:details.general',
             widgets: [
@@ -264,7 +268,7 @@ IPA.automember.rule_details_facet = function(spec) {
             ]
         },
         {
-            $factory: IPA.collapsible_section,
+            $factory: IPA.section,
             name: 'inclusive',
             label: '@i18n:objects.automember.inclusive',
             widgets: [
@@ -285,7 +289,8 @@ IPA.automember.rule_details_facet = function(spec) {
                             },
                             {
                                 name: 'automemberinclusiveregex',
-                                label: '@i18n:objects.automember.expression'
+                                label: '@i18n:objects.automember.expression',
+                                required: true
                             }
                         ]
                     }
@@ -293,7 +298,7 @@ IPA.automember.rule_details_facet = function(spec) {
             ]
         },
         {
-            $factory: IPA.collapsible_section,
+            $factory: IPA.section,
             name: 'exclusive',
             label: '@i18n:objects.automember.exclusive',
             widgets: [
@@ -314,7 +319,8 @@ IPA.automember.rule_details_facet = function(spec) {
                             },
                             {
                                 name: 'automemberexclusiveregex',
-                                label: '@i18n:objects.automember.expression'
+                                label: '@i18n:objects.automember.expression',
+                                required: true
                             }
                         ]
                     }
@@ -442,32 +448,31 @@ IPA.automember.parse_condition_regex = function(regex) {
 IPA.automember.condition_field = function(spec) {
 
     spec = spec || {};
+    spec.adapter = spec.adapter || IPA.automember.condition_adapter;
     var that = IPA.field(spec);
-
-    that.attr_name = spec.attribute || that.name;
-
-    that.load = function(record) {
-
-        var regexes = record[that.attr_name];
-        that.values = [];
-
-        if (regexes) {
-            for (var i=0, j=0; i<regexes.length; i++) {
-                var condition = IPA.automember.parse_condition_regex(regexes[i]);
-                that.values.push(condition);
-            }
-        }
-
-        that.load_writable(record);
-        that.reset();
-    };
-
     return that;
 };
 
+IPA.automember.condition_adapter = declare([field_mod.Adapter], {
+
+    load: function(data) {
+        var regexes = this.inherited(arguments);
+        var values = [];
+        if (regexes) {
+            for (var i=0, j=0; i<regexes.length; i++) {
+                if (regexes[i] === '') continue;
+                var condition = IPA.automember.parse_condition_regex(regexes[i]);
+                values.push(condition);
+            }
+        }
+        return values;
+    }
+});
+
 IPA.automember.condition_widget = function(spec) {
 
     spec = spec || {};
+    spec.footer = spec.footer === undefined ? false : spec.footer;
 
     spec.columns = $.merge(spec.columns || [], [
         {
@@ -522,7 +527,7 @@ IPA.automember.condition_widget = function(spec) {
 
     that.create_remove_command = function(values, on_success, on_error) {
 
-        var batch = IPA.batch_command({
+        var batch = rpc.batch_command({
             name: 'automember_remove_condition',
             on_success: on_success,
             on_error: on_error
@@ -612,6 +617,7 @@ IPA.automember.default_group_widget = function(spec) {
         }
 
         that.update(group);
+        that.set_visible(true);
     };
 
     that.update = function(group) {
@@ -626,7 +632,7 @@ IPA.automember.default_group_widget = function(spec) {
         method = 'default_group_' + method;
         var command_name = that.entity.name + that.group_type + '_' + method;
 
-        var command  = IPA.command({
+        var command  = rpc.command({
             name: command_name,
             entity: that.entity.name,
             method: method,
@@ -641,7 +647,11 @@ IPA.automember.default_group_widget = function(spec) {
     that.refresh = function() {
 
         var command = that.create_command('show');
+        command.retry = false;
         command.on_success = that.load;
+        command.on_error = function() {
+            that.set_visible(false);
+        };
 
         command.execute();
     };
@@ -672,18 +682,20 @@ IPA.automember.default_group_widget = function(spec) {
     that.create = function(container) {
 
         var title = that.get_title();
+        that.container = container;
 
-        var default_group = $('<div />', {
-            'class': 'default_group'
-        }).appendTo(container);
-
-        that.header = $('<h2/>', {
+        that.header = $('<div/>', {
+            'class': 'automember-header-label',
             name: 'header',
             text: title,
             title: title
-        }).appendTo(default_group);
+        }).appendTo(container);
 
-        that.group_select.create(default_group);
+        that.group_select_node = $('<div/>', { 'class': 'automember-header-control' });
+        that.group_select.create(that.group_select_node);
+        that.group_select_node.appendTo(container);
+        that.group_select.update([]); // preload groups
+        that.set_visible(false);
     };
 
     that.get_title = function() {
@@ -699,12 +711,57 @@ IPA.automember.default_group_widget = function(spec) {
     return that;
 };
 
+IPA.automember.rebuild_action = function(spec) {
+
+    spec = spec || {};
+    spec.name = spec.name || 'automember_rebuild';
+    spec.label = spec.label || '@i18n:actions.automember_rebuild';
+    spec.needs_confirm = spec.needs_confirm !== undefined ? spec.needs_confirm : true;
+    spec.confirm_msg = spec.confirm_msg || '@i18n:actions.automember_rebuild_confirm';
+
+    var that = IPA.action(spec);
+
+    that.execute_action = function(facet) {
+        var entity = facet.entity.name;
+        if (facet.name == 'search') {
+            var entries = facet.get_selected_values();
+        } else {
+            entries = facet.get_pkeys();
+        }
+
+        var options = {};
+        if (entries.length > 0) {
+            options[entity + 's'] = entries;
+        } else if (entity == 'user') {
+            options['type'] = 'group';
+        } else {
+            options['type'] = 'hostgroup';
+        }
+
+        var command = rpc.command({
+            entity: 'automember',
+            method: 'rebuild',
+            options: options,
+            on_success: function() {
+                IPA.notify_success('@i18n:actions.automember_rebuild_success');
+            },
+            on_error: function() {
+            }
+        });
+
+        command.execute();
+    };
+
+    return that;
+};
+
 exp.entity_spec = make_spec();
 
 exp.register = function() {
     var e = reg.entity;
     var w = reg.widget;
     var f = reg.field;
+    var a = reg.action;
 
     e.register({
         type: 'automember',
@@ -713,6 +770,7 @@ exp.register = function() {
     });
     w.register('automember_condition', IPA.automember.condition_widget);
     f.register('automember_condition', IPA.automember.condition_field);
+    a.register('automember_rebuild', exp.rebuild_action);
 };
 
 phases.on('registration', exp.register);
