@@ -27,9 +27,10 @@ define([
     './menu',
     './phases',
     './reg',
+    './rpc',
     './text',
     './dialog'],
-        function(lang, metadata_provider, IPA, $, menu, phases, reg, text) {
+        function(lang, metadata_provider, IPA, $, menu, phases, reg, rpc, text) {
 
 var exp = IPA.cert = {};
 
@@ -197,7 +198,7 @@ IPA.cert.download_dialog = function(spec) {
         }
     });
 
-    that.create = function() {
+    that.create_content = function() {
         var textarea = $('<textarea/>', {
             'class': 'certificate',
             readonly: 'yes'
@@ -227,7 +228,7 @@ IPA.cert.revoke_dialog = function(spec) {
         return that.select.val();
     };
 
-    that.create = function() {
+    that.create_content = function() {
 
         var table = $('<table/>').appendTo(that.container);
 
@@ -286,7 +287,7 @@ IPA.cert.view_dialog = function(spec) {
         }
     });
 
-    that.create = function() {
+    that.create_content = function() {
 
         var table = $('<table/>').appendTo(that.container);
 
@@ -424,7 +425,7 @@ IPA.cert.request_dialog = function(spec) {
         }
     });
 
-    that.create = function() {
+    that.create_content = function() {
         that.container.append(that.message);
 
         that.textarea = $('<textarea/>', {
@@ -517,11 +518,13 @@ IPA.cert.load_policy = function(spec) {
     };
 
     that.load_revocation_reason = function(serial_number) {
-        IPA.command({
+        rpc.command({
             entity: 'cert',
             method: 'show',
             args: [serial_number],
             on_success: function(data, text_status, xhr) {
+                // copy it so consumers can notice the difference
+                that.container.certificate = lang.clone(that.container.certificate);
                 var cert = that.container.certificate;
                 cert.revocation_reason = data.result.result.revocation_reason;
                 that.notify_loaded();
@@ -547,7 +550,7 @@ IPA.cert.view_action = function(spec) {
 
     spec = spec || {};
     spec.name = spec.name || 'view_cert';
-    spec.label = spec.label || '@i18n:buttons.view';
+    spec.label = spec.label || '@i18n:objects.cert.view_certificate_btn';
     spec.enable_cond = spec.enable_cond || ['has_certificate'];
 
     var that = IPA.action(spec);
@@ -580,7 +583,7 @@ IPA.cert.get_action = function(spec) {
 
     spec = spec || {};
     spec.name = spec.name || 'get_cert';
-    spec.label = spec.label || '@i18n:buttons.get';
+    spec.label = spec.label || '@i18n:objects.cert.get_certificate';
     spec.enable_cond = spec.enable_cond || ['has_certificate'];
 
     var that = IPA.action(spec);
@@ -643,7 +646,7 @@ IPA.cert.request_action = function(spec) {
             message: request_message,
             request: function(values) {
 
-                IPA.command({
+                rpc.command({
                     entity: 'cert',
                     method: 'request',
                     args: [values.request],
@@ -669,7 +672,7 @@ IPA.cert.revoke_action = function(spec) {
 
     spec = spec || {};
     spec.name = spec.name || 'revoke_cert';
-    spec.label = spec.label || '@i18n:buttons.revoke';
+    spec.label = spec.label || '@i18n:objects.cert.revoke_certificate_simple';
     spec.enable_cond = spec.enable_cond || ['has_certificate'];
     spec.disable_cond = spec.disable_cond || ['certificate_revoked'];
     spec.hide_cond = spec.hide_cond || ['ra_disabled'];
@@ -702,7 +705,7 @@ IPA.cert.revoke_action = function(spec) {
 
         var certificate = facet.certificate;
 
-        IPA.command({
+        rpc.command({
             entity: 'cert',
             method: 'revoke',
             args: [certificate.serial_number],
@@ -724,7 +727,7 @@ IPA.cert.restore_action = function(spec) {
 
     spec = spec || {};
     spec.name = spec.name || 'restore_cert';
-    spec.label = spec.label || '@i18n:buttons.restore';
+    spec.label = spec.label || '@i18n:objects.cert.restore_certificate_simple';
     spec.enable_cond = spec.enable_cond || ['has_certificate', 'certificate_hold'];
     spec.hide_cond = spec.hide_cond || ['ra_disabled'];
     spec.confirm_msg = spec.confirm_msg || '@i18n:objects.cert.restore_confirmation';
@@ -759,7 +762,7 @@ IPA.cert.restore_action = function(spec) {
 
         var certificate = facet.certificate;
 
-        IPA.command({
+        rpc.command({
             entity: 'cert',
             method: 'remove_hold',
             args: [certificate.serial_number],
@@ -823,60 +826,44 @@ IPA.cert.status_widget = function(spec) {
 
         that.widget_create(container);
 
-        that.status_valid = $('<div/>', {
-            name: 'certificate-valid',
-            style: 'display: none;'
-        }).appendTo(container);
+        that.status_valid = that.create_status('certificate-valid',
+                                               text.get('@i18n:objects.cert.valid'),
+                                               'fa fa-check');
+        that.status_valid.appendTo(container);
 
-        $('<img/>', {
-            src: 'images/check-icon.png',
-            style: 'float: left;',
-            'class': 'status-icon'
-        }).appendTo(that.status_valid);
+        that.status_revoked = that.create_status('certificate-revoked',
+                                               text.get('@i18n:objects.cert.revoked'),
+                                               'fa fa-warning');
+        that.status_revoked.appendTo(container);
 
-        var content_div = $('<div/>', {
-            style: 'float: left;'
-        }).appendTo(that.status_valid);
-
-        content_div.append('<b>'+text.get('@i18n:objects.cert.valid')+'</b>');
-
-        that.status_revoked = $('<div/>', {
-            name: 'certificate-revoked',
-            style: 'display: none;'
-        }).appendTo(container);
-
-        $('<img/>', {
-            src: 'images/caution-icon.png',
-            style: 'float: left;',
-            'class': 'status-icon'
-        }).appendTo(that.status_revoked);
-
-        content_div = $('<div/>', {
-            style: 'float: left;'
-        }).appendTo(that.status_revoked);
-
-        content_div.append('<b>'+text.get('@i18n:objects.cert.revoked')+'</b>');
-        content_div.append(' ');
         that.revocation_reason = $('<span/>', {
             'name': 'revocation_reason'
-        }).appendTo(content_div);
+        }).appendTo(that.status_revoked);
 
-        that.status_missing = $('<div/>', {
-            name: 'certificate-missing',
+        that.status_missing = that.create_status('certificate-missing',
+                                               text.get('@i18n:objects.cert.missing'),
+                                               'fa fa-warning');
+        that.status_missing.appendTo(container);
+    };
+
+    that.create_status = function(name, text, icon) {
+
+        var container = $('<div>', {
+            name: name,
             style: 'display: none;'
+        });
+
+        var status = $('<label/>', {
+            'class': 'certificate-status'
         }).appendTo(container);
 
-        $('<img/>', {
-            src: 'images/caution-icon.png',
-            style: 'float: left;',
-            'class': 'status-icon'
-        }).appendTo(that.status_missing);
+        $('<i/>', {
+            'class': icon
+        }).appendTo(status);
 
-        content_div = $('<div/>', {
-            style: 'float: left;'
-        }).appendTo(that.status_missing);
+        status.append(" " +text);
 
-        content_div.append('<b>'+text.get('@i18n:objects.cert.missing')+'</b>');
+        return container;
     };
 
     that.update = function(certificate) {
@@ -893,7 +880,7 @@ IPA.cert.status_widget = function(spec) {
             status = IPA.cert.CERTIFICATE_STATUS_REVOKED;
         }
         that.set_status(status, certificate.revocation_reason);
-        that.updated.notify([], that);
+        that.on_value_changed();
     };
 
     that.clear = function() {
@@ -911,7 +898,9 @@ IPA.cert.status_widget = function(spec) {
             that.status_revoked.css('display', status === IPA.cert.CERTIFICATE_STATUS_REVOKED ? '' : 'none');
 
             var reason = IPA.cert.CRL_REASON[revocation_reason];
-            that.revocation_reason.html(revocation_reason === undefined || reason === null ? '' : text.get('@i18n:objects.cert.'+reason));
+            var reason_text = revocation_reason === undefined || reason === null ? '' : text.get('@i18n:objects.cert.'+reason);
+            reason_text = ' ('+reason_text+')';
+            that.revocation_reason.html(reason_text);
         }
     };
 
@@ -927,12 +916,11 @@ IPA.cert.status_field = function(spec) {
 
     that.load = function(result) {
         that.register_listener();
-        that.reset();
+        that.field_load(result);
     };
 
     that.set_certificate = function(certificate) {
-        that.values = certificate;
-        that.reset();
+        that.set_value(certificate);
     };
 
     that.register_listener = function() {
@@ -1140,17 +1128,11 @@ return {
                     IPA.cert.certificate_evaluator
                 ]
             },
+            header_actions: ['revoke_cert', 'restore_cert'],
             sections: [
                 {
                     name: 'details',
                     label: '@i18n:objects.cert.certificate',
-                    action_panel: {
-                        $factory: IPA.action_panel,
-                        name: 'cert_actions',
-                        actions: [
-                            'revoke_cert', 'restore_cert'
-                        ]
-                    },
                     fields: [
                         'serial_number',
                         'serial_number_hex',
@@ -1203,7 +1185,7 @@ IPA.cert.search_facet = function(spec) {
 
         that.search_option = $('<select/>', {
             name: 'search_option',
-            'class': 'search-option'
+            'class': 'search-option form-control'
         });
 
         that.filter_container.before(that.search_option);
@@ -1252,7 +1234,7 @@ IPA.cert.search_facet = function(spec) {
     that.show = function() {
         that.search_facet_show();
 
-        if (that.search_option) {
+        if (that.search_option && that.state.search_option) {
             that.search_option.val(that.state.search_option);
         }
     };
@@ -1311,7 +1293,7 @@ IPA.cert.cert_update_policy = function(spec) {
 
 exp.remove_menu_item = function() {
     if (!IPA.cert.is_enabled()) {
-        menu.remove_item('identity/cert');
+        menu.remove_item('authentication/cert');
     }
 };
 
