@@ -21,19 +21,24 @@
 
 
 define([
+    'dojo/_base/declare',
     './ipa',
     './jquery',
     './net',
+    './field',
     './navigation',
     './menu',
     './phases',
     './reg',
+    './rpc',
+    './util',
     './text',
     './details',
     './search',
     './association',
     './entity'],
-       function(IPA, $, NET, navigation, menu, phases, reg, text) {
+       function(declare, IPA, $, NET, field_mod, navigation, menu, phases,
+        reg, rpc, util, text) {
 
 var exp = IPA.dns = {
     zone_permission_name: 'Manage DNS zone ${dnszone}'
@@ -119,12 +124,12 @@ return {
                 {
                     name: 'disable',
                     label: '@i18n:buttons.disable',
-                    icon: 'disabled-icon'
+                    icon: 'fa-minus'
                 },
                 {
                     name: 'enable',
                     label: '@i18n:buttons.enable',
-                    icon: 'enabled-icon'
+                    icon: 'fa-check'
                 }
             ]
         },
@@ -213,7 +218,12 @@ return {
                     {
                         $type: 'checkbox',
                         name: 'idnsallowsyncptr'
-                    }
+                    },
+                    {
+                        $type: 'checkbox',
+                        name: 'idnssecinlinesigning'
+                    },
+                    'nsec3paramrecord'
                 ]
             }],
             actions: [
@@ -224,7 +234,7 @@ return {
                 'dns_add_permission',
                 'dns_remove_permission'
             ],
-            header_actions: ['select_action', 'enable', 'disable', 'delete',
+            header_actions: ['enable', 'disable', 'delete',
                 'add_permission', 'remove_permission'],
             state: {
                 evaluators: [
@@ -277,8 +287,8 @@ return {
         height: 300,
         sections: [
             {
-                $factory: IPA.dnszone_name_section,
                 name: 'name',
+                layout: IPA.dnszone_name_section_layout,
                 fields: [
                     {
                         $type: 'dnszone_name',
@@ -321,6 +331,165 @@ return {
     }
 };};
 
+var make_forwardzone_spec = function() {
+return {
+    name: 'dnsforwardzone',
+    enable_test: function() {
+        return IPA.dns_enabled;
+    },
+    facet_groups: [ 'dnsrecord', 'settings' ],
+    facets: [
+        {
+            $type: 'search',
+            row_enabled_attribute: 'idnszoneactive',
+            title: '@mo:dnszone.label',
+            columns: [
+                'idnsname',
+                {
+                    name: 'idnszoneactive',
+                    label: '@i18n:status.label',
+                    formatter: 'boolean_status'
+                },
+                'idnsforwarders',
+                'idnsforwardpolicy'
+            ],
+            actions: [
+                'batch_disable',
+                'batch_enable'
+            ],
+            control_buttons: [
+                {
+                    name: 'disable',
+                    label: '@i18n:buttons.disable',
+                    icon: 'fa-minus'
+                },
+                {
+                    name: 'enable',
+                    label: '@i18n:buttons.enable',
+                    icon: 'fa-check'
+                }
+            ]
+        },
+        {
+            $type: 'details',
+            $factory: IPA.dnszone_details_facet,
+            command_mode: 'info',
+            sections: [
+            {
+                name: 'identity',
+                fields: [
+                    'idnsname',
+                    {
+                        $type: 'multivalued',
+                        name: 'idnsforwarders',
+                        validators: ['dnsforwarder']
+                    },
+                    {
+                        $type: 'radio',
+                        name: 'idnsforwardpolicy',
+                        default_value: 'first',
+                        options: [
+                            {
+                                value: 'first',
+                                label: '@i18n:objects.dnsconfig.forward_first'
+                            },
+                            {
+                                value: 'only',
+                                label: '@i18n:objects.dnsconfig.forward_only'
+                            },
+                            {
+                                value: 'none',
+                                label: '@i18n:objects.dnsconfig.forward_none'
+                            }
+                        ]
+                    }
+                ]
+            }],
+            actions: [
+                'enable',
+                'disable',
+                'delete',
+                'dns_add_permission',
+                'dns_remove_permission'
+            ],
+            header_actions: ['enable', 'disable', 'delete',
+                'add_permission', 'remove_permission'],
+            state: {
+                evaluators: [
+                    {
+                        $factory: IPA.enable_state_evaluator,
+                        field: 'idnszoneactive'
+                    },
+                    {
+                        $factory: IPA.acl_state_evaluator,
+                        attribute: 'managedby'
+                    },
+                    IPA.dns.zone_has_permission_evaluator
+                ],
+                summary_conditions: [
+                    IPA.enabled_summary_cond,
+                    IPA.disabled_summary_cond
+                ]
+            }
+        }
+    ],
+    adder_dialog: {
+        sections: [
+            {
+                name: 'name',
+                layout: IPA.dnszone_name_section_layout,
+                fields: [
+                    {
+                        $type: 'dnszone_name',
+                        name: 'idnsname',
+                        required: false,
+                        radio_name: 'dnszone_name_type'
+                    },
+                    {
+                        $type: 'dnszone_name',
+                        name: 'name_from_ip',
+                        radio_name: 'dnszone_name_type',
+                        validators: ['network']
+                    }
+                ]
+            },
+            {
+                name: 'forwarders',
+                fields: [
+                    {
+                        $type: 'multivalued',
+                        name: 'idnsforwarders',
+                        required: true,
+                        validators: ['dnsforwarder']
+                    },
+                    {
+                        $type: 'radio',
+                        name: 'idnsforwardpolicy',
+                        default_value: 'first',
+                        options: [
+                            {
+                                value: 'first',
+                                label: '@i18n:objects.dnsconfig.forward_first'
+                            },
+                            {
+                                value: 'only',
+                                label: '@i18n:objects.dnsconfig.forward_only'
+                            },
+                            {
+                                value: 'none',
+                                label: '@i18n:objects.dnsconfig.forward_none'
+                            }
+                        ]
+                    }
+                ]
+            }
+        ],
+        policies: [
+            IPA.add_dns_zone_name_policy
+        ]
+    }
+};};
+
 
 IPA.dnszone_details_facet = function(spec, no_init) {
 
@@ -330,34 +499,20 @@ IPA.dnszone_details_facet = function(spec, no_init) {
     that.permission_load = IPA.observer();
     that.permission_status = 'unknown'; // [unknown, set, none]
 
-    that.refresh_on_success = function(data, text_status, xhr) {
-        // do not load data from batch
-
-        that.show_content();
-    };
-
     that.create_refresh_command = function() {
 
         var pkey = that.get_pkey();
 
-        var batch = IPA.batch_command({
-            name: 'dnszone_details_refresh'
+        var batch = rpc.batch_command({
+            name: this.entity.name + '_details_refresh'
         });
 
         var dnszone_command = that.details_facet_create_refresh_command();
-
-        dnszone_command.on_success = function(data, text_status, xhr) {
-            // create data that mimics dnszone-show output
-            var dnszone_data = {};
-            dnszone_data.result = data;
-            that.load(dnszone_data);
-        };
-
         batch.add_command(dnszone_command);
 
         var permission_name = IPA.dns.zone_permission_name.replace('${dnszone}', pkey);
 
-        var permission_command = IPA.command({
+        var permission_command = rpc.command({
             entity: 'permission',
             method: 'show',
             args: [permission_name],
@@ -401,93 +556,26 @@ IPA.dnszone_details_facet = function(spec, no_init) {
     return that;
 };
 
-IPA.dnszone_name_section = function(spec) {
+IPA.dnszone_name_section_layout = function(spec) {
 
-    spec = spec || {};
+    var that = IPA.fluid_layout(spec);
 
-    var that = IPA.details_table_section(spec);
+    that.radio_names = spec.radio_names || 'idnsname_method';
 
-    that.create = function(container) {
-        that.container = container;
+    that.create_label = function(widget) {
 
-        that.message_container = $('<div/>', {
-            style: 'display: none',
-            'class': 'dialog-message ui-state-highlight ui-corner-all'
-        }).appendTo(that.container);
+        var label_text = widget.label + that.get_measurement_unit_text(widget);
 
-        var table = $('<table/>', {
-            'class': 'section-table'
-        }).appendTo(that.container);
+        var label_cont = $('<div/>', { 'class': that.label_cls });
 
-        var idnsname = that.widgets.get_widget('idnsname');
+        widget.create_radio(label_cont);
 
-        var tr = $('<tr/>').appendTo(table);
-
-        var td = $('<td/>', {
-            'class': 'section-cell-label',
-            title: idnsname.label
-        }).appendTo(tr);
-
-        var label = $('<label/>', {
-            name: 'idnsname',
-            'class': 'field-label',
-            'for': idnsname.radio_id
-        }).appendTo(td);
-
-        idnsname.create_radio(label);
-
-        label.append(idnsname.label+':');
-
-        idnsname.create_required(td);
-
-        td = $('<td/>', {
-            'class': 'section-cell-field',
-            title: idnsname.label
-        }).appendTo(tr);
-
-        var widget_cont = $('<div/>', {
-            name: 'idnsname',
-            'class': 'field'
-        }).appendTo(td);
-
-        idnsname.create(widget_cont);
-
-        var idnsname_input = $('input', widget_cont);
-
-        var name_from_ip = that.widgets.get_widget('name_from_ip');
-
-        tr = $('<tr/>').appendTo(table);
-
-        td = $('<td/>', {
-            'class': 'section-cell-label',
-            title: name_from_ip.label
-        }).appendTo(tr);
-
-        label = $('<label/>', {
-            name: 'name_from_ip',
-            'class': 'field-label',
-            'for': name_from_ip.radio_id
-        }).appendTo(td);
-
-        name_from_ip.create_radio(label);
-
-        label.append(name_from_ip.label+':');
-
-        name_from_ip.create_required(td);
-
-        td = $('<td/>', {
-            'class': 'section-cell-field',
-            title: name_from_ip.label
-        }).appendTo(tr);
-
-        widget_cont = $('<div/>', {
-            name: 'name_from_ip',
-            'class': 'field'
-        }).appendTo(td);
-
-        name_from_ip.create(widget_cont);
-
-        idnsname.radio.click();
+        $('<label/>', {
+            name: widget.name,
+            'for': widget.radio_id,
+            text: label_text
+        }).appendTo(label_cont);
+        return label_cont;
     };
 
 
@@ -526,6 +614,13 @@ IPA.add_dns_zone_name_policy = function() {
         });
     };
 
+    that.post_create = function() {
+
+        var idnsname_w = this.container.widgets.get_widget('name.idnsname');
+        idnsname_w.radio.prop('checked', true);
+        idnsname_w.radio_clicked.notify([], idnsname_w);
+    };
+
     return that;
 };
 
@@ -559,6 +654,7 @@ IPA.dnszone_name_widget = function(spec) {
             value: that.name,
             click: function() {
                 that.radio_clicked.notify([], that);
+                that.emit('radio-click', { source: that });
             }
         }).appendTo(container);
     };
@@ -631,8 +727,8 @@ IPA.dnszone_adder_dialog = function(spec) {
         }
     };
 
-    that.create = function() {
-        that.entity_adder_dialog_create();
+    that.create_content = function() {
+        that.entity_adder_dialog_create_content();
         that.container.addClass('dnszone-adder-dialog');
     };
 
@@ -645,29 +741,13 @@ IPA.dns.add_permission_action = function(spec) {
 
     spec = spec || {};
     spec.name = spec.name || 'add_permission';
+    spec.method = spec.method || 'add_permission';
     spec.label = spec.label || '@i18n:objects.dnszone.add_permission';
     spec.enable_cond = spec.enable_cond || ['permission-none', 'managedby_w'];
+    spec.needs_confirm = spec.needs_confirm !== undefined ? spec.needs_confirm : true;
+    spec.confirm_msg = spec.confirm_msg || '@i18n:objects.dnszone.add_permission_confirm';
 
-    var that = IPA.action(spec);
-
-    that.execute_action = function(facet) {
-
-        var pkey = facet.get_pkey();
-
-         var command = IPA.command({
-            entity: 'dnszone',
-            method: 'add_permission',
-            args: [pkey],
-            options: {},
-            on_success: function(data, text_status, xhr) {
-                facet.refresh();
-                IPA.notify_success(data.result.summary);
-            }
-        });
-
-        command.execute();
-    };
-
+    var that = IPA.object_action(spec);
     return that;
 };
 
@@ -675,29 +755,13 @@ IPA.dns.remove_permission_action = function(spec) {
 
     spec = spec || {};
     spec.name = spec.name || 'remove_permission';
+    spec.method = spec.method || 'remove_permission';
     spec.label = spec.label || '@i18n:objects.dnszone.remove_permission';
     spec.enable_cond = spec.enable_cond || ['permission-set', 'managedby_w'];
+    spec.needs_confirm = spec.needs_confirm !== undefined ? spec.needs_confirm : true;
+    spec.confirm_msg = spec.confirm_msg || '@i18n:objects.dnszone.remove_permission_confirm';
 
-    var that = IPA.action(spec);
-
-    that.execute_action = function(facet) {
-
-        var pkey = facet.get_pkey();
-
-         var command = IPA.command({
-            entity: 'dnszone',
-            method: 'remove_permission',
-            args: [pkey],
-            options: {},
-            on_success: function(data, text_status, xhr) {
-                facet.refresh();
-                IPA.notify_success(data.result.summary);
-            }
-        });
-
-        command.execute();
-    };
-
+    var that = IPA.object_action(spec);
     return that;
 };
 
@@ -727,7 +791,7 @@ IPA.dns.record_search_facet = function(spec) {
 
     that.get_records = function(pkeys, on_success, on_error) {
 
-        var batch = IPA.batch_command({
+        var batch = rpc.batch_command({
             name: that.get_records_command_name(),
             on_success: on_success,
             on_error: on_error
@@ -738,7 +802,7 @@ IPA.dns.record_search_facet = function(spec) {
         for (var i=0; i<pkeys.length; i++) {
             var pkey = pkeys[i];
 
-            var command = IPA.command({
+            var command = rpc.command({
                 entity: that.table.entity.name,
                 method: 'show',
                 args: [zone, pkey],
@@ -761,7 +825,7 @@ IPA.dns.record_search_facet = function(spec) {
 
             var original = records[i];
             var record = {
-                idnsname: original.idnsname,
+                idnsname: rpc.extract_objects(original.idnsname),
                 values: []
             };
 
@@ -926,18 +990,18 @@ IPA.dns.get_record_metadata = function() {
                       'ds_part_digest_type']
         },
         {
-            name: 'keyrecord',
+            name: 'dlvrecord',
             attributes: [
-                'key_part_flags',
-                'key_part_protocol',
-                'key_part_algorithm',
+                'dlv_part_key_tag',
+                'dlv_part_algorithm',
+                'dlv_part_digest_type',
                 {
-                    name: 'key_part_public_key',
+                    name: 'dlv_part_digest',
                     $type: 'textarea'
                 }
             ],
-            columns: ['key_part_flags', 'key_part_protocol',
-                      'key_part_algorithm']
+            columns: ['dlv_part_key_tag', 'dlv_part_algorithm',
+                      'dlv_part_digest_type']
         },
         {
             name: 'kxrecord',
@@ -1012,26 +1076,6 @@ IPA.dns.get_record_metadata = function() {
             adder_attributes: [],
             columns: ['ns_part_hostname']
         },
-        {
-            name: 'nsecrecord',
-            attributes: [
-                'nsec_part_next',
-                'nsec_part_types'
-//             TODO: nsec_part_types is multivalued attribute. New selector
-//             widget or at least new validator should be created.
-//                 {
-//                     name: 'nsec_part_types',
-//                     options: IPA.create_options(['SOA', 'A', 'AAAA', 'A6', 'AFSDB',
-//                         'APL', 'CERT', 'CNAME', 'DHCID', 'DLV', 'DNAME', 'DNSKEY',
-//                         'DS', 'HIP', 'IPSECKEY', 'KEY', 'KX', 'LOC', 'MX', 'NAPTR',
-//                         'NS', 'NSEC','NSEC3', 'NSEC3PARAM', 'PTR', 'RRSIG', 'RP',
-//                         'SIG', 'SPF', 'SRV', 'SSHFP', 'TA', 'TKEY', 'TSIG', 'TXT']),
-//                     $type: 'select'
-//                 }
-            ],
-            adder_attributes: [],
-            columns: [ 'nsec_part_next', 'nsec_part_types']
-        },
         {
             name: 'ptrrecord',
             attributes: [
@@ -1040,62 +1084,6 @@ IPA.dns.get_record_metadata = function() {
             adder_attributes: [],
             columns: [ 'ptr_part_hostname']
         },
-        {
-            name: 'rrsigrecord',
-            attributes: [
-                {
-                    name: 'rrsig_part_type_covered',
-                    $type: 'select',
-                    options:  IPA.create_options(['SOA', 'A', 'AAAA', 'A6', 'AFSDB',
-                                'APL', 'CERT', 'CNAME', 'DHCID', 'DLV', 'DNAME',
-                                'DNSKEY', 'DS', 'HIP', 'IPSECKEY', 'KEY', 'KX',
-                                'LOC', 'MX', 'NAPTR', 'NS', 'NSEC', 'NSEC3',
-                                'NSEC3PARAM', 'PTR', 'RRSIG', 'RP', 'SPF', 'SRV',
-                                'SSHFP', 'TA', 'TKEY', 'TSIG', 'TXT'])
-                },
-                'rrsig_part_algorithm',
-                'rrsig_part_labels',
-                'rrsig_part_original_ttl',
-                'rrsig_part_signature_expiration',
-                'rrsig_part_signature_inception',
-                'rrsig_part_key_tag',
-                'rrsig_part_signers_name',
-                {
-                    name: 'rrsig_part_signature',
-                    $type: 'textarea'
-                }
-            ],
-            adder_attributes: [],
-            columns: ['dnsdata']
-        },
-        {
-            name: 'sigrecord',
-            attributes: [
-                {
-                    name: 'sig_part_type_covered',
-                    $type: 'select',
-                    options:  IPA.create_options(['SOA', 'A', 'AAAA', 'A6', 'AFSDB',
-                                'APL', 'CERT', 'CNAME', 'DHCID', 'DLV', 'DNAME',
-                                'DNSKEY', 'DS', 'HIP', 'IPSECKEY', 'KEY', 'KX',
-                                'LOC', 'MX', 'NAPTR', 'NS', 'NSEC', 'NSEC3',
-                                'NSEC3PARAM', 'PTR', 'RRSIG', 'RP', 'SPF', 'SRV',
-                                'SSHFP', 'TA', 'TKEY', 'TSIG', 'TXT'])
-                },
-                'sig_part_algorithm',
-                'sig_part_labels',
-                'sig_part_original_ttl',
-                'sig_part_signature_expiration',
-                'sig_part_signature_inception',
-                'sig_part_key_tag',
-                'sig_part_signers_name',
-                {
-                    name: 'sig_part_signature',
-                    $type: 'textarea'
-                }
-            ],
-            adder_attributes: [],
-            columns: ['dnsdata']
-        },
         {
             name: 'srvrecord',
             attributes: [
@@ -1121,6 +1109,23 @@ IPA.dns.get_record_metadata = function() {
             adder_attributes: [],
             columns: ['sshfp_part_algorithm', 'sshfp_part_fp_type']
         },
+        {
+            name: 'tlsarecord',
+            attributes: [
+                'tlsa_part_cert_usage',
+                'tlsa_part_selector',
+                'tlsa_part_matching_type',
+                {
+                    name: 'tlsa_part_cert_association_data',
+                    $type: 'textarea'
+                }
+            ],
+            adder_attributes: [],
+            columns: [
+                'tlsa_part_cert_usage', 'tlsa_part_selector',
+                'tlsa_part_matching_type'
+            ]
+        },
         {
             name: 'txtrecord',
             attributes: [
@@ -1205,7 +1210,7 @@ return {
                 {
                     name: 'identity',
                     label: '@i18n:details.identity',
-                    $type: 'details_table_section',
+                    $type: 'details_section',
                     widgets: [
                         {
                             $type: 'dnsrecord_host_link',
@@ -1235,7 +1240,7 @@ return {
         widgets: [
             {
                 name: 'general',
-                $type: 'details_table_section_nc',
+                $type: 'details_section',
                 widgets: [
                     'idnsname',
                     {
@@ -1264,7 +1269,7 @@ IPA.dns.record_adder_dialog = function(spec) {
 
     var that = IPA.entity_adder_dialog(spec);
 
-    that.on_error = IPA.create_4304_error_handler(that);
+    that.on_error = rpc.create_4304_error_handler(that);
 
     return that;
 };
@@ -1280,7 +1285,7 @@ IPA.dns.record_details_facet = function(spec) {
         if (!data.result.result.idnsname) {
             that.reset();
             var dialog = IPA.dnsrecord_redirection_dialog();
-            dialog.open(that.container);
+            dialog.open();
             return;
         }
 
@@ -1303,7 +1308,7 @@ IPA.dnsrecord_redirection_dialog = function(spec) {
 
     var that = IPA.message_dialog(spec);
 
-    that.create = function() {
+    that.create_content = function() {
         $('<p/>', {
             'text': text.get('@i18n:objects.dnsrecord.deleted_no_data')
         }).appendTo(that.container);
@@ -1373,7 +1378,7 @@ IPA.dns.record_prepare_editor_for_type = function(type, fields, widgets, update)
 
     var section = {
         name: type.name,
-        $type: 'details_table_section_nc',
+        $type: 'details_section',
         widgets: []
     };
     widgets.push(section);
@@ -1428,14 +1433,14 @@ IPA.dns.record_prepare_details_spec = function(spec, type_prepare_method) {
 
     var standard_record_section = {
         name: 'standard_types',
-        $type: 'details_table_section',
+        $type: 'details_section',
         label: '@i18n:objects.dnsrecord.standard',
         widgets: []
     };
 
     var other_record_section = {
         name: 'other_types',
-        $type: 'details_table_section',
+        $type: 'details_section',
         label: '@i18n:objects.dnsrecord.other',
         widgets: []
     };
@@ -1492,8 +1497,8 @@ IPA.dns.record_prepare_details_for_type = function(type, fields, container) {
  */
 
 
-IPA.dnsrecord_host_link_field = function(spec) {
-    var that = IPA.link_field(spec);
+IPA.dnsrecord_host_link_widget = function(spec) {
+    var that = IPA.link_widget(spec);
     that.other_pkeys = function() {
         var pkey = that.facet.get_pkeys();
         return [pkey[1]+'.'+pkey[0]];
@@ -1505,8 +1510,8 @@ IPA.dns_record_types = function() {
 
     //only supported
     var attrs = ['A', 'AAAA', 'A6', 'AFSDB', 'CERT', 'CNAME', 'DNAME',
-                   'DS','KEY', 'KX', 'LOC', 'MX', 'NAPTR', 'NS', 'NSEC',
-                   'PTR', 'RRSIG', 'SRV', 'SIG', 'SSHFP', 'TXT'];
+                   'DS', 'DLV', 'KX', 'LOC', 'MX', 'NAPTR', 'NS',
+                   'PTR', 'SRV', 'SSHFP', 'TLSA', 'TXT'];
     var record_types = [];
     for (var i=0; i<attrs.length; i++) {
         var attr = attrs[i];
@@ -1645,13 +1650,22 @@ IPA.dnsrecord_adder_dialog_type_policy = function(spec) {
 IPA.dns.record_type_table_field = function(spec) {
 
     spec = spec || {};
+    spec.adapter = spec.adapter || IPA.dns.record_type_adapter;
 
     var that = IPA.field(spec);
 
     that.dnstype = spec.dnstype;
 
-    that.load = function(record) {
+    return that;
+};
 
+IPA.dns.record_type_adapter = declare([field_mod.Adapter], {
+
+    separator: ';',
+
+    load: function(rpcdata) {
+
+        var record = this.get_record(rpcdata);
         var data = {};
 
         data.idnsname = record.idnsname;
@@ -1660,27 +1674,22 @@ IPA.dns.record_type_table_field = function(spec) {
         for (var i=0, j=0; i<record.dnsrecords.length; i++) {
 
             var dnsrecord = record.dnsrecords[i];
-            if(dnsrecord.dnstype === that.dnstype) {
+            if(dnsrecord.dnstype === this.context.dnstype) {
 
                 dnsrecord.position = j;
                 j++;
                 data.dnsrecords.push(dnsrecord);
             }
         }
-
-        that.values = data;
-
-        that.load_writable(record);
-        that.reset();
-    };
-
-    return that;
-};
+        return data;
+    }
+});
 
 IPA.dns.record_type_table_widget = function(spec) {
 
     spec = spec || {};
     spec.columns = spec.columns || [];
+    spec.footer = spec.footer === undefined ? false : spec.footer;
 
     spec.columns.push({
         name: 'position',
@@ -1730,42 +1739,35 @@ IPA.dns.record_type_table_widget = function(spec) {
 
         container.addClass('dnstype-table');
 
-        that.remove_button = IPA.action_button({
+        that.remove_button = IPA.button_widget({
             name: 'remove',
             label: '@i18n:buttons.remove',
-            icon: 'remove-icon',
-            'class': 'action-button-disabled',
-            click: function() {
-                if (!that.remove_button.hasClass('action-button-disabled')) {
-                    that.remove_handler();
-                }
-                return false;
-            }
-        }).appendTo(that.buttons);
+            icon: 'fa-trash-o',
+            enabled: false,
+            button_class: 'btn btn-link',
+            click: that.remove_handler
+        });
+        that.remove_button.create(that.buttons);
 
-        that.add_button = IPA.action_button({
+        that.add_button = IPA.button_widget({
             name: 'add',
             label: '@i18n:buttons.add',
-            icon: 'add-icon',
-            click: function() {
-                if (!that.add_button.hasClass('action-button-disabled')) {
-                    that.add_handler();
-                }
-                return false;
-            }
-        }).appendTo(that.buttons);
+            icon: 'fa-plus',
+            button_class: 'btn btn-link',
+            click: that.add_handler
+        });
+        that.add_button.create(that.buttons);
     };
 
     that.set_enabled = function(enabled) {
         that.table_set_enabled(enabled);
-        if (enabled) {
-            if(that.add_button) {
-                that.add_button.removeClass('action-button-disabled');
-            }
-        } else {
-            $('.action-button', that.table).addClass('action-button-disabled');
+        if (!enabled) {
             that.unselect_all();
         }
+        if (that.add_button) {
+            that.add_button.set_enabled(enabled);
+            that.remove_button.set_enabled(false);
+        }
     };
 
     that.select_changed = function() {
@@ -1773,11 +1775,7 @@ IPA.dns.record_type_table_widget = function(spec) {
         var values = that.get_selected_values();
 
         if (that.remove_button) {
-            if (values.length === 0) {
-                that.remove_button.addClass('action-button-disabled');
-            } else {
-                that.remove_button.removeClass('action-button-disabled');
-            }
+            that.remove_button.set_enabled(values.length > 0);
         }
     };
 
@@ -1794,7 +1792,7 @@ IPA.dns.record_type_table_widget = function(spec) {
                 that.show_add_dialog();
             };
 
-            dialog.open(that.container);
+            dialog.open();
 
         } else {
             that.show_add_dialog();
@@ -1814,7 +1812,7 @@ IPA.dns.record_type_table_widget = function(spec) {
                 that.show_remove_dialog();
             };
 
-            dialog.open(that.container);
+            dialog.open();
 
         } else {
             that.show_remove_dialog();
@@ -1852,13 +1850,13 @@ IPA.dns.record_type_table_widget = function(spec) {
         };
 
 
-        dialog.open(that.container);
+        dialog.open();
     };
 
     that.remove = function(values, pkey, on_success, on_error) {
 
         var pkeys = that.facet.get_pkeys();
-        var command = IPA.command({
+        var command = rpc.command({
             entity: that.entity.name,
             method: 'del',
             args: pkeys,
@@ -1897,7 +1895,7 @@ IPA.dns.record_type_table_widget = function(spec) {
         var cancel_button = dialog.buttons.get('cancel');
         dialog.buttons.empty();
 
-        dialog.on_error = IPA.create_4304_error_handler(dialog);
+        dialog.on_error = rpc.create_4304_error_handler(dialog);
 
         dialog.get_add_message = function() {
             var label = that.entity.metadata.label_singular;
@@ -1935,7 +1933,7 @@ IPA.dns.record_type_table_widget = function(spec) {
                 dialog.add(
                     function(data, text_status, xhr) {
 
-                        dialog.show_message(dialog.get_add_message());
+                        dialog.show_message(dialog.get_add_message(), 'success');
 
                         if (data.result.result.dnsrecords) {
                             that.reload_facet(data);
@@ -1967,7 +1965,7 @@ IPA.dns.record_type_table_widget = function(spec) {
     that.show_add_dialog = function() {
 
         var dialog = that.create_add_dialog();
-        dialog.open(that.container);
+        dialog.open();
     };
 
     that.create_mod_dialog = function() {
@@ -2098,70 +2096,35 @@ IPA.dns.record_type_table_widget = function(spec) {
 IPA.dns.netaddr_field = function(spec) {
 
     spec = spec || {};
-
-    var that = IPA.multivalued_field(spec);
-
-    that.load = function(record) {
-
-        that.record = record;
-
-        that.values = that.get_value(record, that.name);
-        that.values = that.values[0].split(';');
-
-        that.load_writable(record);
-
-        that.reset();
-    };
-
-    that.test_dirty = function() {
-
-        if (that.read_only) return false;
-
-        var values = that.field_save();
-
-        //check for empty value: null, [''], '', []
-        var orig_empty = IPA.is_empty(that.values);
-        var new_empty= IPA.is_empty(values);
-        if (orig_empty && new_empty) return false;
-        if (orig_empty != new_empty) return true;
-
-        //strict equality - checks object's ref equality, numbers, strings
-        if (values === that.values) return false;
-
-        //compare values in array
-        if (values.length !== that.values.length) return true;
-
-        for (var i=0; i<values.length; i++) {
-            if (values[i] != that.values[i]) {
-                return true;
-            }
-        }
-
-        return that.widget.test_dirty();
-    };
-
-    that.save = function(record) {
-
-        var values = that.field_save();
-        var new_val = values.join(';');
-
-        if (record) {
-            record[that.name] = new_val;
-        }
-
-        return [new_val];
-    };
-
-    that.validate = function() {
-
-        var values = that.field_save();
-
-        return that.validate_core(values);
-    };
-
+    spec.adapter = IPA.dns.netaddr_adapter;
+    var that = IPA.field(spec);
     return that;
 };
 
+IPA.dns.netaddr_adapter = declare([field_mod.Adapter], {
+
+    separator: ';',
+
+    load: function(data) {
+        var value = this.inherited(arguments)[0];
+        if (value) {
+            if (value[value.length-1] === this.separator) {
+                value = value.substring(0, value.length-1);
+            }
+            value = value.split(this.separator);
+        }
+        value = util.normalize_value(value);
+        return value;
+    },
+
+    save: function(value, record) {
+        if (value[0]) {
+            value = [value.join(this.separator)];
+        }
+        return this.inherited(arguments, [value, record]);
+    }
+});
+
 IPA.dns.record_modify_column = function(spec) {
 
     spec = spec || {};
@@ -2229,7 +2192,7 @@ IPA.dns.ptr_redirection_dialog = function(spec) {
 
     that.address = spec.address;
 
-    that.create = function() {
+    that.create_content = function() {
 
         that.status_div = $('<div />', {
             'class': 'redirection-status'
@@ -2288,7 +2251,7 @@ IPA.dns.ptr_redirection_dialog = function(spec) {
 
         that.append_status(text.get('@i18n:objects.dnsrecord.ptr_redir_zones'));
 
-        var command = IPA.command({
+        var command = rpc.command({
             entity: 'dnszone',
             method: 'find',
             options: {
@@ -2310,7 +2273,7 @@ IPA.dns.ptr_redirection_dialog = function(spec) {
 
         for (var i=0; i<zones.length; i++) {
 
-            var zone_name = zones[i].idnsname[0];
+            var zone_name = rpc.extract_objects(zones[i].idnsname)[0];
             if (that.reverse_address.indexOf(zone_name) > -1) {
                 var msg = text.get('@i18n:objects.dnsrecord.ptr_redir_zone');
                 msg = msg.replace('${zone}', zone_name);
@@ -2343,7 +2306,7 @@ IPA.dns.ptr_redirection_dialog = function(spec) {
         var record_name = that.reverse_address.substring(0,i1 - 1);
         that.record_keys = [that.zone, record_name];
 
-        var command = IPA.command({
+        var command = rpc.command({
             entity: 'dnsrecord',
             method: 'show',
             args: that.record_keys,
@@ -2380,7 +2343,7 @@ IPA.dns.ptr_redirection_dialog = function(spec) {
 
         var ptr = that.dns_record.name +'.' + that.dns_record.zone;
 
-        var command = IPA.command({
+        var command = rpc.command({
             entity: 'dnsrecord',
             method: 'add',
             args: that.record_keys,
@@ -2556,13 +2519,14 @@ IPA.network_validator = function(spec) {
 
 exp.remove_menu_item = function() {
     if (!IPA.dns_enabled) {
-        menu.remove_item('identity/dns');
+        menu.remove_item('network_services/dns');
     }
 };
 
 exp.config_spec = make_config_spec();
 exp.zone_spec = make_zone_spec();
 exp.record_spec = make_record_spec();
+exp.forwardzone_spec = make_forwardzone_spec();
 exp.register = function() {
     var e = reg.entity;
     var w = reg.widget;
@@ -2573,12 +2537,13 @@ exp.register = function() {
     e.register({type: 'dnsconfig', spec: exp.config_spec});
     e.register({type: 'dnszone', spec: exp.zone_spec});
     e.register({type: 'dnsrecord', spec: exp.record_spec});
+    e.register({type: 'dnsforwardzone', spec: exp.forwardzone_spec});
 
     w.register('dnszone_name', IPA.dnszone_name_widget);
     w.register('force_dnszone_add_checkbox', IPA.force_dnszone_add_checkbox_widget);
     f.register('force_dnszone_add_checkbox', IPA.checkbox_field);
-    w.register('dnsrecord_host_link', IPA.link_widget);
-    f.register('dnsrecord_host_link', IPA.dnsrecord_host_link_field);
+    w.register('dnsrecord_host_link', IPA.dnsrecord_host_link_widget);
+    f.register('dnsrecord_host_link', IPA.field);
     w.register('dnsrecord_type', IPA.dnsrecord_type_widget);
     f.register('dnsrecord_type', IPA.dnsrecord_type_field);
     w.register('dnsrecord_type_table', IPA.dns.record_type_table_widget);
