@@ -23,13 +23,14 @@ define(['./ipa',
         './jquery',
         './phases',
         './reg',
+        './rpc',
         './text',
         './details',
         './search',
         './association',
         './entity',
         './certificate'],
-    function(IPA, $, phases, reg, text) {
+    function(IPA, $, phases, reg, rpc, text) {
 
 var exp = IPA.host = {};
 
@@ -63,7 +64,16 @@ return {
                     label: '@i18n:objects.host.enrolled',
                     formatter: 'boolean'
                 }
-            ]
+            ],
+            actions: [
+                'select',
+                {
+                    $type: 'automember_rebuild',
+                    name: 'automember_rebuild',
+                    label: '@i18n:actions.automember_rebuild'
+                }
+            ],
+            header_actions: ['automember_rebuild']
         },
         {
             $type: 'details',
@@ -82,6 +92,7 @@ return {
                             $type: 'textarea',
                             name: 'description'
                         },
+                        'userclass',
                         'l',
                         'nshostlocation',
                         'nshardwareplatform',
@@ -106,11 +117,6 @@ return {
                 },
                 {
                     name: 'enrollment',
-                    action_panel: {
-                        $factory: IPA.action_panel,
-                        name: 'enrollment_actions',
-                        actions: ['unprovision', 'set_otp', 'reset_otp']
-                    },
                     fields: [
                         {
                             $factory: IPA.host_keytab_widget,
@@ -126,14 +132,6 @@ return {
                 },
                 {
                     name: 'certificate',
-                    action_panel: {
-                        $factory: IPA.action_panel,
-                        name: 'cert_actions',
-                        actions: [
-                            'request_cert', 'view_cert', 'get_cert',
-                            'revoke_cert', 'restore_cert'
-                        ]
-                    },
                     fields: [
                         {
                             $type: 'certificate_status',
@@ -144,19 +142,37 @@ return {
                 }
             ],
             actions: [
+                'select',
+                {
+                    $type: 'automember_rebuild',
+                    name: 'automember_rebuild',
+                    label: '@i18n:actions.automember_rebuild'
+                },
                 'host_unprovision',
                 {
-                    $type: 'set_otp',
+                    $type: 'password',
                     name: 'set_otp',
                     label: '@i18n:objects.host.password_set_title',
-                    status: 'missing',
+                    dialog: {
+                        title: '@i18n:objects.host.password_set_title',
+                        confirm_button_label: '@i18n:objects.host.password_set_button',
+                        password_name: 'userpassword',
+                        success_message: '@i18n:objects.host.password_set_success'
+                    },
+                    enable_cond: ['userpassword_w'],
                     hide_cond: ['has_password']
                 },
                 {
-                    $type: 'set_otp',
+                    $type: 'password',
                     name: 'reset_otp',
                     label: '@i18n:objects.host.password_reset_title',
-                    status: 'present',
+                    dialog: {
+                        title: '@i18n:objects.host.password_reset_title',
+                        confirm_button_label: '@i18n:objects.host.password_reset_button',
+                        password_name: 'userpassword',
+                        success_message: '@i18n:objects.host.password_set_success'
+                    },
+                    enable_cond: ['userpassword_w'],
                     show_cond: ['has_password']
                 },
                 'cert_view',
@@ -165,6 +181,8 @@ return {
                 'cert_revoke',
                 'cert_restore'
             ],
+            header_actions: ['automember_rebuild', 'unprovision', 'set_otp', 'reset_otp',
+                'request_cert', 'view_cert', 'get_cert', 'revoke_cert', 'restore_cert'],
             state: {
                 evaluators: [
                     IPA.host.has_password_evaluator,
@@ -234,6 +252,7 @@ return {
             {
                 name: 'other',
                 fields: [
+                    'userclass',
                     {
                         name: 'ip_address',
                         validators: [ 'ip_address' ],
@@ -294,57 +313,20 @@ IPA.host_fqdn_widget = function(spec) {
 
     that.create = function(container) {
         that.container = container;
+        container.addClass('col-sm-12');
 
         var hostname = that.widgets.get_widget('hostname');
         var dnszone = that.widgets.get_widget('dnszone');
 
-        var table = $('<table/>', {
-            'class': 'fqdn'
-        }).appendTo(that.container);
+        var layout = IPA.fluid_layout({
+            cont_cls: 'row fluid-row',
+            group_cls: 'col-sm-6 form-group',
+            widget_cls: 'controls',
+            label_cls: 'control-label'
+        });
 
-        var tr = $('<tr/>').appendTo(table);
-
-        var th = $('<th/>', {
-            'class': 'hostname',
-            title: hostname.label,
-            text: hostname.label
-        }).appendTo(tr);
-
-        $('<span/>', {
-            'class': 'required-indicator',
-            text: IPA.required_indicator
-        }).appendTo(th);
-
-        th = $('<th/>', {
-            'class': 'dnszone',
-            title: dnszone.label,
-            text: dnszone.label
-        }).appendTo(tr);
-
-        $('<span/>', {
-            'class': 'required-indicator',
-            text: IPA.required_indicator
-        }).appendTo(th);
-
-        tr = $('<tr/>').appendTo(table);
-
-        var td = $('<td/>', {
-            'class': 'hostname'
-        }).appendTo(tr);
-
-        var widget_cont = $('<div/>', {
-            name: hostname.name
-        }).appendTo(td);
-        hostname.create(widget_cont);
-
-        td = $('<td/>', {
-            'class': 'dnszone'
-        }).appendTo(tr);
-
-        widget_cont = $('<div/>', {
-            name: dnszone.name
-        }).appendTo(td);
-        dnszone.create(widget_cont);
+        var html = layout.create([hostname, dnszone]);
+        that.container.append(html);
 
         var hostname_input = $('input', hostname.container);
         var dnszone_input = $('input', dnszone.container);
@@ -365,6 +347,19 @@ IPA.host_fqdn_widget = function(spec) {
         });
     };
 
+    that.save = function() {
+
+        var hw = that.widgets.get_widget('hostname');
+        var dw = that.widgets.get_widget('dnszone');
+
+        var hostname = hw.save()[0];
+        var dnszone = dw.save()[0];
+
+        var fqdn = hostname && dnszone ? [ hostname+'.'+dnszone ] : [];
+        return fqdn;
+
+    };
+
     return that;
 };
 
@@ -374,19 +369,24 @@ IPA.host_fqdn_field = function(spec) {
 
     var that = IPA.field(spec);
 
-    that.validate_required = function() {
 
-        var hostname = that.hostname_widget.save();
-        var dnszone = that.dns_zone_widget.save();
+    that.has_value = function(widget) {
+
+        var value = widget.save();
+        var has_value = !!value.length && value[0] !== '';
+        return has_value;
+    };
+
+    that.validate_required = function() {
 
         var valid = true;
 
-        if(!hostname.length || hostname[0] === '') {
+        if (!that.has_value(that.hostname_widget)) {
             that.hostname_widget.show_error(text.get('@i18n:widget.validation.required'));
             that.valid = valid = false;
         }
 
-        if(!dnszone.length || dnszone[0] === '') {
+        if (!that.has_value(that.dns_zone_widget)) {
             that.dns_zone_widget.show_error(text.get('@i18n:widget.validation.required'));
             that.valid = valid = false;
         }
@@ -395,20 +395,12 @@ IPA.host_fqdn_field = function(spec) {
     };
 
     that.hide_error = function() {
-        that.hostname_widget.hide_error();
-        that.dns_zone_widget.hide_error();
-    };
-
-    that.save = function(record) {
-
-        if(!record) record = {};
-
-        var hostname = that.hostname_widget.save()[0];
-        var dnszone = that.dns_zone_widget.save()[0];
-
-        record.fqdn = hostname && dnszone ? [ hostname+'.'+dnszone ] : [];
-
-        return record.fqdn;
+        if (that.has_value(that.hostname_widget)) {
+            that.hostname_widget.hide_error();
+        }
+        if (that.has_value(that.dns_zone_widget)) {
+            that.dns_zone_widget.hide_error();
+        }
     };
 
     that.reset = function() {
@@ -422,6 +414,12 @@ IPA.host_fqdn_field = function(spec) {
         that.widget = that.container.widgets.get_widget(that.widget_name);
         that.hostname_widget = that.widget.widgets.get_widget('hostname');
         that.dns_zone_widget = that.widget.widgets.get_widget('dnszone');
+        that.hostname_widget.value_changed.attach(that.child_value_changed);
+        that.dns_zone_widget.value_changed.attach(that.child_value_changed);
+    };
+
+    that.child_value_changed = function() {
+        that.set_value(that.widget.save());
     };
 
     return that;
@@ -446,12 +444,12 @@ IPA.host_adder_dialog = function(spec) {
 
     var that = IPA.entity_adder_dialog(spec);
 
-    that.create = function() {
-        that.entity_adder_dialog_create();
+    that.create_content = function() {
+        that.entity_adder_dialog_create_content();
         that.container.addClass('host-adder-dialog');
     };
 
-    that.on_error = IPA.create_4304_error_handler(that);
+    that.on_error = rpc.create_4304_error_handler(that);
 
     return that;
 };
@@ -462,21 +460,19 @@ IPA.host_deleter_dialog = function(spec) {
 
     var that = IPA.search_deleter_dialog(spec);
 
-    that.create = function() {
+    that.create_content = function() {
 
-        that.deleter_dialog_create();
+        that.deleter_dialog_create_content();
 
         var metadata = IPA.get_command_option('host_del', 'updatedns');
 
-        that.updatedns = $('<input/>', {
+        var updatedns = IPA.standalone_option({
             type: 'checkbox',
             name: 'updatedns',
             title: metadata.doc
-        }).appendTo(that.container);
+        }, that.container, metadata.doc);
 
-        that.container.append(' ');
-
-        that.container.append(metadata.doc);
+        that.updatedns = updatedns[0];
     };
 
     that.create_command = function() {
@@ -503,7 +499,7 @@ IPA.dnszone_select_widget = function(spec) {
     var that = IPA.entity_select_widget(spec);
 
     that.create_search_command = function(filter) {
-        return IPA.command({
+        return rpc.command({
             entity: that.other_entity.name,
             method: 'find',
             args: [filter],
@@ -516,8 +512,9 @@ IPA.dnszone_select_widget = function(spec) {
     return that;
 };
 
-IPA.host_dnsrecord_entity_link_field = function(spec){
-    var that = IPA.link_field(spec);
+IPA.host_dnsrecord_entity_link_widget = function(spec) {
+
+    var that = IPA.link_widget(spec);
 
     that.other_pkeys = function(){
         var pkey = that.facet.get_pkey();
@@ -573,48 +570,46 @@ IPA.host_keytab_widget = function(spec) {
 
         that.widget_create(container);
 
-        that.missing_span = $('<span/>', {
+        that.missing_el = $('<label/>', {
             name: 'missing',
             style: 'display: none;'
         }).appendTo(container);
 
-        $('<img/>', {
-            src: 'images/caution-icon.png',
-            'class': 'status-icon'
-        }).appendTo(that.missing_span);
+        $('<i/>', {
+            'class': 'fa fa-warning'
+        }).appendTo(that.missing_el);
 
-        that.missing_span.append(' ');
+        that.missing_el.append(' ');
 
-        that.missing_span.append(text.get('@i18n:objects.host.keytab_missing'));
+        that.missing_el.append(text.get('@i18n:objects.host.keytab_missing'));
 
-        that.present_span = $('<span/>', {
+        that.present_el = $('<label/>', {
             name: 'present',
             style: 'display: none;'
         }).appendTo(container);
 
-        $('<img/>', {
-            src: 'images/check-icon.png',
-            'class': 'status-icon'
-        }).appendTo(that.present_span);
+        $('<i/>', {
+            'class': 'fa fa-check'
+        }).appendTo(that.present_el);
 
-        that.present_span.append(' ');
+        that.present_el.append(' ');
 
-        that.present_span.append(text.get('@i18n:objects.host.keytab_present'));
+        that.present_el.append(text.get('@i18n:objects.host.keytab_present'));
     };
 
     that.update = function(values) {
         set_status(values[0] ? 'present' : 'missing');
-        that.updated.notify([], that);
+        that.on_value_changed();
     };
 
     that.clear = function() {
-        that.present_span.css('display', 'none');
-        that.missing_span.css('display', 'none');
+        that.present_el.css('display', 'none');
+        that.missing_el.css('display', 'none');
     };
 
     function set_status(status) {
-        that.present_span.css('display', status == 'present' ? 'inline' : 'none');
-        that.missing_span.css('display', status == 'missing' ? 'inline' : 'none');
+        that.present_el.css('display', status == 'present' ? '' : 'none');
+        that.missing_el.css('display', status == 'missing' ? '' : 'none');
     }
 
     return that;
@@ -631,7 +626,7 @@ IPA.host_unprovision_dialog = function(spec) {
 
     that.title = that.title.replace('${entity}', that.entity.metadata.label_singular);
 
-    that.create = function() {
+    that.create_content = function() {
         that.container.append(text.get('@i18n:objects.host.unprovision_confirmation'));
     };
 
@@ -667,7 +662,7 @@ IPA.host_unprovision_dialog = function(spec) {
 
         var pkey = that.facet.get_pkeys();
 
-        var command = IPA.command({
+        var command = rpc.command({
             name: that.entity.name+'_disable_'+pkey,
             entity: that.entity.name,
             method: 'disable',
@@ -736,42 +731,40 @@ IPA.host_password_widget = function(spec) {
 
         that.widget_create(container);
 
-        that.missing_span = $('<span/>', {
+        that.missing_el = $('<label/>', {
             name: 'missing'
         }).appendTo(container);
 
-        $('<img/>', {
-            src: 'images/caution-icon.png',
-            'class': 'status-icon'
-        }).appendTo(that.missing_span);
+        $('<i/>', {
+            'class': 'fa fa-warning'
+        }).appendTo(that.missing_el);
 
-        that.missing_span.append(' ');
+        that.missing_el.append(' ');
 
-        that.missing_span.append(text.get('@i18n:objects.host.password_missing'));
+        that.missing_el.append(text.get('@i18n:objects.host.password_missing'));
 
-        that.present_span = $('<span/>', {
+        that.present_el = $('<label/>', {
             name: 'present',
             style: 'display: none;'
         }).appendTo(container);
 
-        $('<img/>', {
-            src: 'images/check-icon.png',
-            'class': 'status-icon'
-        }).appendTo(that.present_span);
+        $('<i/>', {
+            'class': 'fa fa-check'
+        }).appendTo(that.present_el);
 
-        that.present_span.append(' ');
+        that.present_el.append(' ');
 
-        that.present_span.append(text.get('@i18n:objects.host.password_present'));
+        that.present_el.append(text.get('@i18n:objects.host.password_present'));
     };
 
     that.update = function(values) {
         set_status(values[0] ? 'present' : 'missing');
-        that.updated.notify([], that);
+        that.on_value_changed();
     };
 
     that.clear = function() {
-        that.missing_span.css('display', 'none');
-        that.present_span.css('display', 'none');
+        that.missing_el.css('display', 'none');
+        that.present_el.css('display', 'none');
     };
 
     function set_status(status) {
@@ -779,147 +772,17 @@ IPA.host_password_widget = function(spec) {
         that.status = status;
 
         if (status == 'missing') {
-            that.missing_span.css('display', 'inline');
-            that.present_span.css('display', 'none');
+            that.missing_el.css('display', '');
+            that.present_el.css('display', 'none');
         } else {
-            that.missing_span.css('display', 'none');
-            that.present_span.css('display', 'inline');
+            that.missing_el.css('display', 'none');
+            that.present_el.css('display', '');
         }
     }
 
     return that;
 };
 
-IPA.host.set_otp_dialog = function(spec) {
-
-    spec = spec || {};
-    spec.width = spec.width || 400;
-    spec.sections = spec.sections || [
-        {
-            fields: [
-                {
-                    name: 'password1',
-                    label: '@i18n:password.new_password',
-                    $type: 'password',
-                    required: true
-                },
-                {
-                    name: 'password2',
-                    label: '@i18n:password.verify_password',
-                    $type: 'password',
-                    required: true,
-                    validators: [{
-                        $type: 'same_password',
-                        other_field: 'password1'
-                    }]
-                }
-            ]
-        }
-    ];
-
-    var that = IPA.dialog(spec);
-
-    IPA.confirm_mixin().apply(that);
-
-    that.facet = spec.facet;
-
-    that.set_status = function(status) {
-
-        var button = that.get_button('set_password');
-
-        if (status == 'missing') {
-            that.title = text.get('@i18n:objects.host.password_set_title');
-            button.label = text.get('@i18n:objects.host.password_set_button');
-        } else {
-            that.title = text.get('@i18n:objects.host.password_reset_title');
-            button.label = text.get('@i18n:objects.host.password_reset_button');
-        }
-    };
-
-    that.on_confirm = function() {
-
-        if (!that.validate()) return;
-
-        var new_password = that.fields.get_field('password1').save()[0];
-        that.set_otp(new_password);
-
-        that.close();
-    };
-
-    that.create_buttons = function() {
-
-        that.create_button({
-            name: 'set_password',
-            label: '@i18n:objects.host.password_set_button',
-            click: function() {
-                that.on_confirm();
-            }
-        });
-
-        that.create_button({
-            name: 'cancel',
-            label: '@i18n:buttons.cancel',
-            click: function() {
-                that.close();
-            }
-        });
-    };
-
-    that.set_otp = function(password) {
-        var pkey = that.facet.get_pkeys();
-
-        var command = IPA.command({
-            entity: that.entity.name,
-            method: 'mod',
-            args: pkey,
-            options: {
-                all: true,
-                rights: true,
-                userpassword: password
-            },
-            on_success: function(data) {
-                that.facet.load(data);
-                that.close();
-                IPA.notify_success('@i18n:objects.host.password_set_success');
-            },
-            on_error: function() {
-                that.close();
-            }
-        });
-
-        command.execute();
-    };
-
-    that.create_buttons();
-
-    return that;
-};
-
-IPA.host.set_otp_action = function(spec) {
-
-    spec = spec || {};
-    spec.name = spec.name || 'set_otp';
-    spec.label = spec.label || '@i18n:objects.host.password_set_title';
-    spec.enable_cond = spec.enable_cond || ['userpassword_w'];
-
-    var that = IPA.action(spec);
-    that.status = spec.status || 'missing';
-
-    that.execute_action = function(facet) {
-
-        var dialog = IPA.host.set_otp_dialog({
-            entity: facet.entity,
-            facet: facet
-        });
-
-        dialog.set_status(that.status);
-
-        dialog.open();
-    };
-
-    return that;
-};
-
 IPA.host.userpassword_acl_evaluator = function(spec) {
 
     spec.name = spec.name || 'userpassword_acl_evaluator';
@@ -965,7 +828,6 @@ IPA.host.certificate_policy = function(spec) {
     return that;
 };
 
-
 exp.entity_spec = make_spec();
 exp.register = function() {
     var e = reg.entity;
@@ -978,15 +840,14 @@ exp.register = function() {
     w.register('host_fqdn', IPA.host_fqdn_widget);
     f.register('dnszone_select', IPA.field);
     w.register('dnszone_select', IPA.dnszone_select_widget);
-    f.register('host_dnsrecord_entity_link', IPA.host_dnsrecord_entity_link_field);
-    w.register('host_dnsrecord_entity_link', IPA.link_widget);
+    f.register('host_dnsrecord_entity_link', IPA.field);
+    w.register('host_dnsrecord_entity_link', IPA.host_dnsrecord_entity_link_widget);
     f.register('force_host_add_checkbox', IPA.checkbox_field);
     w.register('force_host_add_checkbox', IPA.force_host_add_checkbox_widget);
     f.register('host_password', IPA.field);
     w.register('host_password', IPA.host_password_widget);
 
     a.register('host_unprovision', exp.unprovision_action);
-    a.register('set_otp', exp.set_otp_action);
 };
 phases.on('registration', exp.register);
 
