@@ -23,11 +23,12 @@ define([
     './jquery',
     './phases',
     './reg',
+    './rpc',
     './details',
     './search',
     './association',
     './entity'],
-        function(IPA, $, phases, reg) {
+        function(IPA, $, phases, reg, rpc) {
 
 IPA.rule_details_widget = function(spec) {
 
@@ -42,14 +43,14 @@ IPA.rule_details_widget = function(spec) {
 
     that.init = function() {
 
-        that.enable_radio = IPA.radio_widget({
+        that.enable_radio = IPA.rule_radio_widget({
             name: that.radio_name,
-            options: that.options
+            options: that.options,
+            entity: that.entity
         });
 
         that.widgets.add_widget(that.enable_radio);
         that.enable_radio.value_changed.attach(that.on_enable_radio_changed);
-        that.enable_radio.updated.attach(that.on_enable_radio_changed);
     };
 
     that.on_enable_radio_changed = function() {
@@ -65,41 +66,38 @@ IPA.rule_details_widget = function(spec) {
         }
     };
 
+    that.init();
+
+    return that;
+};
+
+/**
+ * Rule radio widget
+ *
+ * Intended to be used especially by rule widget.
+ *
+ * @class IPA.rule_radio_widget
+ * @extends IPA.radio_widget
+ */
+IPA.rule_radio_widget = function(spec) {
+
+    spec = spec || {};
+    var that = IPA.radio_widget(spec);
+
+    /** @inheritDoc */
     that.create = function(container) {
 
-        that.container = container;
+        var param_info = IPA.get_entity_param(that.entity.name, that.name);
+        var title = param_info ? param_info.doc : that.name;
 
-        //enable radios
-        var param_info = IPA.get_entity_param(that.entity.name, that.radio_name);
-        var title = param_info ? param_info.doc : that.radio_name;
-        var enable_radio_container = $('<div/>', {
-            name: that.radio_name,
-            title: title,
-            'class': 'field'
-        }).appendTo(container);
-
-        enable_radio_container.append(title+': ');
-        that.enable_radio.create(enable_radio_container);
-
-        //tables
-        for (var j=0; j<that.tables.length; j++) {
-            var table = that.tables[j];
-
-            var metadata = IPA.get_entity_param(that.entity.name, table.name);
-
-            var table_container = $('<div/>', {
-                name: table.name,
-                title: metadata ? metadata.doc : table.name,
-                'class': 'field'
-            }).appendTo(container);
-
-            var widget = that.widgets.get_widget(table.name);
-            widget.create(table_container);
+        container.append(title + ': ');
+        that.widget_create(container);
+        that.owb_create(container);
+        if (that.undo) {
+            that.create_undo(container);
         }
     };
 
-    that.init();
-
     return that;
 };
 
@@ -107,6 +105,7 @@ IPA.rule_details_widget = function(spec) {
 IPA.rule_association_table_widget = function(spec) {
 
     spec = spec || {};
+    spec.footer = spec.footer === undefined ? false : spec.footer;
 
     var that = IPA.association_table_widget(spec);
 
@@ -194,12 +193,12 @@ IPA.rule_association_table_field = function(spec) {
         }
     };
 
-    that.load = function(result) {
-        that.values = result[that.param] || [];
+    that.load = function(data) {
+        that.values = that.adapter.load(data);
 
         if (that.external) {
             that.set_values_external(that.values, '');
-            var external_values = result[that.external] || [];
+            var external_values = that.adapter.load(data, that.external, []);
             that.set_values_external(external_values, 'true');
             $.merge(that.values, external_values);
         }
@@ -217,11 +216,11 @@ IPA.rule_association_table_field = function(spec) {
         //performs delete operation.
 
         if (!that.widget.enabled) {
-            var values = that.save();
+            var values = that.widget.save();
 
             if (values.length > 0) { //no need to delete if has no values
 
-                var command = IPA.command({
+                var command = rpc.command({
                     entity: that.entity.name,
                     method: that.widget.remove_method,
                     args: that.facet.get_pkeys()
