@@ -26,10 +26,11 @@ define([
         './jquery',
         './phases',
         './reg',
+        './rpc',
         './spec_util',
         './text',
         './facet'],
-    function(IPA, $, phases, reg, su, text, mod_facet) {
+    function(IPA, $, phases, reg, rpc, su, text, mod_facet) {
 
 var exp = {};
 
@@ -48,19 +49,19 @@ exp.search_facet_control_buttons_pre_op = function(spec, context) {
             {
                 name: 'remove',
                 label: '@i18n:buttons.remove',
-                icon: 'remove-icon'
+                icon: 'fa-trash-o'
             },
             {
                 name: 'add',
                 label: '@i18n:buttons.add',
-                icon: 'add-icon'
+                icon: 'fa-plus'
             });
     }
     spec.control_buttons.unshift(
         {
             name: 'refresh',
             label: '@i18n:buttons.refresh',
-            icon: 'reset-icon'
+            icon: 'fa-refresh'
         });
 
     spec.state = spec.state || {};
@@ -103,20 +104,22 @@ IPA.search_facet = function(spec, no_init) {
     that.create_header = function(container) {
 
         that.facet_create_header(container);
+        that.create_search_filter(that.controls_left);
+        that.create_control_buttons(that.controls_right);
+        that.create_action_dropdown(that.controls_right);
+    };
 
-        var div = $('<div/>', {
-            'class': 'right-aligned-facet-controls'
-        }).appendTo(that.controls);
-
-        div.append(IPA.create_network_spinner());
+    that.create_search_filter = function(container) {
 
         that.filter_container = $('<div/>', {
             'class': 'search-filter'
-        }).appendTo(div);
+        }).appendTo(container);
 
         that.filter = $('<input/>', {
             type: 'text',
-            name: 'filter'
+            'class': 'form-control',
+            name: 'filter',
+            placeholder: text.get('@i18n:search.placeholder')
         }).appendTo(that.filter_container);
 
         that.filter.keypress(function(e) {
@@ -128,14 +131,12 @@ IPA.search_facet = function(spec, no_init) {
 
         that.find_button = IPA.action_button({
             name: 'find',
-            icon: 'search-icon',
+            icon: 'fa-search',
             click: function() {
                 that.find();
                 return false;
             }
         }).appendTo(that.filter_container);
-
-        that.create_control_buttons(that.controls);
     };
 
     that.managed_entity_pkey_prefix = function() {
@@ -165,7 +166,7 @@ IPA.search_facet = function(spec, no_init) {
             });
         }
         dialog.pkey_prefix = that.managed_entity_pkey_prefix();
-        dialog.open(that.container);
+        dialog.open();
     };
 
     that.create_remove_dialog = function() {
@@ -201,7 +202,7 @@ IPA.search_facet = function(spec, no_init) {
     that.show_remove_dialog = function() {
 
         var dialog = that.create_remove_dialog();
-        dialog.open(that.container);
+        dialog.open();
     };
 
     that.find = function() {
@@ -235,7 +236,7 @@ IPA.search_facet = function(spec, no_init) {
 
         var args = that.get_refresh_command_args();
 
-        var command = IPA.command({
+        var command = rpc.command({
             name: that.get_search_command_name(),
             entity: that.managed_entity.name,
             method: 'find',
@@ -312,12 +313,13 @@ IPA.search_deleter_dialog = function(spec) {
     that.pkey_prefix = spec.pkey_prefix || [];
 
     that.create_command = function() {
-        var batch = IPA.batch_command({
-            error_message: '@i18n:search.partial_delete'
+        var batch = rpc.batch_command({
+            error_message: '@i18n:search.partial_delete',
+            name: that.entity.name + '_batch_del'
         });
 
         for (var i=0; i<that.values.length; i++) {
-            var command = IPA.command({
+            var command = rpc.command({
                 entity: that.entity.name,
                 method: 'del'
             });
@@ -352,6 +354,7 @@ IPA.search_deleter_dialog = function(spec) {
         batch.on_success = function(data, text_status, xhr) {
             that.facet.refresh();
             that.facet.on_update.notify([],that.facet);
+            that.facet.table.unselect_all();
             var succeeded = batch.commands.length - batch.errors.errors.length;
             var msg = text.get('@i18n:search.deleted').replace('${count}', succeeded);
             IPA.notify_success(msg);
@@ -466,6 +469,7 @@ IPA.batch_items_action = function(spec) {
 
     that.method = spec.method || 'disable';
     that.success_msg = text.get(spec.success_msg);
+    that.options = spec.options || {};
 
     that.execute_action = function(facet, on_success, on_error) {
 
@@ -474,7 +478,7 @@ IPA.batch_items_action = function(spec) {
         var pkeys = facet.get_pkeys();
         if (!pkeys[0]) pkeys = []; // correction for search facet
 
-        that.batch = IPA.batch_command({
+        that.batch = rpc.batch_command({
             name: entity.name + '_batch_'+ that.method,
             on_success: that.get_on_success(facet, on_success)
         });
@@ -483,21 +487,27 @@ IPA.batch_items_action = function(spec) {
             var item_keys = pkeys.splice(0);
             item_keys.push(selected_keys[i]);
 
-            var command = IPA.command({
-                entity: entity.name,
-                method: that.method,
-                args: item_keys
-            });
-
+            var command = that.create_action_command(facet, item_keys);
             that.batch.add_command(command);
         }
 
         that.batch.execute();
     };
 
+    that.create_action_command = function(facet, keys) {
+        var command = rpc.command({
+            entity: facet.managed_entity.name,
+            method: that.method,
+            args: keys,
+            options: that.options
+        });
+        return command;
+    };
+
     that.on_success = function(facet, data, text_status, xhr) {
         facet.on_update.notify();
         facet.refresh();
+        facet.table.unselect_all();
 
         if (that.success_msg) {
             var succeeded = that.batch.commands.length - that.batch.errors.errors.length;
