@@ -73,6 +73,12 @@ return {
                                 'KDC:Disable Last Success', 'KDC:Disable Lockout'
                             ])
                         },
+                        {
+                            $type: 'checkboxes',
+                            name: 'ipauserauthtype',
+                            flags: ['w_if_no_aci'],
+                            options: ['password', 'radius', 'otp']
+                        },
                         {
                             $type: 'checkbox',
                             name: 'ipamigrationenabled'
