@@ -22,18 +22,27 @@
  */
 
 define([
+        './builder',
         './ipa',
         './jquery',
         './phases',
         './reg',
+        './rpc',
         './text',
+        './dialogs/password',
         './details',
         './search',
         './association',
         './entity',
         './certificate'],
-    function(IPA, $, phases, reg, text) {
+    function(builder, IPA, $, phases, reg, rpc, text, password_dialog) {
 
+/**
+ * User module
+ * @class user
+ * @alternateClassName IPA.user
+ * @singleton
+ */
 var exp = IPA.user = {};
 
 var make_spec = function() {
@@ -61,6 +70,12 @@ return {
                 'title'
             ],
             actions: [
+                'select',
+                {
+                    $type: 'automember_rebuild',
+                    name: 'automember_rebuild',
+                    label: '@i18n:actions.automember_rebuild'
+                },
                 {
                     $type: 'batch_disable',
                     hide_cond: ['self-service']
@@ -70,16 +85,17 @@ return {
                     hide_cond: ['self-service']
                 }
             ],
+            header_actions: ['automember_rebuild'],
             control_buttons: [
                 {
                     name: 'disable',
                     label: '@i18n:buttons.disable',
-                    icon: 'disabled-icon'
+                    icon: 'fa-minus'
                 },
                 {
                     name: 'enable',
                     label: '@i18n:buttons.enable',
-                    icon: 'enabled-icon'
+                    icon: 'fa-check'
                 }
             ]
         },
@@ -97,16 +113,15 @@ return {
                         'cn',
                         'displayname',
                         'initials',
-                        'gecos'
+                        'gecos',
+                        {
+                            name: 'userclass',
+                            flags: ['w_if_no_aci']
+                        }
                     ]
                 },
                 {
                     name: 'account',
-                    action_panel: {
-                        $factory: IPA.action_panel,
-                        name: 'account_actions',
-                        actions: ['reset_password']
-                    },
                     fields: [
                         'uid',
                         {
@@ -115,25 +130,37 @@ return {
                             metadata: '@mo-param:user:userpassword'
                         },
                         {
+                            $type: 'datetime',
                             name: 'krbpasswordexpiration',
                             label: '@i18n:objects.user.krbpasswordexpiration',
-                            read_only: true,
-                            formatter: 'utc_date'
+                            read_only: true
                         },
                         'uidnumber',
                         'gidnumber',
+                        'krbprincipalname',
+                        {
+                            $type: 'datetime',
+                            name: 'krbprincipalexpiration'
+                        },
                         'loginshell',
                         'homedirectory',
                         {
                             $type: 'sshkeys',
                             name: 'ipasshpubkey',
                             label: '@i18n:objects.sshkeystore.keys'
+                        },
+                        {
+                            $type: 'checkboxes',
+                            name: 'ipauserauthtype',
+                            flags: ['w_if_no_aci'],
+                            options: ['password', 'radius', 'otp']
                         }
                     ]
                 },
                 {
                     name: 'pwpolicy',
                     label: '@i18n:objects.pwpolicy.identity',
+                    field_adapter: { result_index: 1 },
                     fields: [
                         {
                             name: 'krbmaxpwdlife',
@@ -183,6 +210,7 @@ return {
                 {
                     name: 'krbtpolicy',
                     label: '@i18n:objects.krbtpolicy.identity',
+                    field_adapter: { result_index: 2 },
                     fields: [
                         {
                             name: 'krbmaxrenewableage',
@@ -221,12 +249,18 @@ return {
                             name: 'manager',
                             other_entity: 'user',
                             other_field: 'uid'
-                        }
+                        },
+                        { $type: 'multivalued', name: 'departmentnumber' },
+                        'employeenumber',
+                        'employeetype',
+                        'preferredlanguage'
                     ]
                 },
                 {
                     name: 'misc',
-                    fields: [ 'carlicense' ]
+                    fields: [
+                        { $type: 'multivalued', name: 'carlicense' }
+                    ]
                 }
             ],
             actions: [
@@ -234,17 +268,36 @@ return {
                 'enable',
                 'disable',
                 'delete',
-                'reset_password'
+                'reset_password',
+                {
+                    $factory: IPA.object_action,
+                    name: 'unlock',
+                    method: 'unlock',
+                    label: '@i18n:objects.user.unlock',
+                    needs_confirm: true,
+                    confirm_msg: '@i18n:objects.user.unlock_confirm'
+                },
+                {
+                    $type: 'automember_rebuild',
+                    name: 'automember_rebuild',
+                    label: '@i18n:actions.automember_rebuild'
+                }
             ],
-            header_actions: ['select_action', 'enable', 'disable', 'delete'],
+            header_actions: ['reset_password', 'enable', 'disable', 'delete', 'unlock', 'automember_rebuild'],
             state: {
                 evaluators: [
                     {
                         $factory: IPA.enable_state_evaluator,
                         field: 'nsaccountlock',
+                        adapter: { $type: 'batch', result_index: 0 },
                         invert_value: true
                     },
-                    IPA.user.reset_password_acl_evaluator
+                    {
+                        $factory: IPA.acl_state_evaluator,
+                        name: 'reset_password_acl_evaluator',
+                        adapter: { $type: 'batch', result_index: 0 },
+                        attribute: 'userpassword'
+                    }
                 ],
                 summary_conditions: [
                     IPA.enabled_summary_cond,
@@ -300,7 +353,8 @@ return {
                         required: false
                     },
                     'givenname',
-                    'sn'
+                    'sn',
+                    'userclass'
                 ]
             },
             {
@@ -327,34 +381,21 @@ IPA.user.details_facet = function(spec) {
 
     var that = IPA.details_facet(spec);
 
-    that.refresh_on_success = function(data, text_status, xhr) {
-        // do not load data from batch
-
-        that.show_content();
-    };
-
     that.create_refresh_command = function() {
 
         var pkey = that.get_pkey();
 
-        var batch = IPA.batch_command({
+        var batch = rpc.batch_command({
             name: 'user_details_refresh'
         });
 
         var user_command = that.details_facet_create_refresh_command();
-
-        user_command.on_success = function(data, text_status, xhr) {
-            // create data that mimics user-show output
-            var user_data = {};
-            user_data.result = data;
-            that.load(user_data);
-        };
-
         batch.add_command(user_command);
 
-        var pwpolicy_command = IPA.command({
+        var pwpolicy_command = rpc.command({
             entity: 'pwpolicy',
             method: 'show',
+            retry: false,
             options: {
                 user: pkey,
                 all: true,
@@ -363,24 +404,20 @@ IPA.user.details_facet = function(spec) {
         });
 
         pwpolicy_command.on_success = function(data, text_status, xhr) {
-            // TODO: Use nested fields: that.fields.get_field('pwpolicy').get_fields();
-            var fields = that.fields.get_fields();
-            for (var i=0; i<fields.length; i++) {
-                var field = fields[i];
+            that.widgets.get_widget('pwpolicy').set_visible(true);
+        };
 
-                // load result into pwpolicy fields
-                if (field.widget_name.match(/^pwpolicy\./)) {
-                    field.load(data.result);
-                }
-            }
+        pwpolicy_command.on_error = function(xhr, text_status, error_thrown) {
+            that.widgets.get_widget('pwpolicy').set_visible(false);
         };
 
         batch.add_command(pwpolicy_command);
 
-        var krbtpolicy_command = IPA.command({
+        var krbtpolicy_command = rpc.command({
             entity: 'krbtpolicy',
             method: 'show',
             args: [ pkey ],
+            retry: false,
             options: {
                 all: true,
                 rights: true
@@ -388,16 +425,11 @@ IPA.user.details_facet = function(spec) {
         });
 
         krbtpolicy_command.on_success = function(data, text_status, xhr) {
-            // TODO: Use nested fields: that.fields.get_field('krbtpolicy').get_fields();
-            var fields = that.fields.get_fields();
-            for (var i=0; i<fields.length; i++) {
-                var field = fields[i];
+            that.widgets.get_widget('krbtpolicy').set_visible(true);
+        };
 
-                // load result into krbtpolicy fields
-                if (field.widget_name.match(/^krbtpolicy\./)) {
-                    field.load(data.result);
-                }
-            }
+        krbtpolicy_command.on_error = function(xhr, text_status, error_thrown) {
+            that.widgets.get_widget('krbtpolicy').set_visible(false);
         };
 
         batch.add_command(krbtpolicy_command);
@@ -409,6 +441,7 @@ IPA.user.details_facet = function(spec) {
 };
 
 /**
+ * @member user
  * Makes user association facets read-only in self service
  */
 IPA.user.association_facet_ss_pre_op = function(spec, context) {
@@ -436,7 +469,7 @@ IPA.user_adder_dialog = function(spec) {
         var password2 = field2.save()[0];
 
         if (password1 !== password2) {
-            field2.show_error(text.get('@i18n:password.password_must_match'));
+            field2.set_valid({ valid: false, message: text.get('@i18n:password.password_must_match') });
             valid = false;
         }
 
@@ -476,7 +509,7 @@ IPA.user_password_widget = function(spec) {
         } else {
             that.display_control.text(that.unset_value);
         }
-        that.updated.notify([], that);
+        that.on_value_changed();
     };
 
     that.clear = function() {
@@ -486,155 +519,57 @@ IPA.user_password_widget = function(spec) {
     return that;
 };
 
-IPA.user_password_dialog = function(spec) {
+IPA.user.password_dialog_pre_op0 = function(spec) {
 
-    spec = spec || {};
+    spec.password_name = spec.password_name || 'password';
+    return spec;
+};
 
-    spec.width = spec.width || 400;
-    spec.title = spec.title || '@i18n:password.reset_password';
-    spec.sections = spec.sections || [];
+IPA.user.password_dialog_pre_op = function(spec) {
 
-    spec.sections.push(
-        {
-            name: 'input',
-            fields: [
-                {
-                    name: 'current_password',
-                    label: '@i18n:password.current_password',
-                    $type: 'password',
-                    required: true
-                },
-                {
-                    name: 'password1',
-                    label: '@i18n:password.new_password',
-                    $type: 'password',
-                    required: true
-                },
-                {
-                    name: 'password2',
-                    label: '@i18n:password.verify_password',
-                    $type: 'password',
-                    validators: [{
-                        $type: 'same_password',
-                        other_field: 'password1'
-                    }],
-                    required: true
-                }
-            ]
-        });
+    spec.sections[0].fields.splice(0, 0, {
+        name: 'current_password',
+        label: '@i18n:password.current_password',
+        $type: 'password',
+        required: true
+    }, {
+         name: 'otp',
+        label: '@i18n:password.otp',
+        $type: 'password'
+    });
 
-    var that = IPA.dialog(spec);
+    spec.method = spec.method || 'passwd';
 
-    IPA.confirm_mixin().apply(that);
+    return spec;
+};
 
-    that.success_handler = spec.on_success;
-    that.error_handler = spec.on_error;
-    that.pkey = spec.pkey;
+IPA.user.password_dialog = function(spec) {
+
+    var that = password_dialog.dialog(spec);
 
     that.is_self_service = function() {
-        var self_service = that.pkey === IPA.whoami.uid[0];
+        var self_service = that.args[0] === IPA.whoami.uid[0];
         return self_service;
     };
 
     that.open = function() {
+        that.dialog_open();
 
         var self_service = that.is_self_service();
-        var section = that.widgets.get_widget('input');
-        var current_password_f = that.fields.get_field('current_password');
+        var current_pw_f = that.fields.get_field('current_password');
+        var current_pw_w = that.widgets.get_widget('general.current_password');
+        var otp_f = that.fields.get_field('otp');
+        var otp_w = that.widgets.get_widget('general.otp');
+
+        current_pw_f.set_required(self_service);
+        current_pw_f.set_enabled(self_service);
+        current_pw_w.set_visible(self_service);
+        otp_f.set_enabled(self_service);
+        otp_w.set_visible(self_service);
 
-        that.dialog_open();
-        section.set_row_visible('current_password', self_service);
-        current_password_f.set_required(self_service);
         that.focus_first_element();
     };
 
-    that.create_buttons = function() {
-
-        that.create_button({
-            name: 'reset_password',
-            label: '@i18n:password.reset_password',
-            click: that.on_reset_click
-        });
-
-        that.create_button({
-            name: 'cancel',
-            label: '@i18n:buttons.cancel',
-            click: function() {
-                that.close();
-            }
-        });
-    };
-
-    that.on_confirm = function() {
-        that.on_reset_click();
-    };
-
-    that.on_reset_click = function() {
-
-        if (!that.validate()) return;
-
-        var self_service = that.is_self_service();
-
-        var record = {};
-        that.save(record);
-
-        var current_password = self_service ? record.current_password[0] : undefined;
-        var new_password = record.password1[0];
-        var repeat_password = record.password2[0];
-
-        that.set_password(
-            that.pkey,
-            current_password,
-            new_password,
-            that.on_reset_success,
-            that.on_reset_error);
-    };
-
-    that.set_password = function(pkey, current_password, password, on_success, on_error) {
-
-        var command = IPA.command({
-            method: 'passwd',
-            args: [ pkey ],
-            options: {
-                current_password: current_password,
-                password: password
-            },
-            on_success: on_success,
-            on_error: on_error
-        });
-
-        command.execute();
-    };
-
-    that.on_reset_success = function(data, text_status, xhr) {
-
-        if (that.success_handler) {
-            that.success_handler.call(this, data, text_status, xhr);
-        } else {
-            IPA.notify_success('@i18n:password.password_change_complete');
-            that.close();
-
-            // refresh password expiration field
-            that.facet.refresh();
-
-            if (that.is_self_service()) {
-                var command = IPA.get_whoami_command();
-                command.execute();
-            }
-        }
-    };
-
-    that.on_reset_error = function(xhr, text_status, error_thrown) {
-
-        if (that.error_handler) {
-            that.error_handler.call(this, xhr, text_status, error_thrown);
-        } else {
-            that.close();
-        }
-    };
-
-    that.create_buttons();
-
     return that;
 };
 
@@ -649,10 +584,17 @@ IPA.user.reset_password_action = function(spec) {
 
     that.execute_action = function(facet) {
 
-        var dialog = IPA.user_password_dialog({
-            entity: facet.entity,
-            facet: facet,
-            pkey: facet.get_pkey()
+        var dialog = builder.build('dialog', {
+            $type: 'user_password',
+            args: [facet.get_pkey()]
+        });
+
+        dialog.succeeded.attach(function() {
+            facet.refresh();
+            if (dialog.is_self_service()) {
+                var command = IPA.get_whoami_command();
+                command.execute();
+            }
         });
 
         dialog.open();
@@ -661,21 +603,18 @@ IPA.user.reset_password_action = function(spec) {
     return that;
 };
 
-IPA.user.reset_password_acl_evaluator = function(spec) {
-
-    spec.name = spec.name || 'reset_password_acl_evaluator';
-    spec.attribute = spec.attribute || 'userpassword';
-
-    var that = IPA.acl_state_evaluator(spec);
-    return that;
-};
-
 exp.entity_spec = make_spec();
 exp.register = function() {
     var e = reg.entity;
     var a = reg.action;
+    var d = reg.dialog;
     e.register({type: 'user', spec: exp.entity_spec});
     a.register('reset_password', IPA.user.reset_password_action);
+    d.copy('password', 'user_password', {
+        factory: IPA.user.password_dialog,
+        pre_ops: [IPA.user.password_dialog_pre_op]
+    });
+    d.register_pre_op('user_password', IPA.user.password_dialog_pre_op0, true);
 };
 phases.on('registration', exp.register);
 
