@@ -18,10 +18,11 @@
 #
 
 from ipapython import ipautil
-from ipapython import services as ipaservices
 import shutil
 import os
-import sys
+from ipaplatform.tasks import tasks
+from ipaplatform import services
+from ipaplatform.paths import paths
 
 ntp_conf = """# Permit time synchronization with our time source, but do not
 # permit the source to query or modify the service on this system.
@@ -96,9 +97,9 @@ def __write_config(path, content):
     fd.close()
 
 def config_ntp(server_fqdn, fstore = None, sysstore = None):
-    path_step_tickers = "/etc/ntp/step-tickers"
-    path_ntp_conf = "/etc/ntp.conf"
-    path_ntp_sysconfig = "/etc/sysconfig/ntpd"
+    path_step_tickers = paths.NTP_STEP_TICKERS
+    path_ntp_conf = paths.NTP_CONF
+    path_ntp_sysconfig = paths.SYSCONFIG_NTPD
     sub_dict = { }
     sub_dict["SERVER"] = server_fqdn
 
@@ -111,49 +112,48 @@ def config_ntp(server_fqdn, fstore = None, sysstore = None):
         ns = ipautil.template_str(ntp_step_tickers, sub_dict)
         __backup_config(path_step_tickers, fstore)
         __write_config(path_step_tickers, ns)
-        ipaservices.restore_context(path_step_tickers)
+        tasks.restore_context(path_step_tickers)
 
     if sysstore:
         module = 'ntp'
-        sysstore.backup_state(module, "enabled", ipaservices.knownservices.ntpd.is_enabled())
+        sysstore.backup_state(module, "enabled", services.knownservices.ntpd.is_enabled())
         if config_step_tickers:
             sysstore.backup_state(module, "step-tickers", True)
 
     __backup_config(path_ntp_conf, fstore)
     __write_config(path_ntp_conf, nc)
-    ipaservices.restore_context(path_ntp_conf)
+    tasks.restore_context(path_ntp_conf)
 
     __backup_config(path_ntp_sysconfig, fstore)
     __write_config(path_ntp_sysconfig, ntp_sysconfig)
-    ipaservices.restore_context(path_ntp_sysconfig)
+    tasks.restore_context(path_ntp_sysconfig)
 
     # Set the ntpd to start on boot
-    ipaservices.knownservices.ntpd.enable()
+    services.knownservices.ntpd.enable()
 
     # Restart ntpd
-    ipaservices.knownservices.ntpd.restart()
+    services.knownservices.ntpd.restart()
 
 
 def synconce_ntp(server_fqdn):
     """
-    Syncs time with specified server using ntpdate.
+    Syncs time with specified server using ntpd.
     Primarily designed to be used before Kerberos setup
     to get time following the KDC time
 
     Returns True if sync was successful
     """
-    ntpdate="/usr/sbin/ntpdate"
-    if os.path.exists(ntpdate):
-        # retry several times -- logic follows /etc/init.d/ntpdate
-        # implementation
-        cmd = [ntpdate, "-U", "ntp", "-s", "-b", "-v", server_fqdn]
-        for retry in range(0, 3):
-            try:
-                ipautil.run(cmd)
-                return True
-            except:
-                pass
-    return False
+    ntpd = paths.NTPD
+    if not os.path.exists(ntpd):
+        return False
+
+    tmp_ntp_conf = ipautil.write_tmp_file('server %s' % server_fqdn)
+    try:
+        ipautil.run([ntpd, '-qgc', tmp_ntp_conf.name])
+        return True
+    except ipautil.CalledProcessError:
+        return False
+
 
 class NTPConfigurationError(Exception):
     pass
@@ -175,49 +175,49 @@ def check_timedate_services():
       https://fedorahosted.org/freeipa/ticket/2974
       http://fedoraproject.org/wiki/Features/ChronyDefaultNTP
     """
-    for service in ipaservices.timedate_services:
+    for service in services.timedate_services:
         if service == 'ntpd':
             continue
         # Make sure that the service is not enabled
-        service = ipaservices.service(service)
-        if service.is_enabled() or service.is_running():
-            raise NTPConflictingService(conflicting_service=service.service_name)
+        instance = services.service(service)
+        if instance.is_enabled() or instance.is_running():
+            raise NTPConflictingService(conflicting_service=instance.service_name)
 
 def force_ntpd(statestore):
     """
     Force ntpd configuration and disable and stop any other conflicting
     time&date service
     """
-    for service in ipaservices.timedate_services:
+    for service in services.timedate_services:
         if service == 'ntpd':
             continue
-        service = ipaservices.service(service)
-        enabled = service.is_enabled()
-        running = service.is_running()
+        instance = services.service(service)
+        enabled = instance.is_enabled()
+        running = instance.is_running()
 
         if enabled or running:
-            statestore.backup_state(service.service_name, 'enabled', enabled)
-            statestore.backup_state(service.service_name, 'running', running)
+            statestore.backup_state(instance.service_name, 'enabled', enabled)
+            statestore.backup_state(instance.service_name, 'running', running)
 
             if running:
-                service.stop()
+                instance.stop()
 
             if enabled:
-                service.disable()
+                instance.disable()
 
 def restore_forced_ntpd(statestore):
     """
     Restore from --force-ntpd installation and enable/start service that were
     disabled/stopped during installation
     """
-    for service in ipaservices.timedate_services:
+    for service in services.timedate_services:
         if service == 'ntpd':
             continue
         if statestore.has_state(service):
-            service = ipaservices.service(service)
-            enabled = statestore.restore_state(service.service_name, 'enabled')
-            running = statestore.restore_state(service.service_name, 'running')
+            instance = services.service(service)
+            enabled = statestore.restore_state(instance.service_name, 'enabled')
+            running = statestore.restore_state(instance.service_name, 'running')
             if enabled:
-                service.enable()
+                instance.enable()
             if running:
-                service.start()
+                instance.start()
