@@ -140,7 +140,7 @@ implement a ``run()`` method, like this:
 >>> api = create_api()
 >>> api.register(my_command)
 >>> api.finalize()
->>> api.Command.my_command() # Call your command
+>>> api.Command.my_command(version=u'2.47') # Call your command
 {'result': 'My run() method was called!'}
 
 When `frontend.Command.__call__()` is called, it first validates any arguments
@@ -359,7 +359,7 @@ False
 
 And yet we can call ``my_command()``:
 
->>> api.Command.my_command()
+>>> api.Command.my_command(version=u'2.47')
 {'result': 'Just my_command.forward() getting called here.'}
 
 
@@ -883,10 +883,11 @@ import os
 import plugable
 from backend import Backend
 from frontend import Command, LocalOrRemote, Updater, Advice
-from frontend import Object, Method, Property
+from frontend import Object, Method
 from crud import Create, Retrieve, Update, Delete, Search
 from parameters import DefaultFrom, Bool, Flag, Int, Decimal, Bytes, Str, IA5Str, Password, DNParam, DeprecatedParam
-from parameters import BytesEnum, StrEnum, AccessTime, File
+from parameters import (BytesEnum, StrEnum, IntEnum, AccessTime, File,
+                        DateTime, DNSNameParam)
 from errors import SkipPluginModule
 from text import _, ngettext, GettextFactory, NGettextFactory
 
@@ -910,14 +911,11 @@ def create_api(mode='dummy'):
 
         - `frontend.Method`
 
-        - `frontend.Property`
-
         - `frontend.Advice`
 
         - `backend.Backend`
     """
-    api = plugable.API(Command, Object, Method, Property, Backend, Updater,
-                       Advice)
+    api = plugable.API(Command, Object, Method, Backend, Updater, Advice)
     if mode is not None:
         api.env.mode = mode
     assert mode != 'production'
