@@ -419,6 +419,11 @@ class Command(HasParam):
         XML-RPC and the executed an the nearest IPA server.
         """
         self.ensure_finalized()
+        version_provided = 'version' in options
+        if version_provided:
+            self.verify_client_version(unicode(options['version']))
+        else:
+            options['version'] = API_VERSION
         params = self.args_options_2_params(*args, **options)
         self.debug(
             'raw: %s(%s)', self.name, ', '.join(self._repr_iter(**params))
@@ -429,22 +434,21 @@ class Command(HasParam):
         self.debug(
             '%s(%s)', self.name, ', '.join(self._repr_iter(**params))
         )
-        if not self.api.env.in_server and 'version' not in params:
-            params['version'] = API_VERSION
         self.validate(**params)
         (args, options) = self.params_2_args_options(**params)
         ret = self.run(*args, **options)
+        if not version_provided and isinstance(ret, dict):
+            messages.add_message(
+                API_VERSION, ret,
+                messages.VersionMissing(server_version=API_VERSION))
         if (
             isinstance(ret, dict)
             and 'summary' in self.output
             and 'summary' not in ret
         ):
-            if self.msg_summary:
-                ret['summary'] = self.msg_summary % ret
-            else:
-                ret['summary'] = None
+            ret['summary'] = self.get_summary_default(ret)
         if self.use_output_validation and (self.output or ret is not None):
-            self.validate_output(ret)
+            self.validate_output(ret, options['version'])
         return ret
 
     def soft_validate(self, values):
@@ -742,22 +746,12 @@ class Command(HasParam):
         actually work this command performs is executed locally.
 
         If running in a non-server context, `Command.forward` is called,
-        which forwards this call over XML-RPC to the exact same command
+        which forwards this call over RPC to the exact same command
         on the nearest IPA server and the actual work this command
         performs is executed remotely.
         """
         if self.api.env.in_server:
-            version_provided = 'version' in options
-            if version_provided:
-                self.verify_client_version(options['version'])
-            else:
-                options['version'] = API_VERSION
-            result = self.execute(*args, **options)
-            if not version_provided:
-                messages.add_message(
-                    API_VERSION, result,
-                    messages.VersionMissing(server_version=API_VERSION))
-            return result
+            return self.execute(*args, **options)
         return self.forward(*args, **options)
 
     def execute(self, *args, **kw):
@@ -777,9 +771,9 @@ class Command(HasParam):
 
     def forward(self, *args, **kw):
         """
-        Forward call over XML-RPC to this same command on server.
+        Forward call over RPC to this same command on server.
         """
-        return self.Backend.xmlclient.forward(self.name, *args, **kw)
+        return self.Backend.rpcclient.forward(self.name, *args, **kw)
 
     def _on_finalize(self):
         """
@@ -918,7 +912,7 @@ class Command(HasParam):
             flags=['no_option', 'no_output'],
         )
 
-    def validate_output(self, output):
+    def validate_output(self, output, version=API_VERSION):
         """
         Validate the return value to make sure it meets the interface contract.
         """
@@ -947,7 +941,7 @@ class Command(HasParam):
                     nice, o.name, o.type, type(value), value)
                 )
             if callable(o.validate):
-                o.validate(self, value)
+                o.validate(self, value, version)
 
     def get_output_params(self):
         for param in self._get_param_iterable('output_params', verb='has'):
@@ -959,6 +953,10 @@ class Command(HasParam):
                 continue
             yield param
 
+    def get_summary_default(self, output):
+        if self.msg_summary:
+            return self.msg_summary % output
+
     def log_messages(self, output, logger):
         logger_functions = dict(
             debug=logger.debug,
@@ -1126,7 +1124,6 @@ class Object(HasParam):
     # Create stubs for attributes that are set in _on_finalize()
     backend = Plugin.finalize_attr('backend')
     methods = Plugin.finalize_attr('methods')
-    properties = Plugin.finalize_attr('properties')
     params = Plugin.finalize_attr('params')
     primary_key = Plugin.finalize_attr('primary_key')
     params_minus_pk = Plugin.finalize_attr('params_minus_pk')
@@ -1139,9 +1136,6 @@ class Object(HasParam):
         self.methods = NameSpace(
             self.__get_attrs('Method'), sort=False, name_attr='attr_name'
         )
-        self.properties = NameSpace(
-            self.__get_attrs('Property'), sort=False, name_attr='attr_name'
-        )
         self._create_param_namespace('params')
         pkeys = filter(lambda p: p.primary_key, self.params())
         if len(pkeys) > 1:
@@ -1196,25 +1190,19 @@ class Object(HasParam):
         """
         This method gets called by `HasParam._create_param_namespace()`.
         """
-        props = self.properties.__todict__()
         for spec in self._get_param_iterable('params'):
             if type(spec) is str:
                 key = spec.rstrip('?*+')
             else:
                 assert isinstance(spec, Param)
                 key = spec.name
-            if key in props:
-                yield props.pop(key).param
-            else:
-                yield create_param(spec)
+            yield create_param(spec)
         def get_key(p):
             if p.param.required:
                 if p.param.default_from is None:
                     return 0
                 return 1
             return 2
-        for prop in sorted(props.itervalues(), key=get_key):
-            yield prop.param
 
 
 class Attribute(Plugin):
@@ -1249,8 +1237,7 @@ class Attribute(Plugin):
     'add'
 
     In practice the `Attribute` class is not used directly, but rather is
-    only the base class for the `Method` and `Property` classes.  Also see
-    the `Object` class.
+    only the base class for the `Method` class.  Also see the `Object` class.
     """
     finalize_early = False
 
@@ -1328,15 +1315,18 @@ class Method(Attribute, Command):
 
     >>> list(api.Method)
     ['user_add']
-    >>> api.Method.user_add() # Will call user_add.run()
+    >>> api.Method.user_add(version=u'2.88')  # Will call user_add.run()
     {'result': 'Added the user!'}
 
+    (The "version" argument is the API version to use.
+    The current API version can be found in ipalib.version.API_VERSION.)
+
     Second, because `Method` is a subclass of `Command`, the ``user_add``
     plugin can also be accessed through the ``api.Command`` namespace:
 
     >>> list(api.Command)
     ['user_add']
-    >>> api.Command.user_add() # Will call user_add.run()
+    >>> api.Command.user_add(version=u'2.88') # Will call user_add.run()
     {'result': 'Added the user!'}
 
     And third, ``user_add`` can be accessed as an attribute on the ``user``
@@ -1346,12 +1336,11 @@ class Method(Attribute, Command):
     ['user']
     >>> list(api.Object.user.methods)
     ['add']
-    >>> api.Object.user.methods.add() # Will call user_add.run()
+    >>> api.Object.user.methods.add(version=u'2.88') # Will call user_add.run()
     {'result': 'Added the user!'}
 
     The `Attribute` base class implements the naming convention for the
-    attribute-to-object association.  Also see the `Object` and the
-    `Property` classes.
+    attribute-to-object association.  Also see the `Object` class.
     """
     extra_options_first = False
     extra_args_first = False
@@ -1373,45 +1362,6 @@ class Method(Attribute, Command):
             yield param
 
 
-class Property(Attribute):
-    klass = Str
-    default = None
-    default_from = None
-    normalizer = None
-
-    def __init__(self):
-        super(Property, self).__init__()
-        # FIXME: This is a hack till Param.label is updated to require a
-        # LazyText instance:
-        self.label = None
-        self.rules = tuple(
-            sorted(self.__rules_iter(), key=lambda f: getattr(f, '__name__'))
-        )
-        self.kwargs = tuple(
-            sorted(self.__kw_iter(), key=lambda keyvalue: keyvalue[0])
-        )
-        kw = dict(self.kwargs)
-        self.param = self.klass(self.attr_name, *self.rules, **kw)
-
-    def __kw_iter(self):
-        for (key, kind, default) in self.klass.kwargs:
-            if getattr(self, key, None) is not None:
-                yield (key, getattr(self, key))
-
-    def __rules_iter(self):
-        """
-        Iterates through the attributes in this instance to retrieve the
-        methods implementing validation rules.
-        """
-        for name in dir(self.__class__):
-            if name.startswith('_'):
-                continue
-            base_attr = getattr(self.__class__, name)
-            if is_rule(base_attr):
-                attr = getattr(self, name)
-                if is_rule(attr):
-                    yield attr
-
 class Updater(Method):
     """
     An LDAP update with an associated object (always update).
