@@ -43,6 +43,7 @@ from text import _
 from base import ReadOnly, NameSpace, lock, islocked, check_name
 from constants import DEFAULT_CONFIG
 from ipapython.ipa_log_manager import *
+from ipapython.version import VERSION, API_VERSION
 
 # FIXME: Updated constants.TYPE_ERROR to use this clearer format from wehjit:
 TYPE_ERROR = '%s: need a %r; got a %r: %r'
@@ -59,6 +60,33 @@ def is_production_mode(obj):
     return obj.env.mode == 'production'
 
 
+class Registry(object):
+    """A decorator that makes plugins available to the API
+
+    Usage::
+
+        register = Registry()
+
+        @register()
+        class obj_mod(...):
+            ...
+
+    For forward compatibility, make sure that the module-level instance of
+    this object is named "register".
+    """
+    # TODO: Instead of auto-loading when plugin modules are imported,
+    # plugins should be stored in this object.
+    # The API should examine it and load plugins explicitly.
+    def __call__(self):
+        from ipalib import api
+
+        def decorator(cls):
+            api.register(cls)
+            return cls
+
+        return decorator
+
+
 class SetProxy(ReadOnly):
     """
     A read-only container with set/sequence behaviour.
@@ -466,6 +494,12 @@ class API(DictProxy):
         object.__setattr__(self, 'log_mgr', log_mgr)
         log = log_mgr.root_logger
         object.__setattr__(self, 'log', log)
+
+        # Add the argument parser
+        if not parser:
+            parser = self.build_global_parser()
+        object.__setattr__(self, 'parser', parser)
+
         # If logging has already been configured somewhere else (like in the
         # installer), don't add handlers or change levels:
         if log_mgr.configure_state != 'default' or self.env.validate_api:
@@ -491,10 +525,6 @@ class API(DictProxy):
                                           level=level,
                                           format=LOGGING_FORMAT_STDERR)])
 
-        if not parser:
-            parser = self.build_global_parser()
-        object.__setattr__(self, 'parser', parser)
-
         # Add file handler:
         if self.env.mode in ('dummy', 'unit_test'):
             return  # But not if in unit-test mode
@@ -530,6 +560,8 @@ class API(DictProxy):
                 formatter=IPAHelpFormatter(),
                 usage='%prog [global-options] COMMAND [command-options]',
                 description='Manage an IPA domain',
+                version=('VERSION: %s, API_VERSION: %s'
+                                % (VERSION, API_VERSION)),
                 epilog='\n'.join([
                     'See "ipa help topics" for available help topics.',
                     'See "ipa help <TOPIC>" for more information on a '
