@@ -32,7 +32,9 @@ from ipalib.cli import to_cli
 from ipalib import output
 from ipalib.text import _
 from ipalib.util import json_serialize, validate_hostname
+from ipalib.capabilities import client_has_capability
 from ipapython.dn import DN, RDN
+from ipapython.version import API_VERSION
 
 DNA_MAGIC = -1
 
@@ -203,11 +205,15 @@ def get_effective_rights(ldap, dn, attrs=None):
         attrs = ['*', 'nsaccountlock', 'cospriority']
     rights = ldap.get_effective_rights(dn, attrs)
     rdict = {}
-    if 'attributelevelrights' in rights[1]:
-        rights = rights[1]['attributelevelrights']
+    if 'attributelevelrights' in rights:
+        rights = rights['attributelevelrights']
         rights = rights[0].split(', ')
         for r in rights:
             (k,v) = r.split(':')
+            if v == 'none':
+                # the string "none" means "no rights found"
+                # see https://fedorahosted.org/freeipa/ticket/4359
+                v = u''
             rdict[k.strip().lower()] = v
 
     return rdict
@@ -233,13 +239,40 @@ def entry_from_entry(entry, newentry):
 
 def entry_to_dict(entry, **options):
     if options.get('raw', False):
-        result = dict(entry)
+        result = {}
+        for attr in entry.iterkeys():
+            if attr.lower() == 'attributelevelrights':
+                value = entry[attr]
+            elif entry.conn.get_type(attr) is str:
+                value = entry.raw[attr]
+            else:
+                value = list(entry.raw[attr])
+                for (i, v) in enumerate(value):
+                    try:
+                        value[i] = v.decode('utf-8')
+                    except UnicodeDecodeError:
+                        pass
+            result[attr] = value
     else:
         result = dict((k.lower(), v) for (k, v) in entry.iteritems())
     if options.get('all', False):
         result['dn'] = entry.dn
     return result
 
+def pkey_to_unicode(key):
+    if key is None:
+        key = []
+    elif not isinstance(key, (tuple, list)):
+        key = [key]
+    key = u','.join(unicode(k) for k in key)
+    return key
+
+def pkey_to_value(key, options):
+    version = options.get('version', API_VERSION)
+    if client_has_capability(version, 'primary_key_types'):
+        return key
+    return pkey_to_unicode(key)
+
 def wait_for_value(ldap, dn, attr, value):
     """
     389-ds postoperation plugins are executed after the data has been
@@ -262,7 +295,7 @@ def wait_for_value(ldap, dn, attr, value):
 
         # FIXME: put a try/except around here? I think it is probably better
         # to just let the exception filter up to the caller.
-        (dn, entry_attrs) = ldap.get_entry( dn, ['*'])
+        entry_attrs = ldap.get_entry(dn, ['*'])
         if attr in entry_attrs:
             if isinstance(entry_attrs[attr], (list, tuple)):
                 values = map(lambda y:y.lower(), entry_attrs[attr])
@@ -297,6 +330,7 @@ def add_external_pre_callback(membertype, ldap, dn, keys, options):
     membertype is the type of member
     """
     assert isinstance(dn, DN)
+
     # validate hostname with allowed underscore characters, non-fqdn
     # hostnames are allowed
     def validate_host(hostname):
@@ -316,32 +350,45 @@ def add_external_pre_callback(membertype, ldap, dn, keys, options):
                 raise errors.ValidationError(name=membertype, error=e)
     return dn
 
-def add_external_post_callback(memberattr, membertype, externalattr, ldap, completed, failed, dn, entry_attrs, *keys, **options):
+
+def add_external_post_callback(ldap, dn, entry_attrs, failed, completed,
+                               memberattr, membertype, externalattr,
+                               normalize=True):
     """
-    Post callback to add failed members as external members.
+    Takes the following arguments:
+        failed - the list of failed entries, these are candidates for possible
+                 external entries to add
+        completed - the number of successfully added entries so far
+        memberattr - the attribute name that IPA uses for membership natively
+                     (e.g. memberhost)
+        membertype - the object type of the member (e.g. host)
+        externalattr - the attribute name that IPA uses to store the membership
+                       of the entries that are not managed by IPA
+                       (e.g externalhost)
 
-    This should be called by a commands post callback directly.
-
-    memberattr is one of memberuser,
-    membertype is the type of member: user,
-    externalattr is one of externaluser,
+    Returns the number of completed entries so far (the number of entries
+    handled by IPA incremented by the number of handled external entries) and
+    dn.
     """
     assert isinstance(dn, DN)
+
     completed_external = 0
-    normalize = options.get('external_callback_normalize', True)
+
     # Sift through the failures. We assume that these are all
     # entries that aren't stored in IPA, aka external entries.
     if memberattr in failed and membertype in failed[memberattr]:
-        (dn, entry_attrs_) = ldap.get_entry(dn, [externalattr])
-        assert isinstance(dn, DN)
+        entry_attrs_ = ldap.get_entry(dn, [externalattr])
+        dn = entry_attrs_.dn
         members = entry_attrs.get(memberattr, [])
         external_entries = entry_attrs_.get(externalattr, [])
         lc_external_entries = set(e.lower() for e in external_entries)
+
         failed_entries = []
         for entry in failed[memberattr][membertype]:
             membername = entry[0].lower()
             member_dn = api.Object[membertype].get_dn(membername)
             assert isinstance(member_dn, DN)
+
             if (membername not in lc_external_entries and
                 member_dn not in members):
                 # Not an IPA entry, assume external
@@ -364,8 +411,9 @@ def add_external_post_callback(memberattr, membertype, externalattr, ldap, compl
                 failed_entries.append(membername)
 
         if completed_external:
+            entry_attrs_[externalattr] = external_entries
             try:
-                ldap.update_entry(dn, {externalattr: external_entries})
+                ldap.update_entry(entry_attrs_)
             except errors.EmptyModlist:
                 pass
             failed[memberattr][membertype] = failed_entries
@@ -373,15 +421,37 @@ def add_external_post_callback(memberattr, membertype, externalattr, ldap, compl
 
     return (completed + completed_external, dn)
 
-def remove_external_post_callback(memberattr, membertype, externalattr, ldap, completed, failed, dn, entry_attrs, *keys, **options):
+
+def remove_external_post_callback(ldap, dn, entry_attrs, failed, completed,
+                                  memberattr, membertype, externalattr):
+    """
+    Takes the following arguments:
+        failed - the list of failed entries, these are candidates for possible
+                 external entries to remove
+        completed - the number of successfully removed entries so far
+        memberattr - the attribute name that IPA uses for membership natively
+                     (e.g. memberhost)
+        membertype - the object type of the member (e.g. host)
+        externalattr - the attribute name that IPA uses to store the membership
+                       of the entries that are not managed by IPA
+                       (e.g externalhost)
+
+    Returns the number of completed entries so far (the number of entries
+    handled by IPA incremented by the number of handled external entries) and
+    dn.
+    """
+
     assert isinstance(dn, DN)
+
     # Run through the failures and gracefully remove any member defined
     # as an external member.
     if memberattr in failed and membertype in failed[memberattr]:
-        (dn, entry_attrs_) = ldap.get_entry(dn, [externalattr])
+        entry_attrs_ = ldap.get_entry(dn, [externalattr])
+        dn = entry_attrs_.dn
         external_entries = entry_attrs_.get(externalattr, [])
         failed_entries = []
         completed_external = 0
+
         for entry in failed[memberattr][membertype]:
             membername = entry[0].lower()
             if membername in external_entries or entry[0] in external_entries:
@@ -398,8 +468,9 @@ def remove_external_post_callback(memberattr, membertype, externalattr, ldap, co
                 failed_entries.append(membername)
 
         if completed_external:
+            entry_attrs_[externalattr] = external_entries
             try:
-                ldap.update_entry(dn, {externalattr: external_entries})
+                ldap.update_entry(entry_attrs_)
             except errors.EmptyModlist:
                 pass
             failed[memberattr][membertype] = failed_entries
@@ -407,6 +478,7 @@ def remove_external_post_callback(memberattr, membertype, externalattr, ldap, co
 
     return (completed + completed_external, dn)
 
+
 def host_is_master(ldap, fqdn):
     """
     Check to see if this host is a master.
@@ -415,7 +487,7 @@ def host_is_master(ldap, fqdn):
     """
     master_dn = DN(('cn', fqdn), ('cn', 'masters'), ('cn', 'ipa'), ('cn', 'etc'), api.env.basedn)
     try:
-        (dn, entry_attrs) = ldap.get_entry(master_dn, ['objectclass'])
+        ldap.get_entry(master_dn, ['objectclass'])
         raise errors.ValidationError(name='hostname', error=_('An IPA master host cannot be deleted or disabled'))
     except errors.NotFound:
         # Good, not a master
@@ -439,6 +511,7 @@ class LDAPObject(Object):
     possible_objectclasses = []
     limit_object_classes = [] # Only attributes in these are allowed
     disallow_object_classes = [] # Disallow attributes in these
+    permission_filter_objectclasses = None
     search_attributes = []
     search_attributes_config = None
     default_attributes = []
@@ -465,6 +538,7 @@ class LDAPObject(Object):
     }
     label = _('Entry')
     label_singular = _('Entry')
+    managed_permissions = {}
 
     container_not_found_msg = _('container entry (%(container)s) not found')
     parent_not_found_msg = _('%(parent)s: %(oname)s not found')
@@ -478,14 +552,14 @@ class LDAPObject(Object):
             parent_dn = DN(self.container_dn, api.env.basedn)
         if self.rdn_attribute:
             try:
-                (dn, entry_attrs) = self.backend.find_entry_by_attr(
+                entry_attrs = self.backend.find_entry_by_attr(
                     self.primary_key.name, keys[-1], self.object_class, [''],
                     DN(self.container_dn, api.env.basedn)
                 )
             except errors.NotFound:
                 pass
             else:
-                return dn
+                return entry_attrs.dn
         if self.primary_key and keys[-1] is not None:
             return self.backend.make_dn_from_attr(
                 self.primary_key.name, keys[-1], parent_dn
@@ -502,7 +576,7 @@ class LDAPObject(Object):
         assert isinstance(dn, DN)
         try:
             if self.rdn_attribute:
-                (dn, entry_attrs) = self.backend.get_entry(
+                entry_attrs = self.backend.get_entry(
                     dn, [self.primary_key.name]
                 )
                 try:
@@ -528,7 +602,7 @@ class LDAPObject(Object):
                 yield key
             if parent_obj.primary_key:
                 pkey = parent_obj.primary_key
-                yield pkey.__class__(
+                yield pkey.clone_rename(
                     parent_obj.name + pkey.name, required=True, query=True,
                     cli_name=parent_obj.name, label=pkey.label
                 )
@@ -582,9 +656,10 @@ class LDAPObject(Object):
         )
 
     def handle_duplicate_entry(self, *keys):
-        pkey = ''
-        if self.primary_key:
+        try:
             pkey = keys[-1]
+        except KeyError:
+            pkey = ''
         raise errors.DuplicateEntry(
             message=self.already_exists_msg % {
                 'pkey': pkey, 'oname': self.object_name,
@@ -608,7 +683,7 @@ class LDAPObject(Object):
             json_dict['primary_key'] = self.primary_key.name
         objectclasses = self.object_class
         if self.object_class_config:
-            config = ldap.get_ipa_config()[1]
+            config = ldap.get_ipa_config()
             objectclasses = config.get(
                 self.object_class_config, objectclasses
             )
@@ -626,6 +701,8 @@ class LDAPObject(Object):
         json_dict['aciattrs'] = attrlist
         attrlist.sort()
         json_dict['methods'] = [m for m in self.methods]
+        json_dict['can_have_permissions'] = bool(
+            self.permission_filter_objectclasses)
         return json_dict
 
 
@@ -760,6 +837,12 @@ last, after all sets and adds."""),
 
     _callback_registry = dict(pre={}, post={}, exc={}, interactive_prompt={})
 
+    def get_summary_default(self, output):
+        if 'value' in output:
+            output = dict(output)
+            output['value'] = pkey_to_unicode(output['value'])
+        return super(BaseLDAPCommand, self).get_summary_default(output)
+
     def _convert_2_dict(self, ldap, attrs):
         """
         Convert a string in the form of name/value pairs into a dictionary.
@@ -871,7 +954,7 @@ last, after all sets and adds."""),
 
         if needldapattrs:
             try:
-                (dn, old_entry) = self._exc_wrapper(keys, options, ldap.get_entry)(
+                old_entry = self._exc_wrapper(keys, options, ldap.get_entry)(
                     dn, needldapattrs
                 )
             except errors.NotFound:
@@ -922,6 +1005,8 @@ last, after all sets and adds."""),
                     raise errors.ValidationError(name=attr, error=err.error)
                 except errors.ConversionError, err:
                     raise errors.ConversionError(name=attr, error=err.error)
+                if isinstance(value, tuple):
+                    value = list(value)
                 entry_attrs[attr] = value
             else:
                 # unknown attribute: remove duplicite and invalid values
@@ -991,12 +1076,9 @@ class LDAPCreate(BaseLDAPCommand, crud.Create):
     takes_options = (BaseLDAPCommand.setattr_option, BaseLDAPCommand.addattr_option)
 
     def get_args(self):
-        #pylint: disable=E1003
         for key in self.obj.get_ancestor_primary_keys():
             yield key
-        if self.obj.primary_key:
-            yield self.obj.primary_key.clone(attribute=True)
-        for arg in super(crud.Create, self).get_args():
+        for arg in super(LDAPCreate, self).get_args():
             yield arg
 
     has_output_params = global_output_params
@@ -1004,15 +1086,16 @@ class LDAPCreate(BaseLDAPCommand, crud.Create):
     def execute(self, *keys, **options):
         ldap = self.obj.backend
 
-        entry_attrs = self.args_options_2_entry(*keys, **options)
-        entry_attrs = ldap.make_entry(DN(), entry_attrs)
+        dn = self.obj.get_dn(*keys, **options)
+        entry_attrs = ldap.make_entry(
+            dn, self.args_options_2_entry(*keys, **options))
 
         self.process_attr_options(entry_attrs, None, keys, options)
 
         entry_attrs['objectclass'] = deepcopy(self.obj.object_class)
 
         if self.obj.object_class_config:
-            config = ldap.get_ipa_config()[1]
+            config = ldap.get_ipa_config()
             entry_attrs['objectclass'] = config.get(
                 self.obj.object_class_config, entry_attrs['objectclass']
             )
@@ -1020,8 +1103,6 @@ class LDAPCreate(BaseLDAPCommand, crud.Create):
         if self.obj.uuid_attribute:
             entry_attrs[self.obj.uuid_attribute] = 'autogenerate'
 
-        dn = self.obj.get_dn(*keys, **options)
-        assert isinstance(dn, DN)
         if self.obj.rdn_attribute:
             try:
                 dn_attr = dn[0].attr
@@ -1029,10 +1110,9 @@ class LDAPCreate(BaseLDAPCommand, crud.Create):
                 dn_attr = None
             if dn_attr != self.obj.primary_key.name:
                 self.obj.handle_duplicate_entry(*keys)
-            dn = ldap.make_dn(
+            entry_attrs.dn = ldap.make_dn(
                 entry_attrs, self.obj.rdn_attribute,
-                DN(self.obj.container_dn, api.env.basedn)
-            )
+                DN(self.obj.container_dn, api.env.basedn))
 
         if options.get('all', False):
             attrs_list = ['*'] + self.obj.default_attributes
@@ -1044,16 +1124,16 @@ class LDAPCreate(BaseLDAPCommand, crud.Create):
             attrs_list = list(attrs_list)
 
         for callback in self.get_callbacks('pre'):
-            dn = callback(
-                self, ldap, dn, entry_attrs, attrs_list, *keys, **options)
-            assert isinstance(dn, DN)
+            entry_attrs.dn = callback(
+                self, ldap, entry_attrs.dn, entry_attrs, attrs_list,
+                *keys, **options)
 
         _check_single_value_attrs(self.params, entry_attrs)
         _check_limit_object_class(self.api.Backend.ldap2.schema.attribute_types(self.obj.limit_object_classes), entry_attrs.keys(), allow_only=True)
         _check_limit_object_class(self.api.Backend.ldap2.schema.attribute_types(self.obj.disallow_object_classes), entry_attrs.keys(), allow_only=False)
 
         try:
-            self._exc_wrapper(keys, options, ldap.add_entry)(dn, entry_attrs)
+            self._exc_wrapper(keys, options, ldap.add_entry)(entry_attrs)
         except errors.NotFound:
             parent = self.obj.parent_object
             if parent:
@@ -1078,31 +1158,32 @@ class LDAPCreate(BaseLDAPCommand, crud.Create):
                     object_class = self.obj.object_class
                 else:
                     object_class = None
-                (dn, entry_attrs) = self._exc_wrapper(keys, options, ldap.find_entry_by_attr)(
+                entry_attrs = self._exc_wrapper(keys, options, ldap.find_entry_by_attr)(
                     self.obj.primary_key.name, keys[-1], object_class, attrs_list,
                     DN(self.obj.container_dn, api.env.basedn)
                 )
-                assert isinstance(dn, DN)
             else:
-                (dn, entry_attrs) = self._exc_wrapper(keys, options, ldap.get_entry)(
-                    dn, attrs_list
-                )
-                assert isinstance(dn, DN)
+                entry_attrs = self._exc_wrapper(keys, options, ldap.get_entry)(
+                    entry_attrs.dn, attrs_list)
         except errors.NotFound:
             self.obj.handle_not_found(*keys)
 
         for callback in self.get_callbacks('post'):
-            dn = callback(self, ldap, dn, entry_attrs, *keys, **options)
+            entry_attrs.dn = callback(
+                self, ldap, entry_attrs.dn, entry_attrs, *keys, **options)
 
         self.obj.convert_attribute_members(entry_attrs, *keys, **options)
 
-        assert isinstance(dn, DN)
+        dn = entry_attrs.dn
         entry_attrs = entry_to_dict(entry_attrs, **options)
         entry_attrs['dn'] = dn
 
-        if self.obj.primary_key and keys[-1] is not None:
-            return dict(result=entry_attrs, value=keys[-1])
-        return dict(result=entry_attrs, value=u'')
+        if self.obj.primary_key:
+            pkey = keys[-1]
+        else:
+            pkey = None
+
+        return dict(result=entry_attrs, value=pkey_to_value(pkey, options))
 
     def pre_callback(self, ldap, dn, entry_attrs, attrs_list, *keys, **options):
         assert isinstance(dn, DN)
@@ -1135,12 +1216,9 @@ class LDAPQuery(BaseLDAPCommand, crud.PKQuery):
     Base class for commands that need to retrieve an existing entry.
     """
     def get_args(self):
-        #pylint: disable=E1003
         for key in self.obj.get_ancestor_primary_keys():
             yield key
-        if self.obj.primary_key:
-            yield self.obj.primary_key.clone(attribute=True, query=True)
-        for arg in super(crud.PKQuery, self).get_args():
+        for arg in super(LDAPQuery, self).get_args():
             yield arg
 
     # list of attributes we want exported to JSON
@@ -1167,15 +1245,11 @@ class LDAPMultiQuery(LDAPQuery):
     )
 
     def get_args(self):
-        #pylint: disable=E1003
-        for key in self.obj.get_ancestor_primary_keys():
-            yield key
-        if self.obj.primary_key:
-            yield self.obj.primary_key.clone(
-                attribute=True, query=True, multivalue=True
-            )
-        for arg in super(crud.PKQuery, self).get_args():
-            yield arg
+        for arg in super(LDAPMultiQuery, self).get_args():
+            if self.obj.primary_key and arg.name == self.obj.primary_key.name:
+                yield arg.clone(multivalue=True)
+            else:
+                yield arg
 
 
 class LDAPRetrieve(LDAPQuery):
@@ -1211,29 +1285,32 @@ class LDAPRetrieve(LDAPQuery):
             assert isinstance(dn, DN)
 
         try:
-            (dn, entry_attrs) = self._exc_wrapper(keys, options, ldap.get_entry)(
+            entry_attrs = self._exc_wrapper(keys, options, ldap.get_entry)(
                 dn, attrs_list
             )
-            assert isinstance(dn, DN)
         except errors.NotFound:
             self.obj.handle_not_found(*keys)
 
         if options.get('rights', False) and options.get('all', False):
-            entry_attrs['attributelevelrights'] = get_effective_rights(ldap, dn)
+            entry_attrs['attributelevelrights'] = get_effective_rights(
+                ldap, entry_attrs.dn)
 
         for callback in self.get_callbacks('post'):
-            dn = callback(self, ldap, dn, entry_attrs, *keys, **options)
-            assert isinstance(dn, DN)
+            entry_attrs.dn = callback(
+                self, ldap, entry_attrs.dn, entry_attrs, *keys, **options)
 
         self.obj.convert_attribute_members(entry_attrs, *keys, **options)
 
-        assert isinstance(dn, DN)
+        dn = entry_attrs.dn
         entry_attrs = entry_to_dict(entry_attrs, **options)
         entry_attrs['dn'] = dn
 
-        if self.obj.primary_key and keys[-1] is not None:
-            return dict(result=entry_attrs, value=keys[-1])
-        return dict(result=entry_attrs, value=u'')
+        if self.obj.primary_key:
+            pkey = keys[-1]
+        else:
+            pkey = None
+
+        return dict(result=entry_attrs, value=pkey_to_value(pkey, options))
 
     def pre_callback(self, ldap, dn, attrs_list, *keys, **options):
         assert isinstance(dn, DN)
@@ -1289,10 +1366,7 @@ class LDAPUpdate(LDAPQuery, crud.Update):
             raise errors.EmptyModlist()
 
         dn = self.obj.get_dn(*keys, **options)
-        assert isinstance(dn, DN)
-
-        entry_attrs = self.args_options_2_entry(**options)
-        entry_attrs = ldap.make_entry(dn, entry_attrs)
+        entry_attrs = ldap.make_entry(dn, self.args_options_2_entry(**options))
 
         self.process_attr_options(entry_attrs, dn, keys, options)
 
@@ -1309,9 +1383,9 @@ class LDAPUpdate(LDAPQuery, crud.Update):
         _check_empty_attrs(self.obj.params, entry_attrs)
 
         for callback in self.get_callbacks('pre'):
-            dn = callback(
-                self, ldap, dn, entry_attrs, attrs_list, *keys, **options)
-            assert isinstance(dn, DN)
+            entry_attrs.dn = callback(
+                self, ldap, entry_attrs.dn, entry_attrs, attrs_list,
+                *keys, **options)
 
         _check_limit_object_class(self.api.Backend.ldap2.schema.attribute_types(self.obj.limit_object_classes), entry_attrs.keys(), allow_only=True)
         _check_limit_object_class(self.api.Backend.ldap2.schema.attribute_types(self.obj.disallow_object_classes), entry_attrs.keys(), allow_only=False)
@@ -1325,11 +1399,12 @@ class LDAPUpdate(LDAPQuery, crud.Update):
 
             if self.obj.rdn_is_primary_key and self.obj.primary_key.name in entry_attrs:
                 # RDN change
-                self._exc_wrapper(keys, options, ldap.update_entry_rdn)(dn,
-                    RDN((self.obj.primary_key.name, entry_attrs[self.obj.primary_key.name])))
+                self._exc_wrapper(keys, options, ldap.update_entry_rdn)(
+                    entry_attrs.dn,
+                    RDN((self.obj.primary_key.name,
+                         entry_attrs[self.obj.primary_key.name])))
                 rdnkeys = keys[:-1] + (entry_attrs[self.obj.primary_key.name], )
-                dn = self.obj.get_dn(*rdnkeys)
-                assert isinstance(dn, DN)
+                entry_attrs.dn = self.obj.get_dn(*rdnkeys)
                 del entry_attrs[self.obj.primary_key.name]
                 options['rdnupdate'] = True
                 rdnupdate = True
@@ -1338,8 +1413,11 @@ class LDAPUpdate(LDAPQuery, crud.Update):
             # to decide what to do. An EmptyModlist in this context doesn't
             # mean an error occurred, just that there were no other updates to
             # perform.
-            assert isinstance(dn, DN)
-            self._exc_wrapper(keys, options, ldap.update_entry)(dn, entry_attrs)
+            update = self._exc_wrapper(keys, options, ldap.get_entry)(
+                entry_attrs.dn, entry_attrs.keys())
+            update.update(entry_attrs)
+
+            self._exc_wrapper(keys, options, ldap.update_entry)(update)
         except errors.EmptyModlist, e:
             if not rdnupdate:
                 raise e
@@ -1347,28 +1425,31 @@ class LDAPUpdate(LDAPQuery, crud.Update):
             self.obj.handle_not_found(*keys)
 
         try:
-            (dn, entry_attrs) = self._exc_wrapper(keys, options, ldap.get_entry)(
-                dn, attrs_list
-            )
+            entry_attrs = self._exc_wrapper(keys, options, ldap.get_entry)(
+                entry_attrs.dn, attrs_list)
         except errors.NotFound:
             raise errors.MidairCollision(
                 format=_('the entry was deleted while being modified')
             )
 
         if options.get('rights', False) and options.get('all', False):
-            entry_attrs['attributelevelrights'] = get_effective_rights(ldap, dn)
+            entry_attrs['attributelevelrights'] = get_effective_rights(
+                ldap, entry_attrs.dn)
 
         for callback in self.get_callbacks('post'):
-            dn = callback(self, ldap, dn, entry_attrs, *keys, **options)
-            assert isinstance(dn, DN)
+            entry_attrs.dn = callback(
+                self, ldap, entry_attrs.dn, entry_attrs, *keys, **options)
 
         self.obj.convert_attribute_members(entry_attrs, *keys, **options)
 
         entry_attrs = entry_to_dict(entry_attrs, **options)
 
-        if self.obj.primary_key and keys[-1] is not None:
-            return dict(result=entry_attrs, value=keys[-1])
-        return dict(result=entry_attrs, value=u'')
+        if self.obj.primary_key:
+            pkey = keys[-1]
+        else:
+            pkey = None
+
+        return dict(result=entry_attrs, value=pkey_to_value(pkey, options))
 
     def pre_callback(self, ldap, dn, entry_attrs, attrs_list, *keys, **options):
         assert isinstance(dn, DN)
@@ -1389,7 +1470,7 @@ class LDAPDelete(LDAPMultiQuery):
     """
     Delete an LDAP entry and all of its direct subentries.
     """
-    has_output = output.standard_delete
+    has_output = output.standard_multi_delete
 
     has_output_params = global_output_params
 
@@ -1416,8 +1497,8 @@ class LDAPDelete(LDAPMultiQuery):
                     except errors.NotFound:
                         break
                     else:
-                        for (dn_, entry_attrs) in subentries:
-                            delete_subtree(dn_)
+                        for entry_attrs in subentries:
+                            delete_subtree(entry_attrs.dn)
                 try:
                     self._exc_wrapper(nkeys, options, ldap.delete_entry)(base_dn)
                 except errors.NotFound:
@@ -1436,28 +1517,28 @@ class LDAPDelete(LDAPMultiQuery):
 
             return result
 
-        if not self.obj.primary_key or not isinstance(keys[-1], (list, tuple)):
-            pkeyiter = (keys[-1], )
-        else:
+        if self.obj.primary_key and isinstance(keys[-1], (list, tuple)):
             pkeyiter = keys[-1]
+        elif keys[-1] is not None:
+            pkeyiter = [keys[-1]]
+        else:
+            pkeyiter = []
 
         deleted = []
         failed = []
-        result = True
         for pkey in pkeyiter:
             try:
-                if not delete_entry(pkey):
-                    result = False
+                delete_entry(pkey)
             except errors.ExecutionError:
                 if not options.get('continue', False):
                     raise
                 failed.append(pkey)
             else:
                 deleted.append(pkey)
+        deleted = pkey_to_value(deleted, options)
+        failed = pkey_to_value(failed, options)
 
-        if self.obj.primary_key and pkeyiter[0] is not None:
-            return dict(result=dict(failed=u','.join(failed)), value=u','.join(deleted))
-        return dict(result=dict(failed=u''), value=u'')
+        return dict(result=dict(failed=failed), value=deleted)
 
     def pre_callback(self, ldap, dn, *keys, **options):
         assert isinstance(dn, DN)
@@ -1579,21 +1660,20 @@ class LDAPAddMember(LDAPModMember):
             attrs_list = list(attrs_list)
 
         try:
-            (dn, entry_attrs) = self._exc_wrapper(keys, options, ldap.get_entry)(
+            entry_attrs = self._exc_wrapper(keys, options, ldap.get_entry)(
                 dn, attrs_list
             )
         except errors.NotFound:
             self.obj.handle_not_found(*keys)
 
         for callback in self.get_callbacks('post'):
-            (completed, dn) = callback(
-                self, ldap, completed, failed, dn, entry_attrs, *keys,
-                **options)
-            assert isinstance(dn, DN)
+            (completed, entry_attrs.dn) = callback(
+                self, ldap, completed, failed, entry_attrs.dn, entry_attrs,
+                *keys, **options)
 
         self.obj.convert_attribute_members(entry_attrs, *keys, **options)
 
-        assert isinstance(dn, DN)
+        dn = entry_attrs.dn
         entry_attrs = entry_to_dict(entry_attrs, **options)
         entry_attrs['dn'] = dn
 
@@ -1682,21 +1762,20 @@ class LDAPRemoveMember(LDAPModMember):
         time.sleep(.3)
 
         try:
-            (dn, entry_attrs) = self._exc_wrapper(keys, options, ldap.get_entry)(
+            entry_attrs = self._exc_wrapper(keys, options, ldap.get_entry)(
                 dn, attrs_list
             )
         except errors.NotFound:
             self.obj.handle_not_found(*keys)
 
         for callback in self.get_callbacks('post'):
-            (completed, dn) = callback(
-                self, ldap, completed, failed, dn, entry_attrs, *keys,
-                **options)
-            assert isinstance(dn, DN)
+            (completed, entry_attrs.dn) = callback(
+                self, ldap, completed, failed, entry_attrs.dn, entry_attrs,
+                *keys, **options)
 
         self.obj.convert_attribute_members(entry_attrs, *keys, **options)
 
-        assert isinstance(dn, DN)
+        dn = entry_attrs.dn
         entry_attrs = entry_to_dict(entry_attrs, **options)
         entry_attrs['dn'] = dn
 
@@ -1758,13 +1837,9 @@ class LDAPSearch(BaseLDAPCommand, crud.Search):
     )
 
     def get_args(self):
-        #pylint: disable=E1003
         for key in self.obj.get_ancestor_primary_keys():
             yield key
-        yield Str('criteria?',
-                  noextrawhitespace=False,
-                  doc=_('A string searched in all relevant object attributes'))
-        for arg in super(crud.Search, self).get_args():
+        for arg in super(LDAPSearch, self).get_args():
             yield arg
 
     def get_member_options(self, attr):
@@ -1863,7 +1938,7 @@ class LDAPSearch(BaseLDAPCommand, crud.Search):
         else:
             search_attrs = self.obj.default_attributes
         if self.obj.search_attributes_config:
-            config = ldap.get_ipa_config()[1]
+            config = ldap.get_ipa_config()
             config_attrs = config.get(
                 self.obj.search_attributes_config, [])
             if len(config_attrs) == 1 and (
@@ -1905,12 +1980,13 @@ class LDAPSearch(BaseLDAPCommand, crud.Search):
         if self.sort_result_entries:
             if self.obj.primary_key:
                 def sort_key(x):
-                    return x[1][self.obj.primary_key.name][0].lower()
+                    return self.obj.primary_key.sort_key(
+                        x[self.obj.primary_key.name][0])
                 entries.sort(key=sort_key)
 
         if not options.get('raw', False):
             for e in entries:
-                self.obj.convert_attribute_members(e[1], *args, **options)
+                self.obj.convert_attribute_members(e, *args, **options)
 
         for (i, e) in enumerate(entries):
             entries[i] = entry_to_dict(e, **options)
@@ -2042,16 +2118,15 @@ class LDAPAddReverseMember(LDAPModReverseMember):
                 failed['member'][self.reverse_attr].append((attr, unicode(msg)))
 
         # Update the member data.
-        (dn, entry_attrs) = ldap.get_entry(dn, ['*'])
+        entry_attrs = ldap.get_entry(dn, ['*'])
         self.obj.convert_attribute_members(entry_attrs, *keys, **options)
 
         for callback in self.get_callbacks('post'):
-            (completed, dn) = callback(
-                self, ldap, completed, failed, dn, entry_attrs, *keys,
-                **options)
-            assert isinstance(dn, DN)
+            (completed, entry_attrs.dn) = callback(
+                self, ldap, completed, failed, entry_attrs.dn, entry_attrs,
+                *keys, **options)
 
-        assert isinstance(dn, DN)
+        dn = entry_attrs.dn
         entry_attrs = entry_to_dict(entry_attrs, **options)
         entry_attrs['dn'] = dn
 
@@ -2147,16 +2222,15 @@ class LDAPRemoveReverseMember(LDAPModReverseMember):
                 failed['member'][self.reverse_attr].append((attr, unicode(msg)))
 
         # Update the member data.
-        (dn, entry_attrs) = ldap.get_entry(dn, ['*'])
+        entry_attrs = ldap.get_entry(dn, ['*'])
         self.obj.convert_attribute_members(entry_attrs, *keys, **options)
 
         for callback in self.get_callbacks('post'):
-            (completed, dn) = callback(
-                self, ldap, completed, failed, dn, entry_attrs, *keys,
-                **options)
-            assert isinstance(dn, DN)
+            (completed, entry_attrs.dn) = callback(
+                self, ldap, completed, failed, entry_attrs.dn, entry_attrs,
+                *keys, **options)
 
-        assert isinstance(dn, DN)
+        dn = entry_attrs.dn
         entry_attrs = entry_to_dict(entry_attrs, **options)
         entry_attrs['dn'] = dn
 
