@@ -31,6 +31,7 @@ from ipalib import pkcs10
 from ipalib import x509
 from ipalib import util
 from ipalib import ngettext
+from ipalib.plugable import Registry
 from ipalib.plugins.virtual import *
 from ipalib.plugins.service import split_principal
 import base64
@@ -41,6 +42,7 @@ from ipalib import output
 from ipalib.plugins.service import validate_principal
 import nss.nss as nss
 from nss.error import NSPRError
+from pyasn1.error import PyAsn1Error
 
 __doc__ = _("""
 IPA certificate operations
@@ -122,6 +124,8 @@ http://www.ietf.org/rfc/rfc5280.txt
 
 """)
 
+register = Registry()
+
 def validate_pkidate(ugettext, value):
     """
     A date in the format of %Y-%m-%d
@@ -133,32 +137,6 @@ def validate_pkidate(ugettext, value):
 
     return None
 
-def get_csr_hostname(csr):
-    """
-    Return the value of CN in the subject of the request or None
-    """
-    try:
-        request = pkcs10.load_certificate_request(csr)
-        subject = pkcs10.get_subject(request)
-        return subject.common_name
-    except NSPRError, nsprerr:
-        raise errors.CertificateOperationError(
-            error=_('Failure decoding Certificate Signing Request: %s') % nsprerr)
-
-def get_subjectaltname(csr):
-    """
-    Return the first value of the subject alt name, if any
-    """
-    try:
-        request = pkcs10.load_certificate_request(csr)
-        for extension in request.extensions:
-            if extension.oid_tag == nss.SEC_OID_X509_SUBJECT_ALT_NAME:
-                return nss.x509_alt_name(extension.value)[0]
-        return None
-    except NSPRError, nsprerr:
-        raise errors.CertificateOperationError(
-            error=_('Failure decoding Certificate Signing Request: %s') % nsprerr)
-
 def validate_csr(ugettext, csr):
     """
     Ensure the CSR is base64-encoded and can be decoded by our PKCS#10
@@ -236,6 +214,7 @@ def get_host_from_principal(principal):
 
     return hostname
 
+@register()
 class cert_request(VirtualCommand):
     __doc__ = _('Submit a certificate signing request.')
 
@@ -301,6 +280,14 @@ class cert_request(VirtualCommand):
         ),
     )
 
+    _allowed_extensions = {
+        '2.5.29.14': None,      # Subject Key Identifier
+        '2.5.29.15': None,      # Key Usage
+        '2.5.29.17': 'request certificate with subjectaltname',
+        '2.5.29.19': None,      # Basic Constraints
+        '2.5.29.37': None,      # Extended Key Usage
+    }
+
     def execute(self, csr, **kw):
         ldap = self.api.Backend.ldap2
         principal = kw.get('principal')
@@ -324,10 +311,22 @@ class cert_request(VirtualCommand):
         if not bind_principal.startswith('host/'):
             self.check_access()
 
-        # FIXME: add support for subject alt name
+        try:
+            subject = pkcs10.get_subject(csr)
+            extensions = pkcs10.get_extensions(csr)
+            subjectaltname = pkcs10.get_subjectaltname(csr) or ()
+        except (NSPRError, PyAsn1Error), e:
+            raise errors.CertificateOperationError(
+                error=_("Failure decoding Certificate Signing Request: %s") % e)
+
+        if not bind_principal.startswith('host/'):
+            for ext in extensions:
+                operation = self._allowed_extensions.get(ext)
+                if operation:
+                    self.check_access(operation)
 
         # Ensure that the hostname in the CSR matches the principal
-        subject_host = get_csr_hostname(csr)
+        subject_host = subject.common_name  #pylint: disable=E1101
         if not subject_host:
             raise errors.ValidationError(name='csr',
                 error=_("No hostname was found in subject of request."))
@@ -339,28 +338,36 @@ class cert_request(VirtualCommand):
                     "does not match principal hostname '%(hostname)s'") % dict(
                         subject_host=subject_host, hostname=hostname))
 
+        for ext in extensions:
+            if ext not in self._allowed_extensions:
+                raise errors.ValidationError(
+                    name='csr', error=_("extension %s is forbidden") % ext)
+
+        for name_type, name in subjectaltname:
+            if name_type not in (pkcs10.SAN_DNSNAME,
+                                 pkcs10.SAN_OTHERNAME_KRB5PRINCIPALNAME,
+                                 pkcs10.SAN_OTHERNAME_UPN):
+                raise errors.ValidationError(
+                    name='csr',
+                    error=_("subject alt name type %s is forbidden") %
+                          name_type)
+
         dn = None
         service = None
         # See if the service exists and punt if it doesn't and we aren't
         # going to add it
         try:
-            if not principal.startswith('host/'):
-                service = api.Command['service_show'](principal, all=True)['result']
-                dn = service['dn']
+            if servicename != 'host':
+                service = api.Command['service_show'](principal, all=True)
             else:
-                hostname = get_host_from_principal(principal)
-                service = api.Command['host_show'](hostname, all=True)['result']
-                dn = service['dn']
+                service = api.Command['host_show'](hostname, all=True)
         except errors.NotFound, e:
             if not add:
                 raise errors.NotFound(reason=_("The service principal for "
                     "this request doesn't exist."))
-            try:
-                service = api.Command['service_add'](principal, **{'force': True})['result']
-                dn = service['dn']
-            except errors.ACIError:
-                raise errors.ACIError(info=_('You need to be a member of '
-                    'the serviceadmin role to add services'))
+            service = api.Command['service_add'](principal, force=True)
+        service = service['result']
+        dn = service['dn']
 
         # We got this far so the service entry exists, can we write it?
         if not ldap.can_write(dn, "usercertificate"):
@@ -368,26 +375,38 @@ class cert_request(VirtualCommand):
                 "to the 'userCertificate' attribute of entry '%s'.") % dn)
 
         # Validate the subject alt name, if any
-        request = pkcs10.load_certificate_request(csr)
-        subjectaltname = pkcs10.get_subjectaltname(request)
-        if subjectaltname is not None:
-            for name in subjectaltname:
+        for name_type, name in subjectaltname:
+            if name_type == pkcs10.SAN_DNSNAME:
                 name = unicode(name)
                 try:
-                    hostentry = api.Command['host_show'](name, all=True)['result']
-                    hostdn = hostentry['dn']
+                    if servicename == 'host':
+                        altservice = api.Command['host_show'](name, all=True)
+                    else:
+                        altprincipal = '%s/%s@%s' % (servicename, name, realm)
+                        altservice = api.Command['service_show'](
+                            altprincipal, all=True)
                 except errors.NotFound:
                     # We don't want to issue any certificates referencing
                     # machines we don't know about. Nothing is stored in this
                     # host record related to this certificate.
-                    raise errors.NotFound(reason=_('no host record for '
-                        'subject alt name %s in certificate request') % name)
-                authprincipal = getattr(context, 'principal')
-                if authprincipal.startswith("host/"):
-                    if not hostdn in service.get('managedby_host', []):
-                        raise errors.ACIError(info=_(
-                            "Insufficient privilege to create a certificate "
-                            "with subject alt name '%s'.") % name)
+                    raise errors.NotFound(reason=_('The service principal for '
+                        'subject alt name %s in certificate request does not '
+                        'exist') % name)
+                altdn = altservice['result']['dn']
+                if not ldap.can_write(altdn, "usercertificate"):
+                    raise errors.ACIError(info=_(
+                        "Insufficient privilege to create a certificate with "
+                        "subject alt name '%s'.") % name)
+            elif name_type in (pkcs10.SAN_OTHERNAME_KRB5PRINCIPALNAME,
+                               pkcs10.SAN_OTHERNAME_UPN):
+                if name != principal:
+                    raise errors.ACIError(
+                        info=_("Principal '%s' in subject alt name does not "
+                               "match requested service principal") % name)
+            else:
+                raise errors.ACIError(
+                    info=_("Subject alt name type %s is forbidden") %
+                         name_type)
 
         if 'usercertificate' in service:
             serial = x509.get_serial_number(service['usercertificate'][0], datatype=x509.DER)
@@ -435,9 +454,9 @@ class cert_request(VirtualCommand):
             result=result
         )
 
-api.register(cert_request)
 
 
+@register()
 class cert_status(VirtualCommand):
     __doc__ = _('Check the status of a certificate signing request.')
 
@@ -461,7 +480,6 @@ class cert_status(VirtualCommand):
             result=self.Backend.ra.check_request_status(request_id)
         )
 
-api.register(cert_status)
 
 
 _serial_number = Str('serial_number',
@@ -471,6 +489,7 @@ _serial_number = Str('serial_number',
     normalizer=normalize_serial_number,
 )
 
+@register()
 class cert_show(VirtualCommand):
     __doc__ = _('Retrieve an existing certificate.')
 
@@ -556,9 +575,9 @@ class cert_show(VirtualCommand):
             return super(cert_show, self).forward(*keys, **options)
 
 
-api.register(cert_show)
 
 
+@register()
 class cert_revoke(VirtualCommand):
     __doc__ = _('Revoke a certificate.')
 
@@ -603,9 +622,9 @@ class cert_revoke(VirtualCommand):
                 serial_number, revocation_reason=revocation_reason)
         )
 
-api.register(cert_revoke)
 
 
+@register()
 class cert_remove_hold(VirtualCommand):
     __doc__ = _('Take a revoked certificate off hold.')
 
@@ -627,9 +646,9 @@ class cert_remove_hold(VirtualCommand):
             result=self.Backend.ra.take_certificate_off_hold(serial_number)
         )
 
-api.register(cert_remove_hold)
 
 
+@register()
 class cert_find(Command):
     __doc__ = _('Search for existing certificates.')
 
@@ -650,10 +669,12 @@ class cert_find(Command):
             doc=_("minimum serial number"),
             autofill=False,
             minvalue=0,
+            maxvalue=2147483647,
         ),
         Int('max_serial_number?',
             doc=_("maximum serial number"),
             autofill=False,
+            minvalue=0,
             maxvalue=2147483647,
         ),
         Flag('exactly?',
@@ -726,4 +747,3 @@ class cert_find(Command):
         ret['truncated'] = False
         return ret
 
-api.register(cert_find)
