@@ -20,6 +20,7 @@
 
 from ipalib import api
 from ipalib import Int, Str
+from ipalib.plugable import Registry
 from ipalib.plugins.baseldap import *
 from ipalib.plugins import baseldap
 from ipalib import _, ngettext
@@ -110,8 +111,12 @@ Example:
    ipa group-show ad_admins_external
 """)
 
+register = Registry()
+
 PROTECTED_GROUPS = (u'admins', u'trust admins', u'default smb group')
 
+
+@register()
 class group(LDAPObject):
     """
     Group object.
@@ -122,6 +127,7 @@ class group(LDAPObject):
     object_class = ['ipausergroup']
     object_class_config = 'ipagroupobjectclasses'
     possible_objectclasses = ['posixGroup', 'mepManagedEntry', 'ipaExternalGroup']
+    permission_filter_objectclasses = ['posixgroup', 'ipausergroup']
     search_attributes_config = 'ipagroupsearchfields'
     default_attributes = [
         'cn', 'description', 'gidnumber', 'member', 'memberof',
@@ -136,6 +142,77 @@ class group(LDAPObject):
         'sudorule'],
     }
     rdn_is_primary_key = True
+    managed_permissions = {
+        'System: Read Groups': {
+            'replaces_global_anonymous_aci': True,
+            'ipapermbindruletype': 'anonymous',
+            'ipapermright': {'read', 'search', 'compare'},
+            'ipapermdefaultattr': {
+                'businesscategory', 'cn', 'description', 'gidnumber',
+                'ipaexternalmember', 'ipauniqueid', 'mepmanagedby', 'o',
+                'objectclass', 'ou', 'owner', 'seealso',
+                'ipantsecurityidentifier'
+            },
+        },
+        'System: Read Group Membership': {
+            'replaces_global_anonymous_aci': True,
+            'ipapermbindruletype': 'all',
+            'ipapermright': {'read', 'search', 'compare'},
+            'ipapermdefaultattr': {
+                'member', 'memberof', 'memberuid', 'memberuser', 'memberhost',
+            },
+        },
+        'System: Add Groups': {
+            'ipapermright': {'add'},
+            'replaces': [
+                '(target = "ldap:///cn=*,cn=groups,cn=accounts,$SUFFIX")(version 3.0;acl "permission:Add Groups";allow (add) groupdn = "ldap:///cn=Add Groups,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Group Administrators'},
+        },
+        'System: Modify Group Membership': {
+            'ipapermright': {'write'},
+            'ipapermtargetfilter': [
+                '(objectclass=ipausergroup)',
+                '(!(cn=admins))',
+            ],
+            'ipapermdefaultattr': {'member'},
+            'replaces': [
+                '(targetattr = "member")(target = "ldap:///cn=*,cn=groups,cn=accounts,$SUFFIX")(version 3.0;acl "permission:Modify Group membership";allow (write) groupdn = "ldap:///cn=Modify Group membership,cn=permissions,cn=pbac,$SUFFIX";)',
+                '(targetfilter = "(!(cn=admins))")(targetattr = "member")(target = "ldap:///cn=*,cn=groups,cn=accounts,$SUFFIX")(version 3.0;acl "permission:Modify Group membership";allow (write) groupdn = "ldap:///cn=Modify Group membership,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {
+                'Group Administrators', 'Modify Group membership'
+            },
+        },
+        'System: Modify Groups': {
+            'ipapermright': {'write'},
+            'ipapermdefaultattr': {
+                'cn', 'description', 'gidnumber', 'ipauniqueid',
+                'mepmanagedby', 'objectclass'
+            },
+            'replaces': [
+                '(targetattr = "cn || description || gidnumber || objectclass || mepmanagedby || ipauniqueid")(target = "ldap:///cn=*,cn=groups,cn=accounts,$SUFFIX")(version 3.0;acl "permission:Modify Groups";allow (write) groupdn = "ldap:///cn=Modify Groups,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Group Administrators'},
+        },
+        'System: Remove Groups': {
+            'ipapermright': {'delete'},
+            'replaces': [
+                '(target = "ldap:///cn=*,cn=groups,cn=accounts,$SUFFIX")(version 3.0;acl "permission:Remove Groups";allow (delete) groupdn = "ldap:///cn=Remove Groups,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Group Administrators'},
+        },
+        'System: Read Group Compat Tree': {
+            'non_object': True,
+            'ipapermbindruletype': 'anonymous',
+            'ipapermlocation': api.env.basedn,
+            'ipapermtarget': DN('cn=groups', 'cn=compat', api.env.basedn),
+            'ipapermright': {'read', 'search', 'compare'},
+            'ipapermdefaultattr': {
+                'objectclass', 'cn', 'memberuid', 'gidnumber',
+            },
+        },
+    }
 
     label = _('User Groups')
     label_singular = _('User Group')
@@ -163,7 +240,6 @@ class group(LDAPObject):
         ),
     )
 
-api.register(group)
 
 ipaexternalmember_param = Str('ipaexternalmember*',
             cli_name='external',
@@ -173,6 +249,8 @@ ipaexternalmember_param = Str('ipaexternalmember*',
             flags=['no_create', 'no_update', 'no_search'],
         )
 
+
+@register()
 class group_add(LDAPCreate):
     __doc__ = _('Create a new group.')
 
@@ -207,9 +285,7 @@ class group_add(LDAPCreate):
         return dn
 
 
-api.register(group_add)
-
-
+@register()
 class group_del(LDAPDelete):
     __doc__ = _('Delete group.')
 
@@ -217,7 +293,7 @@ class group_del(LDAPDelete):
 
     def pre_callback(self, ldap, dn, *keys, **options):
         assert isinstance(dn, DN)
-        config = ldap.get_ipa_config()[1]
+        config = ldap.get_ipa_config()
         def_primary_group = config.get('ipadefaultprimarygroup', '')
         def_primary_group_dn = group_dn = self.obj.get_dn(def_primary_group)
         if dn == def_primary_group_dn:
@@ -241,9 +317,8 @@ class group_del(LDAPDelete):
 
         return True
 
-api.register(group_del)
-
 
+@register()
 class group_mod(LDAPUpdate):
     __doc__ = _('Modify a group.')
 
@@ -272,7 +347,8 @@ class group_mod(LDAPUpdate):
                     reason=u'Cannot be renamed')
 
         if ('posix' in options and options['posix']) or 'gidnumber' in options:
-            (dn, old_entry_attrs) = ldap.get_entry(dn, ['objectclass'])
+            old_entry_attrs = ldap.get_entry(dn, ['objectclass'])
+            dn = old_entry_attrs.dn
             if 'ipaexternalgroup' in old_entry_attrs['objectclass']:
                 raise errors.ExternalGroupViolation()
             if 'posixgroup' in old_entry_attrs['objectclass']:
@@ -288,7 +364,8 @@ class group_mod(LDAPUpdate):
             if is_protected_group:
                 raise errors.ProtectedEntryError(label=u'group', key=keys[-1],
                     reason=u'Cannot support external non-IPA members')
-            (dn, old_entry_attrs) = ldap.get_entry(dn, ['objectclass'])
+            old_entry_attrs = ldap.get_entry(dn, ['objectclass'])
+            dn = old_entry_attrs.dn
             if 'posixgroup' in old_entry_attrs['objectclass']:
                 raise errors.PosixGroupViolation()
             if 'ipaexternalgroup' in old_entry_attrs['objectclass']:
@@ -311,9 +388,8 @@ class group_mod(LDAPUpdate):
                     raise errors.RequirementError(name='gid')
         raise exc
 
-api.register(group_mod)
-
 
+@register()
 class group_find(LDAPSearch):
     __doc__ = _('Search for groups.')
 
@@ -367,7 +443,7 @@ class group_find(LDAPSearch):
 
             # filter based on 'criteria' argument
             search_kw = {}
-            config = ldap.get_ipa_config()[1]
+            config = ldap.get_ipa_config()
             attrs = config.get(self.obj.search_attributes_config, [])
             if len(attrs) == 1 and isinstance(attrs[0], basestring):
                 search_attrs = attrs[0].split(',')
@@ -381,9 +457,8 @@ class group_find(LDAPSearch):
             filter = ldap.combine_filters(filters, rules=ldap.MATCH_ALL)
         return (filter, base_dn, scope)
 
-api.register(group_find)
-
 
+@register()
 class group_show(LDAPRetrieve):
     __doc__ = _('Display information about a named group.')
     has_output_params = LDAPRetrieve.has_output_params + (ipaexternalmember_param,)
@@ -402,9 +477,9 @@ class group_show(LDAPRetrieve):
                 except ValueError:
                     pass
         return dn
-api.register(group_show)
 
 
+@register()
 class group_add_member(LDAPAddMember):
     __doc__ = _('Add members to a group.')
 
@@ -437,16 +512,19 @@ class group_add_member(LDAPAddMember):
             restore = []
             if 'member' in failed and 'group' in failed['member']:
                 restore = failed['member']['group']
-            failed['member']['group'] = list((id,id) for id in sids)
-            result = add_external_post_callback('member', 'group', 'ipaexternalmember',
-                                                ldap, completed, failed, dn, entry_attrs,
-                                                keys, options, external_callback_normalize=False)
+            failed['member']['group'] = list((id, id) for id in sids)
+            result = add_external_post_callback(ldap, dn, entry_attrs,
+                                                failed=failed,
+                                                completed=completed,
+                                                memberattr='member',
+                                                membertype='group',
+                                                externalattr='ipaexternalmember',
+                                                normalize=False)
             failed['member']['group'] += restore + failed_sids
         return result
 
-api.register(group_add_member)
-
 
+@register()
 class group_remove_member(LDAPRemoveMember):
     __doc__ = _('Remove members from a group.')
 
@@ -454,7 +532,7 @@ class group_remove_member(LDAPRemoveMember):
 
     def pre_callback(self, ldap, dn, found, not_found, *keys, **options):
         assert isinstance(dn, DN)
-        if keys[0] in PROTECTED_GROUPS:
+        if keys[0] in PROTECTED_GROUPS and 'user' in options:
             protected_group_name = keys[0]
             result = api.Command.group_show(protected_group_name)
             users_left = set(result['result'].get('member_user', []))
@@ -491,16 +569,19 @@ class group_remove_member(LDAPRemoveMember):
             restore = []
             if 'member' in failed and 'group' in failed['member']:
                 restore = failed['member']['group']
-            failed['member']['group'] = list((id,id) for id in sids)
-            result = remove_external_post_callback('member', 'group', 'ipaexternalmember',
-                                                ldap, completed, failed, dn, entry_attrs,
-                                                keys, options)
+            failed['member']['group'] = list((id, id) for id in sids)
+            result = remove_external_post_callback(ldap, dn, entry_attrs,
+                                                failed=failed,
+                                                completed=completed,
+                                                memberattr='member',
+                                                membertype='group',
+                                                externalattr='ipaexternalmember',
+                                                )
             failed['member']['group'] += restore + failed_sids
         return result
 
-api.register(group_remove_member)
-
 
+@register()
 class group_detach(LDAPQuery):
     __doc__ = _('Detach a managed group from a user.')
 
@@ -520,7 +601,7 @@ class group_detach(LDAPQuery):
         user_dn = self.api.Object['user'].get_dn(*keys)
 
         try:
-            (user_dn, user_attrs) = ldap.get_entry(user_dn)
+            user_attrs = ldap.get_entry(user_dn)
         except errors.NotFound:
             self.obj.handle_not_found(*keys)
         is_managed = self.obj.has_objectclass(user_attrs['objectclass'], 'mepmanagedentry')
@@ -528,7 +609,7 @@ class group_detach(LDAPQuery):
             not (ldap.can_write(user_dn, "mepManagedEntry")) and is_managed):
             raise errors.ACIError(info=_('not allowed to modify user entries'))
 
-        (group_dn, group_attrs) = ldap.get_entry(group_dn)
+        group_attrs = ldap.get_entry(group_dn)
         is_managed = self.obj.has_objectclass(group_attrs['objectclass'], 'mepmanagedby')
         if (not ldap.can_write(group_dn, "objectclass") or
             not (ldap.can_write(group_dn, "mepManagedBy")) and is_managed):
@@ -538,14 +619,14 @@ class group_detach(LDAPQuery):
         try:
             i = objectclasses.index('mepOriginEntry')
             del objectclasses[i]
-            update_attrs = {'objectclass': objectclasses, 'mepManagedEntry': None}
-            ldap.update_entry(user_dn, update_attrs)
+            user_attrs['mepManagedEntry'] = None
+            ldap.update_entry(user_attrs)
         except ValueError:
             # Somehow the user isn't managed, let it pass for now. We'll
             # let the group throw "Not managed".
             pass
 
-        (group_dn, group_attrs) = ldap.get_entry(group_dn)
+        group_attrs = ldap.get_entry(group_dn)
         objectclasses = group_attrs['objectclass']
         try:
             i = objectclasses.index('mepManagedEntry')
@@ -555,18 +636,17 @@ class group_detach(LDAPQuery):
         del objectclasses[i]
 
         # Make sure the resulting group has the default group objectclasses
-        config = ldap.get_ipa_config()[1]
+        config = ldap.get_ipa_config()
         def_objectclass = config.get(
             self.obj.object_class_config, objectclasses
         )
         objectclasses = list(set(def_objectclass + objectclasses))
 
-        update_attrs = {'objectclass': objectclasses, 'mepManagedBy': None}
-        ldap.update_entry(group_dn, update_attrs)
+        group_attrs['mepManagedBy'] = None
+        ldap.update_entry(group_attrs)
 
         return dict(
             result=True,
-            value=keys[0],
+            value=pkey_to_value(keys[0], options),
         )
 
-api.register(group_detach)
