@@ -18,6 +18,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from ipalib import api, errors
+from ipalib.plugable import Registry
 from ipalib.plugins.baseldap import *
 from ipalib import _, ngettext
 
@@ -42,8 +43,11 @@ EXAMPLES:
    ipa hbacsvcgroup-del login
 """)
 
+register = Registry()
+
 topic = ('hbac', _('Host based access control commands'))
 
+@register()
 class hbacsvcgroup(LDAPObject):
     """
     HBAC service group object.
@@ -52,11 +56,46 @@ class hbacsvcgroup(LDAPObject):
     object_name = _('HBAC service group')
     object_name_plural = _('HBAC service groups')
     object_class = ['ipaobject', 'ipahbacservicegroup']
+    permission_filter_objectclasses = ['ipahbacservicegroup']
     default_attributes = [ 'cn', 'description', 'member' ]
     uuid_attribute = 'ipauniqueid'
     attribute_members = {
         'member': ['hbacsvc'],
     }
+    managed_permissions = {
+        'System: Read HBAC Service Groups': {
+            'replaces_global_anonymous_aci': True,
+            'ipapermbindruletype': 'all',
+            'ipapermright': {'read', 'search', 'compare'},
+            'ipapermdefaultattr': {
+                'businesscategory', 'cn', 'description', 'ipauniqueid',
+                'member', 'o', 'objectclass', 'ou', 'owner', 'seealso',
+                'memberuser', 'memberhost',
+            },
+        },
+        'System: Add HBAC Service Groups': {
+            'ipapermright': {'add'},
+            'replaces': [
+                '(target = "ldap:///cn=*,cn=hbacservicegroups,cn=hbac,$SUFFIX")(version 3.0;acl "permission:Add HBAC service groups";allow (add) groupdn = "ldap:///cn=Add HBAC service groups,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'HBAC Administrator'},
+        },
+        'System: Delete HBAC Service Groups': {
+            'ipapermright': {'delete'},
+            'replaces': [
+                '(target = "ldap:///cn=*,cn=hbacservicegroups,cn=hbac,$SUFFIX")(version 3.0;acl "permission:Delete HBAC service groups";allow (delete) groupdn = "ldap:///cn=Delete HBAC service groups,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'HBAC Administrator'},
+        },
+        'System: Manage HBAC Service Group Membership': {
+            'ipapermright': {'write'},
+            'ipapermdefaultattr': {'member'},
+            'replaces': [
+                '(targetattr = "member")(target = "ldap:///cn=*,cn=hbacservicegroups,cn=hbac,$SUFFIX")(version 3.0;acl "permission:Manage HBAC service group membership";allow (write) groupdn = "ldap:///cn=Manage HBAC service group membership,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'HBAC Administrator'},
+        },
+    }
 
     label = _('HBAC Service Groups')
     label_singular = _('HBAC Service Group')
@@ -75,33 +114,33 @@ class hbacsvcgroup(LDAPObject):
         ),
     )
 
-api.register(hbacsvcgroup)
 
 
+@register()
 class hbacsvcgroup_add(LDAPCreate):
     __doc__ = _('Add a new HBAC service group.')
 
     msg_summary = _('Added HBAC service group "%(value)s"')
 
-api.register(hbacsvcgroup_add)
 
 
+@register()
 class hbacsvcgroup_del(LDAPDelete):
     __doc__ = _('Delete an HBAC service group.')
 
     msg_summary = _('Deleted HBAC service group "%(value)s"')
 
-api.register(hbacsvcgroup_del)
 
 
+@register()
 class hbacsvcgroup_mod(LDAPUpdate):
     __doc__ = _('Modify an HBAC service group.')
 
     msg_summary = _('Modified HBAC service group "%(value)s"')
 
-api.register(hbacsvcgroup_mod)
 
 
+@register()
 class hbacsvcgroup_find(LDAPSearch):
     __doc__ = _('Search for an HBAC service group.')
 
@@ -109,22 +148,21 @@ class hbacsvcgroup_find(LDAPSearch):
         '%(count)d HBAC service group matched', '%(count)d HBAC service groups matched', 0
     )
 
-api.register(hbacsvcgroup_find)
 
 
+@register()
 class hbacsvcgroup_show(LDAPRetrieve):
     __doc__ = _('Display information about an HBAC service group.')
 
-api.register(hbacsvcgroup_show)
 
 
+@register()
 class hbacsvcgroup_add_member(LDAPAddMember):
     __doc__ = _('Add members to an HBAC service group.')
 
-api.register(hbacsvcgroup_add_member)
 
 
+@register()
 class hbacsvcgroup_remove_member(LDAPRemoveMember):
     __doc__ = _('Remove members from an HBAC service group.')
 
-api.register(hbacsvcgroup_remove_member)
