@@ -28,20 +28,21 @@ import string
 
 from ipalib import api, errors, util
 from ipalib import Str, Flag, Bytes
+from ipalib.plugable import Registry
 from ipalib.plugins.baseldap import *
 from ipalib.plugins.service import (split_principal, validate_certificate,
     set_certificate_attrs, ticket_flags_params, update_krbticketflags,
     set_kerberos_attrs)
 from ipalib.plugins.dns import (dns_container_exists, _record_types,
         add_records_for_host_validation, add_records_for_host,
-        _hostname_validator, get_reverse_zone)
-from ipalib.plugins.dns import get_reverse_zone
+        get_reverse_zone)
 from ipalib import _, ngettext
 from ipalib import x509
 from ipalib.request import context
 from ipalib.util import (normalize_sshpubkey, validate_sshpubkey_no_options,
-    convert_sshpubkey_post)
+    convert_sshpubkey_post, validate_hostname)
 from ipapython.ipautil import ipa_generate_password, CheckedIPAddress
+from ipapython.dnsutil import DNSName
 from ipapython.ssh import SSHPublicKey
 from ipapython.dn import DN
 
@@ -53,7 +54,7 @@ A host represents a machine. It can be used in a number of contexts:
 - a host stores the host/ service principal
 - a host can be used in Host-based Access Control (HBAC) rules
 - every enrolled client generates a host entry
-
+""") + _("""
 ENROLLMENT:
 
 There are three enrollment scenarios when enrolling a new client:
@@ -65,8 +66,7 @@ There are three enrollment scenarios when enrolling a new client:
    exist. A limited administrator is a member a role with the
    Host Enrollment privilege.
 3. The host has been created with a one-time password.
-
-
+""") + _("""
 RE-ENROLLMENT:
 
 Host that has been enrolled at some point, and lost its configuration (e.g. VM
@@ -76,34 +76,36 @@ For more information, consult the manual pages for ipa-client-install.
 
 A host can optionally store information such as where it is located,
 the OS that it runs, etc.
-
+""") + _("""
 EXAMPLES:
-
+""") + _("""
  Add a new host:
    ipa host-add --location="3rd floor lab" --locality=Dallas test.example.com
-
+""") + _("""
  Delete a host:
    ipa host-del test.example.com
-
+""") + _("""
  Add a new host with a one-time password:
    ipa host-add --os='Fedora 12' --password=Secret123 test.example.com
-
+""") + _("""
  Add a new host with a random one-time password:
    ipa host-add --os='Fedora 12' --random test.example.com
-
+""") + _("""
  Modify information about a host:
    ipa host-mod --os='Fedora 12' test.example.com
-
+""") + _("""
  Remove SSH public keys of a host and update DNS to reflect this change:
    ipa host-mod --sshpubkey= --updatedns test.example.com
-
+""") + _("""
  Disable the host Kerberos key, SSL certificate and all of its services:
    ipa host-disable test.example.com
-
+""") + _("""
  Add a host that can manage this host's keytab and certificate:
    ipa host-add-managedby --hosts=test2 test
 """)
 
+register = Registry()
+
 # Characters to be used by random password generator
 # The set was chosen to avoid the need for escaping the characters by user
 host_pwd_chars=string.digits + string.ascii_letters + '_,.@+-='
@@ -215,6 +217,16 @@ def normalize_hostname(hostname):
     hostname = hostname.lower()
     return hostname
 
+def _hostname_validator(ugettext, value):
+    try:
+        validate_hostname(value)
+    except ValueError, e:
+        return _('invalid domain-name: %s') \
+            % unicode(e)
+
+    return None
+
+@register()
 class host(LDAPObject):
     """
     Host object.
@@ -223,6 +235,7 @@ class host(LDAPObject):
     object_name = _('host')
     object_name_plural = _('hosts')
     object_class = ['ipaobject', 'nshost', 'ipahost', 'pkiuser', 'ipaservice']
+    permission_filter_objectclasses = ['ipahost']
     # object_class_config = 'ipahostobjectclasses'
     search_attributes = [
         'fqdn', 'description', 'l', 'nshostlocation', 'krbprincipalname',
@@ -252,6 +265,120 @@ class host(LDAPObject):
     }
     password_attributes = [('userpassword', 'has_password'),
                            ('krbprincipalkey', 'has_keytab')]
+    managed_permissions = {
+        'System: Read Hosts': {
+            'replaces_global_anonymous_aci': True,
+            'ipapermbindruletype': 'all',
+            'ipapermright': {'read', 'search', 'compare'},
+            'ipapermdefaultattr': {
+                'cn', 'description', 'fqdn', 'ipaclientversion',
+                'ipakrbauthzdata', 'ipasshpubkey', 'ipauniqueid',
+                'krbprincipalname', 'l', 'macaddress', 'nshardwareplatform',
+                'nshostlocation', 'nsosversion', 'objectclass',
+                'serverhostname', 'usercertificate', 'userclass',
+                'enrolledby', 'managedby',
+                'krbprincipalname', 'krbcanonicalname', 'krbprincipalaliases',
+                'krbprincipalexpiration', 'krbpasswordexpiration',
+                'krblastpwdchange',
+            },
+        },
+        'System: Read Host Membership': {
+            'replaces_global_anonymous_aci': True,
+            'ipapermbindruletype': 'all',
+            'ipapermright': {'read', 'search', 'compare'},
+            'ipapermdefaultattr': {
+                'memberof',
+            },
+        },
+        'System: Add Hosts': {
+            'ipapermright': {'add'},
+            'replaces': [
+                '(target = "ldap:///fqdn=*,cn=computers,cn=accounts,$SUFFIX")(version 3.0;acl "permission:Add Hosts";allow (add) groupdn = "ldap:///cn=Add Hosts,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Host Administrators'},
+        },
+        'System: Add krbPrincipalName to a Host': {
+            # Allow an admin to enroll a host that has a one-time password.
+            # When a host is created with a password no krbPrincipalName is set.
+            # This will let it be added if the client ends up enrolling with
+            # an administrator instead.
+            'ipapermright': {'write'},
+            'ipapermtargetfilter': [
+                '(objectclass=ipahost)',
+                '(!(krbprincipalname=*))',
+            ],
+            'ipapermdefaultattr': {'krbprincipalname'},
+            'replaces': [
+                '(target = "ldap:///fqdn=*,cn=computers,cn=accounts,$SUFFIX")(targetfilter = "(!(krbprincipalname=*))")(targetattr = "krbprincipalname")(version 3.0;acl "permission:Add krbPrincipalName to a host"; allow (write) groupdn = "ldap:///cn=Add krbPrincipalName to a host,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Host Administrators', 'Host Enrollment'},
+        },
+        'System: Enroll a Host': {
+            'ipapermright': {'write'},
+            'ipapermdefaultattr': {'objectclass', 'enrolledby'},
+            'replaces': [
+                '(targetattr = "objectclass")(target = "ldap:///fqdn=*,cn=computers,cn=accounts,$SUFFIX")(version 3.0;acl "permission:Enroll a host";allow (write) groupdn = "ldap:///cn=Enroll a host,cn=permissions,cn=pbac,$SUFFIX";)',
+                '(targetattr = "enrolledby || objectclass")(target = "ldap:///fqdn=*,cn=computers,cn=accounts,$SUFFIX")(version 3.0;acl "permission:Enroll a host";allow (write) groupdn = "ldap:///cn=Enroll a host,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Host Administrators', 'Host Enrollment'},
+        },
+        'System: Manage Host SSH Public Keys': {
+            'ipapermright': {'write'},
+            'ipapermdefaultattr': {'ipasshpubkey'},
+            'replaces': [
+                '(targetattr = "ipasshpubkey")(target = "ldap:///fqdn=*,cn=computers,cn=accounts,$SUFFIX")(version 3.0;acl "permission:Manage Host SSH Public Keys";allow (write) groupdn = "ldap:///cn=Manage Host SSH Public Keys,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Host Administrators'},
+        },
+        'System: Manage Host Keytab': {
+            'ipapermright': {'write'},
+            'ipapermdefaultattr': {'krblastpwdchange', 'krbprincipalkey'},
+            'replaces': [
+                '(targetattr = "krbprincipalkey || krblastpwdchange")(target = "ldap:///fqdn=*,cn=computers,cn=accounts,$SUFFIX")(version 3.0;acl "permission:Manage host keytab";allow (write) groupdn = "ldap:///cn=Manage host keytab,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Host Administrators', 'Host Enrollment'},
+        },
+        'System: Modify Hosts': {
+            'ipapermright': {'write'},
+            'ipapermdefaultattr': {
+                'description', 'l', 'nshardwareplatform', 'nshostlocation',
+                'nsosversion', 'macaddress', 'userclass',
+            },
+            'replaces': [
+                '(targetattr = "description || l || nshostlocation || nshardwareplatform || nsosversion")(target = "ldap:///fqdn=*,cn=computers,cn=accounts,$SUFFIX")(version 3.0;acl "permission:Modify Hosts";allow (write) groupdn = "ldap:///cn=Modify Hosts,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Host Administrators'},
+        },
+        'System: Remove Hosts': {
+            'ipapermright': {'delete'},
+            'replaces': [
+                '(target = "ldap:///fqdn=*,cn=computers,cn=accounts,$SUFFIX")(version 3.0;acl "permission:Remove Hosts";allow (delete) groupdn = "ldap:///cn=Remove Hosts,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Host Administrators'},
+        },
+        'System: Manage Host Certificates': {
+            'ipapermbindruletype': 'permission',
+            'ipapermright': {'write'},
+            'ipapermdefaultattr': {'usercertificate'},
+            'default_privileges': {'Host Administrators', 'Host Enrollment'},
+        },
+        'System: Manage Host Enrollment Password': {
+            'ipapermbindruletype': 'permission',
+            'ipapermright': {'write'},
+            'ipapermdefaultattr': {'userpassword'},
+            'default_privileges': {'Host Administrators', 'Host Enrollment'},
+        },
+        'System: Read Host Compat Tree': {
+            'non_object': True,
+            'ipapermbindruletype': 'anonymous',
+            'ipapermlocation': api.env.basedn,
+            'ipapermtarget': DN('cn=computers', 'cn=compat', api.env.basedn),
+            'ipapermright': {'read', 'search', 'compare'},
+            'ipapermdefaultattr': {
+                'objectclass', 'cn', 'macaddress',
+            },
+        },
+    }
 
     label = _('Hosts')
     label_singular = _('Host')
@@ -341,9 +468,10 @@ class host(LDAPObject):
             self.backend.get_entry(dn, [''])
         except errors.NotFound:
             try:
-                (dn, entry_attrs) = self.backend.find_entry_by_attr(
+                entry_attrs = self.backend.find_entry_by_attr(
                     'serverhostname', hostname, self.object_class, [''],
                     DN(self.container_dn, api.env.basedn))
+                dn = entry_attrs.dn
             except errors.NotFound:
                 pass
         return dn
@@ -360,7 +488,7 @@ class host(LDAPObject):
                 filter=host_filter, attrs_list=host_attrs)
 
             for host in hosts:
-                managed_hosts.append(host[0])
+                managed_hosts.append(host.dn)
         except errors.NotFound:
             return []
 
@@ -385,9 +513,9 @@ class host(LDAPObject):
             else:
                 entry_attrs['memberofindirect'].remove(member)
 
-api.register(host)
 
 
+@register()
 class host_add(LDAPCreate):
     __doc__ = _('Add a new host.')
 
@@ -415,7 +543,9 @@ class host_add(LDAPCreate):
             host = parts[0]
             domain = unicode('.'.join(parts[1:]))
             check_reverse = not options.get('no_reverse', False)
-            add_records_for_host_validation('ip_address', host, domain,
+            add_records_for_host_validation('ip_address',
+                    DNSName(host),
+                    DNSName(domain).make_absolute(),
                     options['ip_address'],
                     check_forward=True,
                     check_reverse=check_reverse)
@@ -467,7 +597,9 @@ class host_add(LDAPCreate):
                 if options.get('ip_address'):
                     add_reverse = not options.get('no_reverse', False)
 
-                    add_records_for_host(host, domain, options['ip_address'],
+                    add_records_for_host(DNSName(host),
+                                         DNSName(domain).make_absolute(),
+                                         options['ip_address'],
                                          add_forward=True,
                                          add_reverse=add_reverse)
                     del options['ip_address']
@@ -501,9 +633,9 @@ class host_add(LDAPCreate):
 
         return dn
 
-api.register(host_add)
 
 
+@register()
 class host_del(LDAPDelete):
     __doc__ = _('Delete a host.')
 
@@ -582,10 +714,10 @@ class host_del(LDAPDelete):
 
         if self.api.env.enable_ra:
             try:
-                (dn, entry_attrs) = ldap.get_entry(dn, ['usercertificate'])
+                entry_attrs = ldap.get_entry(dn, ['usercertificate'])
             except errors.NotFound:
                 self.obj.handle_not_found(*keys)
-            cert = entry_attrs.single_value('usercertificate', None)
+            cert = entry_attrs.single_value.get('usercertificate')
             if cert:
                 cert = x509.normalize_certificate(cert)
                 try:
@@ -613,9 +745,9 @@ class host_del(LDAPDelete):
 
         return dn
 
-api.register(host_del)
 
 
+@register()
 class host_mod(LDAPUpdate):
     __doc__ = _('Modify information about a host.')
 
@@ -652,7 +784,7 @@ class host_mod(LDAPUpdate):
         if 'locality' in entry_attrs:
             entry_attrs['l'] = entry_attrs['locality']
         if 'krbprincipalname' in entry_attrs:
-            (dn, entry_attrs_old) = ldap.get_entry(
+            entry_attrs_old = ldap.get_entry(
                 dn, ['objectclass', 'krbprincipalname']
             )
             if 'krbprincipalname' in entry_attrs_old:
@@ -666,8 +798,8 @@ class host_mod(LDAPUpdate):
         if cert:
             if self.api.env.enable_ra:
                 x509.verify_cert_subject(ldap, keys[-1], cert)
-                (dn, entry_attrs_old) = ldap.get_entry(dn, ['usercertificate'])
-                oldcert = entry_attrs_old.single_value('usercertificate', None)
+                entry_attrs_old = ldap.get_entry(dn, ['usercertificate'])
+                oldcert = entry_attrs_old.single_value.get('usercertificate')
                 if oldcert:
                     oldcert = x509.normalize_certificate(oldcert)
                     try:
@@ -704,9 +836,7 @@ class host_mod(LDAPUpdate):
             if 'objectclass' in entry_attrs:
                 obj_classes = entry_attrs['objectclass']
             else:
-                (_dn, _entry_attrs) = ldap.get_entry(
-                    dn, ['objectclass']
-                )
+                _entry_attrs = ldap.get_entry(dn, ['objectclass'])
                 obj_classes = _entry_attrs['objectclass']
             if 'ieee802device' not in obj_classes:
                 obj_classes.append('ieee802device')
@@ -726,7 +856,7 @@ class host_mod(LDAPUpdate):
             if 'objectclass' in entry_attrs:
                 obj_classes = entry_attrs['objectclass']
             else:
-                (_dn, _entry_attrs) = ldap.get_entry(dn, ['objectclass'])
+                _entry_attrs = ldap.get_entry(dn, ['objectclass'])
                 obj_classes = entry_attrs['objectclass'] = _entry_attrs['objectclass']
             if 'ipasshhost' not in obj_classes:
                 obj_classes.append('ipasshhost')
@@ -763,9 +893,9 @@ class host_mod(LDAPUpdate):
 
         return dn
 
-api.register(host_mod)
 
 
+@register()
 class host_find(LDAPSearch):
     __doc__ = _('Search for hosts.')
 
@@ -793,7 +923,7 @@ class host_find(LDAPSearch):
                 for pkey in options.get('man_host', []):
                     dn = self.obj.get_dn(pkey)
                     try:
-                        (dn, entry_attrs) = ldap.get_entry(dn, ['managedby'])
+                        entry_attrs = ldap.get_entry(dn, ['managedby'])
                     except errors.NotFound:
                         self.obj.handle_not_found(pkey)
                     hosts.append(set(entry_attrs.get('managedby', '')))
@@ -810,7 +940,7 @@ class host_find(LDAPSearch):
                 for pkey in options.get('not_man_host', []):
                     dn = self.obj.get_dn(pkey)
                     try:
-                        (dn, entry_attrs) = ldap.get_entry(dn, ['managedby'])
+                        entry_attrs = ldap.get_entry(dn, ['managedby'])
                     except errors.NotFound:
                         self.obj.handle_not_found(pkey)
                     not_hosts += entry_attrs.get('managedby', [])
@@ -831,11 +961,10 @@ class host_find(LDAPSearch):
     def post_callback(self, ldap, entries, truncated, *args, **options):
         if options.get('pkey_only', False):
             return truncated
-        for entry in entries:
-            (dn, entry_attrs) = entry
+        for entry_attrs in entries:
             set_certificate_attrs(entry_attrs)
             set_kerberos_attrs(entry_attrs, options)
-            self.obj.get_password_attributes(ldap, dn, entry_attrs)
+            self.obj.get_password_attributes(ldap, entry_attrs.dn, entry_attrs)
             self.obj.suppress_netgroup_memberof(ldap, entry_attrs)
             if entry_attrs['has_password']:
                 # If an OTP is set there is no keytab, at least not one
@@ -843,15 +972,15 @@ class host_find(LDAPSearch):
                 entry_attrs['has_keytab'] = False
 
             if options.get('all', False):
-                entry_attrs['managing'] = self.obj.get_managed_hosts(entry[0])
+                entry_attrs['managing'] = self.obj.get_managed_hosts(entry_attrs.dn)
 
-            convert_sshpubkey_post(ldap, dn, entry_attrs)
+            convert_sshpubkey_post(ldap, entry_attrs.dn, entry_attrs)
 
         return truncated
 
-api.register(host_find)
 
 
+@register()
 class host_show(LDAPRetrieve):
     __doc__ = _('Display information about a host.')
 
@@ -897,9 +1026,9 @@ class host_show(LDAPRetrieve):
         else:
             return super(host_show, self).forward(*keys, **options)
 
-api.register(host_show)
 
 
+@register()
 class host_disable(LDAPQuery):
     __doc__ = _('Disable the Kerberos key, SSL certificate and all services of a host.')
 
@@ -942,10 +1071,10 @@ class host_disable(LDAPQuery):
 
         dn = self.obj.get_dn(*keys, **options)
         try:
-            (dn, entry_attrs) = ldap.get_entry(dn, ['usercertificate'])
+            entry_attrs = ldap.get_entry(dn, ['usercertificate'])
         except errors.NotFound:
             self.obj.handle_not_found(*keys)
-        cert = entry_attrs.single_value('usercertificate', None)
+        cert = entry_attrs.single_value.get('usercertificate')
         if cert:
             if self.api.env.enable_ra:
                 cert = x509.normalize_certificate(cert)
@@ -973,7 +1102,8 @@ class host_disable(LDAPQuery):
                         raise nsprerr
 
             # Remove the usercertificate altogether
-            ldap.update_entry(dn, {'usercertificate': None})
+            entry_attrs['usercertificate'] = None
+            ldap.update_entry(entry_attrs)
             done_work = True
 
         self.obj.get_password_attributes(ldap, dn, entry_attrs)
@@ -986,7 +1116,7 @@ class host_disable(LDAPQuery):
 
         return dict(
             result=True,
-            value=keys[0],
+            value=pkey_to_value(keys[0], options),
         )
 
     def post_callback(self, ldap, dn, entry_attrs, *keys, **options):
@@ -994,8 +1124,8 @@ class host_disable(LDAPQuery):
         self.obj.suppress_netgroup_memberof(ldap, entry_attrs)
         return dn
 
-api.register(host_disable)
 
+@register()
 class host_add_managedby(LDAPAddMember):
     __doc__ = _('Add hosts that can manage this host.')
 
@@ -1008,9 +1138,9 @@ class host_add_managedby(LDAPAddMember):
         self.obj.suppress_netgroup_memberof(ldap, entry_attrs)
         return (completed, dn)
 
-api.register(host_add_managedby)
 
 
+@register()
 class host_remove_managedby(LDAPRemoveMember):
     __doc__ = _('Remove hosts that can manage this host.')
 
@@ -1022,4 +1152,3 @@ class host_remove_managedby(LDAPRemoveMember):
         self.obj.suppress_netgroup_memberof(ldap, entry_attrs)
         return (completed, dn)
 
-api.register(host_remove_managedby)
