@@ -18,6 +18,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+from ipalib.plugable import Registry
 from ipalib.plugins.baseldap import *
 from ipalib import api, Int, _, ngettext, errors
 from ipalib.plugins.netgroup import NETGROUP_PATTERN, NETGROUP_PATTERN_ERRMSG
@@ -53,6 +54,9 @@ EXAMPLES:
    ipa hostgroup-del baltimore
 """)
 
+register = Registry()
+
+@register()
 class hostgroup(LDAPObject):
     """
     Hostgroup object.
@@ -61,6 +65,7 @@ class hostgroup(LDAPObject):
     object_name = _('host group')
     object_name_plural = _('host groups')
     object_class = ['ipaobject', 'ipahostgroup']
+    permission_filter_objectclasses = ['ipahostgroup']
     default_attributes = ['cn', 'description', 'member', 'memberof',
         'memberindirect', 'memberofindirect',
     ]
@@ -71,6 +76,55 @@ class hostgroup(LDAPObject):
         'memberindirect': ['host', 'hostgroup'],
         'memberofindirect': ['hostgroup', 'hbacrule', 'sudorule'],
     }
+    managed_permissions = {
+        'System: Read Hostgroups': {
+            'replaces_global_anonymous_aci': True,
+            'ipapermbindruletype': 'all',
+            'ipapermright': {'read', 'search', 'compare'},
+            'ipapermdefaultattr': {
+                'businesscategory', 'cn', 'description', 'ipauniqueid', 'o',
+                'objectclass', 'ou', 'owner', 'seealso',
+            },
+        },
+        'System: Read Hostgroup Membership': {
+            'replaces_global_anonymous_aci': True,
+            'ipapermbindruletype': 'all',
+            'ipapermright': {'read', 'search', 'compare'},
+            'ipapermdefaultattr': {
+                'member', 'memberof', 'memberuser', 'memberhost',
+            },
+        },
+        'System: Add Hostgroups': {
+            'ipapermright': {'add'},
+            'replaces': [
+                '(target = "ldap:///cn=*,cn=hostgroups,cn=accounts,$SUFFIX")(version 3.0;acl "permission:Add Hostgroups";allow (add) groupdn = "ldap:///cn=Add Hostgroups,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Host Group Administrators'},
+        },
+        'System: Modify Hostgroup Membership': {
+            'ipapermright': {'write'},
+            'ipapermdefaultattr': {'member'},
+            'replaces': [
+                '(targetattr = "member")(target = "ldap:///cn=*,cn=hostgroups,cn=accounts,$SUFFIX")(version 3.0;acl "permission:Modify Hostgroup membership";allow (write) groupdn = "ldap:///cn=Modify Hostgroup membership,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Host Group Administrators'},
+        },
+        'System: Modify Hostgroups': {
+            'ipapermright': {'write'},
+            'ipapermdefaultattr': {'cn', 'description'},
+            'replaces': [
+                '(targetattr = "cn || description")(target = "ldap:///cn=*,cn=hostgroups,cn=accounts,$SUFFIX")(version 3.0; acl "permission:Modify Hostgroups";allow (write) groupdn = "ldap:///cn=Modify Hostgroups,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Host Group Administrators'},
+        },
+        'System: Remove Hostgroups': {
+            'ipapermright': {'delete'},
+            'replaces': [
+                '(target = "ldap:///cn=*,cn=hostgroups,cn=accounts,$SUFFIX")(version 3.0;acl "permission:Remove Hostgroups";allow (delete) groupdn = "ldap:///cn=Remove Hostgroups,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Host Group Administrators'},
+        },
+    }
 
     label = _('Host Groups')
     label_singular = _('Host Group')
@@ -111,9 +165,9 @@ class hostgroup(LDAPObject):
             else:
                 entry_attrs['memberof'].remove(member)
 
-api.register(hostgroup)
 
 
+@register()
 class hostgroup_add(LDAPCreate):
     __doc__ = _('Add a new hostgroup.')
 
@@ -152,17 +206,17 @@ class hostgroup_add(LDAPCreate):
         return dn
 
 
-api.register(hostgroup_add)
 
 
+@register()
 class hostgroup_del(LDAPDelete):
     __doc__ = _('Delete a hostgroup.')
 
     msg_summary = _('Deleted hostgroup "%(value)s"')
 
-api.register(hostgroup_del)
 
 
+@register()
 class hostgroup_mod(LDAPUpdate):
     __doc__ = _('Modify a hostgroup.')
 
@@ -173,9 +227,9 @@ class hostgroup_mod(LDAPUpdate):
         self.obj.suppress_netgroup_memberof(ldap, dn, entry_attrs)
         return dn
 
-api.register(hostgroup_mod)
 
 
+@register()
 class hostgroup_find(LDAPSearch):
     __doc__ = _('Search for hostgroups.')
 
@@ -188,13 +242,12 @@ class hostgroup_find(LDAPSearch):
         if options.get('pkey_only', False):
             return truncated
         for entry in entries:
-            (dn, entry_attrs) = entry
-            self.obj.suppress_netgroup_memberof(ldap, dn, entry_attrs)
+            self.obj.suppress_netgroup_memberof(ldap, entry.dn, entry)
         return truncated
 
-api.register(hostgroup_find)
 
 
+@register()
 class hostgroup_show(LDAPRetrieve):
     __doc__ = _('Display information about a hostgroup.')
 
@@ -203,9 +256,9 @@ class hostgroup_show(LDAPRetrieve):
         self.obj.suppress_netgroup_memberof(ldap, dn, entry_attrs)
         return dn
 
-api.register(hostgroup_show)
 
 
+@register()
 class hostgroup_add_member(LDAPAddMember):
     __doc__ = _('Add members to a hostgroup.')
 
@@ -214,9 +267,9 @@ class hostgroup_add_member(LDAPAddMember):
         self.obj.suppress_netgroup_memberof(ldap, dn, entry_attrs)
         return (completed, dn)
 
-api.register(hostgroup_add_member)
 
 
+@register()
 class hostgroup_remove_member(LDAPRemoveMember):
     __doc__ = _('Remove members from a hostgroup.')
 
@@ -225,4 +278,3 @@ class hostgroup_remove_member(LDAPRemoveMember):
         self.obj.suppress_netgroup_memberof(ldap, dn, entry_attrs)
         return (completed, dn)
 
-api.register(hostgroup_remove_member)
