@@ -31,7 +31,11 @@ from ipalib import Str
 from ipalib.output import Output
 from ipalib.text import _
 from ipalib.util import json_serialize
+from ipalib.plugable import Registry
 
+register = Registry()
+
+@register()
 class json_metadata(Command):
     """
     Export plugin meta-data for the webUI.
@@ -136,8 +140,8 @@ class json_metadata(Command):
     def output_for_cli(self, textui, result, *args, **options):
         print json.dumps(result, default=json_serialize)
 
-api.register(json_metadata)
 
+@register()
 class i18n_messages(Command):
     NO_CLI = True
 
@@ -149,10 +153,13 @@ class i18n_messages(Command):
         },
         "actions": {
             "apply": _("Apply"),
-            "confirm": _("Are you sure you want to proceed with the action."),
-            "delete_confirm": _("Are you sure you want to delete ${object}"),
-            "disable_confirm": _("Are you sure you want to disable ${object}"),
-            "enable_confirm": _("Are you sure you want to enable ${object}"),
+            "automember_rebuild": _("Rebuild auto membership"),
+            "automember_rebuild_confirm": _("Are you sure you want to rebuild auto membership?"),
+            "automember_rebuild_success": _("Automember rebuild membership task completed"),
+            "confirm": _("Are you sure you want to proceed with the action?"),
+            "delete_confirm": _("Are you sure you want to delete ${object}?"),
+            "disable_confirm": _("Are you sure you want to disable ${object}?"),
+            "enable_confirm": _("Are you sure you want to enable ${object}?"),
             "title": _("Actions"),
         },
         "association": {
@@ -167,6 +174,7 @@ class i18n_messages(Command):
             },
             "added": _("${count} item(s) added"),
             "direct_membership": _("Direct Membership"),
+            "filter_placeholder": _("Filter available ${other_entity}"),
             "indirect_membership": _("Indirect Membership"),
             "no_entries": _("No entries."),
             "paging": _("Showing ${start} to ${end} of ${total} entries."),
@@ -183,6 +191,7 @@ class i18n_messages(Command):
             "show_results": _("Show Results"),
         },
         "buttons": {
+            "about": _("About"),
             "add": _("Add"),
             "add_and_add_another": _("Add and Add Another"),
             "add_and_close": _("Add and Close"),
@@ -194,6 +203,7 @@ class i18n_messages(Command):
             "disable": _("Disable"),
             "edit": _("Edit"),
             "enable": _("Enable"),
+            "filter": _("Filter"),
             "find": _("Find"),
             "get": _("Get"),
             "issue": _("Issue"),
@@ -229,6 +239,8 @@ class i18n_messages(Command):
             "dirty_title": _("Unsaved Changes"),
             "edit_title": _("Edit ${entity}"),
             "hide_details": _("Hide details"),
+            "about_title": _("About"),
+            "about_message": _("${product}, version: ${version}"),
             "prospective": _("Prospective"),
             "redirection": _("Redirection"),
             "remove_empty": _("Select entries to be removed."),
@@ -278,6 +290,7 @@ class i18n_messages(Command):
             "logout": _("Logout"),
             "logout_error": _("Logout error"),
             "password": _("Password"),
+            "sync_otp_token": _("Sync OTP Token"),
             "username": _("Username"),
         },
         "measurement_units": {
@@ -335,6 +348,7 @@ class i18n_messages(Command):
                 "find_validnotbefore_from": _("Valid not before from"),
                 "find_validnotbefore_to": _("Valid not before to"),
                 "fingerprints": _("Fingerprints"),
+                "get_certificate": _("Get Certificate"),
                 "issue_certificate": _("Issue New Certificate for ${entity} ${primary_key}"),
                 "issued_by": _("Issued By"),
                 "issued_on": _("Issued On"),
@@ -369,6 +383,7 @@ class i18n_messages(Command):
                 "valid": _("Valid Certificate Present"),
                 "validity": _("Validity"),
                 "view_certificate": _("Certificate for ${entity} ${primary_key}"),
+                "view_certificate_btn": _("View Certificate"),
             },
             "config": {
                 "group": _("Group Options"),
@@ -408,7 +423,9 @@ class i18n_messages(Command):
             "dnszone": {
                 "identity": _("DNS Zone Settings"),
                 "add_permission":_("Add Permission"),
+                "add_permission_confirm":_("Are you sure you want to add permission for DNS Zone ${object}?"),
                 "remove_permission": _("Remove Permission"),
+                "remove_permission_confirm": _("Are you sure you want to remove permission for DNS Zone ${object}?"),
             },
             "group": {
                 "details": _("Group Settings"),
@@ -480,7 +497,7 @@ class i18n_messages(Command):
             },
             "krbtpolicy": {
                 "identity": _("Kerberos Ticket Policy"),
-                },
+            },
             "netgroup": {
                 "any_host": _("Any Host"),
                 "anyone": _("Anyone"),
@@ -494,11 +511,18 @@ class i18n_messages(Command):
                 "user": _("User"),
                 "usergroups": _("User Groups"),
                 "users": _("Users"),
-                },
+            },
+            "otptoken": {
+                "details": "OTP Token Settings",
+                "disable": "Disable token",
+                "enable": "Enable token",
+            },
             "permission": {
-                "identity": _("Identity"),
-                "invalid_target": _("Permission with invalid target specification"),
-                "rights": _("Rights"),
+                "add_custom_attr": _("Add custom attribute"),
+                "attribute": _("Attribute"),
+                "filter": _("Filter"),
+                "identity": _("Permission settings"),
+                "managed": _("Attribute breakdown"),
                 "target": _("Target"),
             },
             "privilege": {
@@ -522,6 +546,9 @@ class i18n_messages(Command):
                 "type_ipa": _("IPA trust"),
                 "type_winsync": _("Active Directory winsync"),
             },
+            "radiusproxy": {
+                "details": _("RADIUS Proxy Server Settings"),
+            },
             "realmdomains": {
                 "identity": _("Realm Domains"),
                 "check_dns": _("Check DNS"),
@@ -622,21 +649,32 @@ class i18n_messages(Command):
                 "misc": _("Misc. Information"),
                 "status_confirmation": _("Are you sure you want to ${action} the user?<br/>The change will take effect immediately."),
                 "status_link": _("Click to ${action}"),
+                "unlock": _("Unlock"),
+                "unlock_confirm": _("Are you sure you want to unlock user ${object}?"),
             },
         },
         "password": {
             "current_password": _("Current Password"),
             "current_password_required": _("Current password is required"),
             "expires_in": _("Your password expires in ${days} days."),
+            "first_otp": _("First OTP"),
             "invalid_password": _("The password or username you entered is incorrect."),
             "new_password": _("New Password"),
             "new_password_required": _("New password is required"),
+            "otp": _("OTP"),
+            "otp_long": _("One-Time-Password"),
+            "otp_sync_fail": _("Token synchronization failed"),
+            "otp_sync_invalid": _("The username, password or token codes are not correct"),
+            "otp_sync_success":_("Token was synchronized"),
             "password": _("Password"),
+            "password_and_otp": _("Password or Password+One-Time-Password"),
             "password_change_complete": _("Password change complete"),
             "password_must_match": _("Passwords must match"),
             "reset_failure": _("Password reset was not successful."),
             "reset_password": _("Reset Password"),
             "reset_password_sentence": _("Reset your password."),
+            "second_otp": _("Second OTP"),
+            "token_id": _("Token ID"),
             "verify_password": _("Verify Password"),
         },
         "search": {
@@ -647,6 +685,7 @@ class i18n_messages(Command):
             "enable_confirm": _("Are you sure you want to enable selected entries?"),
             "enabled": _("${count} item(s) enabled"),
             "partial_delete": _("Some entries were not deleted"),
+            "placeholder": _("Search"),
             "quick_links": _("Quick Links"),
             "select_all": _("Select All"),
             "truncated": _("Query returned more results than the configured size limit. Displaying the first ${counter} results."),
@@ -658,9 +697,11 @@ class i18n_messages(Command):
             "enable": _("Enable"),
             "enabled": _("Enabled"),
             "label": _("Status"),
+            "working": _("Working"),
         },
         "tabs": {
             "audit": _("Audit"),
+            "authentication": _("Authentication"),
             "automember": _("Automember"),
             "automount": _("Automount"),
             "cert": _("Certificates"),
@@ -668,6 +709,7 @@ class i18n_messages(Command):
             "hbac": _("Host Based Access Control"),
             "identity": _("Identity"),
             "ipaserver": _("IPA Server"),
+            "network_services": _("Network Services"),
             "policy": _("Policy"),
             "role": _("Role Based Access Control"),
             "sudo": _("Sudo"),
@@ -675,14 +717,18 @@ class i18n_messages(Command):
         },
         "true": _("True"),
         "widget": {
+            "first": _("First"),
+            "last": _("Last"),
             "next": _("Next"),
             "page": _("Page"),
             "prev": _("Prev"),
-            "undo": _("undo"),
-            "undo_all": _("undo all"),
+            "undo": _("Undo"),
+            "undo_all": _("Undo All"),
             "validation": {
                 "error": _("Text does not match field pattern"),
+                "datetime": _("Must be an UTC date/time value (e.g., \"2014-01-20 17:58:01Z\")"),
                 "decimal": _("Must be a decimal number"),
+                "format": _("Format error"),
                 "integer": _("Must be an integer"),
                 "ip_address": _('Not a valid IP address'),
                 "ip_v4_address": _('Not a valid IPv4 address'),
@@ -690,6 +736,7 @@ class i18n_messages(Command):
                 "max_value": _("Maximum value is ${value}"),
                 "min_value": _("Minimum value is ${value}"),
                 "net_address": _("Not a valid network address"),
+                "parse": _("Parse error"),
                 "port": _("'${port}' is not a valid port"),
                 "required": _("Required field"),
                 "unsupported": _("Unsupported value"),
@@ -706,4 +753,3 @@ class i18n_messages(Command):
         print json.dumps(result, default=json_serialize)
 
 
-api.register(i18n_messages)
