@@ -21,6 +21,7 @@
 
 from ipalib import api, errors
 from ipalib import Str, StrEnum
+from ipalib.plugable import Registry
 from ipalib.plugins.baseldap import *
 from ipalib import _, ngettext
 from ipalib.plugins.hbacrule import is_all
@@ -49,6 +50,7 @@ EXAMPLES:
    ipa netgroup-del admins
 """)
 
+register = Registry()
 
 NETGROUP_PATTERN='^[a-zA-Z0-9_.][a-zA-Z0-9_.-]*$'
 NETGROUP_PATTERN_ERRMSG='may only include letters, numbers, _, -, and .'
@@ -73,6 +75,8 @@ output_params = (
         ),
     )
 
+
+@register()
 class netgroup(LDAPObject):
     """
     Netgroup object.
@@ -81,6 +85,7 @@ class netgroup(LDAPObject):
     object_name = _('netgroup')
     object_name_plural = _('netgroups')
     object_class = ['ipaobject', 'ipaassociation', 'ipanisnetgroup']
+    permission_filter_objectclasses = ['ipanisnetgroup']
     default_attributes = [
         'cn', 'description', 'memberof', 'externalhost', 'nisdomainname',
         'memberuser', 'memberhost', 'member', 'memberindirect',
@@ -104,6 +109,68 @@ class netgroup(LDAPObject):
         'memberuser': ('Member', '', 'no_'),
         'memberhost': ('Member', '', 'no_'),
     }
+    managed_permissions = {
+        'System: Read Netgroups': {
+            'replaces_global_anonymous_aci': True,
+            'ipapermbindruletype': 'all',
+            'ipapermright': {'read', 'search', 'compare'},
+            'ipapermdefaultattr': {
+                'cn', 'description', 'hostcategory', 'ipaenabledflag',
+                'ipauniqueid', 'nisdomainname', 'usercategory', 'objectclass',
+            },
+        },
+        'System: Read Netgroup Membership': {
+            'replaces_global_anonymous_aci': True,
+            'ipapermbindruletype': 'all',
+            'ipapermright': {'read', 'search', 'compare'},
+            'ipapermdefaultattr': {
+                'externalhost', 'member', 'memberof', 'memberuser',
+                'memberhost', 'objectclass',
+            },
+        },
+        'System: Add Netgroups': {
+            'ipapermright': {'add'},
+            'replaces': [
+                '(target = "ldap:///ipauniqueid=*,cn=ng,cn=alt,$SUFFIX")(version 3.0;acl "permission:Add netgroups";allow (add) groupdn = "ldap:///cn=Add netgroups,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Netgroups Administrators'},
+        },
+        'System: Modify Netgroup Membership': {
+            'ipapermright': {'write'},
+            'ipapermdefaultattr': {
+                'externalhost', 'member', 'memberhost', 'memberuser'
+            },
+            'replaces': [
+                '(targetattr = "memberhost || externalhost || memberuser || member")(target = "ldap:///ipauniqueid=*,cn=ng,cn=alt,$SUFFIX")(version 3.0;acl "permission:Modify netgroup membership";allow (write) groupdn = "ldap:///cn=Modify netgroup membership,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Netgroups Administrators'},
+        },
+        'System: Modify Netgroups': {
+            'ipapermright': {'write'},
+            'ipapermdefaultattr': {'description'},
+            'replaces': [
+                '(targetattr = "description")(target = "ldap:///ipauniqueid=*,cn=ng,cn=alt,$SUFFIX")(version 3.0; acl "permission:Modify netgroups";allow (write) groupdn = "ldap:///cn=Modify netgroups,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Netgroups Administrators'},
+        },
+        'System: Remove Netgroups': {
+            'ipapermright': {'delete'},
+            'replaces': [
+                '(target = "ldap:///ipauniqueid=*,cn=ng,cn=alt,$SUFFIX")(version 3.0;acl "permission:Remove netgroups";allow (delete) groupdn = "ldap:///cn=Remove netgroups,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Netgroups Administrators'},
+        },
+        'System: Read Netgroup Compat Tree': {
+            'non_object': True,
+            'ipapermbindruletype': 'anonymous',
+            'ipapermlocation': api.env.basedn,
+            'ipapermtarget': DN('cn=ng', 'cn=compat', api.env.basedn),
+            'ipapermright': {'read', 'search', 'compare'},
+            'ipapermdefaultattr': {
+                'objectclass', 'cn', 'membernisnetgroup', 'nisnetgrouptriple',
+            },
+        },
+    }
 
     label = _('Netgroups')
     label_singular = _('Netgroup')
@@ -149,9 +216,8 @@ class netgroup(LDAPObject):
         external_host_param,
     )
 
-api.register(netgroup)
-
 
+@register()
 class netgroup_add(LDAPCreate):
     __doc__ = _('Add a new netgroup.')
 
@@ -167,7 +233,7 @@ class netgroup_add(LDAPCreate):
 
         try:
             test_dn = self.obj.get_dn(keys[-1])
-            (test_dn_, netgroup) = ldap.get_entry(test_dn, ['objectclass'])
+            netgroup = ldap.get_entry(test_dn, ['objectclass'])
             if 'mepManagedEntry' in netgroup.get('objectclass', []):
                 raise errors.DuplicateEntry(message=unicode(self.msg_collision % keys[-1]))
             else:
@@ -186,17 +252,16 @@ class netgroup_add(LDAPCreate):
 
         return dn
 
-api.register(netgroup_add)
-
 
+@register()
 class netgroup_del(LDAPDelete):
     __doc__ = _('Delete a netgroup.')
 
     msg_summary = _('Deleted netgroup "%(value)s"')
 
-api.register(netgroup_del)
 
 
+@register()
 class netgroup_mod(LDAPUpdate):
     __doc__ = _('Modify a netgroup.')
 
@@ -206,7 +271,8 @@ class netgroup_mod(LDAPUpdate):
     def pre_callback(self, ldap, dn, entry_attrs, attrs_list, *keys, **options):
         assert isinstance(dn, DN)
         try:
-            (dn, entry_attrs) = ldap.get_entry(dn, attrs_list)
+            entry_attrs = ldap.get_entry(dn, attrs_list)
+            dn = entry_attrs.dn
         except errors.NotFound:
             self.obj.handle_not_found(*keys)
         if is_all(options, 'usercategory') and 'memberuser' in entry_attrs:
@@ -215,9 +281,8 @@ class netgroup_mod(LDAPUpdate):
             raise errors.MutuallyExclusiveError(reason=_("host category cannot be set to 'all' while there are allowed hosts"))
         return dn
 
-api.register(netgroup_mod)
-
 
+@register()
 class netgroup_find(LDAPSearch):
     __doc__ = _('Search for a netgroup.')
 
@@ -253,40 +318,49 @@ class netgroup_find(LDAPSearch):
         filter = ldap.combine_filters((local_filter, filter), rules=ldap.MATCH_ALL)
         return (filter, base_dn, scope)
 
-api.register(netgroup_find)
-
 
+@register()
 class netgroup_show(LDAPRetrieve):
     __doc__ = _('Display information about a netgroup.')
 
     has_output_params = LDAPRetrieve.has_output_params + output_params
 
-api.register(netgroup_show)
-
 
+@register()
 class netgroup_add_member(LDAPAddMember):
     __doc__ = _('Add members to a netgroup.')
 
     member_attributes = ['memberuser', 'memberhost', 'member']
     has_output_params = LDAPAddMember.has_output_params + output_params
+
     def pre_callback(self, ldap, dn, found, not_found, *keys, **options):
         assert isinstance(dn, DN)
         return add_external_pre_callback('host', ldap, dn, keys, options)
 
-    def post_callback(self, ldap, completed, failed, dn, entry_attrs, *keys, **options):
+    def post_callback(self, ldap, completed, failed, dn, entry_attrs,
+                      *keys, **options):
         assert isinstance(dn, DN)
-        return add_external_post_callback('memberhost', 'host', 'externalhost', ldap, completed, failed, dn, entry_attrs, keys, options)
-
-api.register(netgroup_add_member)
+        return add_external_post_callback(ldap, dn, entry_attrs,
+                                          failed=failed,
+                                          completed=completed,
+                                          memberattr='memberhost',
+                                          membertype='host',
+                                          externalattr='externalhost')
 
 
+@register()
 class netgroup_remove_member(LDAPRemoveMember):
     __doc__ = _('Remove members from a netgroup.')
 
     member_attributes = ['memberuser', 'memberhost', 'member']
     has_output_params = LDAPRemoveMember.has_output_params + output_params
-    def post_callback(self, ldap, completed, failed, dn, entry_attrs, *keys, **options):
-        assert isinstance(dn, DN)
-        return remove_external_post_callback('memberhost', 'host', 'externalhost', ldap, completed, failed, dn, entry_attrs, keys, options)
 
-api.register(netgroup_remove_member)
+    def post_callback(self, ldap, completed, failed, dn, entry_attrs,
+                      *keys, **options):
+        assert isinstance(dn, DN)
+        return remove_external_post_callback(ldap, dn, entry_attrs,
+                                             failed=failed,
+                                             completed=completed,
+                                             memberattr='memberhost',
+                                             membertype='host',
+                                             externalattr='externalhost')
