@@ -18,7 +18,8 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from ipalib.plugins.baseldap import *
-from ipalib import api, _, ngettext
+from ipalib import api, _, ngettext, errors
+from ipalib.plugable import Registry
 
 __doc__ = _("""
 Privileges
@@ -41,6 +42,10 @@ A privilege may not contain other privileges.
 See role and permission for additional information.
 """)
 
+register = Registry()
+
+
+@register()
 class privilege(LDAPObject):
     """
     Privilege object.
@@ -49,6 +54,7 @@ class privilege(LDAPObject):
     object_name = _('privilege')
     object_name_plural = _('privileges')
     object_class = ['nestedgroup', 'groupofnames']
+    permission_filter_objectclasses = ['groupofnames']
     default_attributes = ['cn', 'description', 'member', 'memberof']
     attribute_members = {
         'member': ['role'],
@@ -58,6 +64,34 @@ class privilege(LDAPObject):
         'member': ['permission'],
     }
     rdn_is_primary_key = True
+    managed_permissions = {
+        'System: Read Privileges': {
+            'replaces_global_anonymous_aci': True,
+            'ipapermright': {'read', 'search', 'compare'},
+            'ipapermdefaultattr': {
+                'businesscategory', 'cn', 'description', 'member', 'memberof',
+                'o', 'objectclass', 'ou', 'owner', 'seealso', 'memberuser',
+                'memberhost',
+            },
+            'default_privileges': {'RBAC Readers'},
+        },
+        'System: Add Privileges': {
+            'ipapermright': {'add'},
+            'default_privileges': {'Delegation Administrator'},
+        },
+        'System: Modify Privileges': {
+            'ipapermright': {'write'},
+            'ipapermdefaultattr': {
+                'businesscategory',  'cn', 'description', 'o', 'ou', 'owner',
+                'seealso',
+            },
+            'default_privileges': {'Delegation Administrator'},
+        },
+        'System: Remove Privileges': {
+            'ipapermright': {'delete'},
+            'default_privileges': {'Delegation Administrator'},
+        },
+    }
 
     label = _('Privileges')
     label_singular = _('Privilege')
@@ -75,33 +109,29 @@ class privilege(LDAPObject):
         ),
     )
 
-api.register(privilege)
-
 
+@register()
 class privilege_add(LDAPCreate):
     __doc__ = _('Add a new privilege.')
 
     msg_summary = _('Added privilege "%(value)s"')
 
-api.register(privilege_add)
-
 
+@register()
 class privilege_del(LDAPDelete):
     __doc__ = _('Delete a privilege.')
 
     msg_summary = _('Deleted privilege "%(value)s"')
 
-api.register(privilege_del)
-
 
+@register()
 class privilege_mod(LDAPUpdate):
     __doc__ = _('Modify a privilege.')
 
     msg_summary = _('Modified privilege "%(value)s"')
 
-api.register(privilege_mod)
-
 
+@register()
 class privilege_find(LDAPSearch):
     __doc__ = _('Search for privileges.')
 
@@ -109,32 +139,28 @@ class privilege_find(LDAPSearch):
         '%(count)d privilege matched', '%(count)d privileges matched', 0
     )
 
-api.register(privilege_find)
-
 
+@register()
 class privilege_show(LDAPRetrieve):
     __doc__ = _('Display information about a privilege.')
 
-api.register(privilege_show)
-
 
+@register()
 class privilege_add_member(LDAPAddMember):
     __doc__ = _('Add members to a privilege.')
 
     NO_CLI=True
 
-api.register(privilege_add_member)
-
 
+@register()
 class privilege_remove_member(LDAPRemoveMember):
     """
     Remove members from a privilege
     """
     NO_CLI=True
 
-api.register(privilege_remove_member)
-
 
+@register()
 class privilege_add_permission(LDAPAddReverseMember):
     __doc__ = _('Add permissions to a privilege.')
 
@@ -155,9 +181,39 @@ class privilege_add_permission(LDAPAddReverseMember):
         ),
     )
 
-api.register(privilege_add_permission)
+    def pre_callback(self, ldap, dn, *keys, **options):
+        if options.get('permission'):
+            # We can only add permissions with bind rule type set to
+            # "permission" (or old-style permissions)
+            ldapfilter = ldap.combine_filters(rules='&', filters=[
+                '(objectClass=ipaPermissionV2)',
+                '(!(ipaPermBindRuleType=permission))',
+                ldap.make_filter_from_attr('cn', options['permission'],
+                                           rules='|'),
+            ])
+            try:
+                entries, truncated = ldap.find_entries(
+                    filter=ldapfilter,
+                    attrs_list=['cn', 'ipapermbindruletype'],
+                    base_dn=DN(self.api.env.container_permission,
+                               self.api.env.basedn),
+                    size_limit=1)
+            except errors.NotFound:
+                pass
+            else:
+                entry = entries[0]
+                message = _('cannot add permission "%(perm)s" with bindtype '
+                            '"%(bindtype)s" to a privilege')
+                raise errors.ValidationError(
+                    name='permission',
+                    error=message % {
+                        'perm': entry.single_value['cn'],
+                        'bindtype': entry.single_value.get(
+                            'ipapermbindruletype', 'permission')})
+        return dn
 
 
+@register()
 class privilege_remove_permission(LDAPRemoveReverseMember):
     __doc__ = _('Remove permissions from a privilege.')
 
@@ -179,5 +235,3 @@ class privilege_remove_permission(LDAPRemoveReverseMember):
             doc=_('Number of permissions removed'),
         ),
     )
-
-api.register(privilege_remove_permission)
