@@ -22,6 +22,7 @@ from ipalib import api
 from ipalib import Int, Str, DNParam
 from ipalib.plugins.baseldap import *
 from ipalib import _
+from ipalib.plugable import Registry
 from ipalib.request import context
 from ipapython.ipautil import run
 from ipapython.dn import DN
@@ -70,6 +71,9 @@ EXAMPLES:
    ipa pwpolicy-mod --minclasses=2 localadmins
 """)
 
+register = Registry()
+
+@register()
 class cosentry(LDAPObject):
     """
     Class of Service object used for linking policies with groups
@@ -78,7 +82,43 @@ class cosentry(LDAPObject):
 
     container_dn = DN(('cn', 'costemplates'), api.env.container_accounts)
     object_class = ['top', 'costemplate', 'extensibleobject', 'krbcontainer']
+    permission_filter_objectclasses = ['costemplate']
     default_attributes = ['cn', 'cospriority', 'krbpwdpolicyreference']
+    managed_permissions = {
+        'System: Read Group Password Policy costemplate': {
+            'replaces_global_anonymous_aci': True,
+            'ipapermright': {'read', 'search', 'compare'},
+            'ipapermdefaultattr': {
+                'cn', 'cospriority', 'krbpwdpolicyreference', 'objectclass',
+            },
+            'default_privileges': {
+                'Password Policy Readers',
+                'Password Policy Administrator',
+            },
+        },
+        'System: Add Group Password Policy costemplate': {
+            'ipapermright': {'add'},
+            'replaces': [
+                '(target = "ldap:///cn=*,cn=costemplates,cn=accounts,$SUFFIX")(version 3.0;acl "permission:Add Group Password Policy costemplate";allow (add) groupdn = "ldap:///cn=Add Group Password Policy costemplate,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Password Policy Administrator'},
+        },
+        'System: Delete Group Password Policy costemplate': {
+            'ipapermright': {'delete'},
+            'replaces': [
+                '(target = "ldap:///cn=*,cn=costemplates,cn=accounts,$SUFFIX")(version 3.0;acl "permission:Delete Group Password Policy costemplate";allow (delete) groupdn = "ldap:///cn=Delete Group Password Policy costemplate,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Password Policy Administrator'},
+        },
+        'System: Modify Group Password Policy costemplate': {
+            'ipapermright': {'write'},
+            'ipapermdefaultattr': {'cospriority'},
+            'replaces': [
+                '(targetattr = "cospriority")(target = "ldap:///cn=*,cn=costemplates,cn=accounts,$SUFFIX")(version 3.0;acl "permission:Modify Group Password Policy costemplate";allow (write) groupdn = "ldap:///cn=Modify Group Password Policy costemplate,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Password Policy Administrator'},
+        },
+    }
 
     takes_params = (
         Str('cn', primary_key=True),
@@ -112,9 +152,8 @@ class cosentry(LDAPObject):
                     }
                 )
 
-api.register(cosentry)
-
 
+@register()
 class cosentry_add(LDAPCreate):
     NO_CLI = True
 
@@ -130,15 +169,13 @@ class cosentry_add(LDAPCreate):
         del entry_attrs['cn']
         return dn
 
-api.register(cosentry_add)
-
 
+@register()
 class cosentry_del(LDAPDelete):
     NO_CLI = True
 
-api.register(cosentry_del)
-
 
+@register()
 class cosentry_mod(LDAPUpdate):
     NO_CLI = True
 
@@ -154,24 +191,21 @@ class cosentry_mod(LDAPUpdate):
                 self.obj.check_priority_uniqueness(*keys, **options)
         return dn
 
-api.register(cosentry_mod)
-
 
+@register()
 class cosentry_show(LDAPRetrieve):
     NO_CLI = True
 
-api.register(cosentry_show)
-
 
+@register()
 class cosentry_find(LDAPSearch):
     NO_CLI = True
 
-api.register(cosentry_find)
-
 
 global_policy_name = 'global_policy'
 global_policy_dn = DN(('cn', global_policy_name), ('cn', api.env.realm), ('cn', 'kerberos'), api.env.basedn)
 
+@register()
 class pwpolicy(LDAPObject):
     """
     Password Policy object
@@ -180,12 +214,57 @@ class pwpolicy(LDAPObject):
     object_name = _('password policy')
     object_name_plural = _('password policies')
     object_class = ['top', 'nscontainer', 'krbpwdpolicy']
+    permission_filter_objectclasses = ['krbpwdpolicy']
     default_attributes = [
         'cn', 'cospriority', 'krbmaxpwdlife', 'krbminpwdlife',
         'krbpwdhistorylength', 'krbpwdmindiffchars', 'krbpwdminlength',
         'krbpwdmaxfailure', 'krbpwdfailurecountinterval',
         'krbpwdlockoutduration',
     ]
+    managed_permissions = {
+        'System: Read Group Password Policy': {
+            'replaces_global_anonymous_aci': True,
+            'ipapermbindruletype': 'permission',
+            'ipapermright': {'read', 'search', 'compare'},
+            'ipapermdefaultattr': {
+                'cn', 'cospriority', 'krbmaxpwdlife', 'krbminpwdlife',
+                'krbpwdfailurecountinterval', 'krbpwdhistorylength',
+                'krbpwdlockoutduration', 'krbpwdmaxfailure',
+                'krbpwdmindiffchars', 'krbpwdminlength', 'objectclass',
+            },
+            'default_privileges': {
+                'Password Policy Readers',
+                'Password Policy Administrator',
+            },
+        },
+        'System: Add Group Password Policy': {
+            'ipapermright': {'add'},
+            'replaces': [
+                '(target = "ldap:///cn=*,cn=$REALM,cn=kerberos,$SUFFIX")(version 3.0;acl "permission:Add Group Password Policy";allow (add) groupdn = "ldap:///cn=Add Group Password Policy,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Password Policy Administrator'},
+        },
+        'System: Delete Group Password Policy': {
+            'ipapermright': {'delete'},
+            'replaces': [
+                '(target = "ldap:///cn=*,cn=$REALM,cn=kerberos,$SUFFIX")(version 3.0;acl "permission:Delete Group Password Policy";allow (delete) groupdn = "ldap:///cn=Delete Group Password Policy,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Password Policy Administrator'},
+        },
+        'System: Modify Group Password Policy': {
+            'ipapermright': {'write'},
+            'ipapermdefaultattr': {
+                'krbmaxpwdlife', 'krbminpwdlife', 'krbpwdfailurecountinterval',
+                'krbpwdhistorylength', 'krbpwdlockoutduration',
+                'krbpwdmaxfailure', 'krbpwdmindiffchars', 'krbpwdminlength'
+            },
+            'replaces': [
+                '(targetattr = "krbmaxpwdlife || krbminpwdlife || krbpwdhistorylength || krbpwdmindiffchars || krbpwdminlength || krbpwdmaxfailure || krbpwdfailurecountinterval || krbpwdlockoutduration")(target = "ldap:///cn=*,cn=$REALM,cn=kerberos,$SUFFIX")(version 3.0;acl "permission:Modify Group Password Policy";allow (write) groupdn = "ldap:///cn=Modify Group Password Policy,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Password Policy Administrator'},
+        },
+    }
+
     MIN_KRB5KDC_WITH_LOCKOUT = "1.8"
     has_lockout = False
     lockout_params = ()
@@ -334,9 +413,8 @@ class pwpolicy(LDAPObject):
                     entry['attributelevelrights']['cospriority'] = \
                         cos_entry['attributelevelrights']['cospriority']
 
-api.register(pwpolicy)
-
 
+@register()
 class pwpolicy_add(LDAPCreate):
     __doc__ = _('Add a new group password policy.')
 
@@ -361,9 +439,8 @@ class pwpolicy_add(LDAPCreate):
         self.obj.convert_time_for_output(entry_attrs, **options)
         return dn
 
-api.register(pwpolicy_add)
-
 
+@register()
 class pwpolicy_del(LDAPDelete):
     __doc__ = _('Delete a group password policy.')
 
@@ -389,9 +466,8 @@ class pwpolicy_del(LDAPDelete):
             pass
         return True
 
-api.register(pwpolicy_del)
-
 
+@register()
 class pwpolicy_mod(LDAPUpdate):
     __doc__ = _('Modify a group password policy.')
 
@@ -427,15 +503,14 @@ class pwpolicy_mod(LDAPUpdate):
     def exc_callback(self, keys, options, exc, call_func, *call_args, **call_kwargs):
         if call_func.func_name == 'update_entry':
             if isinstance(exc, errors.EmptyModlist):
-                entry_attrs = call_args[1]
+                entry_attrs = call_args[0]
                 cosupdate = getattr(context, 'cosupdate')
                 if not entry_attrs or cosupdate:
                     return
         raise exc
 
-api.register(pwpolicy_mod)
-
 
+@register()
 class pwpolicy_show(LDAPRetrieve):
     __doc__ = _('Display information about password policy.')
 
@@ -463,9 +538,8 @@ class pwpolicy_show(LDAPRetrieve):
         self.obj.convert_time_for_output(entry_attrs, **options)
         return dn
 
-api.register(pwpolicy_show)
-
 
+@register()
 class pwpolicy_find(LDAPSearch):
     __doc__ = _('Search for group password policies.')
 
@@ -478,13 +552,13 @@ class pwpolicy_find(LDAPSearch):
         returns a pair: (is_global, priority)
         """
         # global policy will be always last in the output
-        if entry[1]['cn'][0] == global_policy_name:
+        if entry['cn'][0] == global_policy_name:
             return True, 0
         else:
             # policies with higher priority (lower number) will be at the
             # beginning of the list
             try:
-                cospriority = int(entry[1]['cospriority'][0])
+                cospriority = int(entry['cospriority'][0])
             except KeyError:
                 # if cospriority is not present in the entry, rather return 0
                 # than crash
@@ -496,9 +570,9 @@ class pwpolicy_find(LDAPSearch):
             # When pkey_only flag is on, entries should contain only a cn.
             # Add a cospriority attribute that will be used for sorting.
             # Attribute rights are not allowed for pwpolicy_find.
-            self.obj.add_cospriority(e[1], e[1]['cn'][0], rights=False)
+            self.obj.add_cospriority(e, e['cn'][0], rights=False)
 
-            self.obj.convert_time_for_output(e[1], **options)
+            self.obj.convert_time_for_output(e, **options)
 
         # do custom entry sorting by its cospriority
         entries.sort(key=self.priority_sort_key)
@@ -507,10 +581,8 @@ class pwpolicy_find(LDAPSearch):
             # remove cospriority that was used for sorting
             for e in entries:
                 try:
-                    del e[1]['cospriority']
+                    del e['cospriority']
                 except KeyError:
                     pass
 
         return truncated
-
-api.register(pwpolicy_find)
