@@ -18,6 +18,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+from ipalib.plugable import Registry
 from ipalib.plugins.baseldap import *
 from ipalib import api, Str, _, ngettext
 from ipalib import Command
@@ -58,6 +59,9 @@ EXAMPLES:
  add users, reset passwords or add a user to the default IPA user group.
 """)
 
+register = Registry()
+
+@register()
 class role(LDAPObject):
     """
     Role object.
@@ -66,6 +70,7 @@ class role(LDAPObject):
     object_name = _('role')
     object_name_plural = _('roles')
     object_class = ['groupofnames', 'nestedgroup']
+    permission_filter_objectclasses = ['groupofnames']
     default_attributes = ['cn', 'description', 'member', 'memberof',
         'memberindirect', 'memberofindirect',
     ]
@@ -77,6 +82,48 @@ class role(LDAPObject):
         'member': ['privilege'],
     }
     rdn_is_primary_key = True
+    managed_permissions = {
+        'System: Read Roles': {
+            'replaces_global_anonymous_aci': True,
+            'ipapermright': {'read', 'search', 'compare'},
+            'ipapermdefaultattr': {
+                'businesscategory', 'cn', 'description', 'member', 'memberof',
+                'o', 'objectclass', 'ou', 'owner', 'seealso', 'memberuser',
+                'memberhost',
+            },
+            'default_privileges': {'RBAC Readers'},
+        },
+        'System: Add Roles': {
+            'ipapermright': {'add'},
+            'replaces': [
+                '(target = "ldap:///cn=*,cn=roles,cn=accounts,$SUFFIX")(version 3.0;acl "permission:Add Roles";allow (add) groupdn = "ldap:///cn=Add Roles,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Delegation Administrator'},
+        },
+        'System: Modify Role Membership': {
+            'ipapermright': {'write'},
+            'ipapermdefaultattr': {'member'},
+            'replaces': [
+                '(targetattr = "member")(target = "ldap:///cn=*,cn=roles,cn=accounts,$SUFFIX")(version 3.0;acl "permission:Modify Role membership";allow (write) groupdn = "ldap:///cn=Modify Role membership,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Delegation Administrator'},
+        },
+        'System: Modify Roles': {
+            'ipapermright': {'write'},
+            'ipapermdefaultattr': {'cn', 'description'},
+            'replaces': [
+                '(targetattr = "cn || description")(target = "ldap:///cn=*,cn=roles,cn=accounts,$SUFFIX")(version 3.0; acl "permission:Modify Roles";allow (write) groupdn = "ldap:///cn=Modify Roles,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Delegation Administrator'},
+        },
+        'System: Remove Roles': {
+            'ipapermright': {'delete'},
+            'replaces': [
+                '(target = "ldap:///cn=*,cn=roles,cn=accounts,$SUFFIX")(version 3.0;acl "permission:Remove Roles";allow (delete) groupdn = "ldap:///cn=Remove Roles,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Delegation Administrator'},
+        },
+    }
 
     label = _('Roles')
     label_singular = _('Role')
@@ -94,33 +141,33 @@ class role(LDAPObject):
         ),
     )
 
-api.register(role)
 
 
+@register()
 class role_add(LDAPCreate):
     __doc__ = _('Add a new role.')
 
     msg_summary = _('Added role "%(value)s"')
 
-api.register(role_add)
 
 
+@register()
 class role_del(LDAPDelete):
     __doc__ = _('Delete a role.')
 
     msg_summary = _('Deleted role "%(value)s"')
 
-api.register(role_del)
 
 
+@register()
 class role_mod(LDAPUpdate):
     __doc__ = _('Modify a role.')
 
     msg_summary = _('Modified role "%(value)s"')
 
-api.register(role_mod)
 
 
+@register()
 class role_find(LDAPSearch):
     __doc__ = _('Search for roles.')
 
@@ -128,27 +175,27 @@ class role_find(LDAPSearch):
         '%(count)d role matched', '%(count)d roles matched', 0
     )
 
-api.register(role_find)
 
 
+@register()
 class role_show(LDAPRetrieve):
     __doc__ = _('Display information about a role.')
 
-api.register(role_show)
 
 
+@register()
 class role_add_member(LDAPAddMember):
     __doc__ = _('Add members to a role.')
 
-api.register(role_add_member)
 
 
+@register()
 class role_remove_member(LDAPRemoveMember):
     __doc__ = _('Remove members from a role.')
 
-api.register(role_remove_member)
 
 
+@register()
 class role_add_privilege(LDAPAddReverseMember):
     __doc__ = _('Add privileges to a role.')
 
@@ -169,9 +216,9 @@ class role_add_privilege(LDAPAddReverseMember):
         ),
     )
 
-api.register(role_add_privilege)
 
 
+@register()
 class role_remove_privilege(LDAPRemoveReverseMember):
     __doc__ = _('Remove privileges from a role.')
 
@@ -192,4 +239,3 @@ class role_remove_privilege(LDAPRemoveReverseMember):
         ),
     )
 
-api.register(role_remove_privilege)
