@@ -24,6 +24,7 @@ import os
 
 from ipalib import api, errors, util
 from ipalib import Str, Flag, Bytes, StrEnum, Bool
+from ipalib.plugable import Registry
 from ipalib.plugins.baseldap import *
 from ipalib import x509
 from ipalib import _, ngettext
@@ -91,6 +92,8 @@ EXAMPLES:
 
 """)
 
+register = Registry()
+
 output_params = (
     Flag('has_keytab',
         label=_('Keytab'),
@@ -259,7 +262,7 @@ def update_krbticketflags(ldap, entry_attrs, attrs_list, options, existing):
         old_entry_attrs = entry_attrs
 
     try:
-        ticket_flags = old_entry_attrs.single_value('krbticketflags')
+        ticket_flags = old_entry_attrs.single_value['krbticketflags']
         ticket_flags = int(ticket_flags)
     except (KeyError, ValueError):
         ticket_flags = _ticket_flags_default
@@ -275,8 +278,8 @@ def set_kerberos_attrs(entry_attrs, options):
         return
 
     try:
-        ticket_flags = entry_attrs.single_value('krbticketflags',
-                                                _ticket_flags_default)
+        ticket_flags = entry_attrs.single_value.get('krbticketflags',
+                                                    _ticket_flags_default)
         ticket_flags = int(ticket_flags)
     except ValueError:
         return
@@ -287,6 +290,7 @@ def set_kerberos_attrs(entry_attrs, options):
         if name in options or all_opt:
             entry_attrs[name] = bool(ticket_flags & value)
 
+@register()
 class service(LDAPObject):
     """
     Service object.
@@ -299,6 +303,7 @@ class service(LDAPObject):
         'ipaservice', 'pkiuser'
     ]
     possible_objectclasses = ['ipakrbprincipal']
+    permission_filter_objectclasses = ['ipaservice']
     search_attributes = ['krbprincipalname', 'managedby', 'ipakrbauthzdata']
     default_attributes = ['krbprincipalname', 'usercertificate', 'managedby',
         'ipakrbauthzdata',]
@@ -311,6 +316,51 @@ class service(LDAPObject):
         'managedby': ('Managed by', 'man_by_', 'not_man_by_'),
     }
     password_attributes = [('krbprincipalkey', 'has_keytab')]
+    managed_permissions = {
+        'System: Read Services': {
+            'replaces_global_anonymous_aci': True,
+            'ipapermbindruletype': 'all',
+            'ipapermright': {'read', 'search', 'compare'},
+            'ipapermdefaultattr': {
+                'objectclass',
+                'ipauniqueid', 'managedby', 'memberof', 'usercertificate',
+                'krbprincipalname', 'krbcanonicalname', 'krbprincipalaliases',
+                'krbprincipalexpiration', 'krbpasswordexpiration',
+                'krblastpwdchange', 'ipakrbauthzdata', 'ipakrbprincipalalias',
+                'krbobjectreferences',
+            },
+        },
+        'System: Add Services': {
+            'ipapermright': {'add'},
+            'replaces': [
+                '(target = "ldap:///krbprincipalname=*,cn=services,cn=accounts,$SUFFIX")(version 3.0;acl "permission:Add Services";allow (add) groupdn = "ldap:///cn=Add Services,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Service Administrators'},
+        },
+        'System: Manage Service Keytab': {
+            'ipapermright': {'write'},
+            'ipapermdefaultattr': {'krblastpwdchange', 'krbprincipalkey'},
+            'replaces': [
+                '(targetattr = "krbprincipalkey || krblastpwdchange")(target = "ldap:///krbprincipalname=*,cn=services,cn=accounts,$SUFFIX")(version 3.0;acl "permission:Manage service keytab";allow (write) groupdn = "ldap:///cn=Manage service keytab,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Service Administrators', 'Host Administrators'},
+        },
+        'System: Modify Services': {
+            'ipapermright': {'write'},
+            'ipapermdefaultattr': {'usercertificate'},
+            'replaces': [
+                '(targetattr = "usercertificate")(target = "ldap:///krbprincipalname=*,cn=services,cn=accounts,$SUFFIX")(version 3.0;acl "permission:Modify Services";allow (write) groupdn = "ldap:///cn=Modify Services,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Service Administrators'},
+        },
+        'System: Remove Services': {
+            'ipapermright': {'delete'},
+            'replaces': [
+                '(target = "ldap:///krbprincipalname=*,cn=services,cn=accounts,$SUFFIX")(version 3.0;acl "permission:Remove Services";allow (delete) groupdn = "ldap:///cn=Remove Services,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Service Administrators'},
+        },
+    }
 
     label = _('Services')
     label_singular = _('Service')
@@ -355,9 +405,9 @@ class service(LDAPObject):
             raise errors.ValidationError(name='ipakrbauthzdata',
                 error=_('NONE value cannot be combined with other PAC types'))
 
-api.register(service)
 
 
+@register()
 class service_add(LDAPCreate):
     __doc__ = _('Add a new IPA new service.')
 
@@ -417,9 +467,9 @@ class service_add(LDAPCreate):
         set_kerberos_attrs(entry_attrs, options)
         return dn
 
-api.register(service_add)
 
 
+@register()
 class service_del(LDAPDelete):
     __doc__ = _('Delete an IPA service.')
 
@@ -434,7 +484,7 @@ class service_del(LDAPDelete):
         check_required_principal(ldap, hostname, service)
         if self.api.env.enable_ra:
             try:
-                (dn, entry_attrs) = ldap.get_entry(dn, ['usercertificate'])
+                entry_attrs = ldap.get_entry(dn, ['usercertificate'])
             except errors.NotFound:
                 self.obj.handle_not_found(*keys)
             cert = entry_attrs.get('usercertificate')
@@ -462,9 +512,9 @@ class service_del(LDAPDelete):
                         raise nsprerr
         return dn
 
-api.register(service_del)
 
 
+@register()
 class service_mod(LDAPUpdate):
     __doc__ = _('Modify an existing IPA service.')
 
@@ -486,8 +536,7 @@ class service_mod(LDAPUpdate):
                 dercert = x509.normalize_certificate(cert)
                 x509.verify_cert_subject(ldap, hostname, dercert)
                 try:
-                    (dn, entry_attrs_old) = ldap.get_entry(
-                        dn, ['usercertificate'])
+                    entry_attrs_old = ldap.get_entry(dn, ['usercertificate'])
                 except errors.NotFound:
                     self.obj.handle_not_found(*keys)
                 if 'usercertificate' in entry_attrs_old:
@@ -510,9 +559,9 @@ class service_mod(LDAPUpdate):
         set_kerberos_attrs(entry_attrs, options)
         return dn
 
-api.register(service_mod)
 
 
+@register()
 class service_find(LDAPSearch):
     __doc__ = _('Search for IPA services.')
 
@@ -541,16 +590,15 @@ class service_find(LDAPSearch):
     def post_callback(self, ldap, entries, truncated, *args, **options):
         if options.get('pkey_only', False):
             return truncated
-        for entry in entries:
-            (dn, entry_attrs) = entry
-            self.obj.get_password_attributes(ldap, dn, entry_attrs)
+        for entry_attrs in entries:
+            self.obj.get_password_attributes(ldap, entry_attrs.dn, entry_attrs)
             set_certificate_attrs(entry_attrs)
             set_kerberos_attrs(entry_attrs, options)
         return truncated
 
-api.register(service_find)
 
 
+@register()
 class service_show(LDAPRetrieve):
     __doc__ = _('Display information about an IPA service.')
 
@@ -584,26 +632,26 @@ class service_show(LDAPRetrieve):
         else:
             return super(service_show, self).forward(*keys, **options)
 
-api.register(service_show)
 
+@register()
 class service_add_host(LDAPAddMember):
     __doc__ = _('Add hosts that can manage this service.')
 
     member_attributes = ['managedby']
     has_output_params = LDAPAddMember.has_output_params + output_params
 
-api.register(service_add_host)
 
 
+@register()
 class service_remove_host(LDAPRemoveMember):
     __doc__ = _('Remove hosts that can manage this service.')
 
     member_attributes = ['managedby']
     has_output_params = LDAPRemoveMember.has_output_params + output_params
 
-api.register(service_remove_host)
 
 
+@register()
 class service_disable(LDAPQuery):
     __doc__ = _('Disable the Kerberos key and SSL certificate of a service.')
 
@@ -615,7 +663,7 @@ class service_disable(LDAPQuery):
         ldap = self.obj.backend
 
         dn = self.obj.get_dn(*keys, **options)
-        (dn, entry_attrs) = ldap.get_entry(dn, ['usercertificate'])
+        entry_attrs = ldap.get_entry(dn, ['usercertificate'])
 
         (service, hostname, realm) = split_principal(keys[-1])
         check_required_principal(ldap, hostname, service)
@@ -648,7 +696,8 @@ class service_disable(LDAPQuery):
                         raise nsprerr
 
             # Remove the usercertificate altogether
-            ldap.update_entry(dn, {'usercertificate': None})
+            entry_attrs['usercertificate'] = None
+            ldap.update_entry(entry_attrs)
             done_work = True
 
         self.obj.get_password_attributes(ldap, dn, entry_attrs)
@@ -661,7 +710,6 @@ class service_disable(LDAPQuery):
 
         return dict(
             result=True,
-            value=keys[0],
+            value=pkey_to_value(keys[0], options),
         )
 
-api.register(service_disable)
