@@ -19,6 +19,7 @@
 
 from ipalib import api
 from ipalib import Str
+from ipalib.plugable import Registry
 from ipalib.plugins.baseldap import *
 from ipalib import _, ngettext
 
@@ -45,8 +46,11 @@ EXAMPLES:
    ipa group-show localadmins
 """)
 
+register = Registry()
+
 topic = ('sudo', _('commands for controlling sudo configuration'))
 
+@register()
 class sudocmdgroup(LDAPObject):
     """
     Sudo Command Group object.
@@ -55,6 +59,7 @@ class sudocmdgroup(LDAPObject):
     object_name = _('sudo command group')
     object_name_plural = _('sudo command groups')
     object_class = ['ipaobject', 'ipasudocmdgrp']
+    permission_filter_objectclasses = ['ipasudocmdgrp']
     default_attributes = [
         'cn', 'description', 'member',
     ]
@@ -62,6 +67,45 @@ class sudocmdgroup(LDAPObject):
     attribute_members = {
         'member': ['sudocmd'],
     }
+    managed_permissions = {
+        'System: Read Sudo Command Groups': {
+            'replaces_global_anonymous_aci': True,
+            'ipapermbindruletype': 'all',
+            'ipapermright': {'read', 'search', 'compare'},
+            'ipapermdefaultattr': {
+                'businesscategory', 'cn', 'description', 'ipauniqueid',
+                'member', 'o', 'objectclass', 'ou', 'owner', 'seealso',
+                'memberuser', 'memberhost',
+            },
+        },
+        'System: Add Sudo Command Group': {
+            'ipapermright': {'add'},
+            'replaces': [
+                '(target = "ldap:///cn=*,cn=sudocmdgroups,cn=sudo,$SUFFIX")(version 3.0;acl "permission:Add Sudo command group";allow (add) groupdn = "ldap:///cn=Add Sudo command group,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Sudo Administrator'},
+        },
+        'System: Delete Sudo Command Group': {
+            'ipapermright': {'delete'},
+            'replaces': [
+                '(target = "ldap:///cn=*,cn=sudocmdgroups,cn=sudo,$SUFFIX")(version 3.0;acl "permission:Delete Sudo command group";allow (delete) groupdn = "ldap:///cn=Delete Sudo command group,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Sudo Administrator'},
+        },
+        'System: Modify Sudo Command Group': {
+            'ipapermright': {'write'},
+            'ipapermdefaultattr': {'description'},
+            'default_privileges': {'Sudo Administrator'},
+        },
+        'System: Manage Sudo Command Group Membership': {
+            'ipapermright': {'write'},
+            'ipapermdefaultattr': {'member'},
+            'replaces': [
+                '(targetattr = "member")(target = "ldap:///cn=*,cn=sudocmdgroups,cn=sudo,$SUFFIX")(version 3.0;acl "permission:Manage Sudo command group membership";allow (write) groupdn = "ldap:///cn=Manage Sudo command group membership,cn=permissions,cn=pbac,$SUFFIX";)',
+            ],
+            'default_privileges': {'Sudo Administrator'},
+        },
+    }
 
     label = _('Sudo Command Groups')
     label_singular = _('Sudo Command Group')
@@ -88,33 +132,33 @@ class sudocmdgroup(LDAPObject):
         ),
     )
 
-api.register(sudocmdgroup)
 
 
+@register()
 class sudocmdgroup_add(LDAPCreate):
     __doc__ = _('Create new Sudo Command Group.')
 
     msg_summary = _('Added Sudo Command Group "%(value)s"')
 
-api.register(sudocmdgroup_add)
 
 
+@register()
 class sudocmdgroup_del(LDAPDelete):
     __doc__ = _('Delete Sudo Command Group.')
 
     msg_summary = _('Deleted Sudo Command Group "%(value)s"')
 
-api.register(sudocmdgroup_del)
 
 
+@register()
 class sudocmdgroup_mod(LDAPUpdate):
     __doc__ = _('Modify Sudo Command Group.')
 
     msg_summary = _('Modified Sudo Command Group "%(value)s"')
 
-api.register(sudocmdgroup_mod)
 
 
+@register()
 class sudocmdgroup_find(LDAPSearch):
     __doc__ = _('Search for Sudo Command Groups.')
 
@@ -123,22 +167,21 @@ class sudocmdgroup_find(LDAPSearch):
         '%(count)d Sudo Command Groups matched', 0
     )
 
-api.register(sudocmdgroup_find)
 
 
+@register()
 class sudocmdgroup_show(LDAPRetrieve):
     __doc__ = _('Display Sudo Command Group.')
 
-api.register(sudocmdgroup_show)
 
 
+@register()
 class sudocmdgroup_add_member(LDAPAddMember):
     __doc__ = _('Add members to Sudo Command Group.')
 
-api.register(sudocmdgroup_add_member)
 
 
+@register()
 class sudocmdgroup_remove_member(LDAPRemoveMember):
     __doc__ = _('Remove members from Sudo Command Group.')
 
-api.register(sudocmdgroup_remove_member)
