@@ -32,11 +32,13 @@ from types import NoneType
 from weakref import WeakKeyDictionary
 from dns import resolver, rdatatype
 from dns.exception import DNSException
+from netaddr.core import AddrFormatError
 
 from ipalib import errors
 from ipalib.text import _
 from ipapython.ssh import SSHPublicKey
 from ipapython.dn import DN, RDN
+from ipapython.dnsutil import DNSName
 
 
 def json_serialize(obj):
@@ -57,12 +59,15 @@ def json_serialize(obj):
 
 def get_current_principal():
     try:
-        # krbV isn't necessarily available on client machines, fail gracefully
-        import krbV
-        return unicode(krbV.default_context().default_ccache().principal().name)
+        import kerberos
+        rc, vc = kerberos.authGSSClientInit("notempty")
+        rc = kerberos.authGSSClientInquireCred(vc)
+        username = kerberos.authGSSClientUserName(vc)
+        kerberos.authGSSClientClean(vc)
+        return unicode(username)
     except ImportError:
-        raise RuntimeError('python-krbV is not available.')
-    except krbV.Krb5Error:
+        raise RuntimeError('python-kerberos is not available.')
+    except kerberos.GSSError, e:
         #TODO: do a kinit?
         raise errors.CCacheError()
 
@@ -81,7 +86,7 @@ def find_modules_in_dir(src_dir):
         if not name.endswith(suffix):
             continue
         pyfile = os.path.join(src_dir, name)
-        if os.path.islink(pyfile) or not os.path.isfile(pyfile):
+        if not os.path.isfile(pyfile):
             continue
         module = name[:-len(suffix)]
         if module == '__init__':
@@ -91,19 +96,51 @@ def find_modules_in_dir(src_dir):
 
 def validate_host_dns(log, fqdn):
     """
-    See if the hostname has a DNS A record.
+    See if the hostname has a DNS A/AAAA record.
     """
     try:
         answers = resolver.query(fqdn, rdatatype.A)
         log.debug(
-            'IPA: found %d records for %s: %s' % (len(answers), fqdn,
+            'IPA: found %d A records for %s: %s' % (len(answers), fqdn,
                 ' '.join(str(answer) for answer in answers))
         )
     except DNSException, e:
         log.debug(
             'IPA: DNS A record lookup failed for %s' % fqdn
         )
-        raise errors.DNSNotARecordError()
+        # A record not found, try to find AAAA record
+        try:
+            answers = resolver.query(fqdn, rdatatype.AAAA)
+            log.debug(
+                'IPA: found %d AAAA records for %s: %s' % (len(answers), fqdn,
+                    ' '.join(str(answer) for answer in answers))
+            )
+        except DNSException, e:
+            log.debug(
+                'IPA: DNS AAAA record lookup failed for %s' % fqdn
+            )
+            raise errors.DNSNotARecordError()
+
+
+def has_soa_or_ns_record(domain):
+    """
+    Checks to see if given domain has SOA or NS record.
+    Returns True or False.
+    """
+    try:
+        resolver.query(domain, rdatatype.SOA)
+        soa_record_found = True
+    except DNSException:
+        soa_record_found = False
+
+    try:
+        resolver.query(domain, rdatatype.NS)
+        ns_record_found = True
+    except DNSException:
+        ns_record_found = False
+
+    return soa_record_found or ns_record_found
+
 
 def normalize_name(name):
     result = dict()
@@ -174,8 +211,7 @@ def check_writable_file(filename):
         raise errors.FileError(reason=str(e))
 
 def normalize_zonemgr(zonemgr):
-    if not zonemgr:
-        # do not normalize empty or None value
+    if not zonemgr or not isinstance(zonemgr, basestring):
         return zonemgr
     if '@' in zonemgr:
         # local-part needs to be normalized
@@ -194,13 +230,21 @@ def normalize_zone(zone):
     else:
         return zone
 
-def validate_dns_label(dns_label, allow_underscore=False):
-    label_chars = r'a-z0-9'
-    underscore_err_msg = ''
+
+def validate_dns_label(dns_label, allow_underscore=False, allow_slash=False):
+    base_chars = 'a-z0-9'
+    extra_chars = ''
+    middle_chars = ''
+
     if allow_underscore:
-        label_chars += "_"
-        underscore_err_msg = u' _,'
-    label_regex = r'^[%(chars)s]([%(chars)s-]?[%(chars)s])*$' % dict(chars=label_chars)
+        extra_chars += '_'
+    if allow_slash:
+        middle_chars += '/'
+
+    middle_chars = middle_chars + '-' #has to be always the last in the regex [....-]
+
+    label_regex = r'^[%(base)s%(extra)s]([%(base)s%(extra)s%(middle)s]?[%(base)s%(extra)s])*$' \
+        % dict(base=base_chars, extra=extra_chars, middle=middle_chars)
     regex = re.compile(label_regex, re.IGNORECASE)
 
     if not dns_label:
@@ -210,66 +254,40 @@ def validate_dns_label(dns_label, allow_underscore=False):
         raise ValueError(_('DNS label cannot be longer that 63 characters'))
 
     if not regex.match(dns_label):
-        raise ValueError(_('only letters, numbers,%(underscore)s and - are allowed. ' \
-                           'DNS label may not start or end with -') \
-                           % dict(underscore=underscore_err_msg))
+        chars = ', '.join("'%s'" % c for c in extra_chars + middle_chars)
+        chars2 = ', '.join("'%s'" % c for c in middle_chars)
+        raise ValueError(_("only letters, numbers, %(chars)s are allowed. " \
+                           "DNS label may not start or end with %(chars2)s") \
+                           % dict(chars=chars, chars2=chars2))
 
-def validate_domain_name(domain_name, allow_underscore=False):
+
+def validate_domain_name(domain_name, allow_underscore=False, allow_slash=False):
     if domain_name.endswith('.'):
         domain_name = domain_name[:-1]
 
     domain_name = domain_name.split(".")
 
     # apply DNS name validator to every name part
-    map(lambda label:validate_dns_label(label,allow_underscore), domain_name)
+    map(lambda label:validate_dns_label(label, allow_underscore, allow_slash), domain_name)
 
-    if not domain_name[-1].isalpha():
-        # see RFC 1123
-        raise ValueError(_('top level domain label must be alphabetic'))
 
 def validate_zonemgr(zonemgr):
+    assert isinstance(zonemgr, DNSName)
+    assert zonemgr.is_absolute()
     """ See RFC 1033, 1035 """
-    regex_local_part = re.compile(r'^[a-z0-9]([a-z0-9-_]?[a-z0-9])*$',
-                                    re.IGNORECASE)
-    local_part_errmsg = _('mail account may only include letters, numbers, -, _ and a dot. There may not be consecutive -, _ and . characters. Its parts may not start or end with - or _')
-    local_part_sep = '.'
-    local_part = None
-    domain = None
-
-    if len(zonemgr) > 255:
-        raise ValueError(_('cannot be longer that 255 characters'))
-
-    if zonemgr.endswith('.'):
-        zonemgr = zonemgr[:-1]
-
-    if zonemgr.count('@') == 1:
-        local_part, dot, domain = zonemgr.partition('@')
-    elif zonemgr.count('@') > 1:
+    if any('@' in label for label in zonemgr.labels):
         raise ValueError(_('too many \'@\' characters'))
-    else:
-        last_fake_sep = zonemgr.rfind('\\.')
-        if last_fake_sep != -1: # there is a 'fake' local-part/domain separator
-            local_part_sep = '\\.'
-            sep = zonemgr.find('.', last_fake_sep+2)
-            if sep != -1:
-                local_part = zonemgr[:sep]
-                domain = zonemgr[sep+1:]
-        else:
-            local_part, dot, domain = zonemgr.partition('.')
-
-    if not domain:
+    if len(zonemgr.labels) < 3:
         raise ValueError(_('missing address domain'))
-
-    validate_domain_name(domain)
-
-    if not local_part:
+    if not zonemgr.labels[0]:
         raise ValueError(_('missing mail account'))
 
-    if not all(regex_local_part.match(part) for part in \
-               local_part.split(local_part_sep)):
-        raise ValueError(local_part_errmsg)
+def validate_zonemgr_str(zonemgr):
+    zonemgr = normalize_zonemgr(zonemgr)
+    zonemgr = DNSName(zonemgr).make_absolute()
+    return validate_zonemgr(zonemgr)
 
-def validate_hostname(hostname, check_fqdn=True, allow_underscore=False):
+def validate_hostname(hostname, check_fqdn=True, allow_underscore=False, allow_slash=False):
     """ See RFC 952, 1123
 
     :param hostname Checked value
@@ -287,9 +305,9 @@ def validate_hostname(hostname, check_fqdn=True, allow_underscore=False):
     if '.' not in hostname:
         if check_fqdn:
             raise ValueError(_('not fully qualified'))
-        validate_dns_label(hostname,allow_underscore)
+        validate_dns_label(hostname, allow_underscore, allow_slash)
     else:
-        validate_domain_name(hostname,allow_underscore)
+        validate_domain_name(hostname, allow_underscore, allow_slash)
 
 def normalize_sshpubkey(value):
     return SSHPublicKey(value).openssh()
@@ -314,7 +332,7 @@ def convert_sshpubkey_post(ldap, dn, entry_attrs):
         pubkeys = entry_attrs['ipasshpubkey']
     else:
         old_entry_attrs = ldap.get_entry(dn, ['ipasshpubkey'])
-        pubkeys = old_entry_attrs[1].get('ipasshpubkey')
+        pubkeys = old_entry_attrs.get('ipasshpubkey')
     if not pubkeys:
         return
 
@@ -514,19 +532,15 @@ def get_dns_reverse_zone_update_policy(realm, reverse_zone, rrtypes=('PTR',)):
 
 # dictionary of valid reverse zone -> number of address components
 REVERSE_DNS_ZONES = {
-    '.in-addr.arpa.' : 4,
-    '.ip6.arpa.' : 32,
+    DNSName.ip4_rev_zone : 4,
+    DNSName.ip6_rev_zone : 32,
 }
 
 def zone_is_reverse(zone_name):
-    zone_name = normalize_zone(zone_name)
-    if any(zone_name.endswith(name) for name in REVERSE_DNS_ZONES):
-        return True
-
-    return False
+    return DNSName(zone_name).is_reverse()
 
 def get_reverse_zone_default(ip_address):
-    ip = netaddr.IPAddress(ip_address)
+    ip = netaddr.IPAddress(str(ip_address))
     items = ip.reverse_dns.split('.')
 
     if ip.version == 4:
@@ -542,3 +556,9 @@ def validate_rdn_param(ugettext, value):
     except Exception, e:
         return str(e)
     return None
+
+def validate_hostmask(ugettext, hostmask):
+    try:
+        netaddr.IPNetwork(hostmask)
+    except (ValueError, AddrFormatError):
+        return _('invalid hostmask')
