@@ -17,13 +17,6 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
-SHARE_DIR = "/usr/share/ipa/"
-PLUGINS_SHARE_DIR = "/usr/share/ipa/plugins"
-
-GEN_PWD_LEN = 12
-
-IPA_BASEDN_INFO = 'ipa v2.0'
-
 import string
 import tempfile
 import subprocess
@@ -42,14 +35,23 @@ import datetime
 import netaddr
 import time
 import krbV
+import pwd
 from dns import resolver, rdatatype
 from dns.exception import DNSException
 
 from ipapython.ipa_log_manager import *
 from ipapython import ipavalidate
 from ipapython import config
+from ipaplatform.paths import paths
 from ipapython.dn import DN
 
+SHARE_DIR = paths.USR_SHARE_IPA_DIR
+PLUGINS_SHARE_DIR = paths.IPA_PLUGINS
+
+GEN_PWD_LEN = 12
+
+IPA_BASEDN_INFO = 'ipa v2.0'
+
 try:
     from subprocess import CalledProcessError
 except ImportError:
@@ -142,7 +144,7 @@ class CheckedIPAddress(netaddr.IPAddress):
             elif addr.version == 6:
                 family = 'inet6'
 
-            ipresult = run(['/sbin/ip', '-family', family, '-oneline', 'address', 'show'])
+            ipresult = run([paths.IP, '-family', family, '-oneline', 'address', 'show'])
             lines = ipresult[0].split('\n')
             for line in lines:
                 fields = line.split()
@@ -246,29 +248,35 @@ def shell_quote(string):
     return "'" + string.replace("'", "'\\''") + "'"
 
 def run(args, stdin=None, raiseonerr=True,
-        nolog=(), env=None, capture_output=True, skip_output=False, cwd=None):
+        nolog=(), env=None, capture_output=True, skip_output=False, cwd=None,
+        runas=None):
     """
     Execute a command and return stdin, stdout and the process return code.
 
-    args is a list of arguments for the command
+    :param args: List of arguments for the command
+    :param stdin: Optional input to the command
+    :param raiseonerr: If True, raises an exception if the return code is
+        not zero
+    :param nolog: Tuple of strings that shouldn't be logged, like passwords.
+        Each tuple consists of a string to be replaced by XXXXXXXX.
 
-    stdin is used if you want to pass input to the command
+        Example:
+        We have a command
+            [paths.SETPASSWD, '--password', 'Secret123', 'someuser']
+        and we don't want to log the password so nolog would be set to:
+        ('Secret123',)
+        The resulting log output would be:
 
-    raiseonerr raises an exception if the return code is not zero
+        /usr/bin/setpasswd --password XXXXXXXX someuser
 
-    nolog is a tuple of strings that shouldn't be logged, like passwords.
-    Each tuple consists of a string to be replaced by XXXXXXXX.
-
-    For example, the command ['/usr/bin/setpasswd', '--password', 'Secret123', 'someuser']
-
-    We don't want to log the password so nolog would be set to:
-    ('Secret123',)
-
-    The resulting log output would be:
-
-    /usr/bin/setpasswd --password XXXXXXXX someuser
-
-    If an value isn't found in the list it is silently ignored.
+        If a value isn't found in the list it is silently ignored.
+    :param env: Dictionary of environment variables passed to the command.
+        When None, current environment is copied
+    :param capture_output: Capture stderr and stdout
+    :param skip_output: Redirect the output to /dev/null and do not capture it
+    :param cwd: Current working directory
+    :param runas: Name of a user that the command shold be run as. The spawned
+        process will have both real and effective UID and GID set.
     """
     p_in = None
     p_out = None
@@ -289,18 +297,28 @@ def run(args, stdin=None, raiseonerr=True,
     if stdin:
         p_in = subprocess.PIPE
     if skip_output:
-        p_out = p_err = open('/dev/null', 'w')
+        p_out = p_err = open(paths.DEV_NULL, 'w')
     elif capture_output:
         p_out = subprocess.PIPE
         p_err = subprocess.PIPE
 
-    arg_string = nolog_replace(' '.join(args), nolog)
+    arg_string = nolog_replace(' '.join(shell_quote(a) for a in args), nolog)
     root_logger.debug('Starting external process')
     root_logger.debug('args=%s' % arg_string)
 
+    preexec_fn = None
+    if runas is not None:
+        pent = pwd.getpwnam(runas)
+        root_logger.debug('runas=%s (UID %d, GID %s)', runas,
+            pent.pw_uid, pent.pw_gid)
+
+        preexec_fn = lambda: (os.setregid(pent.pw_gid, pent.pw_gid),
+                              os.setreuid(pent.pw_uid, pent.pw_uid))
+
     try:
         p = subprocess.Popen(args, stdin=p_in, stdout=p_out, stderr=p_err,
-                             close_fds=True, env=env, cwd=cwd)
+                             close_fds=True, env=env, cwd=cwd,
+                             preexec_fn=preexec_fn)
         stdout,stderr = p.communicate(stdin)
         stdout,stderr = str(stdout), str(stderr)    # Make pylint happy
     except KeyboardInterrupt:
@@ -394,7 +412,7 @@ def encrypt_file(source, dest, password, workdir = None):
             #give gpg a fake dir so that we can leater remove all
             #the cruft when we clean up the tempdir
             os.mkdir(gpgdir)
-            args = ['/usr/bin/gpg-agent', '--batch', '--homedir', gpgdir, '--daemon', '/usr/bin/gpg', '--batch', '--homedir', gpgdir, '--passphrase-fd', '0', '--yes', '--no-tty', '-o', dest, '-c', source]
+            args = [paths.GPG_AGENT, '--batch', '--homedir', gpgdir, '--daemon', paths.GPG, '--batch', '--homedir', gpgdir, '--passphrase-fd', '0', '--yes', '--no-tty', '-o', dest, '-c', source]
             run(args, password, skip_output=True)
         except:
             raise
@@ -424,7 +442,7 @@ def decrypt_file(source, dest, password, workdir = None):
             #give gpg a fake dir so that we can leater remove all
             #the cruft when we clean up the tempdir
             os.mkdir(gpgdir)
-            args = ['/usr/bin/gpg-agent', '--batch', '--homedir', gpgdir, '--daemon', '/usr/bin/gpg', '--batch', '--homedir', gpgdir, '--passphrase-fd', '0', '--yes', '--no-tty', '-o', dest, '-d', source]
+            args = [paths.GPG_AGENT, '--batch', '--homedir', gpgdir, '--daemon', paths.GPG, '--batch', '--homedir', gpgdir, '--passphrase-fd', '0', '--yes', '--no-tty', '-o', dest, '-d', source]
             run(args, password, skip_output=True)
         except:
             raise
@@ -444,27 +462,60 @@ class CIDict(dict):
     If you extend UserDict, isinstance(foo, dict) returns false.
     """
 
-    def __init__(self, default=None):
+    def __init__(self, default=None, **kwargs):
         super(CIDict, self).__init__()
-        self._keys = {}
-        self.update(default or {})
+        self._keys = {}  # mapping of lowercased keys to proper case
+        if default:
+            self.update(default)
+        if kwargs:
+            self.update(kwargs)
 
     def __getitem__(self, key):
         return super(CIDict, self).__getitem__(key.lower())
 
-    def __setitem__(self, key, value):
+    def __setitem__(self, key, value, seen_keys=None):
+        """cidict[key] = value
+
+        The ``seen_keys`` argument is used by ``update()`` to keep track of
+        duplicate keys. It should be an initially empty set that is
+        passed to all calls to __setitem__ that should not set duplicate keys.
+        """
         lower_key = key.lower()
+        if seen_keys is not None:
+            if lower_key in seen_keys:
+                raise ValueError('Duplicate key in update: %s' % key)
+            seen_keys.add(lower_key)
         self._keys[lower_key] = key
         return super(CIDict, self).__setitem__(lower_key, value)
 
     def __delitem__(self, key):
         lower_key = key.lower()
         del self._keys[lower_key]
-        return super(CIDict, self).__delitem__(key.lower())
+        return super(CIDict, self).__delitem__(lower_key)
 
-    def update(self, dict):
-        for key in dict.keys():
-            self[key] = dict[key]
+    def update(self, new=None, **kwargs):
+        """Update self from dict/iterable new and kwargs
+
+        Functions like ``dict.update()``.
+
+        Neither ``new`` nor ``kwargs`` may contain two keys that only differ in
+        case, as this situation would result in loss of data.
+        """
+        seen = set()
+        if new:
+            try:
+                keys = new.keys
+            except AttributeError:
+                self.update(dict(new))
+            else:
+                for key in keys():
+                    self.__setitem__(key, new[key], seen)
+        seen = set()
+        for key, value in kwargs.iteritems():
+            self.__setitem__(key, value, seen)
+
+    def __contains__(self, key):
+        return super(CIDict, self).__contains__(key.lower())
 
     def has_key(self, key):
         return super(CIDict, self).has_key(key.lower())
@@ -475,26 +526,30 @@ class CIDict(dict):
         except KeyError:
             return failobj
 
+    def __iter__(self):
+        return self._keys.itervalues()
+
     def keys(self):
-        return self._keys.values()
+        return list(self.iterkeys())
 
     def items(self):
-        result = []
-        for k in self._keys.values():
-            result.append((k, self[k]))
-        return result
+        return list(self.iteritems())
+
+    def values(self):
+        return list(self.itervalues())
 
     def copy(self):
-        copy = {}
-        for k in self._keys.values():
-            copy[k] = self[k]
-        return copy
+        """Returns a shallow copy of this CIDict"""
+        return CIDict(self.items())
 
     def iteritems(self):
-        return self.copy().iteritems()
+        return ((k, self[k]) for k in self._keys.itervalues())
 
     def iterkeys(self):
-        return self.copy().iterkeys()
+        return self._keys.itervalues()
+
+    def itervalues(self):
+        return (v for k, v in self.iteritems())
 
     def setdefault(self, key, value=None):
         try:
@@ -520,6 +575,19 @@ class CIDict(dict):
 
         return (key, value)
 
+    def clear(self):
+        self._keys.clear()
+        return super(CIDict, self).clear()
+
+    def viewitems(self):
+        raise NotImplementedError('CIDict.viewitems is not implemented')
+
+    def viewkeys(self):
+        raise NotImplementedError('CIDict.viewkeys is not implemented')
+
+    def viewvvalues(self):
+        raise NotImplementedError('CIDict.viewvvalues is not implemented')
+
 
 class GeneralizedTimeZone(datetime.tzinfo):
     """This class is a basic timezone wrapper for the offset specified
@@ -845,7 +913,7 @@ def get_ipa_basedn(conn):
     contexts = entry['namingcontexts']
     if 'defaultnamingcontext' in entry:
         # If there is a defaultNamingContext examine that one first
-        default = entry.single_value('defaultnamingcontext')
+        default = entry.single_value['defaultnamingcontext']
         if default in contexts:
             contexts.remove(default)
         contexts.insert(0, default)
@@ -858,7 +926,7 @@ def get_ipa_basedn(conn):
             root_logger.debug("LDAP server did not return info attribute to "
                               "check for IPA version")
             continue
-        info = entry.single_value('info').lower()
+        info = entry.single_value['info'].lower()
         if info != IPA_BASEDN_INFO:
             root_logger.debug("Detected IPA server version (%s) did not match the client (%s)" \
                 % (info, IPA_BASEDN_INFO))
@@ -1062,24 +1130,13 @@ def backup_config_and_replace_variables(
     return old_values
 
 
-def utf8_encode_value(value):
-    if isinstance(value, unicode):
-        return value.encode('utf-8')
-    return value
-
-
-def utf8_encode_values(values):
-    if isinstance(values, list) or isinstance(values, tuple):
-        return map(utf8_encode_value, values)
-    else:
-        return utf8_encode_value(values)
-
 def wait_for_open_ports(host, ports, timeout=0):
     """
     Wait until the specified port(s) on the remote host are open. Timeout
     in seconds may be specified to limit the wait. If the timeout is
     exceeded, socket.timeout exception is raised.
     """
+    timeout = float(timeout)
     if not isinstance(ports, (tuple, list)):
         ports = [ports]
 
@@ -1101,6 +1158,7 @@ def wait_for_open_socket(socket_name, timeout=0):
     Wait until the specified socket on the local host is open. Timeout
     in seconds may be specified to limit the wait.
     """
+    timeout = float(timeout)
     op_timeout = time.time() + timeout
 
     while True:
@@ -1177,6 +1235,6 @@ def restore_hostname(statestore):
     system_hostname = socket.gethostname()
     if old_hostname is not None and old_hostname != system_hostname:
         try:
-            run(['/bin/hostname', old_hostname])
+            run([paths.BIN_HOSTNAME, old_hostname])
         except CalledProcessError, e:
             print >>sys.stderr, "Failed to set this machine hostname back to %s: %s" % (old_hostname, str(e))
