@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/python2
 # Copyright (C) 2007  Red Hat
 # see file 'COPYING' for use and warranty information
 #
@@ -52,7 +52,7 @@ def setup_package():
     try:
         setup(
             name = "ipapython",
-            version = "3.3.4",
+            version = "4.0.5",
             license = "GPL",
             author = "Karl MacMillan, et.al.",
             author_email = "kmacmill@redhat.com",
@@ -65,12 +65,7 @@ def setup_package():
             classifiers=filter(None, CLASSIFIERS.split('\n')),
             platforms = ["Linux", "Solaris", "Unix"],
             package_dir = {'ipapython': ''},
-            packages = [ "ipapython",
-                         "ipapython.platform",
-                         "ipapython.platform.base",
-                         "ipapython.platform.fedora16",
-                         "ipapython.platform.fedora18",
-                         "ipapython.platform.redhat" ],
+            packages = [ "ipapython" ],
         )
     finally:
         del sys.path[0]
