@@ -32,9 +32,10 @@ import random
 import string
 
 from ipapython import ipautil
-from ipapython import services as ipaservices
+from ipaplatform.tasks import tasks
+from ipaplatform.paths import paths
 
-SYSRESTORE_PATH = "/tmp"
+SYSRESTORE_PATH = paths.TMP
 SYSRESTORE_INDEXFILE = "sysrestore.index"
 SYSRESTORE_STATEFILE = "sysrestore.state"
 
@@ -190,7 +191,7 @@ class FileStore:
         os.chown(path, int(uid), int(gid))
         os.chmod(path, int(mode))
 
-        ipaservices.restore_context(path)
+        tasks.restore_context(path)
 
         del self.files[filename]
         self.save()
@@ -221,7 +222,7 @@ class FileStore:
             os.chown(path, int(uid), int(gid))
             os.chmod(path, int(mode))
 
-            ipaservices.restore_context(path)
+            tasks.restore_context(path)
 
 	#force file to be deleted
         self.files = {}
