@@ -37,9 +37,10 @@ from ipapython.dn import DN
 from ipapython import sysrestore
 from ipapython import ipautil
 from ipapython.ipa_log_manager import *
-from ipapython import services as ipaservices
 
 import ipaclient.ipachangeconf
+from ipaplatform import services
+from ipaplatform.paths import paths
 
 
 ALLOWED_NETBIOS_CHARS = string.ascii_uppercase + string.digits
@@ -60,7 +61,7 @@ and re-run ipa-adtrust-instal again afterwards.
 """
 
 def check_inst():
-    for smbfile in ['/usr/sbin/smbd', '/usr/bin/net']:
+    for smbfile in [paths.SMBD, paths.NET]:
         if not os.path.exists(smbfile):
             print "%s was not found on this system" % smbfile
             print "Please install the 'samba' packages and " \
@@ -73,7 +74,7 @@ def check_inst():
 
 def ipa_smb_conf_exists():
     try:
-        conf_fd = open('/etc/samba/smb.conf', 'r')
+        conf_fd = open(paths.SMB_CONF, 'r')
     except IOError, err:
         if err.errno == errno.ENOENT:
             return False
@@ -134,7 +135,7 @@ class ADTRUSTInstance(service.Service):
         if fstore:
             self.fstore = fstore
         else:
-            self.fstore = sysrestore.FileStore('/var/lib/ipa/sysrestore')
+            self.fstore = sysrestore.FileStore(paths.SYSRESTORE)
 
         self.__setup_default_attributes()
 
@@ -145,8 +146,8 @@ class ADTRUSTInstance(service.Service):
         """
 
         # Constants
-        self.smb_conf = "/etc/samba/smb.conf"
-        self.samba_keytab = "/etc/samba/samba.keytab"
+        self.smb_conf = paths.SMB_CONF
+        self.samba_keytab = paths.SAMBA_KEYTAB
         self.selinux_booleans = ["samba_portmapper"]
         self.cifs_hosts = []
 
@@ -203,7 +204,7 @@ class ADTRUSTInstance(service.Service):
             self.print_msg("Samba domain object not found")
             return
 
-        dom_sid = dom_entry.single_value(self.ATTR_SID, None)
+        dom_sid = dom_entry.single_value.get(self.ATTR_SID)
         if not dom_sid:
             self.print_msg("Samba domain object does not have a SID")
             return
@@ -220,7 +221,7 @@ class ADTRUSTInstance(service.Service):
             self.print_msg("IPA admin group object not found")
             return
 
-        if admin_entry.single_value(self.ATTR_SID, None):
+        if admin_entry.single_value.get(self.ATTR_SID):
             self.print_msg("Admin SID already set, nothing to do")
         else:
             try:
@@ -230,7 +231,7 @@ class ADTRUSTInstance(service.Service):
             except:
                 self.print_msg("Failed to modify IPA admin object")
 
-        if admin_group_entry.single_value(self.ATTR_SID, None):
+        if admin_group_entry.single_value.get(self.ATTR_SID):
             self.print_msg("Admin group SID already set, nothing to do")
         else:
             try:
@@ -261,7 +262,7 @@ class ADTRUSTInstance(service.Service):
             self.print_msg("Samba domain object not found")
             return
 
-        if dom_entry.single_value(self.ATTR_FALLBACK_GROUP, None):
+        if dom_entry.single_value.get(self.ATTR_FALLBACK_GROUP):
             self.print_msg("Fallback group already set, nothing to do")
             return
 
@@ -305,8 +306,8 @@ class ADTRUSTInstance(service.Service):
 
             # Filter out ranges where RID base is already set
             no_rid_base_set = lambda r: not any((
-                                  r.single_value('ipaBaseRID', None),
-                                  r.single_value('ipaSecondaryBaseRID', None)))
+                                  r.single_value.get('ipaBaseRID'),
+                                  r.single_value.get('ipaSecondaryBaseRID')))
 
             ranges_with_no_rid_base = filter(no_rid_base_set, ranges)
 
@@ -323,7 +324,7 @@ class ADTRUSTInstance(service.Service):
 
             # Abort if RID bases are too close
             local_range = ranges_with_no_rid_base[0]
-            size = local_range.single_value('ipaIDRangeSize', None)
+            size = local_range.single_value.get('ipaIDRangeSize')
 
             if abs(self.rid_base - self.secondary_rid_base) > size:
                 self.print_msg("Primary and secondary RID base are too close. "
@@ -407,6 +408,7 @@ class ADTRUSTInstance(service.Service):
         conf_fd = open(self.smb_conf, "w")
         conf_fd.write('### Added by IPA Installer ###\n')
         conf_fd.write('[global]\n')
+        conf_fd.write('debug pid = yes\n')
         conf_fd.write('config backend = registry\n')
         conf_fd.close()
 
@@ -472,7 +474,7 @@ class ADTRUSTInstance(service.Service):
             members = current.get('memberPrincipal', [])
             if not(self.cifs_principal in members):
                 current["memberPrincipal"] = members + [self.cifs_principal]
-                self.admin_conn.update_entry(targets_dn, current)
+                self.admin_conn.update_entry(current)
             else:
                 self.print_msg('cifs principal already targeted, nothing to do.')
         except errors.NotFound:
@@ -485,7 +487,7 @@ class ADTRUSTInstance(service.Service):
         os.write(tmp_fd, conf)
         os.close(tmp_fd)
 
-        args = ["/usr/bin/net", "conf", "import", tmp_name]
+        args = [paths.NET, "conf", "import", tmp_name]
 
         try:
             ipautil.run(args)
@@ -495,28 +497,31 @@ class ADTRUSTInstance(service.Service):
     def __setup_principal(self):
         try:
             api.Command.service_add(unicode(self.cifs_principal))
-            # Add the principal to the 'adtrust agents' group
-            # as 389-ds only operates with GroupOfNames, we have to use
-            # the principal's proper dn as defined in self.cifs_agent
-            try:
-                current = self.admin_conn.get_entry(self.smb_dn)
-                members = current.get('member', [])
-                if not(self.cifs_agent in members):
-                    current["member"] = members + [self.cifs_agent]
-                    self.admin_conn.update_entry(self.smb_dn, current)
-            except errors.NotFound:
-                entry = self.admin_conn.make_entry(
-                    self.smb_dn,
-                    objectclass=["top", "GroupOfNames"],
-                    cn=[self.smb_dn['cn']],
-                    member=[self.cifs_agent],
-                )
-                self.admin_conn.add_entry(entry)
-        except Exception:
+        except errors.DuplicateEntry:
             # CIFS principal already exists, it is not the first time
             # adtrustinstance is managed
             # That's fine, we we'll re-extract the key again.
             pass
+        except Exception, e:
+            self.print_msg("Cannot add CIFS service: %s" % e)
+
+        # Add the principal to the 'adtrust agents' group
+        # as 389-ds only operates with GroupOfNames, we have to use
+        # the principal's proper dn as defined in self.cifs_agent
+        try:
+            current = self.admin_conn.get_entry(self.smb_dn)
+            members = current.get('member', [])
+            if not(self.cifs_agent in members):
+                current["member"] = members + [self.cifs_agent]
+                self.admin_conn.update_entry(current)
+        except errors.NotFound:
+            entry = self.admin_conn.make_entry(
+                self.smb_dn,
+                objectclass=["top", "GroupOfNames"],
+                cn=[self.smb_dn['cn']],
+                member=[self.cifs_agent],
+            )
+            self.admin_conn.add_entry(entry)
 
         self.clean_samba_keytab()
 
@@ -608,8 +613,8 @@ class ADTRUSTInstance(service.Service):
     def __configure_selinux_for_smbd(self):
         selinux = False
         try:
-            if (os.path.exists('/usr/sbin/selinuxenabled')):
-                ipautil.run(["/usr/sbin/selinuxenabled"])
+            if (os.path.exists(paths.SELINUXENABLED)):
+                ipautil.run([paths.SELINUXENABLED])
                 selinux = True
         except ipautil.CalledProcessError:
             # selinuxenabled returns 1 if not enabled
@@ -620,7 +625,7 @@ class ADTRUSTInstance(service.Service):
             sebools = []
             for var in self.selinux_booleans:
                 try:
-                    (stdout, stderr, returncode) = ipautil.run(["/usr/sbin/getsebool", var])
+                    (stdout, stderr, returncode) = ipautil.run([paths.GETSEBOOL, var])
                     if stdout and not stderr and returncode == 0:
                         self.backup_state(var, stdout.split()[2])
                         sebools.append(var)
@@ -629,7 +634,7 @@ class ADTRUSTInstance(service.Service):
 
             if sebools:
                 bools = [var + "=true" for var in sebools]
-                args = ["/usr/sbin/setsebool", "-P"]
+                args = [paths.SETSEBOOL, "-P"]
                 args.extend(bools);
                 try:
                     ipautil.run(args)
@@ -665,7 +670,7 @@ class ADTRUSTInstance(service.Service):
                 {'name':'realms', 'type':'section', 'action':'set',
                  'value':ropts}]
 
-        krbconf.changeConf("/etc/krb5.conf", opts)
+        krbconf.changeConf(paths.KRB5_CONF, opts)
 
     def __update_krb5_conf(self):
         """
@@ -673,7 +678,7 @@ class ADTRUSTInstance(service.Service):
         """
 
         try:
-            krb5conf = open("/etc/krb5.conf", 'r')
+            krb5conf = open(paths.KRB5_CONF, 'r')
         except IOError, e:
             self.print_msg("Cannot open /etc/krb5.conf (%s)\n" % str(e))
             return
@@ -701,7 +706,7 @@ class ADTRUSTInstance(service.Service):
             if len(res) > 1:
                 # there are other CIFS services defined, we are not alone
                 for entry in res:
-                    managedBy = entry.single_value('managedBy', None)
+                    managedBy = entry.single_value.get('managedBy')
                     if managedBy:
                         fqdn = DN(managedBy)['fqdn']
                         if fqdn != unicode(self.fqdn):
@@ -723,28 +728,28 @@ class ADTRUSTInstance(service.Service):
                 lookup_nsswitch = current.get(lookup_nsswitch_name, [])
                 if not(config[1] in lookup_nsswitch):
                     current[lookup_nsswitch_name] = [config[1]]
-                    self.admin_conn.update_entry(entry_dn, current)
+                    self.admin_conn.update_entry(current)
         except Exception, e:
             root_logger.critical("Enabling nsswitch support in slapi-nis failed with error '%s'" % e)
 
     def __start(self):
         try:
             self.start()
-            ipaservices.service('winbind').start()
+            services.service('winbind').start()
         except:
             root_logger.critical("CIFS services failed to start")
 
     def __stop(self):
         self.backup_state("running", self.is_running())
         try:
-            ipaservices.service('winbind').stop()
+            services.service('winbind').stop()
             self.stop()
         except:
             pass
 
     def __restart_dirsrv(self):
         try:
-            ipaservices.knownservices.dirsrv.restart()
+            services.knownservices.dirsrv.restart()
         except:
             pass
 
@@ -813,7 +818,7 @@ class ADTRUSTInstance(service.Service):
             raise ValueError("No local ID range and no admins group found.\n" \
                              "Add local ID range manually and try again!")
 
-        base_id = int(entry.single_value('gidNumber'))
+        base_id = int(entry.single_value['gidNumber'])
         id_range_size = 200000
 
         id_filter = "(&" \
@@ -889,12 +894,15 @@ class ADTRUSTInstance(service.Service):
         self.restore_state("running")
         self.restore_state("enabled")
 
+        winbind = services.service("winbind")
         # Always try to stop and disable smb service, since we do not leave
         # working configuration after uninstall
         try:
             self.stop()
             self.disable()
-        except:
+            winbind.stop()
+            winbind.disable()
+        except Exception:
             pass
 
         # Since we do not guarantee restoring back to working samba state,
@@ -905,20 +913,20 @@ class ADTRUSTInstance(service.Service):
             sebool_state = self.restore_state(var)
             if not sebool_state is None:
                 try:
-                    ipautil.run(["/usr/sbin/setsebool",
+                    ipautil.run([paths.SETSEBOOL,
                                  "-P", var, sebool_state])
-                except:
+                except Exception:
                     self.print_msg(SELINUX_WARNING % dict(var=var))
 
         # Remove samba's credentials cache
-        krb5cc_samba = '/var/run/samba/krb5cc_samba'
+        krb5cc_samba = paths.KRB5CC_SAMBA
         installutils.remove_file(krb5cc_samba)
 
         # Remove samba's configuration file
         installutils.remove_file(self.smb_conf)
 
         # Remove samba's persistent and temporary tdb files
-        tdb_files = [tdb_file for tdb_file in os.listdir("/var/lib/samba/")
+        tdb_files = [tdb_file for tdb_file in os.listdir(paths.SAMBA_DIR)
                                            if tdb_file.endswith(".tdb")]
         for tdb_file in tdb_files:
             installutils.remove_file(tdb_file)
