@@ -22,6 +22,7 @@ import os
 import pwd
 import netaddr
 import re
+import time
 
 import ldap
 
@@ -35,12 +36,14 @@ from ipapython.ipa_log_manager import *
 from ipapython.dn import DN
 import ipalib
 from ipalib import api, errors
-from ipalib.util import (validate_zonemgr, normalize_zonemgr,
+from ipaplatform.paths import paths
+from ipalib.util import (validate_zonemgr_str, normalize_zonemgr,
         get_dns_forward_zone_update_policy, get_dns_reverse_zone_update_policy,
         normalize_zone, get_reverse_zone_default, zone_is_reverse)
+from ipalib.constants import CACERT
 
-NAMED_CONF = '/etc/named.conf'
-RESOLV_CONF = '/etc/resolv.conf'
+NAMED_CONF = paths.NAMED_CONF
+RESOLV_CONF = paths.RESOLV_CONF
 
 named_conf_section_ipa_start_re = re.compile('\s*dynamic-db\s+"ipa"\s+{')
 named_conf_section_options_start_re = re.compile('\s*options\s+{')
@@ -49,19 +52,22 @@ named_conf_arg_ipa_re = re.compile(r'(?P<indent>\s*)arg\s+"(?P<name>\S+)\s(?P<va
 named_conf_arg_options_re = re.compile(r'(?P<indent>\s*)(?P<name>\S+)\s+"(?P<value>[^"]+)"\s*;')
 named_conf_arg_ipa_template = "%(indent)sarg \"%(name)s %(value)s\";\n"
 named_conf_arg_options_template = "%(indent)s%(name)s \"%(value)s\";\n"
+# non string args for options section
+named_conf_arg_options_re_nonstr = re.compile(r'(?P<indent>\s*)(?P<name>\S+)\s+(?P<value>[^"]+)\s*;')
+named_conf_arg_options_template_nonstr = "%(indent)s%(name)s %(value)s;\n"
 
 def check_inst(unattended):
     has_bind = True
     # So far this file is always present in both RHEL5 and Fedora if all the necessary
     # bind packages are installed (RHEL5 requires also the pkg: caching-nameserver)
-    if not os.path.exists('/etc/named.rfc1912.zones'):
+    if not os.path.exists(paths.NAMED_RFC1912_ZONES):
         print "BIND was not found on this system"
         print "Please install the 'bind' package and start the installation again"
         has_bind = False
 
     # Also check for the LDAP BIND plug-in
-    if not os.path.exists('/usr/lib/bind/ldap.so') and \
-       not os.path.exists('/usr/lib64/bind/ldap.so'):
+    if not os.path.exists(paths.BIND_LDAP_SO) and \
+       not os.path.exists(paths.BIND_LDAP_SO_64):
         print "The BIND LDAP plug-in was not found on this system"
         print "Please install the 'bind-dyndb-ldap' package and start the installation again"
         has_bind = False
@@ -92,14 +98,21 @@ def named_conf_exists():
 
 NAMED_SECTION_OPTIONS = "options"
 NAMED_SECTION_IPA = "ipa"
-def named_conf_get_directive(name, section=NAMED_SECTION_IPA):
-    """Get a configuration option in bind-dyndb-ldap section of named.conf"""
+def named_conf_get_directive(name, section=NAMED_SECTION_IPA, str_val=True):
+    """Get a configuration option in bind-dyndb-ldap section of named.conf
+
+    :str_val - set to True if directive value is string
+        (only for NAMED_SECTION_OPTIONS)
+    """
     if section == NAMED_SECTION_IPA:
         named_conf_section_start_re = named_conf_section_ipa_start_re
         named_conf_arg_re = named_conf_arg_ipa_re
     elif section == NAMED_SECTION_OPTIONS:
         named_conf_section_start_re = named_conf_section_options_start_re
-        named_conf_arg_re = named_conf_arg_options_re
+        if str_val:
+            named_conf_arg_re = named_conf_arg_options_re
+        else:
+            named_conf_arg_re = named_conf_arg_options_re_nonstr
     else:
         raise NotImplementedError('Section "%s" is not supported' % section)
 
@@ -119,7 +132,8 @@ def named_conf_get_directive(name, section=NAMED_SECTION_IPA):
                 if match and name == match.group('name'):
                     return match.group('value')
 
-def named_conf_set_directive(name, value, section=NAMED_SECTION_IPA):
+def named_conf_set_directive(name, value, section=NAMED_SECTION_IPA,
+                             str_val=True):
     """
     Set configuration option in bind-dyndb-ldap section of named.conf.
 
@@ -128,6 +142,9 @@ def named_conf_set_directive(name, value, section=NAMED_SECTION_IPA):
 
     If the value is set to None, the configuration option is removed
     from named.conf.
+
+    :str_val - set to True if directive value is string
+        (only for NAMED_SECTION_OPTIONS)
     """
     new_lines = []
 
@@ -137,8 +154,12 @@ def named_conf_set_directive(name, value, section=NAMED_SECTION_IPA):
         named_conf_arg_template = named_conf_arg_ipa_template
     elif section == NAMED_SECTION_OPTIONS:
         named_conf_section_start_re = named_conf_section_options_start_re
-        named_conf_arg_re = named_conf_arg_options_re
-        named_conf_arg_template = named_conf_arg_options_template
+        if str_val:
+            named_conf_arg_re = named_conf_arg_options_re
+            named_conf_arg_template = named_conf_arg_options_template
+        else:
+            named_conf_arg_re = named_conf_arg_options_re_nonstr
+            named_conf_arg_template = named_conf_arg_options_template_nonstr
     else:
         raise NotImplementedError('Section "%s" is not supported' % section)
 
@@ -206,7 +227,7 @@ def dns_container_exists(fqdn, suffix, dm_password=None, ldapi=False, realm=None
         if ldapi:
             conn = ipaldap.IPAdmin(host=fqdn, ldapi=True, realm=realm)
         else:
-            conn = ipaldap.IPAdmin(host=fqdn, port=636, cacert=service.CACERT)
+            conn = ipaldap.IPAdmin(host=fqdn, port=636, cacert=CACERT)
 
         if dm_password:
             conn.do_simple_bind(bindpw=dm_password)
@@ -382,7 +403,7 @@ def zonemgr_callback(option, opt_str, value, parser):
     """
     # validate the value first
     try:
-        validate_zonemgr(value)
+        validate_zonemgr_str(value)
     except ValueError, e:
         parser.error("invalid zonemgr: " + unicode(e))
 
@@ -457,7 +478,7 @@ class BindInstance(service.Service):
         if fstore:
             self.fstore = fstore
         else:
-            self.fstore = sysrestore.FileStore('/var/lib/ipa/sysrestore')
+            self.fstore = sysrestore.FileStore(paths.SYSRESTORE)
 
     suffix = ipautil.dn_attribute_property('_suffix')
 
@@ -522,6 +543,9 @@ class BindInstance(service.Service):
         if installutils.record_in_hosts(self.ip_address, self.fqdn) is None:
             installutils.add_record_to_hosts(self.ip_address, self.fqdn)
 
+        # Make sure generate-rndc-key.sh runs before named restart
+        self.step("generating rndc key file", self.__generate_rndc_key)
+
         if self.first_instance:
             self.step("adding DNS container", self.__setup_dns_container)
 
@@ -605,6 +629,35 @@ class BindInstance(service.Service):
 
     def __setup_dns_container(self):
         self._ldap_mod("dns.ldif", self.sub_dict)
+        self.__fix_dns_privilege_members()
+
+    def __fix_dns_privilege_members(self):
+        ldap = api.Backend.ldap2
+
+        cn = 'Update PBAC memberOf %s' % time.time()
+        task_dn = DN(('cn', cn), ('cn', 'memberof task'), ('cn', 'tasks'),
+                     ('cn', 'config'))
+        basedn = DN(api.env.container_privilege, api.env.basedn)
+        entry = ldap.make_entry(
+            task_dn,
+            objectclass=['top', 'extensibleObject'],
+            cn=[cn],
+            basedn=[basedn],
+            filter=['(objectclass=*)'],
+            ttl=[10])
+        ldap.add_entry(entry)
+
+        start_time = time.time()
+        while True:
+            try:
+                task = ldap.get_entry(task_dn)
+            except errors.NotFound:
+                break
+            if 'nstaskexitcode' in task:
+                break
+            time.sleep(1)
+            if time.time() > (start_time + 60):
+                raise errors.TaskTimeout(task='memberof', task_dn=task_dn)
 
     def __setup_zone(self):
         nameserver_ip_address = self.ip_address
@@ -754,8 +807,8 @@ class BindInstance(service.Service):
         installutils.kadmin_addprinc(dns_principal)
 
         # Store the keytab on disk
-        self.fstore.backup_file("/etc/named.keytab")
-        installutils.create_keytab("/etc/named.keytab", dns_principal)
+        self.fstore.backup_file(paths.NAMED_KEYTAB)
+        installutils.create_keytab(paths.NAMED_KEYTAB, dns_principal)
         p = self.move_service(dns_principal)
         if p is None:
             # the service has already been moved, perhaps we're doing a DNS reinstall
@@ -766,8 +819,8 @@ class BindInstance(service.Service):
 
         # Make sure access is strictly reserved to the named user
         pent = pwd.getpwnam(self.named_user)
-        os.chown("/etc/named.keytab", pent.pw_uid, pent.pw_gid)
-        os.chmod("/etc/named.keytab", 0400)
+        os.chown(paths.NAMED_KEYTAB, pent.pw_uid, pent.pw_gid)
+        os.chmod(paths.NAMED_KEYTAB, 0400)
 
         # modify the principal so that it is marked as an ipa service so that
         # it can host the memberof attribute, then also add it to the
@@ -810,11 +863,18 @@ class BindInstance(service.Service):
     def __setup_resolv_conf(self):
         self.fstore.backup_file(RESOLV_CONF)
         resolv_txt = "search "+self.domain+"\nnameserver "+self.ip_address+"\n"
-        resolv_fd = open(RESOLV_CONF, 'w')
-        resolv_fd.seek(0)
-        resolv_fd.truncate(0)
-        resolv_fd.write(resolv_txt)
-        resolv_fd.close()
+        try:
+            resolv_fd = open(RESOLV_CONF, 'w')
+            resolv_fd.seek(0)
+            resolv_fd.truncate(0)
+            resolv_fd.write(resolv_txt)
+            resolv_fd.close()
+        except IOError as e:
+            root_logger.error('Could not write to resolv.conf: %s', e)
+
+    def __generate_rndc_key(self):
+        installutils.check_entropy()
+        ipautil.run(['/usr/libexec/generate-rndc-key.sh'])
 
     def add_master_dns_records(self, fqdn, ip_address, realm_name, domain_name,
                                reverse_zone, ntp=False, ca_configured=None):
@@ -828,6 +888,7 @@ class BindInstance(service.Service):
         self.reverse_zone = reverse_zone
         self.ca_configured = ca_configured
         self.first_instance = False
+        self.zonemgr = 'hostmaster.%s' % self.domain
 
         self.__add_self()
         self.__add_ipa_ca_record()
