@@ -31,7 +31,6 @@ import grp
 
 from ipapython.ipa_log_manager import *
 from ipapython import ipautil, sysrestore, ipaldap
-from ipapython import services as ipaservices
 import service
 import installutils
 import certs
@@ -40,15 +39,33 @@ from ipaserver.install import ldapupdate
 from ipaserver.install import replication
 from ipaserver.install import sysupgrade
 from ipalib import errors
+from ipaplatform.tasks import tasks
+from ipalib.constants import CACERT
 from ipapython.dn import DN
+from ipaplatform import services
+from ipaplatform.paths import paths
 
-SERVER_ROOT_64 = "/usr/lib64/dirsrv"
-SERVER_ROOT_32 = "/usr/lib/dirsrv"
-CACERT="/etc/ipa/ca.crt"
+SERVER_ROOT_64 = paths.USR_LIB_DIRSRV_64
+SERVER_ROOT_32 = paths.USR_LIB_DIRSRV
 
 DS_USER = 'dirsrv'
 DS_GROUP = 'dirsrv'
 
+IPA_SCHEMA_FILES = ("60kerberos.ldif",
+                    "60samba.ldif",
+                    "60ipaconfig.ldif",
+                    "60basev2.ldif",
+                    "60basev3.ldif",
+                    "60ipadns.ldif",
+                    "61kerberos-ipav3.ldif",
+                    "65ipasudo.ldif",
+                    "70ipaotp.ldif",
+                    "15rfc2307bis.ldif",
+                    "15rfc4876.ldif")
+
+ALL_SCHEMA_FILES = IPA_SCHEMA_FILES + ("05rfc2247.ldif", )
+
+
 def find_server_root():
     if ipautil.dir_exists(SERVER_ROOT_64):
         return SERVER_ROOT_64
@@ -59,32 +76,32 @@ def realm_to_serverid(realm_name):
     return "-".join(realm_name.split("."))
 
 def config_dirname(serverid):
-    return "/etc/dirsrv/slapd-" + serverid + "/"
+    return (paths.ETC_DIRSRV_SLAPD_INSTANCE_TEMPLATE % serverid) + "/"
 
 def schema_dirname(serverid):
     return config_dirname(serverid) + "/schema/"
 
 def erase_ds_instance_data(serverid):
-    installutils.rmtree("/etc/dirsrv/slapd-%s" % serverid)
+    installutils.rmtree(paths.ETC_DIRSRV_SLAPD_INSTANCE_TEMPLATE % serverid)
 
-    installutils.rmtree("/usr/lib/dirsrv/slapd-%s" % serverid)
+    installutils.rmtree(paths.USR_LIB_SLAPD_INSTANCE_TEMPLATE % serverid)
 
-    installutils.rmtree("/usr/lib64/dirsrv/slapd-%s" % serverid)
+    installutils.rmtree(paths.USR_LIB_DIRSRV_SLAPD_INSTANCE_DIR_TEMPLATE % serverid)
 
-    installutils.rmtree("/var/lib/dirsrv/slapd-%s" % serverid)
+    installutils.rmtree(paths.VAR_LIB_SLAPD_INSTANCE_DIR_TEMPLATE % serverid)
 
-    installutils.rmtree("/var/lock/dirsrv/slapd-%s" % serverid)
+    installutils.rmtree(paths.SLAPD_INSTANCE_LOCK_TEMPLATE % serverid)
 
-    installutils.remove_file("/var/run/slapd-%s.socket" % serverid)
+    installutils.remove_file(paths.SLAPD_INSTANCE_SOCKET_TEMPLATE % serverid)
 
-    installutils.rmtree("/var/lib/dirsrv/scripts-%s" % serverid)
+    installutils.rmtree(paths.VAR_LIB_DIRSRV_INSTANCE_SCRIPTS_TEMPLATE % serverid)
 
-    installutils.remove_file("/etc/dirsrv/ds.keytab")
+    installutils.remove_file(paths.DS_KEYTAB)
 
-    installutils.remove_file("/etc/sysconfig/dirsrv-%s" % serverid)
+    installutils.remove_file(paths.SYSCONFIG_DIRSRV_INSTANCE % serverid)
 
 #    try:
-#        shutil.rmtree("/var/log/dirsrv/slapd-%s" % serverid)
+#        shutil.rmtree(paths.VAR_LOG_DIRSRV_INSTANCE_TEMPLATE % serverid)
 #    except:
 #        pass
 
@@ -96,7 +113,7 @@ def get_ds_instances():
     matches 389ds behavior.
     '''
 
-    dirsrv_instance_dir='/etc/dirsrv'
+    dirsrv_instance_dir=paths.ETC_DIRSRV
     instance_prefix = 'slapd-'
 
     instances = []
@@ -129,7 +146,7 @@ def check_ports():
     return (ds_unsecure, ds_secure)
 
 def is_ds_running(server_id=''):
-    return ipaservices.knownservices.dirsrv.is_running(instance_name=server_id)
+    return services.knownservices.dirsrv.is_running(instance_name=server_id)
 
 
 def create_ds_user():
@@ -142,11 +159,11 @@ def create_ds_user():
     except KeyError:
         root_logger.debug('Adding DS user %s', DS_USER)
         args = [
-            '/usr/sbin/useradd',
+            paths.USERADD,
             '-g', DS_GROUP,
             '-c', 'DS System User',
-            '-d', '/var/lib/dirsrv',
-            '-s', '/sbin/nologin',
+            '-d', paths.VAR_LIB_DIRSRV,
+            '-s', paths.NOLOGIN,
             '-M', '-r', DS_USER
         ]
         try:
@@ -168,7 +185,7 @@ def create_ds_group():
     except KeyError:
         group_exists = False
         root_logger.debug('Adding DS group %s', DS_GROUP)
-        args = ['/usr/sbin/groupadd', '-r', DS_GROUP]
+        args = [paths.GROUPADD, '-r', DS_GROUP]
         try:
             ipautil.run(args)
             root_logger.debug('Done adding DS group')
@@ -204,20 +221,21 @@ info: IPA V2.0
 
 class DsInstance(service.Service):
     def __init__(self, realm_name=None, domain_name=None, dm_password=None,
-                 fstore=None, cert_nickname='Server-Cert'):
+                 fstore=None):
         service.Service.__init__(self, "dirsrv",
             service_desc="directory server",
             dm_password=dm_password,
             ldapi=False,
             autobind=service.DISABLED
             )
-        self.nickname = cert_nickname
+        self.nickname = 'Server-Cert'
         self.dm_password = dm_password
         self.realm = realm_name
         self.sub_dict = None
         self.domain = domain_name
         self.serverid = None
         self.pkcs12_info = None
+        self.cacert_name = None
         self.ca_is_configured = True
         self.dercert = None
         self.idstart = None
@@ -234,7 +252,7 @@ class DsInstance(service.Service):
         if fstore:
             self.fstore = fstore
         else:
-            self.fstore = sysrestore.FileStore('/var/lib/ipa/sysrestore')
+            self.fstore = sysrestore.FileStore(paths.SYSRESTORE)
 
 
     subject_base = ipautil.dn_attribute_property('_subject_base')
@@ -258,7 +276,7 @@ class DsInstance(service.Service):
         self.step("creating indices", self.__create_indices)
         self.step("enabling referential integrity plugin", self.__add_referint_module)
         if enable_ssl:
-            self.step("configuring ssl for ds instance", self.enable_ssl)
+            self.step("configuring ssl for ds instance", self.__enable_ssl)
         self.step("configuring certmap.conf", self.__certmap_conf)
         self.step("configure autobind for root", self.__root_autobind)
         self.step("configure new location for managed entries", self.__repoint_managed_entries)
@@ -322,6 +340,15 @@ class DsInstance(service.Service):
 
         self.start_creation(runtime=60)
 
+    def enable_ssl(self):
+        self.steps = []
+
+        self.step("configuring ssl for ds instance", self.__enable_ssl)
+        self.step("restarting directory server", self.__restart_instance)
+        self.step("adding CA certificate entry", self.__upload_ca_cert)
+
+        self.start_creation(runtime=10)
+
     def create_replica(self, realm_name, master_fqdn, fqdn,
                        domain_name, dm_password, subject_base,
                        pkcs12_info=None, ca_file=None, ca_is_configured=None):
@@ -406,13 +433,13 @@ class DsInstance(service.Service):
         pent = pwd.getpwnam(DS_USER)
 
         self.backup_state("serverid", self.serverid)
-        self.fstore.backup_file("/etc/sysconfig/dirsrv")
+        self.fstore.backup_file(paths.SYSCONFIG_DIRSRV)
 
         self.sub_dict['BASEDC'] = self.realm.split('.')[0].lower()
         base_txt = ipautil.template_str(BASE_TEMPLATE, self.sub_dict)
         root_logger.debug(base_txt)
 
-        target_fname = '/var/lib/dirsrv/boot.ldif'
+        target_fname = paths.DIRSRV_BOOT_LDIF
         base_fd = open(target_fname, "w")
         base_fd.write(base_txt)
         base_fd.close()
@@ -426,11 +453,11 @@ class DsInstance(service.Service):
         inf_fd = ipautil.write_tmp_file(inf_txt)
         inf_txt = re.sub(r"RootDNPwd=.*\n", "", inf_txt)
         root_logger.debug(inf_txt)
-        if ipautil.file_exists("/usr/sbin/setup-ds.pl"):
-            args = ["/usr/sbin/setup-ds.pl", "--silent", "--logfile", "-", "-f", inf_fd.name]
+        if ipautil.file_exists(paths.SETUP_DS_PL):
+            args = [paths.SETUP_DS_PL, "--silent", "--logfile", "-", "-f", inf_fd.name]
             root_logger.debug("calling setup-ds.pl")
         else:
-            args = ["/usr/bin/ds_newinst.pl", inf_fd.name]
+            args = [paths.DS_NEWINST_PL, inf_fd.name]
             root_logger.debug("calling ds_newinst.pl")
         try:
             ipautil.run(args)
@@ -449,19 +476,11 @@ class DsInstance(service.Service):
             print "failed to restart ds instance", e
             root_logger.debug("failed to restart ds instance %s" % e)
         inf_fd.close()
-        os.remove("/var/lib/dirsrv/boot.ldif")
+        os.remove(paths.DIRSRV_BOOT_LDIF)
 
     def __add_default_schemas(self):
         pent = pwd.getpwnam(DS_USER)
-        for schema_fname in ("60kerberos.ldif",
-                             "60samba.ldif",
-                             "60ipaconfig.ldif",
-                             "60basev2.ldif",
-                             "60basev3.ldif",
-                             "60ipadns.ldif",
-                             "61kerberos-ipav3.ldif",
-                             "65ipasudo.ldif",
-                             "70ipaotp.ldif"):
+        for schema_fname in IPA_SCHEMA_FILES:
             target_fname = schema_dirname(self.serverid) + schema_fname
             shutil.copyfile(ipautil.SHARE_DIR + schema_fname, target_fname)
             os.chmod(target_fname, 0440)    # read access for dirsrv user/group
@@ -541,7 +560,7 @@ class DsInstance(service.Service):
 
     def __enable_compat_plugin(self):
         ld = ldapupdate.LDAPUpdate(dm_password=self.dm_password, sub_dict=self.sub_dict)
-        rv = ld.update(['/usr/share/ipa/schema_compat.uldif'])
+        rv = ld.update([paths.SCHEMA_COMPAT_ULDIF])
         if not rv:
             raise RuntimeError("Enabling compatibility plugin failed")
 
@@ -569,8 +588,8 @@ class DsInstance(service.Service):
 
     def configure_dirsrv_ccache(self):
         pent = pwd.getpwnam("dirsrv")
-        ccache = '/tmp/krb5cc_%d' % pent.pw_uid
-        filepath = '/etc/sysconfig/dirsrv'
+        ccache = paths.TMP_KRB5CC % pent.pw_uid
+        filepath = paths.SYSCONFIG_DIRSRV
         if not os.path.exists(filepath):
             # file doesn't exist; create it with correct ownership & mode
             open(filepath, 'a').close()
@@ -581,7 +600,7 @@ class DsInstance(service.Service):
         replacevars = {'KRB5CCNAME': ccache}
         old_values = ipautil.backup_config_and_replace_variables(
             self.fstore, filepath, replacevars=replacevars)
-        ipaservices.restore_context(filepath)
+        tasks.restore_context(filepath)
 
     def __managed_entries(self):
         self._ldap_mod("managed-entries.ldif", self.sub_dict)
@@ -598,7 +617,7 @@ class DsInstance(service.Service):
     def generate_random(self):
         return ipautil.ipa_generate_password()
 
-    def enable_ssl(self):
+    def __enable_ssl(self):
         dirname = config_dirname(self.serverid)
         dsdb = certs.CertDB(self.realm, nssdir=dirname, subject_base=self.subject_base)
         if self.pkcs12_info:
@@ -609,33 +628,31 @@ class DsInstance(service.Service):
                 raise RuntimeError("Could not find a suitable server cert in import in %s" % self.pkcs12_info[0])
 
             # We only handle one server cert
-            nickname = server_certs[0][0]
-            self.dercert = dsdb.get_cert_from_db(nickname, pem=False)
+            self.nickname = server_certs[0][0]
+            self.dercert = dsdb.get_cert_from_db(self.nickname, pem=False)
         else:
-            nickname = self.nickname
             cadb = certs.CertDB(self.realm, host_name=self.fqdn, subject_base=self.subject_base)
 
             # FIXME, need to set this nickname in the RA plugin
             cadb.export_ca_cert('ipaCert', False)
             dsdb.create_from_cacert(cadb.cacert_fname, passwd=None)
             self.dercert = dsdb.create_server_cert(
-                nickname, self.fqdn, cadb)
+                self.nickname, self.fqdn, cadb)
             dsdb.create_pin_file()
 
+        self.cacert_name = dsdb.cacert_name
+
         if self.ca_is_configured:
             dsdb.track_server_cert(
-                nickname, self.principal, dsdb.passwd_fname,
+                self.nickname, self.principal, dsdb.passwd_fname,
                 'restart_dirsrv %s' % self.serverid)
 
         conn = ipaldap.IPAdmin(self.fqdn)
         conn.do_simple_bind(DN(('cn', 'directory manager')), self.dm_password)
 
         mod = [(ldap.MOD_REPLACE, "nsSSLClientAuth", "allowed"),
-               (ldap.MOD_REPLACE, "nsSSL3Ciphers",
-                "-rsa_null_md5,+rsa_rc4_128_md5,+rsa_rc4_40_md5,+rsa_rc2_40_md5,\
-+rsa_des_sha,+rsa_fips_des_sha,+rsa_3des_sha,+rsa_fips_3des_sha,+fortezza,\
-+fortezza_rc4_128_sha,+fortezza_null,+tls_rsa_export1024_with_rc4_56_sha,\
-+tls_rsa_export1024_with_des_cbc_sha")]
+               (ldap.MOD_REPLACE, "nsSSL3Ciphers", "+all"),
+               (ldap.MOD_REPLACE, "allowWeakCipher", "off")]
         conn.modify_s(DN(('cn', 'encryption'), ('cn', 'config')), mod)
 
         mod = [(ldap.MOD_ADD, "nsslapd-security", "on")]
@@ -645,7 +662,7 @@ class DsInstance(service.Service):
             DN(('cn', 'RSA'), ('cn', 'encryption'), ('cn', 'config')),
             objectclass=["top", "nsEncryptionModule"],
             cn=["RSA"],
-            nsSSLPersonalitySSL=[nickname],
+            nsSSLPersonalitySSL=[self.nickname],
             nsSSLToken=["internal (software)"],
             nsSSLActivation=["on"],
         )
@@ -656,33 +673,36 @@ class DsInstance(service.Service):
         # check for open secure port 636 from now on
         self.open_ports.append(636)
 
-    def export_ca_cert(self, nickname, location):
-        dirname = config_dirname(self.serverid)
-        dsdb = certs.NSSDatabase(nssdir=dirname)
-        dsdb.export_pem_cert(nickname, location)
-
-    def upload_ca_cert(self, cacert_name=None):
+    def __upload_ca_cert(self):
         """
         Upload the CA certificate from the NSS database to the LDAP directory.
         """
 
         dirname = config_dirname(self.serverid)
-        certdb = certs.CertDB(self.realm, nssdir=dirname, subject_base=self.subject_base)
+        certdb = certs.CertDB(self.realm, nssdir=dirname,
+                              subject_base=self.subject_base)
 
-        if cacert_name is None:
-            cacert_name = certdb.cacert_name
-        dercert = certdb.get_cert_from_db(cacert_name, pem=False)
-        self.upload_ca_dercert(dercert)
+        dercert = certdb.get_cert_from_db(self.cacert_name, pem=False)
 
-    def upload_ca_dercert(self, dercert):
-        """Upload the CA DER certificate to the LDAP directory
-        """
-        # Note: Don't try to optimize if base64 data is already available.
-        # We want to re-encode using Python's b64encode to ensure the
-        # data is normalized (no extra newlines in the ldif)
-        self.sub_dict['CADERCERT'] = base64.b64encode(dercert)
+        conn = ipaldap.IPAdmin(self.fqdn)
+        conn.do_simple_bind(DN(('cn', 'directory manager')), self.dm_password)
 
-        self._ldap_mod('upload-cacert.ldif', self.sub_dict)
+        dn = DN(('cn', 'CAcert'), ('cn', 'ipa'), ('cn', 'etc'), self.suffix)
+        try:
+            entry = conn.get_entry(dn, attrs_list=['cACertificate;binary'])
+            entry['cACertificate;binary'] = [dercert]
+            conn.update_entry(entry)
+        except errors.NotFound:
+            entry = conn.make_entry(
+                dn,
+                {'objectClass': ['nsContainer', 'pkiCA'],
+                 'cn': ['CAcert'],
+                 'cACertificate;binary': [dercert]})
+            conn.add_entry(entry)
+        except errors.EmptyModlist:
+            pass
+
+        conn.unbind()
 
     def __add_default_layout(self):
         self._ldap_mod("bootstrap-template.ldif", self.sub_dict)
@@ -726,15 +746,15 @@ class DsInstance(service.Service):
         admpwdfile = ""
 
         try:
-            (dmpwdfd, dmpwdfile) = tempfile.mkstemp(dir='/var/lib/ipa')
+            (dmpwdfd, dmpwdfile) = tempfile.mkstemp(dir=paths.VAR_LIB_IPA)
             os.write(dmpwdfd, self.dm_password)
             os.close(dmpwdfd)
 
-            (admpwdfd, admpwdfile) = tempfile.mkstemp(dir='/var/lib/ipa')
+            (admpwdfd, admpwdfile) = tempfile.mkstemp(dir=paths.VAR_LIB_IPA)
             os.write(admpwdfd, password)
             os.close(admpwdfd)
 
-            args = ["/usr/bin/ldappasswd", "-h", self.fqdn,
+            args = [paths.LDAPPASSWD, "-h", self.fqdn,
                     "-ZZ", "-x", "-D", str(DN(('cn', 'Directory Manager'))),
                     "-y", dmpwdfile, "-T", admpwdfile,
                     str(DN(('uid', 'admin'), ('cn', 'users'), ('cn', 'accounts'), self.suffix))]
@@ -763,8 +783,8 @@ class DsInstance(service.Service):
         running = self.restore_state("running")
 
         try:
-            self.fstore.restore_file("/etc/security/limits.conf")
-            self.fstore.restore_file("/etc/sysconfig/dirsrv")
+            self.fstore.restore_file(paths.LIMITS_CONF)
+            self.fstore.restore_file(paths.SYSCONFIG_DIRSRV)
         except ValueError, error:
             root_logger.debug(error)
             pass
@@ -792,7 +812,7 @@ class DsInstance(service.Service):
         # (re)start them.
         for ds_instance in get_ds_instances():
             try:
-                ipaservices.knownservices.dirsrv.restart(ds_instance, wait=False)
+                services.knownservices.dirsrv.restart(ds_instance, wait=False)
             except Exception, e:
                 root_logger.error('Unable to restart ds instance %s: %s', ds_instance, e)
 
@@ -857,7 +877,7 @@ class DsInstance(service.Service):
         """
 
         # Do the platform-specific changes
-        proceed = ipaservices.knownservices.dirsrv.tune_nofile_platform(
+        proceed = services.knownservices.dirsrv.tune_nofile_platform(
                     num=num, fstore=self.fstore)
 
         if proceed:
@@ -894,7 +914,7 @@ class DsInstance(service.Service):
         dn = DN(('cn', 'default'), ('ou', 'profile'), self.suffix)
         try:
             entry = self.admin_conn.get_entry(dn)
-            srvlist = entry.single_value('defaultServerList', '')
+            srvlist = entry.single_value.get('defaultServerList', '')
             srvlist = srvlist.split()
             if not self.fqdn in srvlist:
                 srvlist.append(self.fqdn)
