@@ -41,8 +41,9 @@ from ipalib.util import validate_hostname
 from ipapython import config
 from ipalib import errors
 from ipapython.dn import DN
-from ipaserver.install import certs
-from ipapython import services as ipaservices
+from ipaserver.install import certs, service
+from ipaplatform import services
+from ipaplatform.paths import paths
 
 # Used to determine install status
 IPA_MODULES = [
@@ -172,7 +173,7 @@ def verify_fqdn(host_name, no_host_dns=False, local_hostname=True):
             raise HostReverseLookupError("The host name %s does not match the reverse lookup %s" % (host_name, revname))
         verified.add(address)
 
-def record_in_hosts(ip, host_name=None, file="/etc/hosts"):
+def record_in_hosts(ip, host_name=None, file=paths.HOSTS):
     """
     Search record in /etc/hosts - static table lookup for hostnames
 
@@ -209,7 +210,7 @@ def record_in_hosts(ip, host_name=None, file="/etc/hosts"):
 
     return None
 
-def add_record_to_hosts(ip, host_name, file="/etc/hosts"):
+def add_record_to_hosts(ip, host_name, file=paths.HOSTS):
     hosts_fd = open(file, 'r+')
     hosts_fd.seek(0, 2)
     hosts_fd.write(ip+'\t'+host_name+' '+host_name.split('.')[0]+'\n')
@@ -486,9 +487,9 @@ def get_server_ip_address(host_name, fstore, unattended, options):
     hosts_record = record_in_hosts(ip_address)
 
     if hosts_record is None:
-        if ip_add_to_hosts:
+        if ip_add_to_hosts or options.setup_dns:
             print "Adding ["+ip_address+" "+host_name+"] to your /etc/hosts file"
-            fstore.backup_file("/etc/hosts")
+            fstore.backup_file(paths.HOSTS)
             add_record_to_hosts(ip_address, host_name)
     else:
         primary_host = hosts_record[1][0]
@@ -538,6 +539,22 @@ def read_replica_info(dir, rconfig):
     except NoOptionError:
         pass
 
+def read_replica_info_dogtag_port(config_dir):
+    portfile = config_dir + "/dogtag_directory_port.txt"
+    default_port = dogtag.Dogtag9Constants.DS_PORT
+    if not ipautil.file_exists(portfile):
+        dogtag_master_ds_port = default_port
+    else:
+        with open(portfile) as fd:
+            try:
+                dogtag_master_ds_port = int(fd.read())
+            except (ValueError, IOError), e:
+                root_logger.debug('Cannot parse dogtag DS port: %s', e)
+                root_logger.debug('Default to %d', default_port)
+                dogtag_master_ds_port = default_port
+
+    return dogtag_master_ds_port
+
 def check_server_configuration():
     """
     Check if IPA server is configured on the system.
@@ -550,7 +567,7 @@ def check_server_configuration():
     Most convenient use case for the function is in install tools that require
     configured IPA for its function.
     """
-    server_fstore = sysrestore.FileStore('/var/lib/ipa/sysrestore')
+    server_fstore = sysrestore.FileStore(paths.SYSRESTORE)
     if not server_fstore.has_files():
         raise RuntimeError("IPA is not configured on this system.")
 
@@ -581,8 +598,8 @@ def is_ipa_configured():
     """
     installed = False
 
-    sstore = sysrestore.StateFile('/var/lib/ipa/sysrestore')
-    fstore = sysrestore.FileStore('/var/lib/ipa/sysrestore')
+    sstore = sysrestore.StateFile(paths.SYSRESTORE)
+    fstore = sysrestore.FileStore(paths.SYSRESTORE)
 
     for module in IPA_MODULES:
         if sstore.has_state(module):
@@ -673,6 +690,8 @@ def handle_error(error, log_file_name=None):
     if isinstance(error, socket.error):
         return error, 1
 
+    if isinstance(error, errors.ACIError):
+        return error.message, 1
     if isinstance(error, ldap.INVALID_CREDENTIALS):
         return "Invalid password", 1
     if isinstance(error, ldap.INSUFFICIENT_ACCESS):
@@ -715,8 +734,6 @@ def check_pkcs12(pkcs12_info, ca_file, hostname):
 
     This is used for files given to --*_pkcs12 to ipa-server-install and
     ipa-replica-prepare.
-
-    Return a (server cert name, CA cert names) tuple
     """
     pkcs12_filename, pkcs12_passwd = pkcs12_info
     root_logger.debug('Checking PKCS#12 certificate %s', pkcs12_filename)
@@ -727,13 +744,18 @@ def check_pkcs12(pkcs12_info, ca_file, hostname):
         # Import the CA cert first so it has a known nickname
         # (if it's present in the PKCS#12 it won't be overwritten)
         ca_cert_name = 'The Root CA'
-        try:
-            nssdb.import_pem_cert(ca_cert_name, "CT,C,C", ca_file)
-        except (ValueError, RuntimeError) as e:
-            raise ScriptError(str(e))
+        if ca_file:
+            try:
+                nssdb.import_pem_cert(ca_cert_name, "CT,C,C", ca_file)
+            except (ValueError, RuntimeError) as e:
+                raise ScriptError(str(e))
 
         # Import everything in the PKCS#12
-        nssdb.import_pkcs12(pkcs12_filename, db_pwd_file.name, pkcs12_passwd)
+        try:
+            nssdb.import_pkcs12(
+                pkcs12_filename, db_pwd_file.name, pkcs12_passwd)
+        except RuntimeError as e:
+            raise ScriptError(str(e))
 
         # Check we have exactly one server cert (one with a private key)
         server_certs = nssdb.find_server_certs()
@@ -748,21 +770,23 @@ def check_pkcs12(pkcs12_info, ca_file, hostname):
 
         # Check we have the whole cert chain & the CA is in it
         trust_chain = nssdb.get_trust_chain(server_cert_name)
-        while trust_chain:
-            if trust_chain[0] == ca_cert_name:
-                break
-            trust_chain = trust_chain[1:]
-        else:
+        if len(trust_chain) < 2:
+            if ca_file:
+                raise ScriptError(
+                    '%s is not signed by %s, or the full certificate chain is '
+                    'not present in the PKCS#12 file' %
+                    (pkcs12_filename, ca_file))
+            else:
+                raise ScriptError(
+                    'The full certificate chain is not present in %s' %
+                    pkcs12_filename)
+        if ca_file and trust_chain[-2] != ca_cert_name:
             raise ScriptError(
-                '%s is not signed by %s, or the full certificate chain is not '
-                'present in the PKCS#12 file' % (pkcs12_filename, ca_file))
-        if len(trust_chain) != 2:
-            raise ScriptError(
-                'trust chain of the server certificate in %s contains %s '
-                'certificates, expected 2' %
-                (pkcs12_filename, len(trust_chain)))
+                '%s is not signed by %s' % (pkcs12_filename, ca_file))
+        ca_cert_name = trust_chain[-2]
 
         # Check server validity
+        nssdb.trust_root_cert(ca_cert_name)
         try:
             nssdb.verify_server_cert_validity(server_cert_name, hostname)
         except ValueError as e:
@@ -770,8 +794,7 @@ def check_pkcs12(pkcs12_info, ca_file, hostname):
                 'The server certificate in %s is not valid: %s' %
                 (pkcs12_filename, e))
 
-        return server_cert_name
-
+        return nssdb.get_cert(ca_cert_name)
 
 @contextmanager
 def private_ccache(path=None):
@@ -815,16 +838,34 @@ def stopped_service(service, instance_name=""):
                       log_instance_name)
 
     # Figure out if the service is running, if not, yield
-    if not ipaservices.knownservices[service].is_running(instance_name):
+    if not services.knownservices[service].is_running(instance_name):
         root_logger.debug('Service %s%s is not running, continue.', service,
                           log_instance_name)
         yield
     else:
         # Stop the service, do the required stuff and start it again
         root_logger.debug('Stopping %s%s.', service, log_instance_name)
-        ipaservices.knownservices[service].stop(instance_name)
+        services.knownservices[service].stop(instance_name)
         try:
             yield
         finally:
             root_logger.debug('Starting %s%s.', service, log_instance_name)
-            ipaservices.knownservices[service].start(instance_name)
+            services.knownservices[service].start(instance_name)
+
+def check_entropy():
+    '''
+    Checks if the system has enough entropy, if not, displays warning message
+    '''
+    try:
+        with open('/proc/sys/kernel/random/entropy_avail', 'r') as efname:
+            if int(efname.read()) < 200:
+                emsg = 'WARNING: Your system is running out of entropy, ' \
+                        'you may experience long delays'
+                service.print_msg(emsg)
+                root_logger.debug(emsg)
+    except IOError as e:
+        root_logger.debug("Could not open /proc/sys/kernel/random/entropy_avail: %s" % \
+            e)
+    except ValueError as e:
+        root_logger.debug("Invalid value in /proc/sys/kernel/random/entropy_avail %s" % \
+            e)
