@@ -21,9 +21,12 @@
 import os
 import shutil
 import tempfile
+import time
 from optparse import OptionGroup
 from ConfigParser import SafeConfigParser
 
+import dns.resolver
+
 from ipaserver.install import certs, installutils, bindinstance, dsinstance
 from ipaserver.install.replication import enable_replication_version_checking
 from ipaserver.plugins.ldap2 import ldap2
@@ -34,6 +37,8 @@ from ipapython.dn import DN
 from ipapython import version
 from ipalib import api
 from ipalib import errors
+from ipaplatform.paths import paths
+from ipalib.constants import CACERT
 
 
 class ReplicaPrepare(admintool.AdminTool):
@@ -59,9 +64,12 @@ class ReplicaPrepare(admintool.AdminTool):
         parser.add_option("--no-pkinit", dest="setup_pkinit",
             action="store_false", default=True,
             help="disables pkinit setup steps")
-        parser.add_option("--ca", dest="ca_file", default="/root/cacert.p12",
+        parser.add_option("--ca", dest="ca_file", default=paths.CACERT_P12,
             metavar="FILE",
             help="location of CA PKCS#12 file, default /root/cacert.p12")
+        parser.add_option('--no-wait-for-dns', dest='wait_for_dns',
+            action='store_false', default=True,
+            help="do not wait until the replica is resolvable in DNS")
 
         group = OptionGroup(parser, "SSL certificate options",
             "Only used if the server was installed using custom SSL certificates")
@@ -137,9 +145,9 @@ class ReplicaPrepare(admintool.AdminTool):
                 "could not find directory instance: %s" % config_dir)
 
     def check_pkcs12(self, pkcs12_file, pkcs12_pin):
-        installutils.check_pkcs12(
+        return installutils.check_pkcs12(
             pkcs12_info=(pkcs12_file, pkcs12_pin),
-            ca_file='/etc/ipa/ca.crt',
+            ca_file=CACERT,
             hostname=self.replica_fqdn)
 
     def ask_for_options(self):
@@ -162,7 +170,7 @@ class ReplicaPrepare(admintool.AdminTool):
             conn = ldap2(shared_instance=False, base_dn=suffix)
             conn.connect(bind_dn=DN(('cn', 'directory manager')),
                          bind_pw=self.dirman_password)
-            dn, entry_attrs = conn.get_ipa_config()
+            entry_attrs = conn.get_ipa_config()
             conn.disconnect()
         except errors.ACIError:
             raise admintool.ScriptError("The password provided is incorrect "
@@ -202,8 +210,11 @@ class ReplicaPrepare(admintool.AdminTool):
             if not dns_container_exists(api.env.host, api.env.basedn,
                                         dm_password=self.dirman_password,
                                         ldapi=True, realm=api.env.realm):
-                raise admintool.ScriptError("You can't add a DNS record "
-                    "because DNS is not set up.")
+                self.log.error(
+                    "It is not possible to add a DNS record automatically "
+                    "because DNS is not managed by IPA. Please create DNS "
+                    "record manually and then omit --ip-address option.")
+                raise admintool.ScriptError("Cannot add DNS record")
             if options.reverse_zone and not bindinstance.verify_reverse_zone(
                     options.reverse_zone, options.ip_address):
                 raise admintool.ScriptError("Invalid reverse zone")
@@ -216,7 +227,8 @@ class ReplicaPrepare(admintool.AdminTool):
                 if options.http_pin is None:
                     raise admintool.ScriptError(
                         "%s unlock password required" % options.http_pkcs12)
-            self.check_pkcs12(options.http_pkcs12, options.http_pin)
+            http_ca_cert = self.check_pkcs12(
+                options.http_pkcs12, options.http_pin)
 
         if options.dirsrv_pkcs12:
             if options.dirsrv_pin is None:
@@ -226,7 +238,8 @@ class ReplicaPrepare(admintool.AdminTool):
                 if options.dirsrv_pin is None:
                     raise admintool.ScriptError(
                         "%s unlock password required" % options.dirsrv_pkcs12)
-            self.check_pkcs12(options.dirsrv_pkcs12, options.dirsrv_pin)
+            dirsrv_ca_cert = self.check_pkcs12(
+                options.dirsrv_pkcs12, options.dirsrv_pin)
 
         if options.pkinit_pkcs12:
             if options.pkinit_pin is None:
@@ -237,6 +250,12 @@ class ReplicaPrepare(admintool.AdminTool):
                     raise admintool.ScriptError(
                         "%s unlock password required" % options.pkinit_pkcs12)
 
+        if (options.http_pkcs12 and options.dirsrv_pkcs12 and
+            http_ca_cert != dirsrv_ca_cert):
+            raise admintool.ScriptError(
+                "%s and %s are not signed by the same CA certificate" %
+                (options.http_pkcs12, options.dirsrv_pkcs12))
+
         if (not ipautil.file_exists(
                     dogtag.configured_constants().CS_CFG_PATH) and
                 options.dirsrv_pin is None):
@@ -277,6 +296,9 @@ class ReplicaPrepare(admintool.AdminTool):
         if options.ip_address:
             self.add_dns_records()
 
+        if options.wait_for_dns:
+            self.wait_for_dns()
+
     def copy_ds_certificate(self):
         options = self.options
 
@@ -353,17 +375,17 @@ class ReplicaPrepare(admintool.AdminTool):
     def copy_misc_files(self):
         self.log.info("Copying additional files")
 
-        self.copy_info_file("/etc/ipa/ca.crt", "ca.crt")
-        preferences_filename = "/usr/share/ipa/html/preferences.html"
+        self.copy_info_file(CACERT, "ca.crt")
+        preferences_filename = paths.PREFERENCES_HTML
         if ipautil.file_exists(preferences_filename):
             self.copy_info_file(preferences_filename, "preferences.html")
-            self.copy_info_file("/usr/share/ipa/html/krb.js", "krb.js")
+            self.copy_info_file(paths.KRB_JS, "krb.js")
             self.copy_info_file(
-                "/usr/share/ipa/html/kerberosauth.xpi", "kerberosauth.xpi")
-        jar_filename = "/usr/share/ipa/html/configure.jar"
+                paths.KERBEROSAUTH_XPI, "kerberosauth.xpi")
+        jar_filename = paths.CONFIGURE_JAR
         if ipautil.file_exists(jar_filename):
             self.copy_info_file(jar_filename, "configure.jar")
-        cacert_filename = "/var/kerberos/krb5kdc/cacert.pem"
+        cacert_filename = paths.CACERT_PEM
         if ipautil.file_exists(cacert_filename):
             self.copy_info_file(cacert_filename, "cacert.pem")
 
@@ -383,12 +405,12 @@ class ReplicaPrepare(admintool.AdminTool):
             config.write(fd)
 
     def package_replica_file(self):
-        replicafile = "/var/lib/ipa/replica-info-%s" % self.replica_fqdn
+        replicafile = paths.REPLICA_INFO_TEMPLATE % self.replica_fqdn
         encfile = "%s.gpg" % replicafile
 
         self.log.info("Packaging replica information into %s", encfile)
         ipautil.run(
-            ["/bin/tar", "cf", replicafile, "-C", self.top_dir, "realm_info"])
+            [paths.TAR, "cf", replicafile, "-C", self.top_dir, "realm_info"])
         ipautil.encrypt_file(
             replicafile, encfile, self.dirman_password, self.top_dir)
 
@@ -441,6 +463,50 @@ class ReplicaPrepare(admintool.AdminTool):
                 raise admintool.ScriptError(
                     "Could not add reverse DNS record for the replica: %s" % e)
 
+    def check_dns(self, replica_fqdn):
+        """Return true if the replica hostname is resolvable"""
+        resolver = dns.resolver.Resolver()
+        exceptions = (dns.resolver.NXDOMAIN, dns.resolver.NoAnswer,
+                      dns.resolver.Timeout, dns.resolver.NoNameservers)
+
+        try:
+            dns_answer = resolver.query(replica_fqdn, 'A', 'IN')
+        except exceptions:
+            try:
+                dns_answer = resolver.query(replica_fqdn, 'AAAA', 'IN')
+            except exceptions:
+                return False
+        except Exception as e:
+            self.log.warn('Exception while waiting for DNS record: %s: %s',
+                          type(e).__name__, e)
+
+        return True
+
+    def wait_for_dns(self):
+        options = self.options
+
+        # Make sure replica_fqdn has a trailing dot, so the
+        # 'search' directive in /etc/resolv.conf doesn't apply
+        replica_fqdn = self.replica_fqdn
+        if not replica_fqdn.endswith('.'):
+            replica_fqdn += '.'
+
+        if self.check_dns(replica_fqdn):
+            self.log.debug('%s A/AAAA record resolvable', replica_fqdn)
+            return
+
+        self.log.info('Waiting for %s A or AAAA record to be resolvable',
+                      replica_fqdn)
+        print 'This can be safely interrupted (Ctrl+C)'
+
+        try:
+            while not self.check_dns(replica_fqdn):
+                time.sleep(1)
+        except KeyboardInterrupt:
+            self.log.info('Interrupted')
+        else:
+            self.log.debug('%s A/AAAA record resolvable', replica_fqdn)
+
     def copy_info_file(self, source, dest):
         """Copy a file into the info directory
 
@@ -542,7 +608,7 @@ class ReplicaPrepare(admintool.AdminTool):
         dm_pwd_fd = ipautil.write_tmp_file(self.dirman_password)
 
         keydb_pwd = ''
-        with open('/etc/pki/pki-tomcat/password.conf') as f:
+        with open(paths.PKI_TOMCAT_PASSWORD_CONF) as f:
             for line in f.readlines():
                 key, value = line.strip().split('=')
                 if key == 'internal':
@@ -552,8 +618,8 @@ class ReplicaPrepare(admintool.AdminTool):
         keydb_pwd_fd = ipautil.write_tmp_file(keydb_pwd)
 
         ipautil.run([
-            '/usr/bin/PKCS12Export',
-            '-d', '/etc/pki/pki-tomcat/alias/',
+            paths.PKCS12EXPORT,
+            '-d', paths.PKI_TOMCAT_ALIAS_DIR,
             '-p', keydb_pwd_fd.name,
             '-w', dm_pwd_fd.name,
             '-o', ca_file
