@@ -36,9 +36,11 @@ from ipaserver.install.cainstance import PKI_USER
 from ipaserver.install.replication import (wait_for_task, ReplicationManager,
                                            get_cs_replication_manager)
 from ipaserver.install import installutils
-from ipapython import services as ipaservices
 from ipapython import ipaldap
+from ipaplatform.tasks import tasks
 from ipaserver.install.ipa_backup import BACKUP_DIR
+from ipaplatform import services
+from ipaplatform.paths import paths
 
 
 def recursive_chown(path, uid, gid):
@@ -64,7 +66,7 @@ def decrypt_file(tmpdir, filename, keyring):
     dest = os.path.basename(dest)
     dest = os.path.join(tmpdir, dest)
 
-    args = ['/usr/bin/gpg',
+    args = [paths.GPG,
             '--batch',
             '-o', dest]
 
@@ -87,7 +89,7 @@ def decrypt_file(tmpdir, filename, keyring):
 
 class Restore(admintool.AdminTool):
     command_name = 'ipa-restore'
-    log_file_name = '/var/log/iparestore.log'
+    log_file_name = paths.IPARESTORE_LOG
 
     usage = "%prog [options] backup"
 
@@ -179,7 +181,7 @@ class Restore(admintool.AdminTool):
         if not options.instance:
             instances = []
             for instance in [realm_to_serverid(api.env.realm), 'PKI-IPA']:
-                if os.path.exists('/var/lib/dirsrv/slapd-%s' % instance):
+                if os.path.exists(paths.VAR_LIB_SLAPD_INSTANCE_DIR_TEMPLATE % instance):
                     instances.append(instance)
         else:
             instances = [options.instance]
@@ -203,7 +205,7 @@ class Restore(admintool.AdminTool):
 
         cwd = os.getcwd()
         try:
-            dirsrv = ipaservices.knownservices.dirsrv
+            dirsrv = services.knownservices.dirsrv
 
             self.read_header()
             # These two checks would normally be in the validate method but
@@ -276,10 +278,10 @@ class Restore(admintool.AdminTool):
             # have a unified instance we need to restore both userRoot and
             # ipaca.
             for instance in instances:
-                if os.path.exists('/var/lib/dirsrv/slapd-%s' % instance):
+                if os.path.exists(paths.VAR_LIB_SLAPD_INSTANCE_DIR_TEMPLATE % instance):
                     if options.backend is None:
                         self.ldif2db(instance, 'userRoot', online=options.online)
-                        if os.path.exists('/var/lib/dirsrv/slapd-%s/db/ipaca' % instance):
+                        if os.path.exists(paths.IPACA_DIRSRV_INSTANCE_DB_TEMPLATE % instance):
                             self.ldif2db(instance, 'ipaca', online=options.online)
                     else:
                         self.ldif2db(instance, options.backend, online=options.online)
@@ -293,13 +295,13 @@ class Restore(admintool.AdminTool):
             else:
                 # explicitly enable then disable the pki tomcatd service to
                 # re-register its instance. FIXME, this is really wierd.
-                ipaservices.knownservices.pki_tomcatd.enable()
-                ipaservices.knownservices.pki_tomcatd.disable()
+                services.knownservices.pki_tomcatd.enable()
+                services.knownservices.pki_tomcatd.disable()
 
                 self.log.info('Starting IPA services')
                 run(['ipactl', 'start'])
                 self.log.info('Restarting SSSD')
-                sssd = ipaservices.service('sssd')
+                sssd = services.service('sssd')
                 sssd.restart()
         finally:
             try:
@@ -349,7 +351,7 @@ class Restore(admintool.AdminTool):
             raise admintool.ScriptError(
                 "Failed to read master data: %s" % e)
         else:
-            masters = [ent.single_value('cn') for ent in entries]
+            masters = [ent.single_value['cn'] for ent in entries]
 
         for master in masters:
             if master == api.env.host:
@@ -368,10 +370,10 @@ class Restore(admintool.AdminTool):
             except errors.NotFound:
                 continue
 
-            services_cns = [s.single_value('cn') for s in services]
+            services_cns = [s.single_value['cn'] for s in services]
 
             host_entries = repl.find_ipa_replication_agreements()
-            hosts = [rep.single_value('nsds5replicahost', None)
+            hosts = [rep.single_value.get('nsds5replicahost')
                      for rep in host_entries]
 
             for host in hosts:
@@ -386,7 +388,7 @@ class Restore(admintool.AdminTool):
                     self.log.critical("Unable to disable agreement on %s: %s" % (master, e))
 
                 host_entries = repl.find_ipa_replication_agreements()
-                hosts = [rep.single_value('nsds5replicahost', None)
+                hosts = [rep.single_value.get('nsds5replicahost')
                          for rep in host_entries]
                 for host in hosts:
                     self.log.info('Disabling CA replication agreement on %s to %s' % (master, host))
@@ -588,13 +590,13 @@ class Restore(admintool.AdminTool):
         does so we need to probe for it.
         """
         if instance != 'PKI-IPA':
-            return os.path.join('/var/lib/dirsrv', 'scripts-%s' % instance)
+            return os.path.join(paths.VAR_LIB_DIRSRV, 'scripts-%s' % instance)
         else:
             if sys.maxsize > 2**32L:
                 libpath = 'lib64'
             else:
                 libpath = 'lib'
-            return os.path.join('/usr', libpath, 'dirsrv', 'slapd-PKI-IPA')
+            return os.path.join(paths.USR_DIR, libpath, 'dirsrv', 'slapd-PKI-IPA')
 
     def __create_dogtag_log_dirs(self):
         """
@@ -605,16 +607,16 @@ class Restore(admintool.AdminTool):
         or a d10-based installation. We can tell based on whether there is
         a PKI-IPA 389-ds instance.
         """
-        if os.path.exists('/etc/dirsrv/slapd-PKI-IPA'): # dogtag 9
-            topdir = '/var/log/pki-ca'
+        if os.path.exists(paths.ETC_SLAPD_PKI_IPA_DIR): # dogtag 9
+            topdir = paths.PKI_CA_LOG_DIR
             dirs = [topdir,
                     '/var/log/pki-ca/signedAudit,']
         else: # dogtag 10
-            topdir = '/var/log/pki/pki-tomcat'
+            topdir = paths.TOMCAT_TOPLEVEL_DIR
             dirs = [topdir,
-                    '/var/log/pki/pki-tomcat/ca',
-                    '/var/log/pki/pki-tomcat/ca/archive',
-                    '/var/log/pki/pki-tomcat/ca/signedAudit',]
+                    paths.TOMCAT_CA_DIR,
+                    paths.TOMCAT_CA_ARCHIVE_DIR,
+                    paths.TOMCAT_SIGNEDAUDIT_DIR,]
 
         if os.path.exists(topdir):
             return
@@ -630,7 +632,7 @@ class Restore(admintool.AdminTool):
                 self.log.debug('Creating %s' % dir)
                 os.mkdir(dir, 0770)
                 os.chown(dir, pent.pw_uid, pent.pw_gid)
-                ipaservices.restore_context(dir)
+                tasks.restore_context(dir)
             except Exception, e:
                 # This isn't so fatal as to side-track the restore
                 self.log.error('Problem with %s: %s' % (dir, e))
