@@ -33,13 +33,12 @@ class update_upload_cacrt(PostUpdate):
 
     def execute(self, **options):
         ldap = self.obj.backend
-        (cdn, ipa_config) = ldap.get_ipa_config()
+        ipa_config = ldap.get_ipa_config()
         subject_base = ipa_config.get('ipacertificatesubjectbase', [None])[0]
         dirname = config_dirname(realm_to_serverid(api.env.realm))
         certdb = certs.CertDB(api.env.realm, nssdir=dirname, subject_base=subject_base)
 
         dercert = certdb.get_cert_from_db(certdb.cacert_name, pem=False)
-        cadercert = base64.b64encode(dercert)
 
         updates = {}
         dn = DN(('cn', 'CACert'), ('cn', 'ipa'), ('cn','etc'), api.env.basedn)
@@ -47,7 +46,7 @@ class update_upload_cacrt(PostUpdate):
         cacrt_entry = ['objectclass:nsContainer',
                        'objectclass:pkiCA',
                        'cn:CAcert',
-                       'cACertificate;binary:%s' % cadercert,
+                       'cACertificate;binary:%s' % dercert,
                       ]
         updates[dn] = {'dn': dn, 'default': cacrt_entry}
 
