@@ -25,12 +25,13 @@ import os
 import ldap
 
 from ipalib import api, errors
-from ipapython import services as ipaservices
+from ipalib.constants import CACERT
 from ipapython.ipa_log_manager import *
 from ipapython import ipautil, dogtag, ipaldap
 from ipapython.dn import DN
+from ipaplatform import services
+from ipaplatform.paths import paths
 
-CACERT = "/etc/ipa/ca.crt"
 # the default container used by AD for user entries
 WIN_USER_CONTAINER = DN(('cn', 'Users'))
 # the default container used by IPA for user entries
@@ -68,7 +69,7 @@ def replica_conn_check(master_host, host_name, realm, check_ca,
     Does not return a value, will sys.exit() on failure.
     """
     print "Run connection check to master"
-    args = ["/usr/sbin/ipa-replica-conncheck", "--master", master_host,
+    args = [paths.IPA_REPLICA_CONNCHECK, "--master", master_host,
             "--auto-master-check", "--realm", realm,
             "--principal", "admin",
             "--hostname", host_name]
@@ -104,11 +105,11 @@ def enable_replication_version_checking(hostname, realm, dirman_passwd):
     entry = conn.get_entry(DN(('cn', 'IPA Version Replication'),
                               ('cn', 'plugins'),
                               ('cn', 'config')))
-    if entry.single_value('nsslapd-pluginenabled', None) == 'off':
+    if entry.single_value.get('nsslapd-pluginenabled') == 'off':
         conn.modify_s(entry.dn, [(ldap.MOD_REPLACE, 'nsslapd-pluginenabled', 'on')])
         conn.unbind()
         serverid = "-".join(realm.split("."))
-        ipaservices.knownservices.dirsrv.restart(instance_name=serverid)
+        services.knownservices.dirsrv.restart(instance_name=serverid)
     else:
         conn.unbind()
 
@@ -126,8 +127,8 @@ def wait_for_task(conn, dn):
         'nsTaskTotalItems']
     while True:
         entry = conn.get_entry(dn, attrlist)
-        if entry.single_value('nsTaskExitCode', None):
-            exit_code = int(entry.single_value('nsTaskExitCode'))
+        if entry.single_value.get('nsTaskExitCode'):
+            exit_code = int(entry.single_value['nsTaskExitCode'])
             break
         time.sleep(1)
     return exit_code
@@ -224,8 +225,8 @@ class ReplicationManager(object):
         except errors.NotFound:
             pass
         else:
-            if replica.single_value('nsDS5ReplicaId', None):
-                return int(replica.single_value('nsDS5ReplicaId'))
+            if replica.single_value.get('nsDS5ReplicaId'):
+                return int(replica.single_value['nsDS5ReplicaId'])
 
         # Ok, either the entry doesn't exist or the attribute isn't set
         # so get it from the other master
@@ -237,12 +238,17 @@ class ReplicationManager(object):
             root_logger.debug("Unable to retrieve nsDS5ReplicaId from remote server")
             raise
         else:
-            if replica.single_value('nsDS5ReplicaId', None) is None:
+            id_values = replica.get('nsDS5ReplicaId')
+            if not id_values:
                 root_logger.debug("Unable to retrieve nsDS5ReplicaId from remote server")
                 raise RuntimeError("Unable to retrieve nsDS5ReplicaId from remote server")
 
+        # nsDS5ReplicaId is single-valued now, but historically it could
+        # contain multiple values, of which we need the highest.
+        # see bug: https://fedorahosted.org/freeipa/ticket/3394
+        retval = max(int(v) for v in id_values)
+
         # Now update the value on the master
-        retval = int(replica.single_value('nsDS5ReplicaId'))
         mod = [(ldap.MOD_REPLACE, 'nsDS5ReplicaId', str(retval + 1))]
 
         try:
@@ -421,7 +427,7 @@ class ReplicationManager(object):
                 ('cn', 'config'), ('cn', 'ldbm database'),
                 ('cn', 'plugins'), ('cn', 'config')),
             ['nsslapd-directory'])
-        dbdir = os.path.dirname(ent.single_value('nsslapd-directory', None))
+        dbdir = os.path.dirname(ent.single_value.get('nsslapd-directory'))
 
         entry = conn.make_entry(
             DN(('cn', 'changelog5'), ('cn', 'config')),
@@ -501,7 +507,7 @@ class ReplicationManager(object):
             DN(('cn', 'Multimaster Replication Plugin'), ('cn', 'plugins'),
                ('cn', 'config')),
             ['nsslapd-pluginPath'])
-        path = plgent.single_value('nsslapd-pluginPath', None)
+        path = plgent.single_value.get('nsslapd-pluginPath')
 
         mod = [(ldap.MOD_REPLACE, 'nsslapd-state', 'backend'),
                (ldap.MOD_ADD, 'nsslapd-backend', bename),
@@ -803,10 +809,9 @@ class ReplicationManager(object):
             print "Error reading status from agreement", agmtdn
             hasError = 1
         else:
-            refresh = entry.single_value('nsds5BeginReplicaRefresh', None)
-            inprogress = entry.single_value('nsds5replicaUpdateInProgress',
-                                            None)
-            status = entry.single_value('nsds5ReplicaLastInitStatus', None)
+            refresh = entry.single_value.get('nsds5BeginReplicaRefresh')
+            inprogress = entry.single_value.get('nsds5replicaUpdateInProgress')
+            status = entry.single_value.get('nsds5ReplicaLastInitStatus')
             if not refresh: # done - check status
                 if not status:
                     print "No status yet"
@@ -844,15 +849,14 @@ class ReplicationManager(object):
             print "Error reading status from agreement", agmtdn
             hasError = 1
         else:
-            inprogress = entry.single_value('nsds5replicaUpdateInProgress',
-                                            None)
-            status = entry.single_value('nsds5ReplicaLastUpdateStatus', None)
+            inprogress = entry.single_value.get('nsds5replicaUpdateInProgress')
+            status = entry.single_value.get('nsds5ReplicaLastUpdateStatus')
             try:
-                start = int(entry.single_value('nsds5ReplicaLastUpdateStart'))
+                start = int(entry.single_value['nsds5ReplicaLastUpdateStart'])
             except (ValueError, TypeError, KeyError):
                 start = 0
             try:
-                end = int(entry.single_value('nsds5ReplicaLastUpdateEnd'))
+                end = int(entry.single_value['nsds5ReplicaLastUpdateEnd'])
             except (ValueError, TypeError, KeyError):
                 end = 0
             # incremental update is done if inprogress is false and end >= start
@@ -1096,7 +1100,7 @@ class ReplicationManager(object):
             root_logger.error("Using the first one only (%s)" % entries[0].dn)
 
         dn = entries[0].dn
-        schedule = entries[0].single_value('nsds5replicaupdateschedule', None)
+        schedule = entries[0].single_value.get('nsds5replicaupdateschedule')
 
         # On the remote chance of a match. We force a synch to happen right
         # now by setting the schedule to something and quickly removing it.
@@ -1216,7 +1220,7 @@ class ReplicationManager(object):
         try:
             dn = DN(('cn', 'default'), ('ou', 'profile'), self.suffix)
             ret = self.conn.get_entry(dn)
-            srvlist = ret.single_value('defaultServerList', '')
+            srvlist = ret.single_value.get('defaultServerList', '')
             srvlist = srvlist[0].split()
             if replica in srvlist:
                 srvlist.remove(replica)
@@ -1322,15 +1326,15 @@ class ReplicationManager(object):
         """
         entry = self.conn.get_entry(DNA_DN)
 
-        nextvalue = int(entry.single_value("dnaNextValue", 0))
-        maxvalue = int(entry.single_value("dnaMaxValue", 0))
+        nextvalue = int(entry.single_value.get("dnaNextValue", 0))
+        maxvalue = int(entry.single_value.get("dnaMaxValue", 0))
 
-        sharedcfgdn = entry.single_value("dnaSharedCfgDN", None)
+        sharedcfgdn = entry.single_value.get("dnaSharedCfgDN")
         if sharedcfgdn is not None:
             sharedcfgdn = DN(sharedcfgdn)
 
             shared_entry = self.conn.get_entry(sharedcfgdn)
-            remaining = int(shared_entry.single_value("dnaRemainingValues", 0))
+            remaining = int(shared_entry.single_value.get("dnaRemainingValues", 0))
         else:
             remaining = 0
 
@@ -1353,7 +1357,7 @@ class ReplicationManager(object):
         """
         entry = self.conn.get_entry(DNA_DN)
 
-        range = entry.single_value("dnaNextRange", None)
+        range = entry.single_value.get("dnaNextRange")
 
         if range is None:
             return (None, None)
@@ -1379,7 +1383,7 @@ class ReplicationManager(object):
         """
         entry = self.conn.get_entry(DNA_DN)
 
-        range = entry.single_value("dnaNextRange", None)
+        range = entry.single_value.get("dnaNextRange")
 
         if range is not None and next_start != 0 and next_max != 0:
             return False
