@@ -25,12 +25,11 @@ import time
 import datetime
 
 from ipapython import sysrestore, ipautil, dogtag, ipaldap
-from ipapython import services as ipaservices
 from ipapython.dn import DN
 from ipapython.ipa_log_manager import *
 from ipalib import errors
-
-CACERT = "/etc/ipa/ca.crt"
+from ipaplatform import services
+from ipaplatform.paths import paths
 
 # Autobind modes
 AUTO = 1
@@ -58,11 +57,26 @@ def print_msg(message, output_fd=sys.stdout):
     output_fd.flush()
 
 
+def format_seconds(seconds):
+    """Format a number of seconds as an English minutes+seconds message"""
+    parts = []
+    minutes, seconds = divmod(seconds, 60)
+    if minutes:
+        parts.append('%d minute' % minutes)
+        if minutes != 1:
+            parts[-1] += 's'
+    if seconds or not minutes:
+        parts.append('%d second' % seconds)
+        if seconds != 1:
+            parts[-1] += 's'
+    return ' '.join(parts)
+
+
 class Service(object):
     def __init__(self, service_name, service_desc=None, sstore=None, dm_password=None, ldapi=True, autobind=AUTO):
         self.service_name = service_name
         self.service_desc = service_desc
-        self.service = ipaservices.service(service_name)
+        self.service = services.service(service_name)
         self.steps = []
         self.output_fd = sys.stdout
         self.dm_password = dm_password
@@ -75,7 +89,7 @@ class Service(object):
         if sstore:
             self.sstore = sstore
         else:
-            self.sstore = sysrestore.StateFile('/var/lib/ipa/sysrestore')
+            self.sstore = sysrestore.StateFile(paths.SYSRESTORE)
 
         self.realm = None
         self.suffix = DN()
@@ -121,17 +135,15 @@ class Service(object):
 
         self.admin_conn = conn
 
-
     def ldap_disconnect(self):
         self.admin_conn.unbind()
         self.admin_conn = None
 
-    def _ldap_mod(self, ldif, sub_dict = None):
-
+    def _ldap_mod(self, ldif, sub_dict=None):
         pw_name = None
         fd = None
         path = ipautil.SHARE_DIR + ldif
-        nologlist=[]
+        nologlist = []
 
         if sub_dict is not None:
             txt = ipautil.template_file(path, sub_dict)
@@ -139,12 +151,12 @@ class Service(object):
             path = fd.name
 
             # do not log passwords
-            if sub_dict.has_key('PASSWORD'):
+            if 'PASSWORD' in sub_dict:
                 nologlist.append(sub_dict['PASSWORD'])
-            if sub_dict.has_key('RANDOM_PASSWORD'):
+            if 'RANDOM_PASSWORD' in sub_dict:
                 nologlist.append(sub_dict['RANDOM_PASSWORD'])
 
-        args = ["/usr/bin/ldapmodify", "-v", "-f", path]
+        args = [paths.LDAPMODIFY, "-v", "-f", path]
 
         # As we always connect to the local host,
         # use URI of admin connection
@@ -152,16 +164,18 @@ class Service(object):
             self.ldap_connect()
         args += ["-H", self.admin_conn.ldap_uri]
 
-        auth_parms = []
+        # If DM password is available, use it
         if self.dm_password:
             [pw_fd, pw_name] = tempfile.mkstemp()
             os.write(pw_fd, self.dm_password)
             os.close(pw_fd)
             auth_parms = ["-x", "-D", "cn=Directory Manager", "-y", pw_name]
+        # Use GSSAPI auth when not using DM password or not being root
+        elif os.getegid() != 0:
+            auth_parms = ["-Y", "GSSAPI"]
+        # Default to EXTERNAL auth mechanism
         else:
-            # always try GSSAPI auth when not using DM password or not being root
-            if os.getegid() != 0:
-                auth_parms = ["-Y", "GSSAPI"]
+            auth_parms = ["-Y", "EXTERNAL"]
 
         args += auth_parms
 
@@ -341,19 +355,8 @@ class Service(object):
                     end_message = "Done configuring %s." % self.service_desc
 
         if runtime > 0:
-            plural=''
-            est = time.localtime(runtime)
-            if est.tm_min > 0:
-                if est.tm_min > 1:
-                    plural = 's'
-                if est.tm_sec > 0:
-                    self.print_msg('%s: Estimated time %d minute%s %d seconds' % (start_message, est.tm_min, plural, est.tm_sec))
-                else:
-                    self.print_msg('%s: Estimated time %d minute%s' % (start_message, est.tm_min, plural))
-            else:
-                if est.tm_sec > 1:
-                    plural = 's'
-                self.print_msg('%s: Estimated time %d second%s' % (start_message, est.tm_sec, plural))
+            self.print_msg('%s: Estimated time %s' % (start_message,
+                                                      format_seconds(runtime)))
         else:
             self.print_msg(start_message)
 
@@ -371,7 +374,7 @@ class Service(object):
 
         self.steps = []
 
-    def ldap_enable(self, name, fqdn, dm_password, ldap_suffix):
+    def ldap_enable(self, name, fqdn, dm_password, ldap_suffix, config=[]):
         assert isinstance(ldap_suffix, DN)
         self.disable()
         if not self.admin_conn:
@@ -384,7 +387,7 @@ class Service(object):
             objectclass=["nsContainer", "ipaConfigObject"],
             cn=[name],
             ipaconfigstring=[
-                "enabledService", "startOrder " + str(order)],
+                "enabledService", "startOrder " + str(order)] + config,
         )
 
         try:
