@@ -22,14 +22,16 @@ import sys
 import shutil
 import random
 import traceback
+from ipaplatform.paths import paths
 from ipapython.ipa_log_manager import *
 
 from ipaserver.install import installutils
 from ipaserver.install import dsinstance
+from ipaserver.install import schemaupdate
 from ipaserver.install import ldapupdate
 from ipaserver.install import service
 
-DSBASE = '/etc/dirsrv/slapd-'
+DSBASE = paths.ETC_DIRSRV_SLAPD_INSTANCE_TEMPLATE
 DSE = 'dse.ldif'
 
 class IPAUpgrade(service.Service):
@@ -38,7 +40,7 @@ class IPAUpgrade(service.Service):
     listeners and updating over ldapi. This way we know the server is
     quiet.
     """
-    def __init__(self, realm_name, files=[], live_run=True):
+    def __init__(self, realm_name, files=[], live_run=True, schema_files=[]):
         """
         realm_name: kerberos realm name, used to determine DS instance dir
         files: list of update files to process. If none use UPDATEDIR
@@ -52,14 +54,15 @@ class IPAUpgrade(service.Service):
             ext += h
         service.Service.__init__(self, "dirsrv")
         serverid = dsinstance.realm_to_serverid(realm_name)
-        self.filename = '%s%s/%s' % (DSBASE, serverid, DSE)
-        self.savefilename = '%s%s/%s.ipa.%s' % (DSBASE, serverid, DSE, ext)
+        self.filename = '%s/%s' % (DSBASE % serverid, DSE)
+        self.savefilename = '%s/%s.ipa.%s' % (DSBASE % serverid, DSE, ext)
         self.live_run = live_run
         self.files = files
         self.modified = False
         self.badsyntax = False
         self.upgradefailed = False
         self.serverid = serverid
+        self.schema_files = schema_files
 
     def __start_nowait(self):
         # Don't wait here because we've turned off port 389. The connection
@@ -75,6 +78,9 @@ class IPAUpgrade(service.Service):
         self.step("saving configuration", self.__save_config)
         self.step("disabling listeners", self.__disable_listeners)
         self.step("starting directory server", self.__start_nowait)
+        self.step("preparing server upgrade", self.__pre_schema_upgrade)
+        if self.schema_files:
+            self.step("updating schema", self.__update_schema)
         self.step("upgrading server", self.__upgrade)
         self.step("stopping directory server", self.__stop_instance)
         self.step("restoring configuration", self.__restore_config)
@@ -110,12 +116,34 @@ class IPAUpgrade(service.Service):
         installutils.set_directive(self.filename, 'nsslapd-ldapientrysearchbase',
             None, quotes=False, separator=':')
 
+    def __pre_schema_upgrade(self):
+        try:
+            ld = ldapupdate.LDAPUpdate(dm_password='', ldapi=True, live_run=self.live_run, plugins=True)
+            self.modified = (ld.pre_schema_update(ordered=True) or
+                             self.modified)
+        except ldapupdate.BadSyntax, e:
+            root_logger.error('Bad syntax in pre schema upgrade %s' % str(e))
+            self.modified = False
+            self.badsyntax = True
+        except Exception, e:
+            # Bad things happened, return gracefully
+            self.modified = False
+            self.upgradefailed = True
+            root_logger.error('Pre schema upgrade failed with %s' % str(e))
+            root_logger.debug('%s', traceback.format_exc())
+
+    def __update_schema(self):
+        self.modified = schemaupdate.update_schema(
+            self.schema_files,
+            dm_password='', ldapi=True, live_run=self.live_run) or self.modified
+
     def __upgrade(self):
         try:
             ld = ldapupdate.LDAPUpdate(dm_password='', ldapi=True, live_run=self.live_run, plugins=True)
             if len(self.files) == 0:
                 self.files = ld.get_all_files(ldapupdate.UPDATES_DIR)
-            self.modified = ld.update(self.files, ordered=True)
+            self.modified = (ld.update(self.files, ordered=True) or
+                             self.modified)
         except ldapupdate.BadSyntax, e:
             root_logger.error('Bad syntax in upgrade %s' % str(e))
             self.modified = False
