@@ -28,7 +28,6 @@ Backend plugin for LDAP.
 # everything except the CrudBackend methods, where dn is part of the entry dict.
 
 import os
-import re
 import pwd
 
 import krbV
@@ -52,7 +51,7 @@ except ImportError:
         def __init__(self, criticality, authzId=None):
             LDAPControl.__init__(self, '1.3.6.1.4.1.42.2.27.9.5.2', criticality, authzId)
 
-from ipalib import api, errors
+from ipalib import api, errors, _
 from ipalib.crud import CrudBackend
 from ipalib.request import context
 
@@ -85,9 +84,6 @@ class ldap2(LDAPClient, CrudBackend):
         # do not set it
         pass
 
-    def get_api(self):
-        return api
-
     def __del__(self):
         if self.isconnected():
             self.disconnect()
@@ -97,7 +93,7 @@ class ldap2(LDAPClient, CrudBackend):
 
     def create_connection(self, ccache=None, bind_dn=None, bind_pw='',
             tls_cacertfile=None, tls_certfile=None, tls_keyfile=None,
-            debug_level=0, autobind=False):
+            debug_level=0, autobind=False, serverctrls=None, clientctrls=None):
         """
         Connect to LDAP server.
 
@@ -155,16 +151,22 @@ class ldap2(LDAPClient, CrudBackend):
                         context=krbV.default_context()).principal().name
 
                 os.environ['KRB5CCNAME'] = ccache
-                conn.sasl_interactive_bind_s(None, SASL_GSSAPI)
+                conn.sasl_interactive_bind_s(None, SASL_GSSAPI,
+                                             serverctrls=serverctrls,
+                                             clientctrls=clientctrls)
                 setattr(context, 'principal', principal)
             else:
                 # no kerberos ccache, use simple bind or external sasl
                 if autobind:
                     pent = pwd.getpwuid(os.geteuid())
                     auth_tokens = _ldap.sasl.external(pent.pw_name)
-                    conn.sasl_interactive_bind_s(None, auth_tokens)
+                    conn.sasl_interactive_bind_s(None, auth_tokens,
+                                                 serverctrls=serverctrls,
+                                                 clientctrls=clientctrls)
                 else:
-                    conn.simple_bind_s(bind_dn, bind_pw)
+                    conn.simple_bind_s(bind_dn, bind_pw,
+                                       serverctrls=serverctrls,
+                                       clientctrls=clientctrls)
 
         return conn
 
@@ -176,6 +178,121 @@ class ldap2(LDAPClient, CrudBackend):
             # ignore when trying to unbind multiple times
             pass
 
+
+    def find_entries(self, filter=None, attrs_list=None, base_dn=None,
+                     scope=_ldap.SCOPE_SUBTREE, time_limit=None,
+                     size_limit=None, search_refs=False, paged_search=False):
+
+        def _get_limits():
+            """Get configured global limits, caching them for more calls"""
+            if not _lims:
+                config = self.get_ipa_config()
+                _lims['time'] = config.get('ipasearchtimelimit', [None])[0]
+                _lims['size'] = config.get('ipasearchrecordslimit', [None])[0]
+            return _lims
+        _lims = {}
+
+        if time_limit is None:
+            time_limit = _get_limits()['time']
+        if size_limit is None:
+            size_limit = _get_limits()['size']
+
+        has_memberindirect = False
+        has_memberofindirect = False
+        if attrs_list:
+            new_attrs_list = []
+            for attr_name in attrs_list:
+                if attr_name == 'memberindirect':
+                    has_memberindirect = True
+                elif attr_name == 'memberofindirect':
+                    has_memberofindirect = True
+                else:
+                    new_attrs_list.append(attr_name)
+            attrs_list = new_attrs_list
+
+        res, truncated = super(ldap2, self).find_entries(
+            filter=filter, attrs_list=attrs_list, base_dn=base_dn, scope=scope,
+            time_limit=time_limit, size_limit=size_limit,
+            search_refs=search_refs, paged_search=paged_search)
+
+        if has_memberindirect or has_memberofindirect:
+
+            # For the memberof searches, we want to apply the global limit
+            # if it's larger than the requested one, so decreasing limits on
+            # the individual query only affects the query itself.
+            # See https://fedorahosted.org/freeipa/ticket/4398
+            def _max_with_none(a, b):
+                """Maximum of a and b, treating None as infinity"""
+                if a is None or b is None:
+                    return None
+                else:
+                    return max(a, b)
+            time_limit = _max_with_none(time_limit, _get_limits()['time'])
+            size_limit = _max_with_none(size_limit, _get_limits()['size'])
+
+            for entry in res:
+                if has_memberindirect:
+                    self._process_memberindirect(
+                        entry, time_limit=time_limit, size_limit=size_limit)
+                if has_memberofindirect:
+                    self._process_memberofindirect(
+                        entry, time_limit=time_limit, size_limit=size_limit)
+
+        return (res, truncated)
+
+    def _process_memberindirect(self, group_entry, time_limit=None,
+                                size_limit=None):
+        filter = self.make_filter({'memberof': group_entry.dn})
+        try:
+            result, truncated = self.find_entries(
+                base_dn=self.api.env.basedn,
+                filter=filter,
+                attrs_list=['member'],
+                time_limit=time_limit,
+                size_limit=size_limit,
+                paged_search=True)
+            if truncated:
+                raise errors.LimitsExceeded()
+        except errors.NotFound:
+            result = []
+
+        indirect = set()
+        for entry in result:
+            indirect.update(entry.get('member', []))
+        indirect.difference_update(group_entry.get('member', []))
+
+        if indirect:
+            group_entry['memberindirect'] = list(indirect)
+
+    def _process_memberofindirect(self, entry, time_limit=None,
+                                  size_limit=None):
+        dn = entry.dn
+        filter = self.make_filter(
+            {'member': dn, 'memberuser': dn, 'memberhost': dn})
+        try:
+            result, truncated = self.find_entries(
+                base_dn=self.api.env.basedn,
+                filter=filter,
+                attrs_list=[''],
+                time_limit=time_limit,
+                size_limit=size_limit)
+            if truncated:
+                raise errors.LimitsExceeded()
+        except errors.NotFound:
+            result = []
+
+        direct = set()
+        indirect = set(entry.get('memberof', []))
+        for group_entry in result:
+            dn = group_entry.dn
+            if dn in indirect:
+                indirect.remove(dn)
+                direct.add(dn)
+
+        entry['memberof'] = list(direct)
+        if indirect:
+            entry['memberofindirect'] = list(indirect)
+
     config_defaults = {'ipasearchtimelimit': [2], 'ipasearchrecordslimit': [0]}
     def get_ipa_config(self, attrs_list=None):
         """Returns the IPA configuration entry (dn, entry_attrs)."""
@@ -186,47 +303,52 @@ class ldap2(LDAPClient, CrudBackend):
         try:
             config_entry = getattr(context, 'config_entry')
             if config_entry.conn is self.conn:
-                return config_entry.clone()
+                return config_entry
         except AttributeError:
             # Not in our context yet
             pass
         try:
-            (entry, truncated) = self.find_entries(
+            (entries, truncated) = self.find_entries(
                 None, attrs_list, base_dn=dn, scope=self.SCOPE_BASE,
                 time_limit=2, size_limit=10
             )
             if truncated:
                 raise errors.LimitsExceeded()
-            config_entry = entry[0]
+            config_entry = entries[0]
         except errors.NotFound:
             config_entry = self.make_entry(dn)
         for a in self.config_defaults:
             if a not in config_entry:
                 config_entry[a] = self.config_defaults[a]
-        context.config_entry = config_entry.clone()
+        context.config_entry = config_entry
         return config_entry
 
     def has_upg(self):
         """Returns True/False whether User-Private Groups are enabled.
-           This is determined based on whether the UPG Template exists.
+
+        This is determined based on whether the UPG Definition's originfilter
+        contains "(objectclass=disable)".
+
+        If the UPG Definition or its originfilter is not readable,
+        an ACI error is raised.
         """
 
         upg_dn = DN(('cn', 'UPG Definition'), ('cn', 'Definitions'), ('cn', 'Managed Entries'),
                     ('cn', 'etc'), api.env.basedn)
 
         try:
-            upg_entry = self.conn.search_s(upg_dn, _ldap.SCOPE_BASE,
-                                           attrlist=['*'])[0]
-            disable_attr = '(objectclass=disable)'
-            if 'originfilter' in upg_entry[1]:
-                org_filter = upg_entry[1]['originfilter']
-                return not bool(re.search(r'%s' % disable_attr, org_filter[0]))
-            else:
-                return False
-        except _ldap.NO_SUCH_OBJECT, e:
-            return False
+            upg_entries = self.conn.search_s(upg_dn, _ldap.SCOPE_BASE,
+                                             attrlist=['*'])
+        except _ldap.NO_SUCH_OBJECT:
+            upg_entries = None
+        if not upg_entries or 'originfilter' not in upg_entries[0]:
+            raise errors.ACIError(info=_(
+                'Could not read UPG Definition originfilter. '
+                'Check your permissions.'))
+        org_filter = upg_entries[0].single_value['originfilter']
+        return '(objectclass=disable)' not in org_filter
 
-    def get_effective_rights(self, dn, entry_attrs):
+    def get_effective_rights(self, dn, attrs_list):
         """Returns the rights the currently bound user has for the given DN.
 
            Returns 2 attributes, the attributeLevelRights for the given list of
@@ -236,15 +358,16 @@ class ldap2(LDAPClient, CrudBackend):
         assert isinstance(dn, DN)
 
         principal = getattr(context, 'principal')
-        (binddn, attrs) = self.find_entry_by_attr("krbprincipalname", principal,
+        entry = self.find_entry_by_attr("krbprincipalname", principal,
             "krbPrincipalAux", base_dn=api.env.basedn)
-        assert isinstance(binddn, DN)
-        sctrl = [GetEffectiveRightsControl(True, "dn: " + str(binddn))]
+        sctrl = [GetEffectiveRightsControl(True, "dn: " + str(entry.dn))]
         self.conn.set_option(_ldap.OPT_SERVER_CONTROLS, sctrl)
-        (dn, attrs) = self.get_entry(dn, entry_attrs)
-        # remove the control so subsequent operations don't include GER
-        self.conn.set_option(_ldap.OPT_SERVER_CONTROLS, [])
-        return (dn, attrs)
+        try:
+            entry = self.get_entry(dn, attrs_list)
+        finally:
+            # remove the control so subsequent operations don't include GER
+            self.conn.set_option(_ldap.OPT_SERVER_CONTROLS, [])
+        return entry
 
     def can_write(self, dn, attr):
         """Returns True/False if the currently bound user has write permissions
@@ -253,7 +376,7 @@ class ldap2(LDAPClient, CrudBackend):
 
         assert isinstance(dn, DN)
 
-        (dn, attrs) = self.get_effective_rights(dn, [attr])
+        attrs = self.get_effective_rights(dn, [attr])
         if 'attributelevelrights' in attrs:
             attr_rights = attrs.get('attributelevelrights')[0].decode('UTF-8')
             (attr, rights) = attr_rights.split(':')
@@ -268,7 +391,7 @@ class ldap2(LDAPClient, CrudBackend):
         """
         assert isinstance(dn, DN)
 
-        (dn, attrs) = self.get_effective_rights(dn, [attr])
+        attrs = self.get_effective_rights(dn, [attr])
         if 'attributelevelrights' in attrs:
             attr_rights = attrs.get('attributelevelrights')[0].decode('UTF-8')
             (attr, rights) = attr_rights.split(':')
@@ -293,7 +416,7 @@ class ldap2(LDAPClient, CrudBackend):
 
         assert isinstance(dn, DN)
 
-        (dn, attrs) = self.get_effective_rights(dn, ["*"])
+        attrs = self.get_effective_rights(dn, ["*"])
         if 'entrylevelrights' in attrs:
             entry_rights = attrs['entrylevelrights'][0].decode('UTF-8')
             if 'd' in entry_rights:
@@ -306,7 +429,7 @@ class ldap2(LDAPClient, CrudBackend):
            on the entry.
         """
         assert isinstance(dn, DN)
-        (dn, attrs) = self.get_effective_rights(dn, ["*"])
+        attrs = self.get_effective_rights(dn, ["*"])
         if 'entrylevelrights' in attrs:
             entry_rights = attrs['entrylevelrights'][0].decode('UTF-8')
             if 'a' in entry_rights:
@@ -314,18 +437,21 @@ class ldap2(LDAPClient, CrudBackend):
 
         return False
 
-    def modify_password(self, dn, new_pass, old_pass=''):
+    def modify_password(self, dn, new_pass, old_pass='', otp='', skip_bind=False):
         """Set user password."""
 
         assert isinstance(dn, DN)
 
         # The python-ldap passwd command doesn't verify the old password
         # so we'll do a simple bind to validate it.
-        if old_pass != '':
+        if not skip_bind and old_pass != '':
+            pw = old_pass
+            if (otp):
+                pw = old_pass+otp
             with self.error_handler():
                 conn = IPASimpleLDAPObject(
                     self.ldap_uri, force_schema_updates=False)
-                conn.simple_bind_s(dn, old_pass)
+                conn.simple_bind_s(dn, pw)
                 conn.unbind_s()
 
         with self.error_handler():
@@ -392,7 +518,7 @@ class ldap2(LDAPClient, CrudBackend):
         assert isinstance(active, bool)
 
         # get the entry in question
-        (dn, entry_attrs) = self.get_entry(dn, ['nsaccountlock'])
+        entry_attrs = self.get_entry(dn, ['nsaccountlock'])
 
         # check nsAccountLock attribute
         account_lock_attr = entry_attrs.get('nsaccountlock', ['false'])
@@ -409,7 +535,7 @@ class ldap2(LDAPClient, CrudBackend):
         account_lock_attr = str(not active).upper()
 
         entry_attrs['nsaccountlock'] = account_lock_attr
-        self.update_entry(dn, entry_attrs)
+        self.update_entry(entry_attrs)
 
     def activate_entry(self, dn):
         """Mark entry active."""
@@ -443,7 +569,7 @@ class ldap2(LDAPClient, CrudBackend):
 
         assert isinstance(dn, DN)
 
-        (dn, entry_attrs) = self.get_entry(dn, attrs_list)
+        entry_attrs = self.get_entry(dn, attrs_list)
         return entry_attrs
 
     def create(self, **kw):
@@ -456,7 +582,7 @@ class ldap2(LDAPClient, CrudBackend):
         dn = kw['dn']
         assert isinstance(dn, DN)
         del kw['dn']
-        self.add_entry(dn, kw)
+        self.add_entry(self.make_entry(dn, kw))
         return self._get_normalized_entry_for_crud(dn)
 
     def retrieve(self, primary_key, attributes):
@@ -473,7 +599,7 @@ class ldap2(LDAPClient, CrudBackend):
 
         Extends CrudBackend.update.
         """
-        self.update_entry(primary_key, kw)
+        self.update_entry(self.make_entry(primary_key, kw))
         return self._get_normalized_entry_for_crud(primary_key)
 
     def delete(self, primary_key):
@@ -518,7 +644,7 @@ class ldap2(LDAPClient, CrudBackend):
         (entries, truncated) = self.find_entries(
             filter, attrs_list, base_dn, scope
         )
-        for (dn, entry_attrs) in entries:
+        for entry_attrs in entries:
             output.append(entry_attrs)
 
         if truncated:
