@@ -23,9 +23,11 @@ import shutil
 import base64
 import glob
 import contextlib
+import nose
 
 from ipalib import x509
 from ipapython import ipautil
+from ipaplatform.paths import paths
 from ipapython.dn import DN
 from ipatests.test_integration.base import IntegrationTest
 from ipatests.test_integration import tasks
@@ -112,7 +114,7 @@ class CALessBase(IntegrationTest):
 
         # Remove CA cert in /etc/pki/nssdb, in case of failed (un)install
         for host in cls.get_all_hosts():
-            cls.master.run_command(['certutil', '-d', '/etc/pki/nssdb', '-D',
+            cls.master.run_command(['certutil', '-d', paths.NSS_DB_DIR, '-D',
                                     '-n', 'External CA cert'],
                                    raiseonerr=False)
 
@@ -144,11 +146,11 @@ class CALessBase(IntegrationTest):
         for filename in set(files_to_copy):
             cls.copy_cert(host, filename)
 
-        cls.collect_log(host, '/var/log/ipaserver-install.log')
-        cls.collect_log(host, '/var/log/ipaclient-install.log')
+        cls.collect_log(host, paths.IPASERVER_INSTALL_LOG)
+        cls.collect_log(host, paths.IPACLIENT_INSTALL_LOG)
         inst = host.domain.realm.replace('.', '-')
-        cls.collect_log(host, '/var/log/dirsrv/slapd-%s/errors' % inst)
-        cls.collect_log(host, '/var/log/dirsrv/slapd-%s/access' % inst)
+        cls.collect_log(host, paths.SLAPD_INSTANCE_ERROR_LOG_TEMPLATE % inst)
+        cls.collect_log(host, paths.SLAPD_INSTANCE_ACCESS_LOG_TEMPLATE % inst)
 
         args = [
             'ipa-server-install',
@@ -215,11 +217,11 @@ class CALessBase(IntegrationTest):
                 os.path.join(self.cert_dir, filename),
                 os.path.join(master.config.test_dir, filename))
 
-        self.collect_log(replica, '/var/log/ipareplica-install.log')
-        self.collect_log(replica, '/var/log/ipaclient-install.log')
+        self.collect_log(replica, paths.IPAREPLICA_INSTALL_LOG)
+        self.collect_log(replica, paths.IPACLIENT_INSTALL_LOG)
         inst = replica.domain.realm.replace('.', '-')
-        self.collect_log(replica, '/var/log/dirsrv/slapd-%s/errors' % inst)
-        self.collect_log(replica, '/var/log/dirsrv/slapd-%s/access' % inst)
+        self.collect_log(replica, paths.SLAPD_INSTANCE_ERROR_LOG_TEMPLATE % inst)
+        self.collect_log(replica, paths.SLAPD_INSTANCE_ACCESS_LOG_TEMPLATE % inst)
 
         args = [
             'ipa-replica-prepare',
@@ -243,7 +245,7 @@ class CALessBase(IntegrationTest):
 
         if result.returncode == 0:
             replica_bundle = master.get_file_contents(
-                '/var/lib/ipa/replica-info-%s.gpg' % replica.hostname)
+                paths.REPLICA_INFO_GPG_TEMPLATE % replica.hostname)
             replica.put_file_contents(self.get_replica_filename(replica),
                                       replica_bundle)
         else:
@@ -313,7 +315,7 @@ class CALessBase(IntegrationTest):
             ldap = host.ldap_connect()
             entry = ldap.get_entry(DN(('cn', 'CACert'), ('cn', 'ipa'),
                                       ('cn', 'etc'), host.domain.basedn))
-            cert_from_ldap = entry.single_value('cACertificate')
+            cert_from_ldap = entry.single_value['cACertificate']
             self.log.debug('CA cert from LDAP on %s:\n%r',
                            host, cert_from_ldap)
             assert cert_from_ldap == expected_binary_cacrt
@@ -326,7 +328,7 @@ class CALessBase(IntegrationTest):
 
         for host in self.get_all_hosts():
             # Check the cert PEM file
-            remote_cacrt = host.get_file_contents('/etc/ipa/ca.crt')
+            remote_cacrt = host.get_file_contents(paths.IPA_CA_CRT)
             self.log.debug('%s:/etc/ipa/ca.crt contents:\n%s',
                            host, remote_cacrt)
             binary_cacrt = base64.b64decode(x509.strip_header(remote_cacrt))
@@ -343,7 +345,7 @@ class TestServerInstall(CALessBase):
 
         # Remove CA cert in /etc/pki/nssdb, in case of failed (un)install
         for host in self.get_all_hosts():
-            self.master.run_command(['certutil', '-d', '/etc/pki/nssdb', '-D',
+            self.master.run_command(['certutil', '-d', paths.NSS_DB_DIR, '-D',
                                      '-n', 'External CA cert'],
                                     raiseonerr=False)
 
@@ -557,6 +559,12 @@ class TestServerInstall(CALessBase):
 
         result = self.install_server(http_pkcs12='http.p12',
                                      dirsrv_pkcs12='dirsrv.p12')
+
+        if result.returncode == 0:
+            raise nose.SkipTest(
+                "Known CA-less installation defect, see "
+                + "https://fedorahosted.org/freeipa/ticket/4270")
+
         assert result.returncode > 0
 
     def test_revoked_ds(self):
@@ -569,6 +577,12 @@ class TestServerInstall(CALessBase):
 
         result = self.install_server(http_pkcs12='http.p12',
                                      dirsrv_pkcs12='dirsrv.p12')
+
+        if result.returncode == 0:
+            raise nose.SkipTest(
+                "Known CA-less installation defect, see "
+                + "https://fedorahosted.org/freeipa/ticket/4270")
+
         assert result.returncode > 0
 
     def test_http_intermediate_ca(self):
@@ -755,11 +769,11 @@ class TestReplicaInstall(CALessBase):
         self.master.run_command(['ipa', 'host-del', replica.hostname],
                                 raiseonerr=False)
 
-        replica.run_command(['certutil', '-d', '/etc/pki/nssdb', '-D',
+        replica.run_command(['certutil', '-d', paths.NSS_DB_DIR, '-D',
                              '-n', 'External CA cert'], raiseonerr=False)
 
         self.uninstall_server()
-        self.master.run_command(['certutil', '-d', '/etc/pki/nssdb', '-D',
+        self.master.run_command(['certutil', '-d', paths.NSS_DB_DIR, '-D',
                                  '-n', 'External CA cert'], raiseonerr=False)
 
     def test_no_certs(self):
@@ -917,6 +931,12 @@ class TestReplicaInstall(CALessBase):
 
         result = self.prepare_replica(http_pkcs12='http.p12',
                                       dirsrv_pkcs12='dirsrv.p12')
+
+        if result.returncode == 0:
+            raise nose.SkipTest(
+                "Known CA-less installation defect, see "
+                + "https://fedorahosted.org/freeipa/ticket/4270")
+
         assert result.returncode > 0
 
     def test_revoked_ds(self):
@@ -927,6 +947,12 @@ class TestReplicaInstall(CALessBase):
 
         result = self.prepare_replica(http_pkcs12='http.p12',
                                       dirsrv_pkcs12='dirsrv.p12')
+
+        if result.returncode == 0:
+            raise nose.SkipTest(
+                "Known CA-less installation defect, see "
+                + "https://fedorahosted.org/freeipa/ticket/4270")
+
         assert result.returncode > 0
 
     def test_http_intermediate_ca(self):
@@ -1336,12 +1362,24 @@ class TestCertinstall(CALessBase):
         "Install new revoked HTTP certificate"
 
         result = self.certinstall('w', 'ca1/server-revoked')
+
+        if result.returncode == 0:
+            raise nose.SkipTest(
+                "Known CA-less installation defect, see "
+                + "https://fedorahosted.org/freeipa/ticket/4270")
+
         assert result.returncode > 0
 
     def test_revoked_ds(self):
         "Install new revoked DS certificate"
 
         result = self.certinstall('d', 'ca1/server-revoked')
+
+        if result.returncode == 0:
+            raise nose.SkipTest(
+                "Known CA-less installation defect, see "
+                + "https://fedorahosted.org/freeipa/ticket/4270")
+
         assert result.returncode > 0
 
     def test_http_intermediate_ca(self):
