@@ -22,6 +22,7 @@ Delegation tests
 """
 
 from ipatests.test_webui.ui_driver import UI_driver
+from ipatests.test_webui.ui_driver import screenshot
 
 ENTITY = 'delegation'
 PKEY = 'itest-delegation-rule'
@@ -32,17 +33,18 @@ DATA = {
         ('textbox', 'aciname', PKEY),
         ('combobox', 'group', 'editors'),
         ('combobox', 'memberof', 'ipausers'),
-        ('table', 'attrs', 'audio'),
-        ('table', 'attrs', 'businesscategory'),
+        ('checkbox', 'attrs', 'audio'),
+        ('checkbox', 'attrs', 'businesscategory'),
     ],
     'mod': [
-        ('table', 'attrs', 'businesscategory'),
+        ('checkbox', 'attrs', 'businesscategory'),
     ],
 }
 
 
 class test_delegation(UI_driver):
 
+    @screenshot
     def test_crud(self):
         """
         Basic CRUD: delegation
