@@ -468,8 +468,8 @@ class test_role(Declarative):
             desc='Delete %r' % group1,
             command=('group_del', [group1], {}),
             expected=dict(
-                result=dict(failed=u''),
-                value=group1,
+                result=dict(failed=[]),
+                value=[group1],
                 summary=u'Deleted group "testgroup1"',
             )
         ),
@@ -554,8 +554,8 @@ class test_role(Declarative):
             desc='Delete %r' % role1,
             command=('role_del', [role1], {}),
             expected=dict(
-                result=dict(failed=u''),
-                value=role1,
+                result=dict(failed=[]),
+                value=[role1],
                 summary=u'Deleted role "%s"' % role1,
             )
         ),
@@ -604,8 +604,8 @@ class test_role(Declarative):
             desc='Delete %r' % role2,
             command=('role_del', [role2], {}),
             expected=dict(
-                result=dict(failed=u''),
-                value=role2,
+                result=dict(failed=[]),
+                value=[role2],
                 summary=u'Deleted role "%s"' % role2,
             )
         ),
