@@ -21,6 +21,7 @@
 Base class for all XML-RPC tests
 """
 
+import datetime
 import nose
 from ipatests.util import assert_deepequal, Fuzzy
 from ipalib import api, request, errors
@@ -38,6 +39,16 @@ uuid_re = '[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}'
 # Matches an ipauniqueid like u'784d85fd-eae7-11de-9d01-54520012478b'
 fuzzy_uuid = Fuzzy('^%s$' % uuid_re)
 
+# Matches an automember task DN
+fuzzy_automember_dn = Fuzzy(
+    '^cn=%s,cn=automember rebuild membership,cn=tasks,cn=config$' % uuid_re
+)
+
+# Matches an automember task finish message
+fuzzy_automember_message = Fuzzy(
+    '^Automember rebuild task finished\. Processed \(\d+\) entries\.$'
+)
+
 # Matches trusted domain GUID, like u'463bf2be-3456-4a57-979e-120304f2a0eb'
 fuzzy_guid = fuzzy_uuid
 
@@ -76,7 +87,7 @@ fuzzy_hex = Fuzzy('^0x[0-9a-fA-F]+$', type=basestring)
 fuzzy_password = Fuzzy('^\S([\S ]*\S)*$')
 
 # Matches generalized time value. Time format is: %Y%m%d%H%M%SZ
-fuzzy_dergeneralizedtime = Fuzzy('^[0-9]{14}Z$')
+fuzzy_dergeneralizedtime = Fuzzy(type=datetime.datetime)
 
 # match any string
 fuzzy_string = Fuzzy(type=basestring)
@@ -86,8 +97,8 @@ def fuzzy_set_ci(s):
     return Fuzzy(test=lambda other: set(x.lower() for x in other) == set(y.lower() for y in s))
 
 try:
-    if not api.Backend.xmlclient.isconnected():
-        api.Backend.xmlclient.connect(fallback=False)
+    if not api.Backend.rpcclient.isconnected():
+        api.Backend.rpcclient.connect(fallback=False)
     res = api.Command['user_show'](u'notfound')
 except errors.NetworkError:
     server_available = False
@@ -163,8 +174,8 @@ class XMLRPC_test(object):
                                 (cls.__module__, api.env.xmlrpc_uri))
 
     def setUp(self):
-        if not api.Backend.xmlclient.isconnected():
-            api.Backend.xmlclient.connect(fallback=False)
+        if not api.Backend.rpcclient.isconnected():
+            api.Backend.rpcclient.connect(fallback=False)
 
     def tearDown(self):
         """
@@ -241,6 +252,7 @@ class Declarative(XMLRPC_test):
     and must not fail.
     """
 
+    default_version = API_VERSION
     cleanup_commands = tuple()
     tests = tuple()
 
@@ -277,11 +289,18 @@ class Declarative(XMLRPC_test):
         # Iterate through the tests:
         name = self.__class__.__name__
         for (i, test) in enumerate(self.tests):
-            nice = '%s[%d]: %s: %s' % (
-                name, i, test['command'][0], test.get('desc', '')
-            )
-            func = lambda: self.check(nice, **test)
-            func.description = nice
+            if callable(test):
+                func = lambda: test(self)
+                nice = '%s[%d]: call %s: %s' % (
+                    name, i, test.__name__, test.__doc__
+                )
+                func.description = nice
+            else:
+                nice = '%s[%d]: %s: %s' % (
+                    name, i, test['command'][0], test.get('desc', '')
+                )
+                func = lambda: self.check(nice, **test)
+                func.description = nice
             yield (func,)
 
         # Iterate through post-cleanup:
@@ -290,7 +309,7 @@ class Declarative(XMLRPC_test):
 
     def check(self, nice, desc, command, expected, extra_check=None):
         (cmd, args, options) = command
-        options.setdefault('version', API_VERSION)
+        options.setdefault('version', self.default_version)
         if cmd not in api.Command:
             raise nose.SkipTest('%r not in api.Command' % cmd)
         if isinstance(expected, errors.PublicError):
